#! /bin/make
##############################################
# common make definitions for modules and samples
# supported CPUs : i386 and ppc
#===========================================
#        == DO NOT CHANGE !!! ==
#===========================================
##############################################
#---------------------------------------------
include $(mvIMPACT)/config.inc

##############################################
# for CYGWIN user
#---------------------------------------------
ifeq ($(OSTYPE),cygwin)
	OTYPEDEF=-D__cygwin__
	TARGET=ppc603e
	TARGET_SYST=$(OSTYPE)
else
	OTYPEDEF=
	TARGET_SYST=$(TARGET)
endif

##############################################
#---------------------------------------------
ifeq ($(TARGET),ppc603e)
	TARGETARCH:=ppc
else
	ifeq ($(TARGET),ppc823)
		TARGETARCH:=ppc
	else
		ifeq ($(TARGET),ppc_6xx_41)
			TARGETARCH:=ppc_6xx
		else
			TARGETARCH:=$(TARGET)
		endif
	endif
endif
TARGETDEFINE=-D__$(TARGET)__


##############################################
#---------------------------------------------
ifeq ($(TARGETARCH),ppc)
	POWERPC=powerpc-linux-

	DISTCC=$(shell which distcc 2>/dev/null)
	CCACHE=$(shell which ccache 2>/dev/null)

	#HAVE_IPKG:=yes

else
	ifeq ($(TARGETARCH),arm)
		POWERPC=arm-linux-
	endif

	ifeq ($(TARGETARCH),ppc_6xx)
		POWERPC=ppc_6xx-
		#HAVE_IPKG:=yes
	endif

	DISTCC=
	CCACHE=
endif


ifeq ($(CCACHE),)
	# ccache not found, just use distcc if found
	SPEED_PREFIX = $(DISTCC)

else
	# ccache found, use it
	ifneq ($(DISTCC),)
		# use distcc with ccache if found
		export CCACHE_PREFIX = distcc
	endif

	SPEED_PREFIX = ccache

endif


##############################################
#---------------------------------------------
GCCPATH=$(GCC_$(HOSTTYPE)X$(TARGET_SYST))

##############################################
#---------------------------------------------
TOOLS= $(SPEED_PREFIX) $(GCCPATH)/bin/$(POWERPC)

##############################################
#---------------------------------------------
ifeq ($(TARGETARCH),ppc)
	PGUI_INC_PATH=-I$(GCCPATH)/usr/include/picogui
	PPC_USER_INC_PATH=-I$(GCCPATH)/usr/include

	ADDITIONAL_USR_LIB_PATH=				\
		-L$(GCCPATH)/usr/lib
endif

ifeq ($(TARGETARCH),arm)
	PPC_USER_INC_PATH=-I$(GCCPATH)/usr/include

	ADDITIONAL_USR_LIB_PATH=				\
		-L$(GCCPATH)/usr/lib
endif

GLOBINCFILES=								\
	$(PPC_USER_INC_PATH)					\
	$(PGUI_INC_PATH)


############################################
# OPT
# ---
# To get debug-info set OPT=-g; with OPT=-O2
# you get the sources optimized; if OPT=-pg
# -a there will be generated a profiling
# file (bb.out).
#
# If TARGET=ppc you can set OPT=-mcpu=823
# or OPT=-msoft-float and use that way the
# softfloat - lib
#===========================================
ifeq ($(OPT),)
# optimization:
	OPT=-O2
# standard debugging:
#	OPT=-g
# extended debugging (macros etc), but beware: large output files:
#	OPT=-ggdb -g3
## change these options on command line, like "make OPT='-g' ", don't edit this file
## for that purpose


endif

############################################
# compile level options
#  -fPIC needed for SO, but not for the samples
#===========================================
COMP_OPTIONS=	\
	-Wall	\
	-W	\
	-Wno-unknown-pragmas

ifneq ($(TARGETARCH),ppc)
COMP_OPTIONS+=	\
	-Wno-strict-aliasing
endif

LIB_OPTIONS=	\
	-fPIC

#===========================================
UNUSED=				\
	-Werror			\
	-save-temps		\
	unused

############################################
CPPFLAGS_2=				\
	-fsigned-char		\
	$(TARGETDEFINE)		\
	$(OTYPEDEF)			\
	$(INCFILES)			\
   	$(OPT)				\
	$(COMP_OPTIONS)		\
	$(LIB_OPTIONS)		\
	$(CPPFLAGS_1)

############################################
# modifier for built-in rules
#===========================================
CC  = $(TOOLS)gcc
CXX = $(TOOLS)g++
AR	= ar

GCC_OK=$(shell if test -e $(GCCPATH)/bin/$(POWERPC)gcc; then echo "yes"; else echo "no"; fi )

############################################
.PHONY: new default info build $(TARGET)
#===========================================
default:	all

#-------------------------------------------
build: clean_mod all

#-------------------------------------------
new: clean_mod strip

#-------------------------------------------
$(TARGET_SYST):
#	mkdir -p $(TARGET)

#-------------------------------------------
$(TARGET_SYST)/%.o:	%.c
	$(CC) -c $(CPPFLAGS)	-o $@ $<

#-------------------------------------------
$(TARGET_SYST)/%.o:	%.cpp
	$(CXX)	-c $(CPPFLAGS) -o $@ $<

#-------------------------------------------
info:
	@echo -n "PWD         =" ; pwd
	@echo "OTSYPE         =" $(OSTYPE)
	@echo "TARGET         =" $(TARGET)
	@echo "TARGETARCH     =" $(TARGETARCH)
	@echo "IMPACT         =" $(IMPACT)
	@echo "TOOLS          =" $(TOOLS)
	@echo "INCFILES       =" $(INCFILES)
	@echo "CC             =" $(CC)
	@echo "CXX            =" $(CXX)
	@echo "TARGET_SYST    =" $(TARGET_SYST)
	@echo "GCCPATH        =" $(GCCPATH)
	@echo "PGUI_INC_PATH  =" $(PGUI_INC_PATH)
	@echo "IMPACTLIB_PATH =" $(IMPACTLIB_PATH)
	@echo "DISTCC         =" $(DISTCC)
	@echo "CCACHE         =" $(CCACHE)
	@echo "CCACHE_PREFIX  =" $(CCACHE_PREFIX)
	@echo "PUSER_LIB_PATH =" $(PUSER_LIB_PATH)
	@echo "PRVLIB =" $(PRVLIB)
	@echo "HAVELIB =" $(HAVELIB)
	@echo "LICMKFILE =" $(LICMKFILE)
	@echo "USER_AR_OBJ_LIST =" $(USER_AR_OBJ_LIST)
	@echo "IPKG_FOLDER =" $(IPKG_FOLDER)
	@echo "HAVE_IPKG =" $(HAVE_IPKG)

#-------------------------------------------
