// The property id's for the MatrixVision driver.

// These id's can be used with the DEV_InquireCaps and DEV_ControlCaps functions.
// They are implemented across all MatrixVision boards, but they are dynamic in
// nature. It is not guaranteed, that these properties are actually available. 

// The list of properties that are available for a particular board, along with
// their datatypes, can be inquired by DEV_InquireCapsList. Thus, you can inquire
// at run-time, which properties are actually available.

// Properties may be read/write, read only or write only. This cannot be inquired,
// but the respective DEV_InquireCaps/DEV_ControlCaps function will fail.

#include "vl_driver.h"

// properties implemented for mvIMPACT version 3.0
const int PID_INPUT_CHANNEL = DEV_INPUT_CHANNEL;
//const int PID_GAIN = -1;
//const int PID_OFFSET = -2;
const int PID_CLOCK_DELAY = -3;
const int PID_HUE = -4;
const int PID_SATURATION = -5;
const int PID_LUMA = -6;
const int PID_CONTRAST = -7;
//const int PID_SHUTTER = -8;
const int PID_SYNC_INPUT = -9;
//const int PID_INPUT_CLK = -10;
//const int PID_EXT_TRIG = -11;
const int PID_SERIAL = -12;
const int PID_REVISION = -13;
const int PID_BOARD_TYPE = -14;
const int PID_DRIVER_NAME = -15;
const int PID_DRIVER_REV = -16;
const int PID_DISP_MEM = -17;
const int PID_SYS_MEM = -18;
const int PID_AQR_MEM = -19;
const int PID_COMMAND = -20;
const int PID_DIG_VSYNC = -21;
const int PID_DIG_FIELD = -22;
const int PID_DIG_BUSY = -23;
const int PID_DIG_SNAP_READY = -24;
const int PID_DIG_LIVE = -25;
const int PID_DIG_VCOUNT = -26;
// the commented out properties in the block above have been removed
// with mvIMPACT version 4.2

// properties implemented for mvIMPACT version 4.2
const int PID_SNAP_WIDTH = -27;
const int PID_SNAP_HEIGHT = -28;
const int PID_SNAP_IMAGE_MAXIMIZED = -29;
const int PID_SNAP_SCALE = -30;
const int PID_VCR = -31;
const int PID_BRIGHTNESS = -32;
const int PID_CAMERA_NAME = -33;
const int PID_CAMERA_FIELD_TIME = -34;
const int PID_CAMERA_FIELDS_PER_FRAME = -35;
const int PID_CAMERA_WIDTH = -36;
const int PID_CAMERA_HEIGHT = -37;
const int PID_TRIGGER_ENABLE = -38;
const int PID_SNAP_START_MODE = -39;
const int PID_GREYSCALE_ACQUIRE = -40;
const int PID_LONG_INTEGRATION_ENABLE = -41;
const int PID_LONG_INTEGRATION_FIELDS = -42;
const int PID_GRABBER_TYPE = -43;
const int PID_BITS_PER_COMPONENT = -44;
const int PID_POSSIBLE_BITS_PER_COMPONENT = -45;
const int PID_AUTO_TRIGGER_AVAILABLE = -46;
const int PID_AUTO_TRIGGER_ENABLE = -47;
const int PID_AUTO_TRIGGER_PERIOD = -48;
const int PID_SHUTTER_CONTROL_AVAILABLE = -49;
const int PID_SHUTTER_CONTROL_ENABLE = -50;
const int PID_SHUTTER_MODE = -51;
const int PID_SHUTTER_TIME = -52;
const int PID_SHUTTER_HIGH_ACTIV = -53;
const int PID_SHUTTER_VSYNC_SYNCHRON = -54;
const int PID_SHUTTER_ACQUIRE_START_MODE = -55;
const int PID_EXPOSE_AVAILABLE = -56;
const int PID_EXPOSE_ENABLE = -57;
const int PID_EXPOSURE_MODE = -58;
const int PID_EXPOSE_TIME = -59;
const int PID_EXPOSE_TIME_REAL = -60;
const int PID_EXPOSE_PERIOD_TIME = -61;
const int PID_EXPOSE_HIGH_ACTIV = -62;
const int PID_EXPOSE_DIVIDER_AVAILABLE = -63;
const int PID_EXPOSE_DIVIDER = -64;
const int PID_EXPOSE_OUTPUT_LINE_AVAILABLE = -65;
const int PID_EXPOSE_OUTPUT_LINE = -66;
const int PID_SNAP_TIMEOUT = -67;
const int PID_TRIGGER_TIMEOUT = -68;

// Properties implemented for mvIMPACT version 5.2
const int PID_TRIGGER_MODE = -69;
const int PID_TRIGGER_AVAILABLE = -70;
const int PID_STARTSTOP_TRIGGER_AVAILABLE = -71;
const int PID_TRIGGER_RISING_EDGE_ACTIVE_AVAILABLE = -72;
const int PID_TRIGGER_RISING_EDGE_ACTIVE = -73;
const int PID_BRIGHTNESS_AVAILABLE = -74;
const int PID_CONTRAST_AVAILABLE = -75;
const int PID_COLOR_MODE = -76;
const int PID_EXPOSE_FLASH_CURRENT = -77;
const int PID_SIGNAL0_MASK = -78;
const int PID_SIGNAL1_MASK = -79;
const int PID_SIGNAL2_MASK = -80;
const int PID_SIGNAL3_MASK = -81;
const int PID_SIGNAL4_MASK = -82;
const int PID_SIGNAL5_MASK = -83;
const int PID_SIGNAL6_MASK = -84;
const int PID_SIGNAL7_MASK = -85;
const int PID_SIGNAL0_TIME = -86;
const int PID_SIGNAL1_TIME = -87;
const int PID_SIGNAL2_TIME = -88;
const int PID_SIGNAL3_TIME = -89;
const int PID_SIGNAL4_TIME = -90;
const int PID_SIGNAL5_TIME = -91;
const int PID_SIGNAL6_TIME = -92;
const int PID_SIGNAL7_TIME = -93;
const int PID_DIGITAL_INPUTS = -94;
const int PID_SHUTTER_SIGNAL0_OUTPUT = -95;
const int PID_SHUTTER_SIGNAL1_OUTPUT = -96;
const int PID_EXPOSE_FLASH_CURRENT_AVAILABLE = -97;
const int PID_SIGNAL_OUT = -98;

