// This is the master version include file.
// It is (to be) included in all projects.

// To create a new version:
// a) change one or all of the following constants for a different version (___v* and ___sv* must stay in sync)
// b) add a text entry in the version history comment below, explaining the use of the new version
// c) rebuild the projects.
#define ___v_major				6		// major version number, must fit in 4 bits, i.e. 0..15
#define ___sv_major				"6"
#define ___v_minor				6		// minor version number, must fit in 4 bits, i.e. 0..15
#define ___sv_minor				"6"
#define ___v_point				0		// point version number, must fit in 8 bits, i.e. 0..255
#define ___sv_point				"0"
#define ___v_build				5300	// build number, must fit in 16 bits, i.e. 0..65535
#define ___sv_build				"5300"
#define ___sv_version			"6.6.0.5300"
#define ___sv_copyright			"(c) Copyright 1995-2010: Impuls computergestützte Bildanalyse GmbH\0"
#define ___sv_company			"Impuls computergestützte Bildanalyse GmbH\0"
#define ___sv_filename			"\0"
#define ___sv_filedescription	"\0"
#define ___sv_internal			"\0"
#define ___sv_trademarks		"\0"
#define ___sv_productname		"\0"
#define ___sv_comment			"\0"


// This file defines the following public macros
//
// V_MAJOR		to access the major version number
// V_MINOR		to access the minor version number
// V_POINT		to access the point release version number
// V_BUILD		to access the build number
// V_STRING		to access a composite version number for the version resource
// V_TYPELIB	to access a composite version number for the type library
// V_STYPELIB	to access a composite version number for the type library in string form
// V_VERSION	to access a composite version number in a 32bit word
// V_SCOPYRIGHT	to access the copyright string
// V_SCOMPANY	to access the company name
//
// Don't use any user macros defined in this file, they are designed for internal
// use only.



//
// DONT CHANGE BELOW, UNLESS YOU HAVE A GOOD REASON
//

// combine the major and minor version components into one number
#define ___v_tl(major, minor)	major##.##minor
// combine the major and minor version components into one string
#define ___sv_tl(major, minor)	major##"."##minor
// use this macro to get an expanded macro string, such as "3, 0, 0, 4355\0"
#define V_STRING	___sv_version
// use this macro to get a version number for typelibs, such as 3.0
#define V_TYPELIB	___v_tl(___v_major, ___v_minor)
// use this macro to get a version number string for typelibs, such as "3.0"
#define V_STYPELIB	___sv_tl(___sv_major, ___sv_minor)

// for programmatic access
// the BUILD is in the high WORD, the version is in the low WORD
#define V_VERSION	((___v_build << 16) | (___v_major << 12) | (___v_minor << 8) | ___v_point)

// for component access
#define V_MAJOR	___v_major
#define V_MINOR	___v_minor
#define V_POINT	___v_point
#define V_BUILD	___v_build

// check parameter ranges
#if ___v_major < 0
	#error Major version number out of range
#elif ___v_major > 15
	#error Major version number out of range
#endif

#if ___v_minor < 0
	#error Minor version number out of range
#elif ___v_minor > 15
	#error Minor version number out of range
#endif

#if ___v_point < 0
	#error Point version number out of range
#elif ___v_point > 255
	#error Point version number out of range
#endif

#if ___v_build < 0
	#error Build number out of range
#elif ___v_build > 65535
	#error Build number out of range
#endif

#ifndef V_SCOMMENT
#define V_SCOMMENT		___sv_comment
#endif

#ifndef V_SCOMPANY
#define V_SCOMPANY		___sv_company
#endif

#ifndef V_SFILEDESCRIPTION
#define V_SFILEDESCRIPTION	___sv_filedescription
#endif

#ifndef V_SINTERNAL
#define V_SINTERNAL		___sv_internal
#endif

#ifndef V_SCOPYRIGHT
#define V_SCOPYRIGHT	___sv_copyright
#endif

#ifndef V_STRADEMARKS
#define V_STRADEMARKS	___sv_trademarks
#endif

#ifndef V_SFILENAME
#define V_SFILENAME		___sv_filename
#endif

#ifndef V_SPRODUCTNAME
#define V_SPRODUCTNAME	___sv_productname
#endif

#ifndef VS_VERSION_INFO
#define VL_VERSION_INFO 1
#endif


