//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \file pgtools.h
 *  \brief MATRIX VISION GmbH
 *
 * $Id: pgtools.h,v 1.5 2004/10/20 09:07:53 td Exp $
 *
 *
 */

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#ifndef __pgtools_h__
#define __pgtools_h__

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <mk_msg.h>

#include <picogui.h>
#include <system.h>

//-------------------------------------------------------------------
//-------------------------------------------------------------------
#define PGT_MSG_SIZE								80
#define PGT_MAX_CHAR_SIZE							256

#define PGT_COLOR_WHITE								PALETTERGB( 255, 255, 255 )
#define PGT_COLOR_GREY								PALETTERGB( 127, 127, 127 )
#define PGT_COLOR_BLACK								PALETTERGB( 0, 0, 0 )
#define PGT_COLOR_RED								PALETTERGB( 255, 0, 0 )
#define PGT_COLOR_GREEN								PALETTERGB( 0, 255, 0 )
#define PGT_COLOR_BLUE								PALETTERGB( 0, 0, 255 )
#define PGT_COLOR_YELLOW							PALETTERGB( 255, 255, 0 )
#define PGT_COLOR_MAGENTA							PALETTERGB( 255, 0, 255 )
#define PGT_COLOR_L_BLUE							PALETTERGB( 0, 255, 255 )

#define PGT_PREFIX_4_MSG_DATA						"MSG_DATA"

//-------------------------------------------------------------------
#define PGT_FCT_START( b )			make_message( b, " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ );
#define PGT_FCT_END( b, r )			make_message( b, " -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, r );

#define PGT_PERR( s )				make_message( 1, "*** %s (%s: %d) - %s!!\n", __FUNCTION__, __FILE__, __LINE__, s );
#define PGT_PERR_CODE( s, c )		make_message( 1, "*** %s (%s: %d) - %s (err: %d)!!\n", __FUNCTION__, __FILE__, __LINE__, s, c );


//-------------------------------------------------------------------
//-------------------------------------------------------------------
typedef unsigned char BYTE;

enum EPGTFileTypes
{
	PGT_FT_FIRST = 0,
	PGT_FT_INI,
	PGT_FT_DDC,
	PGT_FT_PGM,
	PGT_FT_LAST,
};
typedef enum EPGTFileTypes PGT_FL_TYPES;

//-------------------------------------------------------------------
//-------------------------------------------------------------------
struct SPGTStatusInfo
{
	pghandle wBox;
	pghandle wText;
	char *pMsg;
	int MsgLen;
	pghandle wMsgCol;
};
typedef struct SPGTStatusInfo PGT_STATUS_INFO;

struct SPGTFileType
{
	//PGT_FL_TYPES type;
	char sufix[4];
};
typedef struct SPGTFileType PGT_FILE_TYPES;

//-------------------------------------------------------------------
//-------------------------------------------------------------------
class CPGTools
{
public:
	/* ------------------------------------------------------- */
	CPGTools( )
	{
		PGT_FCT_START( DBG_START_END );

		sprintf( m_pgt_ft[PGT_FT_INI].sufix, ".ini" );
		sprintf( m_pgt_ft[PGT_FT_DDC].sufix, ".ddc" );
		sprintf( m_pgt_ft[PGT_FT_PGM].sufix, ".pgm" );

		PGT_FCT_END( DBG_START_END, 0 );
	};

	/* ------------------------------------------------------- */
	virtual ~CPGTools( )
	{
		PGT_FCT_START( DBG_START_END );
		PGT_FCT_END( DBG_START_END, 0 );
	};

	/* ------------------------------------------------------- */
	void PGT_NotImplemented( )
	{
		pgMessageDialog( "NOTE", "\nThis functionality is not implemented yet!\n\nBut it soon will be :o)\n", PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	};

	/* ------------------------------------------------------- */
	void PGT_WDelete( pghandle& widget );

	int PGT_mkPopupMenu( const pghandle wCanvas, const char *pMenuEntrys, const unsigned int EntryCnt );

	int PGT_CpyImg( const BYTE *pSource, BYTE *PDestination, const unsigned long width,
						const unsigned long height, const unsigned long SrcPitch, const unsigned long DstPitch );

	int PGT_CpyChldImg( const BYTE *pSource, BYTE *pDestination, const unsigned long SrcX, const unsigned long SrcY,
						const unsigned long width, const unsigned long height, const unsigned long SrcPitch,
						const unsigned long DstPitch );

	/* */
	int PGT_ReplaceSIText( PGT_STATUS_INFO si );
	int PGT_mkStatBarLab( const pghandle wParent, const int position, PGT_STATUS_INFO& si );
	pghandle PGT_mkStatBarLab( const pghandle wParent, const int position, pghandle& wText,
						const char *pStr, int size );

	int PGT_LSIUpdate( PGT_STATUS_INFO& si, const char *pMsgPrefix, const char *pMsgSufix,
							const long value, const int ColorMode, const int MsgLen );
	int PGT_DSIUpdate( PGT_STATUS_INFO& si, const char *pMsgPrefix, const char *pMsgSufix,
							const double value, const int ColorMode, const int MsgLen );
	int PGT_CSIUpdate( PGT_STATUS_INFO& si, const char *pMsgPrefix, const char *pMsgSufix, const char *pData,
							const int ColorMode, const int MsgLen );

	/* */
	int PGT_ReplaceWText( pghandle widget, const char *pText );

	/* */
	int  PGT_DrawRect( const pgcontext gc, const long x, const long y, const long w, const long h, const pgcolor wColor );
	int  PGT_DrawLine( const pgcontext gc, const long x0, const long y0, const long x1, const long y1, const pgcolor wColor );
	int  PGT_DrawPoint( const pgcontext gc, const long x, const long y, const pgcolor wColor );
	int  PGT_DrawROI( const pgcontext gc, const long x, const long y, const long w, const long h, const pgcolor wColor );
	int  PGT_DrawText( const pgcontext gc, const long x, const long y, const char *pText, const pgcolor wColor );

	void PGT_DrawRect( const pghandle wCanvas, const long x, const long y, const long w, const long h, const pghandle wColor );
	void PGT_DrawLine( const pghandle wCanvas, const long x0, const long y0, const long x1, const long y1, const pghandle wColor );
	void PGT_DrawPoint( const pghandle wCanvas, const long x, const long y, const pghandle wColor );
	void PGT_DrawROI( const pghandle wCanvas, const long x, const long y, const long w, const long h, const pghandle wColor );

	/* */
	int PGT_GUINoteOrErrMsg( const char *pMsgType, const char *pS, const long ErrCode );
	int PGT_GUINoteOrErrMsg( const char *pMsgType, const char *pS, const char *pErrStr, const long ErrCode );
	int PGT_GUIErrMsg( const char *pS, const long ErrCode );
	int PGT_GUIErrMsg( const char *pS, const char *pErrStr, const long ErrCode );
	int PGT_GUIExitApplication( const char *pS, const long ErrCode );
	int PGT_GUIExitApplication( const char *pS, const char *pErrStr, const long ErrCode );
	int PGT_GUINote4Limit( const char *pType, const long value );

	/* */
	pghandle PGT_mkButton( const pghandle wParent, const int position, const char *pText,
						const int size, pgevthandler evtFctHandle );
	pghandle PGT_mkButton( const pghandle wParent, const int position, const char *pText, pgevthandler evtFctHandle,
						const pghandle wColor, const int FontSize );
	pghandle PGT_mkCheckBox( const pghandle wParent, const int position,
						const char *pText, pgevthandler evtFctHandle, int status );
	pghandle PGT_mkLabel( const pghandle wParent, const int position, const char *pText, pghandle wColor );
	pghandle PGT_mkLabel( const pghandle wParent, const int position, const int FontSize, const char *pText );
	pghandle PGT_mkScroller( const pghandle wParent, const int position, const long MaxSize, const long DefaultValue,
						pgevthandler evtFctHandle );
	pghandle PGT_mkScrollerLabel( const pghandle wParent, int position, const char *pText );
	pghandle PGT_mkWBox( const pghandle wParent, const int position, const int side, const int size );
	pghandle PGT_mkStatusBar( const pghandle wParent, const int position, const int size );
	pghandle PGT_GetStatusColor( const long status );
	pghandle PGT_mkCanvas( const pghandle wParent, const int position, pgevthandler evtFctHandle, const bool bUseTrigg );

	/* ------------------------------------------------------- */
	pgcontext PGT_mkContext( const pghandle wCanvas, const int type )
	{
		PGT_FCT_START( DBG_START_END );

		pgcontext gcRet = NULL;
		if( wCanvas )
		{
			pgWriteCmd( wCanvas, PGCANVAS_NUKE, 0 );
			gcRet = pgNewCanvasContext( wCanvas, type/*PGFX_IMMEDIATE*/ );
		}

		PGT_FCT_END( DBG_START_END, 0 );
		return gcRet;
	}

	/* ------------------------------------------------------- */
	void PGT_rmContext( pgcontext& gcContext )
	{
		PGT_FCT_START( DBG_START_END );

		if( gcContext != NULL )
		{
			pgDeleteContext( gcContext );
			gcContext = NULL;
		}

		PGT_FCT_END( DBG_START_END, 0 );
	}

	/* ------------------------------------------------------- */
	int PGT_GetStrSize( const pghandle wFont, const char *pStr, long& x, long& y );
	int PGT_GetStrSize( const char *pStr, long& x, long& y );
	int PGT_GetStrXSize( const char *pStr );
	int PGT_GetStrYSize( const char *pStr );

	int PGT_SetWColor( const pghandle widget, const pghandle wColor );
	int PGT_EnableWButton( const pghandle wButton, const bool OnOff );


	bool PGT_CheckIfFileExist( const char *pFileName );

	int PGT_LoadFile( char *pFileName );
	int PGT_SavePGMFile( const BYTE *pDataBuf, const FILE *pF, const long FileSize );
	int PGT_SaveFile( const BYTE *pDataBuf, const char *pFileName, const char *pFileType, const long FileSize );
	int PGT_SaveFile( const BYTE *pDataBuf, const char *pFileName, const PGT_FL_TYPES FileType, const long FileSize );

	int PGT_mkPGMFile( const BYTE *pSource, BYTE **pDestination, const unsigned long width,
					unsigned long height, unsigned long pitch, unsigned long bpp );


	/* */
	PGT_FILE_TYPES m_pgt_ft[PGT_FT_LAST];

	/* */
	virtual int PGT_CloseAll( ) {  return 0; };
	virtual int PGT_SaveINIFile( const FILE * /*pF*/ ) { return 0; };
	virtual int PGT_ReadINIFile( const FILE * /*pF*/ ) { return 0; };
	virtual int PGT_SaveDDCFile( const FILE * /*pF*/ ) { return 0; };
	virtual int PGT_ReadDDCFile( const FILE * /*pF*/ ) { return 0; };

	/* ------------------------------------------------------- */
	double PGT_Changetv2ms( struct timeval tv )
	{
		dprintf((" ==> %s - tv_sec: %ld, tv_usec: %ld\n", __FUNCTION__, tv.tv_sec, tv.tv_usec ));
		return (double)(tv.tv_sec * 1000. + tv.tv_usec/1000.);
	};

	/* ------------------------------------------------------- */
private:
	char m_pgt_msg[PGT_MAX_CHAR_SIZE];
	void pgt_CleanMsgBuf( ) { memset( m_pgt_msg, 0, PGT_MAX_CHAR_SIZE * sizeof(char) ); };

	int pgt_bcMenuFromArray( const pghandle wCanvas, const pghandle *pItems, const unsigned int NumItems );
	PGT_FL_TYPES pgt_GetFlType( const char *pFileType );

	/* ------------------------------------------------------- */
	int PGT_PGMHeader( char *pBuf, const long width, const long height )
	{
		PGT_FCT_START( DBG_START_END );
		int ret = -1;

		if( pBuf )
		{
			dprintf((" == %s - width: %ld height: %ld\n", __FUNCTION__, width, height ));
			sprintf( pBuf, "P5\n# Created by mvBlueLXNY-MATRIX VISION GmbH\n%ld\n%ld\n255\n", width, height );
		}

		PGT_FCT_END( DBG_START_END, ret );
		return ret;
	}

	/* ------------------------------------------------------- */

protected:


};

//-------------------------------------------------------------------
//-------------------------------------------------------------------
#endif	// __pgtools_h__

