#ifndef MVIMPACT_DATAMATRIX_IMPL_H_
#define MVIMPACT_DATAMATRIX_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace datamatrix {



inline DatamatrixConfiguration& DatamatrixConfiguration::setSpacing(
		int value) 
	{
		if (!mvIMPACT_C::CR2D_Control(getHandle(), mvIMPACT_C::CR2D_CFG_SPACING, value))
			throwImpactError();
		return *this;
	}

inline int DatamatrixConfiguration::getSpacing() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_CFG_SPACING, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline DatamatrixConfiguration& DatamatrixConfiguration::setContrast(
		int value) 
	{
		if (!mvIMPACT_C::CR2D_Control(getHandle(), mvIMPACT_C::CR2D_CFG_CONTRAST, value))
			throwImpactError();
		return *this;
	}

inline int DatamatrixConfiguration::getContrast() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_CFG_CONTRAST, 0, 0, &value))
			throwImpactError();
		return value;
	}


inline DatamatrixConfiguration& DatamatrixConfiguration::setBlackOnWhite(
		bool value) 
	{
		mvIMPACT_C::IPL_BOOL value_ = value ? IPL_TRUE : IPL_FALSE;
		if (!mvIMPACT_C::CR2D_Control(getHandle(), mvIMPACT_C::CR2D_CFG_MODE_BLACK_ON_WHITE, value_))
			throwImpactError();
		return *this;
	}

inline bool DatamatrixConfiguration::getBlackOnWhite() const
	{
		mvIMPACT_C::IPL_BOOL value = IPL_FALSE;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_CFG_MODE_BLACK_ON_WHITE, 0, 0, &value))
			throwImpactError();
		return value ? true : false;
	}

inline DatamatrixConfiguration& DatamatrixConfiguration::setTrimming(
		GridTrimming value) 
	{
		mvIMPACT_C::CR2D_TRIMMING_STRATEGY value_(static_cast<mvIMPACT_C::CR2D_TRIMMING_STRATEGY>(value));
		if (!mvIMPACT_C::CR2D_Control(getHandle(), mvIMPACT_C::CR2D_CFG_TRIMMING_STRATEGY , value_))
			throwImpactError();
		return *this;
	}

inline GridTrimming DatamatrixConfiguration::getTrimming() const
	{
		mvIMPACT_C::CR2D_TRIMMING_STRATEGY value = mvIMPACT_C::CR2D_TRIM_3_ADJ_PLUS;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_CFG_TRIMMING_STRATEGY , 0, 0, &value))
			throwImpactError();
		return GridTrimming(value);
	}

inline void DatamatrixConfiguration::setCheckPrintQuality(
	bool value) 
{
	mvIMPACT_C::IPL_BOOL value_ = value ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::CR2D_Control(getHandle(), mvIMPACT_C::CR2D_CFG_CHECK_PRINT_QUALITY, value_))
		throwImpactError();
}

inline bool DatamatrixConfiguration::getCheckPrintQuality() const
{
	mvIMPACT_C::IPL_BOOL value = 0;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_CFG_CHECK_PRINT_QUALITY, 0, 0, &value))
		throwImpactError();
	return value ? true : false;
}

inline DatamatrixConfiguration::DatamatrixConfiguration() 
		: handle_(unassigned), counter_(new int(1))
	{ 
		assert(unassigned == getHandle());
		if (!mvIMPACT_C::CR2D_BufAlloc(&handle_))
			throwImpactError();
		assert(unassigned != getHandle());
	}

inline DatamatrixConfiguration::DatamatrixConfiguration(const DatamatrixConfiguration& r) 
	{
		copy(r);
	}

inline DatamatrixConfiguration& DatamatrixConfiguration::operator=(
		const DatamatrixConfiguration& r)
	{ 
		if (&r != this)	
		{
			dispose();
			copy(r);
		}
		return *this;
	}

inline DatamatrixConfiguration::~DatamatrixConfiguration()
	{
		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}

inline mvIMPACT_C::CR2D_BUFHANDLE DatamatrixConfiguration::getHandle  () const { return handle_; }

inline int DatamatrixConfiguration::getUseCount() const { return *counter_; }

inline bool DatamatrixConfiguration::getIsUnique   () const { return 1 == *counter_; }

inline DatamatrixConfiguration::DatamatrixConfiguration(
		mvIMPACT_C::CR2D_BUFHANDLE bufhandle) 
		: handle_(-1), counter_(new int(1))
	{
		assert(unassigned == getHandle());
		handle_ = bufhandle;
		assert(unassigned != getHandle());
	}


inline void DatamatrixConfiguration::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			if (unassigned != getHandle())
			{
				assert(unassigned != getHandle());
				if (!mvIMPACT_C::CR2D_BufFree(getHandle())) throwImpactError();
				handle_ = unassigned;
				assert(unassigned == getHandle());
			}
		}
	}

inline void DatamatrixConfiguration::copy(const DatamatrixConfiguration& src)
{
	handle_ = src.handle_;
	counter_ = src.counter_;
	
	(*counter_)++;
}



inline int DatamatrixResults::getNumberOfCandidates() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_NUMBER_OF_CANDIDATES, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline int DatamatrixResults::getNumberOfCodes(int candidate) const
	{
		int value = 0;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_NUMBER_OF_CODES, candidate, 0, &value))
			throwImpactError();
		return value;
	}

inline std::string DatamatrixResults::getCodeType(int candidate, int code) const
	{
		mvIMPACT_C::IPL_ULONG length = 0;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_CODETYPE_LENGTH, candidate, code, &length))
			throwImpactError();
		DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (char)0);
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_CODETYPE, candidate, code, value.getPointer()))
			throwImpactError();
		return value.getPointer();
	}

inline std::string DatamatrixResults::getCode(int candidate, int code) const
	{
		mvIMPACT_C::IPL_ULONG length = 0;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_CODEDATA_LENGTH, candidate, code, &length))
			throwImpactError();
		DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_CODEDATA, candidate, code, value.getPointer()))
			throwImpactError();
		return value.getPointer();
	}

inline Size DatamatrixResults::getDatamatrixDims(int candidate, int code) const
	{
		int width; 
		int height;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_DM_COLUMNS, candidate, code, &width))
			throwImpactError();
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_DM_ROWS, candidate, code, &height))
			throwImpactError();
		return Size(width, height);
	}

inline bool DatamatrixResults::getFlipped(int candidate, int code) const
{
	mvIMPACT_C::IPL_BOOL mirror = IPL_FALSE;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_DM_MIRROR, candidate, code, &mirror))
		throwImpactError();
	return mirror ? true : false;
}

inline int DatamatrixResults::getNumberOfErrors(int candidate, int code) const
	{
		int value;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_DM_ERRORS, candidate, code, &value))
			throwImpactError();
		return value;
	}

inline double DatamatrixResults::getReadabilityScore(int candidate, int code) const
	{
		double value;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_READABILITY_SCORE, candidate, code, &value))
			throwImpactError();
		return value;
	}

inline bool DatamatrixResults::getIsBlackOnWhite(int candidate, int code) const
	{
		mvIMPACT_C::IPL_BOOL value;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_IS_BLACK_ON_WHITE, candidate, code, &value))
			throwImpactError();
		return value ? true : false;
	}

inline std::vector<Point> DatamatrixResults::getLocation(int candidate) const
	{
		DynArray<mvIMPACT_C::IPL_POINT, Point> iplResult(4);
		if (!mvIMPACT_C::CR2D_Inquire(
				getHandle(),
				mvIMPACT_C::CR2D_GET_LOCATION,
				candidate,
				0,
				iplResult.getPointer()))
			throwImpactError();
		return iplResult.asVector();
	}

inline double DatamatrixResults::getTotalDecodingTime () const
	{
		mvIMPACT_C::IPL_DOUBLE t;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_TOTAL_DECODING_TIME_AS_DOUBLE, 0, 0, &t))
			throwImpactError();
		return t;
	}

inline double DatamatrixResults::getDecodingTime (int candidate) const
	{
		mvIMPACT_C::IPL_DOUBLE t;
		if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_DECODING_TIME_AS_DOUBLE, candidate, 0, &t))
			throwImpactError();
		return t;
	}

inline double DatamatrixResults::getDecodabilityGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_DECODABILITY_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline int DatamatrixResults::getReflectanceMinimum (int candidate) const
{
	mvIMPACT_C::IPL_ULONG t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_REFLECTANCE_MINIMUM, candidate, 0, &t))
		throwImpactError();
	return (int)t;
}

inline int DatamatrixResults::getReflectanceMaximum (int candidate) const
{
	mvIMPACT_C::IPL_ULONG t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_REFLECTANCE_MAXIMUM, candidate, 0, &t))
		throwImpactError();
	return (int)t;
}

inline double DatamatrixResults::getReflectanceDark (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_REFLECTANCE_DARK, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getReflectanceBright (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_REFLECTANCE_BRIGHT, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getSymbolContrast (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_SYMBOL_CONTRAST, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getSymbolContrastGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_SYMBOL_CONTRAST_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getModulation (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_MODULATION, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getModulationGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_MODULATION_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getFixedPatternDamage (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_FIXED_PATTERN_DAMAGE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getFixedPatternDamageGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_FIXED_PATTERN_DAMAGE_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getAxialNonuniformity (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_AXIAL_NONUNIFORMITY, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getAxialNonuniformityGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_AXIAL_NONUNIFORMITY_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getGridNonuniformity (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_GRID_NONUNIFORMITY, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getGridNonuniformityGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_GRID_NONUNIFORMITY_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getUnusedErrorCorrection (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_UNUSED_ERROR_CORRECTION, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getUnusedErrorCorrectionGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_UNUSED_ERROR_CORRECTION_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline double DatamatrixResults::getScanGrade (int candidate) const
{
	mvIMPACT_C::IPL_DOUBLE t;
	if (!mvIMPACT_C::CR2D_Inquire(getHandle(), mvIMPACT_C::CR2D_GET_SCAN_GRADE, candidate, 0, &t))
		throwImpactError();
	return (double)t;
}

inline DatamatrixResults::DatamatrixResults() 
		: handle_(unassigned), counter_(new int(1))
	{ 
	}

inline DatamatrixResults::DatamatrixResults(const DatamatrixResults& r) 
	{
		copy(r);
	}

inline DatamatrixResults& DatamatrixResults::operator=(const DatamatrixResults& r)
	{ 
		// share ownership
		if (&r != this)	
			share(r.getHandle(), r.counter_); 
		return *this;
	}


inline DatamatrixResults::~DatamatrixResults()
	{
		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}

inline mvIMPACT_C::CR2D_BUFHANDLE DatamatrixResults::getHandle  () const 
	{ return handle_; }

inline int DatamatrixResults::getUseCount() const 
	{ return *counter_; }

inline bool DatamatrixResults::getIsUnique   () const { return 1 == *counter_; }

inline DatamatrixResults::DatamatrixResults(
		mvIMPACT_C::CR2D_BUFHANDLE bufhandle) 
		: handle_(unassigned), counter_(new int(1))
	{
		assert(unassigned == getHandle());
		handle_ = bufhandle;
		assert(unassigned != getHandle());
	}

inline void DatamatrixResults::addref() const
	{
		++*counter_;
	}

inline void DatamatrixResults::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			if (unassigned != getHandle())
			{
				assert(unassigned != getHandle());
				if (!mvIMPACT_C::CR2D_BufFree(getHandle()))
					throwImpactError();
				handle_ = unassigned;
				assert(unassigned == getHandle());
			}
		}
	}

inline void DatamatrixResults::share(mvIMPACT_C::CR2D_BUFHANDLE rhandle, int *r_counter)
	{
		if (counter_ != r_counter)
		{
			if(unassigned != getHandle()) { dispose(); }
			// if share() is called from the copy constructor via operator=(), the
			// object can be unassigned, and we shouldn't call dispose() on an
			// unassigned object
			handle_ = rhandle;
			++*(counter_ = r_counter);
		}
	}


inline void DatamatrixResults::copy(const DatamatrixResults& src)
{
	counter_ = src.counter_;
	handle_ = src.handle_;
	
	(*counter_)++;
}


inline DatamatrixResults DatamatrixResults::datamatrixResultsFactory(
	mvIMPACT_C::CR2D_BUFHANDLE buffer_handle) 
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == buffer_handle)
		return DatamatrixResults();

	return DatamatrixResults(buffer_handle);
}





inline DatamatrixResults decodeDM(
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_Decode(S.getHandle(), &C, &D))
		throwImpactError();
	return DatamatrixResults::datamatrixResultsFactory(D);
}

#ifndef I386_LINUX
inline DatamatrixResults decodeDM(
	const Image& S, 
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_DecodeAndVisualizeRuns(
			S.getHandle(), &C, &D, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	return DatamatrixResults::datamatrixResultsFactory(D);
}
#endif // !I386_LINUX^M

inline void decodeDM(
	DatamatrixResults& D, 
	const Image& S,
	const DatamatrixConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR2D_Decode(S.getHandle(), &C, &hD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
}

#ifndef I386_LINUX
inline void decodeDM(
	DatamatrixResults& D, 
	const Image& S,
	const DatamatrixConfiguration& config, 
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_DecodeAndVisualizeRuns(
			S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
}
#endif // !I386_LINUX^M

inline DatamatrixResults decodeDM(
	const Image& S,
	const DatamatrixConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_Decode(S.getHandle(), &C, &D))
		throwImpactError();
	return DatamatrixResults::datamatrixResultsFactory(D);
}

#ifndef I386_LINUX
inline DatamatrixResults decodeDM(
	const Image& S,
	const DatamatrixConfiguration& config, 
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_DecodeAndVisualizeRuns(
			S.getHandle(), &C, &D, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	return DatamatrixResults::datamatrixResultsFactory(D);
}
#endif // !I386_LINUX^M

inline void decodeDM(
	DatamatrixResults& D,
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR2D_Decode(S.getHandle(), &C, &hD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
}

#ifndef I386_LINUX
inline void decodeDM(
	DatamatrixResults& D,
	const Image& S,
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_DecodeAndVisualizeRuns(
			S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
}
#endif // !I386_LINUX^M

inline DatamatrixResults decodeDMAt(
	const Image& S,
	const std::vector<Point>& location) 
//-----------------------------------------------------------------------------
{
	if (location.size() < 4)
		throwImpactException("Parameter location holds less than 4 points.");

	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	DynArray<mvIMPACT_C::IPL_POINT,Point> iplpoints(location);

	if (!mvIMPACT_C::CR2D_DecodeAtLocation(S.getHandle(), iplpoints.getPointer(), &C, &D))
		throwImpactError();

	return DatamatrixResults::datamatrixResultsFactory(D);
}


inline void decodeDMAt(
	DatamatrixResults& D,
	const Image& S,
	const std::vector<Point>& location) 
//-----------------------------------------------------------------------------
{
	if (location.size() < 4)
		throwImpactException("Parameter location holds less than 4 points.");

	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	DynArray<mvIMPACT_C::IPL_POINT,Point> iplpoints(location);

	if (!mvIMPACT_C::CR2D_DecodeAtLocation(S.getHandle(), iplpoints.getPointer(), &C, &hD))
		throwImpactError();

	if (hD != D.getHandle()) 
		D = DatamatrixResults::datamatrixResultsFactory(hD);
}


inline DatamatrixResults decodeDMAt(
	const Image& S,
	const DatamatrixConfiguration& config,
	const std::vector<Point>& location)  
//-----------------------------------------------------------------------------
{
	if (location.size() < 4)
		throwImpactException("Parameter location holds less than 4 points.");

	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	DynArray<mvIMPACT_C::IPL_POINT,Point> iplpoints(location);

	if (!mvIMPACT_C::CR2D_DecodeAtLocation(S.getHandle(), iplpoints.getPointer(), &C, &D))
		throwImpactError();

	return DatamatrixResults::datamatrixResultsFactory(D);
}


inline void decodeDMAt(
	DatamatrixResults& D, 
	const Image& S,
	const DatamatrixConfiguration& config,
	const std::vector<Point>& location)  
//-----------------------------------------------------------------------------
{
	if (location.size() < 4)
		throwImpactException("Parameter location holds less than 4 points.");

	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	DynArray<mvIMPACT_C::IPL_POINT,Point> iplpoints(location);

	if (!mvIMPACT_C::CR2D_DecodeAtLocation(S.getHandle(), iplpoints.getPointer(), &C, &hD))
		throwImpactError();

	if (hD != D.getHandle()) 
		D = DatamatrixResults::datamatrixResultsFactory(hD);
}

inline DatamatrixResults decodeDMBitmap(
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;

	if (!mvIMPACT_C::CR2D_DecodeBitmap(S.getHandle(), &C, &D))
		throwImpactError();

	return DatamatrixResults::datamatrixResultsFactory(D);
}

inline void decodeDMBitmap(
	DatamatrixResults& D,
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();

	if (!mvIMPACT_C::CR2D_DecodeBitmap(S.getHandle(), &C, &hD))
		throwImpactError();

	if (hD != D.getHandle()) 
		D = DatamatrixResults::datamatrixResultsFactory(hD);
}

inline DatamatrixResults decodeDMBitmap(
	const Image& S,
	const DatamatrixConfiguration& config)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;

	if (!mvIMPACT_C::CR2D_DecodeBitmap(S.getHandle(), &C, &D))
		throwImpactError();

	return DatamatrixResults::datamatrixResultsFactory(D);
}

inline void decodeDMBitmap(
	DatamatrixResults& D, 
	const Image& S,
	const DatamatrixConfiguration& config)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();

	if (!mvIMPACT_C::CR2D_DecodeBitmap(S.getHandle(), &C, &hD))
		throwImpactError();

	if (hD != D.getHandle()) 
		D = DatamatrixResults::datamatrixResultsFactory(hD);
}

inline DatamatrixResults locateDM(
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_Locate(S.getHandle(), &C, &D))
		throwImpactError();
	return DatamatrixResults::datamatrixResultsFactory(D);
}

#ifndef I386_LINUX
inline DatamatrixResults locateDM(
	const Image& S, 
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &D, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	return DatamatrixResults::datamatrixResultsFactory(D);
}
#endif // !I386_LINUX^M

inline void locateDM(
	DatamatrixResults& D, 
	const Image& S,
	const DatamatrixConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR2D_Locate(S.getHandle(), &C, &hD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
		
}

#ifndef I386_LINUX
inline void locateDM(
	DatamatrixResults& D, 
	const Image& S,
	const DatamatrixConfiguration& config, 
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
}
#endif // !I386_LINUX^M

inline DatamatrixResults locateDM(
	const Image& S,
	const DatamatrixConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_Locate(S.getHandle(), &C, &D))
		throwImpactError();
	return DatamatrixResults::datamatrixResultsFactory(D);
}

#ifndef I386_LINUX
inline DatamatrixResults locateDM(
	const Image& S,
	const DatamatrixConfiguration& config, 
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &D, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	return DatamatrixResults::datamatrixResultsFactory(D);
}
#endif // !I386_LINUX^M

inline void locateDM(
	DatamatrixResults& D,
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR2D_Locate(S.getHandle(), &C, &hD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
}

#ifndef I386_LINUX
inline void locateDM(
	DatamatrixResults& D,
	const Image& S, 
	Window& W,
	LineArrayOfInterest& B, 
	LineArrayOfInterest& Q) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hBusy = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hQuiet = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hBusy, &hQuiet, &W))
		throwImpactError();

	LineArrayOfInterest busy(W, hBusy);
	B = busy;

	LineArrayOfInterest quiet(W, hQuiet);
	Q = quiet;

	if(hD != D.getHandle()) {
		D = DatamatrixResults::datamatrixResultsFactory(hD);
	}
}
#endif // !I386_LINUX^M


inline Image edgeMapDM(
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_EdgeMap(S.getHandle(), &C, &D))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void edgeMapDM(
	Image& D, 
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE C = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR2D_EdgeMap(S.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) 
		{ D = Image::imageFactory(hD); }
}

inline 	Image edgeMapDM(
	const Image& S,
	const DatamatrixConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	if (!mvIMPACT_C::CR2D_EdgeMap(S.getHandle(), &C, &D))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void edgeMapDM(
	Image& D, 
	const Image& S,
	const DatamatrixConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::CR2D_BUFHANDLE C = config.getHandle();
	if (!mvIMPACT_C::CR2D_EdgeMap(S.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) 
		{ D = Image::imageFactory(hD); }
}


inline void addDMDecoder(
	const std::string& decoder,
	int index)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::CR2D_AddDecoder(decoder.c_str(), index))
		throwImpactError();
}





inline void removeDMDecoder(
	const std::string& decoder)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::CR2D_RemoveDecoder(decoder.c_str()))
		throwImpactError();
}





inline int getDMDecoderCount()
//-----------------------------------------------------------------------------
{
	int value;
	if (!mvIMPACT_C::CR2D_InquireDecoder(0, mvIMPACT_C::CR2D_DECODER_COUNT, &value))
		throwImpactError();
	return value;
}





inline std::string getDMDecoderName(
	int index)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG length = 0;
	if (!mvIMPACT_C::CR2D_InquireDecoder(index, mvIMPACT_C::CR2D_DECODER_NAME_LENGTH, &length))
		throwImpactError();
	DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);
	if (!mvIMPACT_C::CR2D_InquireDecoder(index, mvIMPACT_C::CR2D_DECODER_NAME, value.getPointer()))
		throwImpactError();
	return value.getPointer();
}





inline std::string getDMDecoderFileName(
	int index)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG length = 0;
	if (!mvIMPACT_C::CR2D_InquireDecoder(index, mvIMPACT_C::CR2D_DECODER_FILENAME_LENGTH, &length))
		throwImpactError();
	DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);

	if (!mvIMPACT_C::CR2D_InquireDecoder(index, mvIMPACT_C::CR2D_DECODER_FILENAME, value.getPointer()))
		throwImpactError();
	return value.getPointer();
}

inline int getDMExpectedSize(
	int index)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG value;
	if (!mvIMPACT_C::CR2D_InquireDecoder(index, mvIMPACT_C::CR2D_DECODER_DM_SIZE, &value))
		throwImpactError();
	return (int)value;
}

inline void setDMExpectedSize(
	int index,
	int expectedSize)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG value_ = (mvIMPACT_C::IPL_ULONG)expectedSize;
	if (!mvIMPACT_C::CR2D_ControlDecoder(index, mvIMPACT_C::CR2D_DECODER_DM_SIZE, &value_))
		throwImpactError();
}


inline bool checkLicenseDM()
//-----------------------------------------------------------------------------
{
	return (IPL_TRUE == mvIMPACT_C::CR2D_CheckLic());
}

inline std::string getVersionDM()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::CR2D_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

inline std::string getVersionDMDecoder(const std::string& decoder)
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::CR2D_GetVersionDecoder(decoder.c_str(), sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

	} // namespace datamatrix
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_DATAMATRIX_IMPL_H_
