#! /bin/make
##############################################
# common make definitions for samples
#===========================================
#        == DO NOT CHANGE !!! ==
##############################################
TARGET:=$(shell basename `pwd`)
OSTYPE := $(shell uname | cut -d _ -f 1 | tr [:upper:] [:lower:])

##############################################
# directories
#===========================================
WORK=$(mvIMPACT)/..
MVS=$(WORK)/mvblue

#-------------------------------------------
MVSLIB_PATH=$(MVS)/libs/mvslib
MVSENS_PATH=$(MVS)/kmodules/mvsensor2

ifeq ($(TARGET),ppc603e)
	MVBL_OK:=$(shell if [ $(TARGET) != i386 ]; then if [ -d $(MVS) ]; then echo "yes"; fi; else echo "yes"; fi )
else
	MVBL_OK:=$(shell echo "yes" )
endif

############################################
#===========================================
include $(mvIMPACT)/Makefile.all.inc

# -fPIC is not needed when compiling the samples,
# but only when linking for the ppc
LIB_OPTIONS =

##############################################
#===========================================
SOURCES= $(MODULE_NAME).cpp
OBJECTS= $(MODULE_NAME).o
HEADERS= *.h

############################################
# Entries for the different target-types
#===========================================
############################################
# libs
#===========================================
ifeq ($(TARGETARCH),ppc)
	ifeq ($(OSTYPE),cygwin)
		MVBL_TARGET=$(OSTYPE)603e
		IPL_LIB_TRGT=$(OSTYPE)
	else
		MVBL_TARGET=$(TARGET)
		IPL_LIB_TRGT=$(TARGET)
	endif

	ARCH_LDFLAGS = -fPIC
	MVSLIB=									\
		-L$(MVSLIB_PATH)/$(MVBL_TARGET)		\
			-lmvs
else
	IPL_LIB_TRGT=$(TARGET)
	MVSLIB=
endif

#-------------------------------------------
IMPACTLIB_PATH=$(mvIMPACT)/lib/$(IPL_LIB_TRGT)

############################################
#-------------------------------------------
LICMKFILE:=$(shell if [ -e $(mvIMPACT)/private/Makefile.lic.inc ]; then ls $(mvIMPACT)/private/Makefile.lic.inc; fi )

ifneq ($(LICMKFILE),)
	include $(LICMKFILE)
endif

############################################
#===========================================
ifeq ($(PROFILING),1)
	PROFOPT=-g -pg -a
	OPT=-g
else
	PROFILING=0
endif

############################################
#===========================================
LOCDEFINES=					\
	-DUSE_STOPWATCH=1		\
##	-DNO_DEVICE				\
##	-DEXPORT_RESULTS=0		\
##	-DDISPLAY_RESULTS=0		\

############################################
#===========================================
IMPACT_INC=							\
	-I$(mvIMPACT)/include/common   	\
	-I$(mvIMPACT)/include/linux    	\
	-I$(mvIMPACT)/include

#-------------------------------------------
ifeq ($(TARGET),ppc603e)
MVS_INC=							\
	-I$(MVSENS_PATH)				\
	-I$(MVSLIB_PATH)				\
	-I$(MVS)/mvstools				\

##	-I$(MVSGRAPH_PATH)
endif


#-------------------------------------------
INCLFILES=				\
	$(GLOBINCFILES)		\
	$(USR_INCLS)		\
	$(IMPACT_INC)		\
	$(MVS_INC)			\
	$(UL_INCL)			\
	-I..

############################################
CPPFLAGS=				\
	$(CPPFLAGS_2)		\
	$(CPPFLAGS_3)		\
    $(LOCDEFINES)		\
    $(INCLFILES)

############################################
#===========================================
LDFLAGS = 									\
	$(ARCH_LDFLAGS)							\
	$(ADDITIONAL_USR_LIB_PATH)				\
	-L$(IMPACTLIB_PATH)						\
		$(IMPACT_LIBS)						\
	$(MVSLIB)								\
	$(USR_LIBS)								\
        -lm

MOVED_TO_THE_INC=			\
		-lvl_autofocus		\
		-lvl_device			\
		-lvl_compress		\
		-lvl_autoreg		\
		-lvl_autotrace		\
		-lvl_color 			\
		-lvl_ocr			\
		-lvl_barcode		\
		-lvl_file 			\
		-lvl_extpat 		\
		-lvl_pat 			\
		-lvl_blob 			\
		-lvl_gauge 			\
		-lvl_slice 			\
		-lvl_display 		\
		-lvl_ipl 			\
		-lvl_base

############################################
#===========================================
.PHONY: run strip dbg clean_mod new pack all $(MODULE_NAME) $(MODULE_NAME) extract_ar_objects ipk

ifeq ($(GCC_OK),yes)
#===========================================
ifeq ($(MVBL_OK),yes)
#===========================================
all:	$(MODULE_NAME)

#-------------------------------------------
$(MODULE_NAME): $(OBJECTS) $(OOBJ) extract_ar_objects
	$(CXX) $(CPPFLAGS) -o $(MODULE_NAME) $(OBJECTS) $(OOBJ) $(LLOCLIB) $(LDFLAGS) $(USER_AR_OBJ_LIST) -ldl $(PROFOPT)

else
all:
	@echo "***** NO MVBLUE SOFTWARE ON THIS SYSTEM!! *****"
	@echo "********* PLEASE INSTALL THIS FIRST!! *********"
	@echo " "
endif
#===========================================
else
all:
	@echo "***** WARNING - NO COMPILER: $(CC) AVAILABLE *****"
	@echo " "
endif


#-------------------------------------------
strip: all
	@ls -lh $(MODULE_NAME)
	$(TOOLS)strip $(MODULE_NAME)
	@ls -lh $(MODULE_NAME)

#-------------------------------------------
$(MODULE_NAME).gz:	$(MODULE_NAME)
	@ls -l $(MODULE_NAME)
	$(TOOLS)strip $(MODULE_NAME)
	@gzip -c -f $(MODULE_NAME) > $(MODULE_NAME).gz
	@ls -l $(MODULE_NAME).gz
	@mcopy -n $(MODULE_NAME).gz a:

#-------------------------------------------
dbg:	$(MODULE_NAME)
	gdb -x ../../gdb.inc $(MODULE_NAME)

#-------------------------------------------
run:	$(MODULE_NAME)
#	$(MODULE_NAME) *.bmp

#-------------------------------------------
#getloc:
#	@
#$(IMPACT_LIBS)

#===========================================
pack:	$(MODULE_NAME).gz

#===========================================
clean_mod:
	@rm -f $(OBJECTS) $(OOBJ) $(MODULE_NAME)

#-------------------------------------------
#-------------------------------------------
# make ipkg package
#----------------------------------------
IPKG_BUILD:=ipkg-build
ifeq ($(TARGET),ppc_6xx)
	IPKG_BUILD:=ipkg-build-c
endif


ifeq ($(HAVE_IPKG),yes)
IPKG_FOLDER:=$(shell basename $(IPKG_PATH))

ipk: $(MODULE_NAME)_$(IPK_VERSION)_$(TARGET).ipk

#----------------------------------------
ipkglink: strip
	@(   												\
	    if [ -d $(IPKG_PATH) ]; 						\
	    then    										\
			rm -rf $(IPKG_DST_DIR);						\
			mkdir -p $(IPKG_DST_DIR);					\
			echo "Copy files....";						\
			cp -v $(MODULE_NAME) $(IPKG_DST_DIR)/. ;	\
			for file in $(ADDIPKG_FILES);				\
			do											\
				cp -v $$file $(IPKG_DST_DIR)/. ;		\
			done;										\
	    fi  											\
	)

#----------------------------------------
$(MODULE_NAME)_$(IPK_VERSION)_$(TARGET).ipk: ipkglink
	@(																\
	    if [ -d $(IPKG_PATH) ]; 									\
	    then														\
			cd ..;													\
			rm -if $(MODULE_NAME)_$(IPK_VERSION)_$(TARGET).ipk;		\
			echo "Change control file....";							\
			sed -e "s/^\(Architecture:[\t ]*\)[a-zA-Z0-9.+\-\_]*/\1$(TARGET)/" < $(IPKG_FOLDER)/CONTROL/control > ipk1.tmp; \
	        sed -e "s/^\(Version:[\t ]*\)[a-zA-Z0-9.+]*/\1$(IPK_VERSION)/" < ipk1.tmp > ipk.tmp; \
			rm ipk1.tmp;    										\
			mv -f ipk.tmp $(IPKG_FOLDER)/CONTROL/control; 			\
			echo "Executing: $(IPKG_BUILD) $(IPKG_FOLDER)";			\
			$(IPKG_BUILD) $(IPKG_FOLDER);    						\
			cd -;													\
	    fi  														\
	)
else
ipk:
	@echo "*** NO IPKG possibile for $(TARGET)!! ***"
endif

#===========================================

ifeq ($(OSTYPE),cygwin)
.cpp.o:
	@ echo "Compiling $*.cpp"
	-$(CXX) -c $(CPPFLAGS) $(INCLUDES) -o $@ $< 2>gccErrors.dat
	@ gccerr gccErrors.dat
endif


