//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <vl_gauge.h>

#include <samples.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
IPL_DOUBLE X0 = 0.;
IPL_DOUBLE Y0 = 0.;
IPL_DOUBLE X1 = 0.;
IPL_DOUBLE Y1 = 0.;

DISP_OOISETHANDLE oois = DISP_DONT_CARE;
DISP_OOIHANDLE loi1 = DISP_DONT_CARE;
IPL_BUFHANDLE image = IPL_DONT_CARE;
IPL_HWND hwnd = 0;

//----------------------------------------------------------------------
// function to do the measurement
void OnMeasure( )
{
	//inquire position of LOI
	IPL_LONG xp1 = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, loi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+0), &xp1));
	IPL_LONG xp2 = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, loi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+1), &xp2));
	IPL_LONG yp1 = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, loi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+2), &yp1));
	IPL_LONG yp2 = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, loi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+3), &yp2));

	//inquire width of LOI
	IPL_ULONG measwidth = 0;
    CHECKSDKCALL(DISP_InquireOoi(oois, loi1, (DISP_OOICONTROLTYPE)(DISP_OOI_WIDTH), &measwidth));
	//DISP_InquireOOI delivers half of the width
	measwidth = measwidth * 2;
	printf("%ld \t %ld \t %ld \t %ld \t %ld\n", xp1, yp1, xp2, yp2, measwidth);

	//allocate measurement buffer
	GAU_BUFHANDLE measure = -1;
	CHECKSDKCALL(GAU_BufAlloc(&measure));

	//add features to be calculated
	CHECKSDKCALL(GAU_AddFeature(measure, GAU_EDGE_X0_INTERCEPT, TRUE));
	CHECKSDKCALL(GAU_AddFeature(measure, GAU_EDGE_Y0_INTERCEPT, FALSE));
	CHECKSDKCALL(GAU_AddFeature(measure, GAU_EDGE_X1_INTERCEPT, FALSE));
	CHECKSDKCALL(GAU_AddFeature(measure, GAU_EDGE_Y1_INTERCEPT, FALSE));

	//set some properties
	IPL_DOUBLE score = 40.;
	CHECKSDKCALL(GAU_BufControl(measure, GAU_SCORE, &score));
	GAU_EDGE_PAIR_MODE edgemode = GAU_SINGLE;
	CHECKSDKCALL(GAU_BufControl(measure, GAU_EDGE_MODE, &edgemode));
	IPL_BOOL bestmatchonly = IPL_TRUE;
	CHECKSDKCALL(GAU_BufControl(measure, GAU_BEST_MATCH_ONLY, &bestmatchonly));


	//find edge
	if (GAU_FindEdge((IPL_BUFHANDLE)image, measure, xp1, yp1, xp2, yp2, measwidth))
	{
		//get number of edges
		IPL_DOUBLE edges = 0.;
		CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_EDGE_NUMBER, 0, 0, 0, &edges));

 		if (edges) //edges found ?
		{
			CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_EDGE_X0_INTERCEPT, 0, 0, 0, &X0));
			CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_EDGE_Y0_INTERCEPT, 0, 0, 0, &Y0));
			CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_EDGE_X1_INTERCEPT, 0, 0, 0, &X1));
			CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_EDGE_Y1_INTERCEPT, 0, 0, 0, &Y1));
		}
		else
		{
			X0 = Y0 = X1 = Y1 = 0.;
		}
	}
	else
	{
		X0 = Y0 = X1 = Y1 = 0.;
	}

	//update the image
	CHECKSDKCALL(DISP_UpdateWindow(hwnd, DISP_INVALIDATE));

	//free measurement buffer
	CHECKSDKCALL(GAU_BufFree(measure));
}

//----------------------------------------------------------------------
// callback is called when display module is about to draw
IPL_FRESULT IPL_C_CALLTYPE OnDraw( IPL_HWND /*Window*/, IPL_BUFHANDLE /*Buffer*/, IPL_HDC Dc, const IPL_RECT* /*Source*/, const IPL_RECT* /*Dest*/, IPL_CONST_VOID_PTR /*UserData*/ )
{
	IPL_COLOR Color = PALETTERGB( 0, 0, 255 );
	IPL_POINT From, To;

	From.xpos = (int)X0;
	From.ypos = (int)Y0;
	To.xpos   = (int)X1;
	To.ypos   = (int)Y1;

	CHECKSDKCALL( DISP_DrawLine( Dc, Color, &From, &To ));

	return TRUE;
}

//----------------------------------------------------------------------
// callback is called when button is released
IPL_FRESULT IPL_C_CALLTYPE OnButtonUp(IPL_HWND /*Window*/, const IPL_POINT* /*Point*/, DISP_MOUSEBUTTON Button, BOOL* /*Cancel*/, IPL_CONST_VOID_PTR /*UserData*/)
{
	//start measurement on left button up
	if( DISP_LBUTTON == Button )
		OnMeasure( );

	return TRUE;
}

//----------------------------------------------------------------------
int main(int /*argc*/, char** /*argv[]*/ )
{
	// load any supported file format, into image
	CHECKSDKCALL(IPL_FlImport(&image, NULL, FL_DONT_CARE, "stanz.bmp"));

	// output message
	printf("Drag the LOI over an edge to be measured.\n"
		    "The LOI should be placed perpendicular to the edge.\n"
			"Releasing the left mouse button starts the measurement.\n"
		    "(The coordinates x0, y0, x1, y1 and the width of the line are listed.) \n\n");

	// install callbacks
	CHECKSDKCALL( DISP_Control( DISP_ON_DRAW, (IPL_VOID_PTR)OnDraw ));
	CHECKSDKCALL( DISP_Control( DISP_ON_BUTTONUP, (IPL_VOID_PTR)OnButtonUp ));

	// create an ooiset
	CHECKSDKCALL(DISP_OoiSetAlloc(&oois));

	// create an loi in the set
	IPL_POINT loipoints[2] =
	{
		{ 100, 100 },
		{ 170, 170 },
	};
	CHECKSDKCALL(DISP_OoiAlloc(oois, &loi1, DISP_LOI_TOOL, loipoints, sizeof(loipoints)/sizeof(loipoints[0]), NULL));

	// set some ooi properties
	COLORREF color3 = PALETTERGB(0, 0, 255);
	CHECKSDKCALL(DISP_ControlOoi(oois, loi1, DISP_OOI_COLOR, &color3));
	color3 = PALETTERGB(255, 0, 0);
	CHECKSDKCALL(DISP_ControlOoi(oois, loi1, DISP_OOI_TEXT_COLOR, &color3));
    IPL_ULONG width = 20;
	CHECKSDKCALL(DISP_ControlOoi(oois, loi1, DISP_OOI_WIDTH, &width));
	IPL_BOOL flag = IPL_TRUE;
	CHECKSDKCALL(DISP_ControlOoi(oois, loi1, DISP_OOI_FILL_VISIBLE, &flag));
	CHECKSDKCALL(DISP_ControlOoi(oois, loi1, DISP_OOI_FILL_TRANSPARENT,&flag));
	CHECKSDKCALL(DISP_ControlOoi(oois, loi1, DISP_LOI_DRAG_THICKNESS, &flag));
	CHECKSDKCALL(DISP_ControlOoi(oois, loi1, DISP_LOI_SHOW_ARROW, &flag));

	// associate the ooiset with the thread (and all windows created thereafter)
	CHECKSDKCALL(DISP_Control(DISP_OOISET, &oois));

	// open window to display image
	CHECKSDKCALL( DISP_OpenWindow( image, &hwnd, 0, NULL ));

	// keep sample alive until console window is closed or space bar is pressed
	CHECKSDKCALL( DISP_KeepRunning("Click the space bar to end program.\n"));

	// free allocated buffers
	CHECKSDKCALL(IPL_BufFreeAll());

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
