//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <vl_gauge.h>
#include <samples.h>

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
static IPL_DOUBLE cenx = 0.;
static IPL_DOUBLE ceny = 0.;
static IPL_DOUBLE radius = 0.;

static DISP_OOISETHANDLE oois = DISP_DONT_CARE;
static DISP_OOIHANDLE aoi1 = DISP_DONT_CARE;
static DISP_OOIHANDLE loi1 = DISP_DONT_CARE;
static IPL_BUFHANDLE image = IPL_DONT_CARE;
static IPL_HWND hwnd = 0;

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
static int write_res( )
{
	char message[64];
	sprintf( message, "Radius:%.3f, x-center:%.3f, y-center:%.3f", radius, cenx, ceny );

	// create an aoi in the set
	IPL_POINT loi[2] =
	{
		{ 640/2, 10 },
		{ 640/2, 10 },
	};

	if( loi1 == DISP_DONT_CARE )
		CHECKSDKCALL( DISP_OoiAlloc( oois, &loi1, DISP_LOI_TOOL, loi, sizeof(loi)/sizeof(loi[0]), NULL ));

	// set some ooi properties
	COLORREF color = PALETTERGB( 0,0,255 );
	CHECKSDKCALL( DISP_ControlOoi( oois, loi1, DISP_OOI_COLOR, &color ));
	CHECKSDKCALL( DISP_ControlOoi( oois, loi1, DISP_OOI_TEXT_COLOR, &color ));
	CHECKSDKCALL( DISP_ControlOoi( oois, loi1, DISP_OOI_TEXT, message ));

	return 0;
}

//------------------------------------------------------------------------------------------
void OnMeasure()
{
	//inquire position of AOI
	IPL_LONG left = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, aoi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+0), &left));
	IPL_LONG right = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, aoi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+1), &right));
	IPL_LONG top = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, aoi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+2), &top));
	IPL_LONG bottom = 0;
	CHECKSDKCALL(DISP_InquireOoi(oois, aoi1, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+3), &bottom));

	// swap coordinates if necessary
	if (right < left)
	{
		IPL_LONG t = left;
		left = right;
		right = t;
	}
	if (bottom < top)
	{
		IPL_LONG t = top;
		top = bottom;
		bottom = t;
	}

printf("%ld %ld %ld %ld\n", left, right, top, bottom);

	//allocate measurement buffer
	GAU_BUFHANDLE measure = -1;
	CHECKSDKCALL(GAU_BufAlloc(&measure));

	//add features to be calculated
	CHECKSDKCALL(GAU_AddFeature(measure, GAU_CIRCLE_RADIUS, TRUE));
	CHECKSDKCALL(GAU_AddFeature(measure, GAU_CIRCLE_CEN_X, FALSE));
	CHECKSDKCALL(GAU_AddFeature(measure, GAU_CIRCLE_CEN_Y, FALSE));

	//set some properties
	GAU_EDGE_PAIR_MODE edgemode = GAU_SINGLE;
	CHECKSDKCALL(GAU_BufControl(measure, GAU_EDGE_MODE, &edgemode));
	//IPL_DOUBLE direction = GAU_HORIZONTAL_RIGHT_TO_LEFT;
	//CHECKSDKCALL(GAU_BufControl(measure, GAU_MEASUREMENT_DIRECTION, &direction));

	//find circle
	if (GAU_FindCircle((IPL_BUFHANDLE)image, measure, left, top, abs(right-left), abs(bottom-top)))
	{
		//get number of circles
		IPL_DOUBLE circles = 0.;
		CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_CIRCLE_NUMBER, 0, 0, 0, &circles));

 		if (circles) //circles found ?
		{
			CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_CIRCLE_RADIUS, 0, 0, 0, &radius));
			CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_CIRCLE_CEN_X, 0, 0, 0, &cenx));
			CHECKSDKCALL(GAU_GetResults(measure, 0, GAU_CIRCLE_CEN_Y, 0, 0, 0, &ceny));
		}
		else
		{
			radius = cenx = ceny = 0.;
		}
	}
	else
	{
    	radius = cenx = ceny = 0.;
	}

	write_res( );
	//update the image
	CHECKSDKCALL(DISP_UpdateWindow(hwnd, DISP_INVALIDATE));

	//free measurement buffer
	CHECKSDKCALL(GAU_BufFree(measure));
}

//------------------------------------------------------------------------------------------
// callback is called when display module is about to draw
IPL_FRESULT IPL_C_CALLTYPE OnDraw(IPL_HWND /*Window*/, IPL_BUFHANDLE /*Buffer*/, IPL_HDC Dc,
					const IPL_RECT* /*Source*/, const IPL_RECT* /*Dest*/, IPL_CONST_VOID_PTR /*UserData*/)
{
	IPL_COLOR color = PALETTERGB( 255,0,0 );
	IPL_RECT Bounds;
	IPL_POINT AFrom, ATo;

    int iextent = (int)(radius*2 + 0.5) + 1;

	Bounds.left   = (long)(cenx - radius + 0.5);
	Bounds.top    = (long)(ceny - radius + 0.5);
	Bounds.right  = Bounds.left + iextent;
	Bounds.bottom = Bounds.top + iextent;
	AFrom.xpos    = Bounds.left;
	AFrom.ypos    = Bounds.top;
	ATo.xpos      = AFrom.xpos;
	ATo.ypos      = AFrom.ypos;

	CHECKSDKCALL( DISP_DrawArc( Dc, color, &Bounds, &AFrom, &ATo, 0 ));

	//write_res( );

	return TRUE;
}

//------------------------------------------------------------------------------------------
// callback is called when button is released
IPL_FRESULT IPL_C_CALLTYPE OnButtonUp(IPL_HWND /*Window*/, const IPL_POINT* /*Point*/, DISP_MOUSEBUTTON Button,
					BOOL* /*Cancel*/, IPL_CONST_VOID_PTR /*UserData*/)
{

	if (DISP_LBUTTON == Button)
		OnMeasure( );

	return TRUE;
}

//------------------------------------------------------------------------------------------
int main(int /*argc*/, char** /*argv[]*/ )
{
	// load any supported file format, into image
	CHECKSDKCALL(IPL_FlImport(&image, NULL, FL_DONT_CARE, "stanz.bmp"));

	// output message
	printf("Drag the AOI over a circle or circular arc to be measured.\nReleasing the left mouse button starts the measurement.\n\n");

	// install callbacks
	CHECKSDKCALL(DISP_Control(DISP_ON_DRAW, (IPL_VOID_PTR)OnDraw));
	CHECKSDKCALL(DISP_Control(DISP_ON_BUTTONUP, (IPL_VOID_PTR)OnButtonUp));

	// create an ooiset
	CHECKSDKCALL(DISP_OoiSetAlloc(&oois));

	// create an aoi in the set
	IPL_POINT aoi[5] =
	{
		{ 10, 10 },
		{ 110, 10 },
		{ 110, 110 },
		{ 10, 110 },
		{ 10, 10 },
	};
	CHECKSDKCALL(DISP_OoiAlloc(oois, &aoi1, DISP_AOI_TOOL, aoi, sizeof(aoi)/sizeof(aoi[0]), (IPL_VOID_PTR)image));

	// set some ooi properties
	COLORREF color = PALETTERGB(255, 0, 0);
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi1, DISP_OOI_COLOR, &color));

	// associate the ooiset with the thread (and all windows created thereafter)
	CHECKSDKCALL(DISP_Control(DISP_OOISET, &oois));

	// open window to display image
	CHECKSDKCALL(DISP_OpenWindow(image, &hwnd, 0, NULL));

	// keep sample alive until console window is closed or space bar is pressed
	CHECKSDKCALL(DISP_KeepRunning("Click the space bar to end program.\n"));

	// free allocated buffers
	CHECKSDKCALL(IPL_BufFreeAll());

	return 0;
}

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
