//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX sample >>calib<<
// Fre Jun 10 13:19:20 CEST 2005
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_camcal.h>
#include <vl_display.h>

#include <samples.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define TOOL_PREFIX				"calib"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
int main( int argc, char* argv[] )
{
	char* pImgName = "grid04.bmp";

	if( argc > 1 )
	{
		_skiparg();
		_strarg( pImgName );
	}

	/* Load the grid image. */
	IPL_BUFHANDLE imageHandle = IPL_DONT_CARE;
	CHECKSDKCALL( IPL_FlImport( &imageHandle, NULL, FL_DONT_CARE, pImgName ));

	/* Display the grid image. */
	IPL_HWND hwnd = 0;
	CHECKSDKCALL( DISP_OpenWindow( imageHandle, &hwnd, 0, NULL ));
	CHECKSDKCALL( DISP_KeepRunning( "Click the space bar to start the teach process.\n" ));

	/* Allocate a calibration buffer. All defaults are set. */
	CC_BUFHANDLE camcalHandle = IPL_DONT_CARE;
	CHECKSDKCALL( CC_BufAlloc( &camcalHandle, 0 ));

	double angle = 2.0;
	CHECKSDKCALL( CC_Control( camcalHandle, CC_RW_LINE_ANGLE_TOLERANCE, &angle));


	/* Start the teach process. */
	CHECKSDKCALL( CC_TeachCalibrationByGrid( camcalHandle, imageHandle ));

	/* Read some results. */
	unsigned int icnt = 0;
	CHECKSDKCALL( CC_Inquire( camcalHandle, CC_RO_ITER_CNT, &icnt ));
	printf(" == " TOOL_PREFIX ": needed %d iteration\n", icnt );

	CC_STATISTICS stat;
	CHECKSDKCALL( CC_Inquire( camcalHandle, CC_RO_STATISTICS, &stat ));
	printf(" == " TOOL_PREFIX ": xavg: %8.4f xstddev: %8.4f\n", stat.dxavg,   stat.xstddev );
	printf(" == " TOOL_PREFIX ": yavg: %8.4f ystddev: %8.4f\n", stat.dyavg,   stat.ystddev );
	printf(" == " TOOL_PREFIX ": eavg: %8.4f estddev: %8.4f\n", stat.euclavg, stat.euclstddev );

	/* Transform/calibrate the image. */
	CHECKSDKCALL( CC_TrCalibratedImage( camcalHandle, imageHandle, &imageHandle ));

	/* Update the display. */
	CHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));
	CHECKSDKCALL( DISP_KeepRunning( "Click the space bar to end program.\n" ));

	/* */
	CC_BufFree( camcalHandle );
	IPL_BufFree( imageHandle );

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
