//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>dylth<<
// Die Mr  1 17:10:28 CET 2005
//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \file dylth.cpp
 *  \brief This sample shows how to use the mvIMPACT dynamic line threshold  function \c IPL_PtDynamicLineThreshold .
 *  After the image is loaded the dynamic line threshold will be applyed to this
 *  image. The resulted image will be displayed.
 * \li The usage of the sample is: \c dylth [ \c ImageName ] [ \c polarity ] [ \c OperatorWidth ] [ \c MinDistance ]
 * where \c ImageName is the name of the image which should be loaded. If non
 * is specified the default image will be loaded.
 */
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_display.h>
#include <vl_ipl.h>
#include <vl_file.h>

#include <samples.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
int main( int argc, char* argv[] )
{
	IPL_ULONG OperatorWidth = 30;
	IPL_ULONG MinDistance   = 100;
	int polarity = 0;
	char* pImageName = "milk.bmp";

	if( argc > 1 )
	{
		_skiparg( );
		_intarg( polarity );				/* object polarity: 1 == WHITE_ON_BLACK & 0 == BLACK_ON_WHITE */
		_intarg( OperatorWidth );			/* the width of the used operator. It should have at
											 * least the size of the greatest object which should remain
											 * after the thresholding */
		_intarg( MinDistance );				/* minimum distance between estimates for background and foreground */
		_strarg( pImageName );				/* name of the image which should be loaded */

		if( polarity < 0 || OperatorWidth <= 0 || !pImageName )
		{
			printf( "\n usage: dylth [polarity] [OperatorWidth] [MinDistance] [ImageName]\n\n");
			return 0;
		}
	}

	/* load the image */
	IPL_BUFHANDLE image = IPL_DONT_CARE;
	CHECKSDKCALL( IPL_FlImport( &image, NULL, FL_DONT_CARE, pImageName ));

	/* Apply the threshold function */
	CHECKSDKCALL( IPL_PtDynamicLineThreshold( image, &image, polarity ? IPL_WHITE_ON_BLACK : IPL_BLACK_ON_WHITE, OperatorWidth, MinDistance ));

	/* Open a window and display the image */
	IPL_HWND hwnd = 0;
	CHECKSDKCALL( DISP_OpenWindow( image, &hwnd, 0, NULL ));
	CHECKSDKCALL( DISP_KeepRunning( "Press <SPACE> to end the programm...\n" ));

	IPL_BufFree( image );

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------



