//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX sample >>gauge_video<<
// Mon Jun  7 08:56:56 CEST 2004
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <vl_device.h>
#include <vl_gauge.h>

#include <samples.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define TOOL_PREFIX		"gauge_video"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static IPL_DOUBLE XPos = 0., YPos = 0.;
static IPL_LONG xp1 = 0, yp1 = 0, xp2 = 0, yp2 = 0;

static DISP_OOISETHANDLE ooisHandle   = DISP_DONT_CARE;
static DISP_OOIHANDLE    loiHandle    = DISP_DONT_CARE;
static DISP_OOIHANDLE    loi1Handle   = DISP_DONT_CARE;
static DISP_OOIHANDLE    loi2Handle   = DISP_DONT_CARE;
static IPL_BUFHANDLE     imageHandle  = IPL_DONT_CARE;
static DEV_HANDLE        deviceHandle = IPL_DONT_CARE;
static IPL_HWND hWnd = 0;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static int write_results( )
{
	char msg[512];
	memset( msg, 0, 512 );
	sprintf( msg, "in:  (%ld/%ld)/(%ld/%ld)\nEdge: %.2f/%.2f\n",
								xp1, yp1, xp2, yp2, XPos, YPos );

	IPL_POINT respoint[2] = { {640/2, 30},{640/2, 30}};

	if( loi2Handle == IPL_DONT_CARE )
		CHECKSDKCALL( DISP_OoiAlloc( ooisHandle, &loi2Handle, DISP_LOI_TOOL, respoint, sizeof(respoint)/sizeof(respoint[0]), NULL ));

	COLORREF color = PALETTERGB( 0, 0, 255 );
	CHECKSDKCALL( DISP_ControlOoi( ooisHandle, loi2Handle, DISP_OOI_COLOR, &color ));
	CHECKSDKCALL( DISP_ControlOoi( ooisHandle, loi2Handle, DISP_OOI_TEXT_COLOR, &color ));
	CHECKSDKCALL( DISP_ControlOoi( ooisHandle, loi2Handle, DISP_OOI_TEXT, msg ));

	return 0;
}

//----------------------------------------------------------------------
static int draw_line( IPL_HDC Dc, long x0, long y0, long x1, long y1 )
{
	IPL_POINT LFrom, LTo;
	IPL_COLOR color = PALETTERGB( 0, 255, 0 );

	LFrom.xpos = x0;
	LFrom.ypos = y0;
	LTo.xpos   = x1;
	LTo.ypos   = y1;

	DISP_DrawLine( Dc, color, &LFrom, &LTo );


	return 0;
}

//----------------------------------------------------------------------
IPL_FRESULT IPL_C_CALLTYPE OnDraw(IPL_HWND /*Window*/, IPL_BUFHANDLE /*Buffer*/, IPL_HDC Dc, const IPL_RECT* /*Source*/, const IPL_RECT* /*Dest*/, IPL_CONST_VOID_PTR /*UserData*/)
{
	int ret = IPL_TRUE;

	draw_line( Dc, (long)(XPos+0.5), (long)(YPos-3.5), (long)(XPos+0.5), (long)(YPos+3.5) );
	draw_line( Dc, (long)(XPos-3.5), (long)(YPos+0.5), (long)(XPos+3.5), (long)(YPos+0.5) );
	write_results( );
	return ret;
}

//----------------------------------------------------------------------
// function to do the measurement
static int process( )
{
	//allocate measurement buffer
	GAU_BUFHANDLE measureHandle = IPL_DONT_CARE;
	CHECKSDKCALL( GAU_BufAlloc( &measureHandle ));

	//add features to be calculated
	CHECKSDKCALL( GAU_AddFeature( measureHandle, GAU_EDGE_XPOS, TRUE ));
	CHECKSDKCALL( GAU_AddFeature( measureHandle, GAU_EDGE_YPOS, FALSE ));

	//set some properties
	IPL_DOUBLE score = 40.;
	CHECKSDKCALL( GAU_BufControl( measureHandle, GAU_SCORE, &score ));
	GAU_EDGE_PAIR_MODE edgemode = GAU_SINGLE;
	CHECKSDKCALL( GAU_BufControl( measureHandle, GAU_EDGE_MODE, &edgemode ));
	IPL_BOOL bestmatchonly = IPL_TRUE;
	CHECKSDKCALL( GAU_BufControl( measureHandle, GAU_BEST_MATCH_ONLY, &bestmatchonly ));

    //set measurement window width
	IPL_ULONG measwidth = 51;

	//find edge
	if( GAU_FindEdge( imageHandle, measureHandle, xp1, yp1, xp2, yp2, measwidth ))
	{
		//get number of edges
		IPL_DOUBLE edges = 0.;
		CHECKSDKCALL( GAU_GetResults( measureHandle, 0, GAU_EDGE_NUMBER, 0, 0, 0, &edges ));

 		if( edges ) //edges found ?
		{
			GAU_GetResults( measureHandle, 0, GAU_EDGE_XPOS, 0, 0, 0, &XPos );
			GAU_GetResults( measureHandle, 0, GAU_EDGE_YPOS, 0, 0, 0, &YPos );
		}
		else
		{
			XPos = YPos = 0.;
		}
	}
	else
	{
		XPos = YPos = 0.;
	}

	dprintf((" == " TOOL_PREFIX ": EdgePos: %.2f/%.2f\n", XPos, YPos ));

	//free measurement buffer
	CHECKSDKCALL( GAU_BufFree( measureHandle ));

	return 0;
}

//----------------------------------------------------------------------
static void get_ref_pts( )
{
	//inquire position of LOI
	CHECKSDKCALL( DISP_InquireOoi( ooisHandle, loiHandle, (DISP_OOICONTROLTYPE)( DISP_OOI_POSINDEX + 0 ), &xp1 ));
	CHECKSDKCALL( DISP_InquireOoi( ooisHandle, loiHandle, (DISP_OOICONTROLTYPE)( DISP_OOI_POSINDEX + 2 ), &yp1 ));
	CHECKSDKCALL( DISP_InquireOoi( ooisHandle, loiHandle, (DISP_OOICONTROLTYPE)( DISP_OOI_POSINDEX + 1 ), &xp2 ));
	CHECKSDKCALL( DISP_InquireOoi( ooisHandle, loiHandle, (DISP_OOICONTROLTYPE)( DISP_OOI_POSINDEX + 3 ), &yp2 ));

	dprintf((" == " TOOL_PREFIX ": ref - x/y1: %ld %ld x/y2: %ld %ld\n", xp1, yp1, xp2, yp2 ));
}

//----------------------------------------------------------------------
// callback is called when button is released
IPL_FRESULT IPL_C_CALLTYPE OnButtonUp(IPL_HWND /*Window*/, const IPL_POINT* /*Point*/,
						DISP_MOUSEBUTTON Button, BOOL* /*Cancel*/, IPL_CONST_VOID_PTR /*UserData*/ )
{
	//start measurement on left button up
	if( DISP_LBUTTON == Button )
		get_ref_pts( );

	return TRUE;
}

//----------------------------------------------------------------------
int main( int /*argc*/, char** /*argv[]*/ )
{
	CHECKSDKCALL( DEV_Open( &deviceHandle, "matrixvision.ddc" ));
	CHECKSDKCALL( DEV_Control( deviceHandle, DEV_SINGLE, 0 ));
	CHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));

	// open window to display image
	CHECKSDKCALL( DISP_OpenWindow( imageHandle, &hWnd, 0, NULL ));

	// output message
	printf(" == " TOOL_PREFIX ": Drag the LOI over an edge to be measured.\n"
		   " == " TOOL_PREFIX ": The LOI should be placed perpendicular to the edge.\n"
		   " == " TOOL_PREFIX ": Releasing the left mouse button starts the measurement.\n\n");

	// install callbacks
	CHECKSDKCALL( DISP_ControlWindow( DISP_ON_BUTTONUP, hWnd, (IPL_VOID_PTR)OnButtonUp ));
	CHECKSDKCALL( DISP_ControlWindow( DISP_ON_DRAW,     hWnd, (IPL_VOID_PTR)OnDraw ));

	// create an ooiset
	CHECKSDKCALL( DISP_OoiSetAlloc( &ooisHandle ));

	// create an loi in the set
	IPL_POINT loipoints[2] =
	{
		{ 100, 100 },
		{ 170, 170 },
	};

	CHECKSDKCALL( DISP_OoiAlloc( ooisHandle, &loiHandle, DISP_LOI_TOOL, loipoints, sizeof(loipoints)/sizeof(loipoints[0]), NULL ));

	// set some ooi properties
	COLORREF color = PALETTERGB( 0, 0, 255 );
	CHECKSDKCALL( DISP_ControlOoi( ooisHandle, loiHandle, DISP_OOI_COLOR, &color ));

	// associate the ooiset with the window
	CHECKSDKCALL( DISP_ControlWindow( DISP_OOISET, hWnd, &ooisHandle ));

	get_ref_pts( );

	while( !_kbhit( ))
	{
		if( !DEV_Grab( deviceHandle, &imageHandle ))
		{
			fprintf( stderr, "*** " TOOL_PREFIX ": Grab ERROR! Err: %ld", IPL_GetErrorCode( ));
			break;
		}

		process( );
		DISP_UpdateWindow( hWnd, DISP_INVALIDATE );
		DISP_KeepRunning( NULL );
	}

	// free allocated buffers
	DEV_Close( deviceHandle );

	if( loiHandle != IPL_DONT_CARE )
		DISP_OoiFree( ooisHandle, loiHandle );

	if( loi1Handle != IPL_DONT_CARE )
		DISP_OoiFree( ooisHandle, loi1Handle );
	if( loi2Handle != IPL_DONT_CARE )
		DISP_OoiFree( ooisHandle, loi2Handle );

	DISP_OoiSetFree( ooisHandle );
	IPL_BufFree( imageHandle );
	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------



