/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page affinetrans_page Affinetrans example

\section affinetrans_section_purpose Purpose
This example shows how to do an geometric transformation on an image. 

\section affinetrans_section_intro What it does
The example shows an image. If a key is pressed the image is rotated and is displayed again.

\section affinetrans_section_description How it works
First the image is loaded and displayed. Then the sample waits for user input.
This can be done with the following code:
\code 
		// load any supported file format
		Image image = loadImage("screw.bmp");

		// create a window containing the loaded image:
		Window w(image);

		runMessageLoop();


\endcode

Then the matrix is defined. Assuming that the elements have the following ordering

\verbatim
    m11 m12 m13
    m21 m22 m23 
    m31 m32 m33
\endverbatim

the intensities for the new image I'(x',y') are copied from the original image I(x,y) using the following equations

\verbatim
x' = m11*x + m12*y + m13
y' = m21*x + m22*y + m23
\endverbatim

when calling mvIMPACT::matrix.

\code

		// now populate a matrix that shifts the coordinate system by (150, -150) pixels and 
		// then rotates the image by 30 degrees around top-left corner
		double sin30 = sin(30.0 * M_PI/180.0);
		double cos30 = cos(30.0 * M_PI/180.0);
		double r[] = {
			cos30, -sin30,   150.0,
			sin30,  cos30,  -150.0,
			  0.0,    0.0,     1.0 };
		MatrixDouble rot(3, 3, r);

		// perform affine transformation
		Image dest = matrix(image, rot); 


\endcode

 After that the new image, that was stored in variable \c dest is displayed and the sample
 waits until the user presses a key again.

\code
		// create a window containing the transformed image:
		Window w2(dest);

		// keep sample alive until console window is closed 
		// or space bar is pressed
		runMessageLoop();
\endcode

\section affinetrans_section_source Source file

<matrixdoc>
*/


#include "mvIMPACT.h"

#include <iostream>
using namespace std;

int main(int , char* )
{
	try
	{
		cout << "This sample demonstrates how to perform a ";
		cout << "geometric transformation using an affine ";
		cout << "transformation matrix.\n\n";

		// load any supported file format
		Image image = loadImage("screw.bmp");

		// create a window containing the loaded image:
		Window w(image);

		runMessageLoop();

		// now populate a matrix that shifts the coordinate system by (150, -150) pixels and 
		// then rotates the image by 30 degrees around top-left corner
		double sin30 = sin(30.0 * M_PI/180.0);
		double cos30 = cos(30.0 * M_PI/180.0);
		double r[] = {
			cos30, -sin30,   150.0,
			sin30,  cos30,  -150.0,
			  0.0,    0.0,     1.0 };
		MatrixDouble rot(3, 3, r);

		// perform affine transformation
		Image dest = matrix(image, rot); 

		// create a window containing the transformed image:
		Window w2(dest);

		// keep sample alive until console window is closed 
		// or space bar is pressed
		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
