/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page dylth_page Dynamic line threshold example (CPP)

\section dylth_section_purpose Purpose
This example shows how to use a line based (fast) dynamic threshold to binarize an image which has non-uniform illumination.

\section dylth_section_intro What it does
An image is loaded and the threshold function is applied and the result is displayed in
a window.

\section dylth_section_description How it works
First the input image is loaded into memory.


\code
		// load the image
		Image image = loadImage("milk.bmp");
\endcode

Then a window is opened showing the image and the user is requested to press space.

\code
		// Open a window and display the image 
		Window window = Window(image);
		runMessageLoop("Press <space> to apply the threshold function\n");
\endcode

Afterwards the threshold function is called.

\code
		/// Apply the threshold function 
		dynamicLineThreshold(image, image, pmBlackOnWhite, 30, 100);
\endcode

Finally the window is updated and the message loop is run again before the
program ends.

\code
		window.redraw();	
		runMessageLoop();
\endcode

\section dylth_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"

int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		// load the image
		Image image = loadImage("milk.bmp");
	
		// Open a window and display the image 
		Window window = Window(image);
		runMessageLoop("Press <space> to apply the threshold function\n");

		// Apply the threshold function 
		dynamicLineThreshold(image, image, pmBlackOnWhite, 30, 100);

		// Update the window and wait for user input
		window.redraw();	
		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
