//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include <vl_base.h>

#include "coldemo.h"

//---------------------------------------------------------------------------
#ifdef __linux__

//---------------------------------------------------------------------------
#define BLACK_VALUE	  0 // <= 20 !!!

//---------------------------------
// Bayer - Mosaic to RGB components
// order :
//	G0	B
//	R	G1

#define _gg( p, w )			(( p[0] + p[( w ) + 1] )/2 )		// (g0+g1)/2
//#define _gg(p,w)    p[0]					// g0
//#define _gg(p,w)    p[(w)+1]				// g1


#ifdef OMNIVISION_CMOS_BM
	#define _rr( p, w )		p[w]
	#define _bb( p, w )		p[1]
#else // CCD
	#define _rr( p, w )		p[1]
	#define _bb( p, w )		p[w]
#endif

#if BLACK_VALUE	> 0
	#define _r( p, w )		max(_rr(p,w)-BLACK_VALUE,0)
	#define _g( p, w )		max(_gg(p,w)-BLACK_VALUE,0)
	#define _b( p, w )		max(_bb(p,w)-BLACK_VALUE,0)
#else
	#define _r( p, w)		_rr( p, w )
	#define _g( p, w)		_gg( p, w )
	#define _b( p, w)		_bb( p, w )
#endif

#define _bm_rgb(p,w)	 _r(p,w), _g(p,w), _b(p,w)

//---------------------------------------------------------------------------
typedef unsigned char		BYTE;


//---------------------------------------------------------------------------
void buf_free( BYTE *buf )
{
	if( buf != NULL )
	{
		free( buf );
		buf = NULL;
	}
}

//---------------------------------------------------------------------------
int BM2RGB( IPL_BUFHANDLE src_image, IPL_BUFHANDLE *dst_image, int width, int height, int comp )
{
	int ret = IPL_TRUE;
	int x, y;
	int step = 2;
	IPL_LONG pitch = 0;
	BYTE *pSrcData = NULL;
	BYTE *r_dst_buf = NULL, *g_dst_buf = NULL, *b_dst_buf = NULL;
	BYTE *pp = NULL, *pline = NULL;

	if( *dst_image == IPL_DONT_CARE )
	{
		if( !IPL_BufAlloc( dst_image, width/2, height/2, 3, 1, 1, IPL_DTUCHAR, IPL_IMAGEBUF, NULL ))
		{
			fprintf( stderr, "***  " DEMO_NAME ": BM2RGB - out of memory (err: %ld)!!\n", IPL_GetErrorCode( ));
			return IPL_FALSE;
		}
	}

	if( !( ret = IPL_GetPtr( src_image, IPL_LOCK_READ, 0,0,0, &pitch, &pSrcData )))
		fprintf( stderr, "*** " DEMO_NAME ": BM2RGB - get image pointer failed (err: %ld)!!\n", IPL_GetErrorCode( ));

	if( pSrcData != NULL )
	{
		if(( r_dst_buf = (BYTE*) malloc( sizeof(BYTE) * pitch )) == NULL )
		{
			fprintf( stderr, "*** " DEMO_NAME ": BM2RGB - out of memory !!\n");
			return IPL_FALSE;
		}

		if(( g_dst_buf = (BYTE*) malloc( sizeof(BYTE) * pitch )) == NULL )
		{
			fprintf( stderr, "*** " DEMO_NAME ": BM2RGB - out of memory !!\n");
			return IPL_FALSE;
		}

		if(( b_dst_buf = (BYTE*) malloc( sizeof(BYTE) * pitch )) == NULL )
		{
			fprintf( stderr, "*** " DEMO_NAME ": BM2RGB - out of memory !!\n");
			return IPL_FALSE;
		}

		int rgbcnt = 0, line_cnt = 0;
		for( y = 0; y < height; y += step )
		{
			pline = &pSrcData[ y * pitch ];
			rgbcnt = 0;
			for( pp = pline, x = 0; x < width; x += step, pp += step * comp )
			{
				r_dst_buf[rgbcnt] = _r( pp, pitch );
				g_dst_buf[rgbcnt] = _g( pp, pitch );
				b_dst_buf[rgbcnt] = _b( pp, pitch );
				rgbcnt++;
			}

			IPL_PutBufData( *dst_image, 0, line_cnt, 0,0,0, width/2, 1, r_dst_buf );
			IPL_PutBufData( *dst_image, 0, line_cnt, 1,0,0, width/2, 1, g_dst_buf );
			IPL_PutBufData( *dst_image, 0, line_cnt, 2,0,0, width/2, 1, b_dst_buf );
			line_cnt++;
		}

		buf_free( r_dst_buf );
		buf_free( g_dst_buf );
		buf_free( b_dst_buf );

		IPL_ReleasePtr( src_image, IPL_LOCK_READ, 0, 0, 0, pSrcData );
	}

	return ret;
}
#else
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int BM2RGB( IPL_BUFHANDLE src_image, IPL_BUFHANDLE *dst_image, int width, int height, int comp )
{
	IPL_LONG xsize1, ysize1, xsize2, ysize2;
	IPL_UCHAR *ptr;

	if( *dst_image == IPL_DONT_CARE )
	{
		if( !IPL_BufAlloc( dst_image, width, height, comp, 1, 1, IPL_DTUCHAR, IPL_IMAGEBUF, NULL ))
		{
			fprintf( stderr, "***  " DEMO_NAME ": BM2RGB - out of memory (err: %ld)!!\n", IPL_GetErrorCode( ));
			return IPL_FALSE;
		}
	}

	for ( int b = 0; b < comp; b++ )
	{
		IPL_LONG pitch;

		if ( IPL_GetPtr( src_image, IPL_LOCK_READ, b, 0, 0, &pitch, &ptr) )
		{
			for ( int y = 0; y < height; y++ )
			{
				IPL_PutBufData( dst_image, 0, y, b, 0, 0, width, 1, ptr);
				ptr += pitch;
			}

			IPL_ReleasePtr( src_image, IPL_LOCK_READ, b, 0, 0, ptr );
		}
	}

	return IPL_TRUE;
}
#endif	//__linux__
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
