//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#include <stdio.h>
#include <system.h>

#include <vl_base.h>
#include <vl_ipl.h>
#include <vl_file.h>
#include <vl_color.h>
#include <vl_device.h>
#include <vl_display.h>

#include <mvslib.h>

#include <samples.h>

#include "coldemo.h"

//---------------------------------------------------------------------------
#define MAX_HIST_SIZE		256
#define MAX_COLORS			16
#define MAX_CHAR_SIZE		256

#define DEFAULT_S_THR		150
#define DEFAULT_V_THR		 50

#define DEF_PARAM			"clr_pattern.clr, 150, 50"

#ifdef MALLOC_TRACE
	#include <mcheck.h>
#endif	// MALLOC_TRACE

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
typedef IPL_BUFHANDLE		HSV_BUFFERS[3];

//---------------------------------------------------------------------------
HSV_BUFFERS hHSVBuffers[] = {
	{ IPL_DONT_CARE, IPL_DONT_CARE, IPL_DONT_CARE },
	{ IPL_DONT_CARE, IPL_DONT_CARE, IPL_DONT_CARE }
};

IPL_LONG lSThr = DEFAULT_S_THR, lVThr = DEFAULT_V_THR;				// S and V thresholds
int iClrCount = 0;

struct SClr
{
	int value;
	char name[MAX_CHAR_SIZE];
	int digio;
};

typedef struct SClr DColor;

static DColor m_clr[MAX_COLORS];

IPL_BUFHANDLE hHist  = IPL_DONT_CARE;
static int bgrd_val = -1;

//---------------------------------------------------------------------------
int loadPattern( char* fname )
{
    FILE* file;
	int ret = IPL_FALSE;

	if(( file = fopen( fname, "r" )) != NULL )
	{
		fscanf( file, "%d", &iClrCount );

		char s[MAX_CHAR_SIZE];
		memset( s, 0, MAX_CHAR_SIZE );

		for( int i = 0; i < iClrCount + 1; i++ )
		{
			fgets( s, MAX_CHAR_SIZE, file );

			if( i > 0 )
			{
				memset( m_clr[i - 1].name, 0, MAX_CHAR_SIZE );
				sscanf( s, "%s %d %x", m_clr[i - 1].name, &m_clr[i - 1].value, &m_clr[i - 1].digio );
			}
		}

		fclose( file ) ;
		ret = IPL_TRUE;
	}
	else
		fprintf( stderr, "*** " DEMO_NAME ": CLR-pattern not found: %s\n", fname );

	for( int i = 0; i < iClrCount; i++ )
	{
		if( strncmp("BACKGROUND", m_clr[i].name, strlen("BACKGROUND")) == 0 )
		{
			bgrd_val = m_clr[i].value;
			break;
		}
	}

	if( bgrd_val < 0 )
	{
		fprintf( stderr, "*** " DEMO_NAME ": please set the BACKGROUND into the %s file\n", fname );
		ret = IPL_FALSE;
	}

#ifdef DEBUG
	printf(" bgrd_val: %d\n", bgrd_val );
	for( int i = 0; i < iClrCount; i++ )
		printf(" == [%3d] name: %s, value: %d, digio: 0x%x\n", i, m_clr[i].name, m_clr[i].value, m_clr[i].digio );
#endif	// DEBUG

	return ret;
}

//---------------------------------------------------------------------------
int OpenImpactProcess( IPL_CHAR *pcfg_str )
{
	dprintf((" ++ %s - pcfg_str: %s\n", __FUNCTION__, pcfg_str ));
	int ret;
	char *s, def_cfg[MAX_CHAR_SIZE], cfg_str[MAX_CHAR_SIZE];

	if( strlen( pcfg_str ) == 0 )
	{
		sprintf( def_cfg, "%s%s", getenv("VL_FILE_DIR"), DEF_PARAM );
		pcfg_str = def_cfg;
	}

	// alloc histogramm buffer
	if ( !( ret = IPL_BufAlloc1D( &hHist, MAX_HIST_SIZE, IPL_DTULONG, IPL_HISTBUF, NULL )))
	{
		fprintf( stderr,  "*** " DEMO_NAME ": out of memory!! (err: %ld)\n", IPL_GetErrorCode( ));
		return ret;
	}

	memset( cfg_str, 0, MAX_CHAR_SIZE );
	sprintf( cfg_str, pcfg_str );

	// load pattern from a file
	s = strtok(cfg_str, ",");
	if(( ret = loadPattern( s )))
	{
		s = strtok( NULL, "," );
		sscanf( s, "%ld", &lSThr );
		s = strtok(NULL, "\0");
		sscanf( s, "%ld", &lVThr );
	}

	dprintf((" -- %s - lSThr: %ld, lVThr: %ld\n", __FUNCTION__, lSThr, lVThr ));
	return ret ;
}

//---------------------------------------------------------------------------
void allocHSVChildBuffers( IPL_BUFHANDLE hParBuf, int idx )
{
	// Inquire buffer size
	IPL_ULONG ulXSize, ulYSize;
	IPL_BufInquire( hParBuf, IPL_XSIZE, &ulXSize );

	IPL_BufInquire( hParBuf, IPL_YSIZE, &ulYSize );

	// alloc child buffers
	IPL_BufAllocChild2D( &hHSVBuffers[idx][0], hParBuf, 0, 0, 0, 0, 0, ulXSize, ulYSize );
	IPL_BufAllocChild2D( &hHSVBuffers[idx][1], hParBuf, 0, 0, 1, 0, 0, ulXSize, ulYSize );
	IPL_BufAllocChild2D( &hHSVBuffers[idx][2], hParBuf, 0, 0, 2, 0, 0, ulXSize, ulYSize );
}

//---------------------------------------------------------------------------
int checkHSVChildBuffers( IPL_BUFHANDLE hParBuf )
{
	IPL_BUFHANDLE h = IPL_DONT_CARE;

	// check child buffers for first process buffer
	if ( hHSVBuffers[0][0] == IPL_DONT_CARE )
	{
		allocHSVChildBuffers( hParBuf, 0 );
		return 0;
	}

	if ( !IPL_BufInquire( hHSVBuffers[0][0], IPL_PARBUFHANDLE, &h ))
		return -1;

	if ( h == hParBuf )
		return 0;

	// check child buffers for second process buffer
	if ( hHSVBuffers[1][0] == IPL_DONT_CARE )
	{
		allocHSVChildBuffers( hParBuf, 1 );
		return 1;
	}

	if ( IPL_BufInquire( hHSVBuffers[1][0], IPL_PARBUFHANDLE, &h ))
		return -1;

	if ( h == hParBuf )
		return 1;

	// error
	return -1;
}

//---------------------------------------------------------------------------
bool ImpactProcess( IPL_BUFHANDLE hSrc, char *text )
{
	int idx;

	// check H-, S-, V- child buffers
	if(( idx = checkHSVChildBuffers( hSrc )) == -1 )
		return false;

	IPL_BUFHANDLE hHBuf = hHSVBuffers[idx][0], hSBuf = hHSVBuffers[idx][1], hVBuf = hHSVBuffers[idx][2];

	// RGB->HSV
	if( !COL_ConvertColorSpace( hSrc, &hSrc, ( COL_CONVERSION)(COL_RGB_SRC | COL_HSV_DST )))
		return false;

	if( lSThr <= 0 )
		IPL_StatBestThreshold( hSBuf, &lSThr );

	if( lVThr <= 0 )
		IPL_StatBestThreshold( hVBuf, &lVThr );

	dprintf(( " == %s - %ld %ld\n", __FUNCTION__, lSThr, lVThr ));

	// bynarisize S and V buffers
	IPL_PtToPoint( hSBuf, lSThr, &hSBuf, IPL_GREATER_CST );
	IPL_PtToPoint( hVBuf, lVThr, &hVBuf, IPL_GREATER_CST );

	// S AND H -> H
	IPL_PtToPoint( hSBuf, hHBuf, &hHBuf, IPL_AND );

	// V AND H -> H
	IPL_PtToPoint( hVBuf, hHBuf, &hHBuf, IPL_AND );


    // calculate histogramm for HBuf
	if( !IPL_StatHistogram( hHBuf, &hHist ))
		return false;

	// get histogramm data
	int tmp[MAX_HIST_SIZE];
	memset( tmp, 0, MAX_HIST_SIZE * sizeof( int ));

	if( !IPL_GetBufData( hHist, 0,0,0,0,0, MAX_HIST_SIZE, 1, tmp ))
		return false;

	IPL_BufFree( hHist );
	hHist = IPL_DONT_CARE;

	// find histogramm maximum
	unsigned long maxidx = 1;
	for( int i = 1; i < MAX_HIST_SIZE; i++ )
	{
		dprintf((" i: %d, bgrd_val: %d, tmp: %d\n", i, bgrd_val, tmp[i] ));
		if(( i < bgrd_val - 5 ) || ( i > bgrd_val + 5 ))
		{
			maxidx = tmp[i] > tmp[maxidx] ? i : maxidx;
			dprintf((" i: %d, maxidx: %ld\n", i, maxidx ));
		}
	}

	dprintf((" have maxidx: %ld\n", maxidx ));

	// find nearest pattern
	int clr = 0;
	for( int j = 0; j < iClrCount; j++ )
		clr = abs( m_clr[j].value - maxidx ) < abs( m_clr[clr].value - maxidx ) ? j : clr;

	sprintf( text, "%s ( %ld )", m_clr[clr].name, maxidx );
	write_io( m_clr[clr].digio );

	return true;
}

//---------------------------------------------------------------------------
int disp_txt( DISP_OOISETHANDLE oois, DISP_OOIHANDLE *loi, int width, char *txt )
{
	IPL_POINT txtpoints[2];
	memset( txtpoints, 0, 2 * sizeof( IPL_POINT ));

	txtpoints[0].xpos = width/2;
	txtpoints[0].ypos = 38;

	txtpoints[1].xpos = width/2 + 1;
	txtpoints[1].ypos = 38;

	*loi = DISP_DONT_CARE;
	DISP_OoiAlloc( oois, loi, DISP_LOI_TOOL, txtpoints, sizeof( txtpoints )/sizeof( txtpoints[0] ), NULL );

	COLORREF color = PALETTERGB( 0, 0, 0 );
	DISP_ControlOoi( oois, *loi, DISP_OOI_COLOR, &color );

	COLORREF txtcolor = PALETTERGB( 255, 255, 255 );
	DISP_ControlOoi( oois, *loi, DISP_OOI_TEXT, txt );
	DISP_ControlOoi( oois, *loi, DISP_OOI_TEXT_COLOR, &txtcolor );

	return 0;
}

//---------------------------------------------------------------------------
int get_dev_bayer_para( DEV_HANDLE deviceHandle, double *displ_green )
{
	int ret = IPL_FALSE;
	if( deviceHandle != IPL_DONT_CARE )
		ret = DEV_InquireCaps( deviceHandle, DEV_BAYER_DISPLAY_GREEN_GAIN, displ_green );

	return ret;
}

//---------------------------------------------------------------------------
int set_disp_bayer_para( double displ_green )
{
	return DISP_Control( DISP_BAYER_DISPLAY_GREEN_GAIN, &displ_green );
}

//---------------------------------------------------------------------------
int main( int argc, char *argv[ ] )
{
#ifdef MALLOC_TRACE
	mtrace( );
#endif	// MALLOC_TRACE
	char params[256], *clrfname = "clr_pattern.clr";
	memset( params, 0, 256 );
	printf("\n usage: ./" DEMO_NAME " [S-Threshold V-Threshold Clr-File]\n");
	printf("\t\t defaults: S-Threshold: %ld V-Threshold: %ld Clr-File: >>%s<<\n", lSThr, lVThr, clrfname );
	if( argc > 1 )
	{
		_skiparg();
		_intarg( lSThr );
		_intarg( lVThr );
		_strarg( clrfname );

		sprintf( params, "%s, %ld, %ld", clrfname, lSThr, lVThr );
	}
	else
		sprintf( params, DEF_PARAM );

	IPL_BUFHANDLE image     = IPL_DONT_CARE;
	IPL_BUFHANDLE rgbimage  = IPL_DONT_CARE;
	DEV_HANDLE device       = IPL_DONT_CARE;
	DISP_OOISETHANDLE oois  = DISP_DONT_CARE;
	DISP_OOIHANDLE loi      = DISP_DONT_CARE;

	IPL_HWND hwnd = 0;
	char text[1024];
	int width = 0, height = 0, comp = 0;

	if( !OpenImpactProcess( params ))
		return -1;

	if( !DEV_Open( &device, "matrixvision.ddc |mvTITAN-CL" ))
	{
		fprintf( stderr, "*** " DEMO_NAME ": open device failed (err: %ld)!!\n", IPL_GetErrorCode( ));
		return -1;
	}

	if( !DEV_Control( device, DEV_SINGLE, 0 ))
	{
		fprintf( stderr, "*** " DEMO_NAME ": set control failed (err: %ld)!!\n", IPL_GetErrorCode( ));
		return -1;
	}

	if( !DEV_Grab( device, &image ))
	{
		fprintf( stderr, "*** " DEMO_NAME ": grab image failed (err: %ld)!!\n", IPL_GetErrorCode( ));
		return -1;
	}

	if( !IPL_BufInquire( image, IPL_XSIZE, &width ))
		fprintf( stderr, "*** " DEMO_NAME ": get X size failed (err: %ld)!!\n", IPL_GetErrorCode( ));
	if( !IPL_BufInquire( image, IPL_YSIZE, &height ))
		fprintf( stderr, "*** " DEMO_NAME ": get Y size failed (err: %ld)!!\n", IPL_GetErrorCode( ));
	if( !IPL_BufInquire( image, IPL_NB_BANDS, &comp ))
		fprintf( stderr, "*** " DEMO_NAME ": get nb bands failed (err: %ld)!!\n", IPL_GetErrorCode( ));

	double displ_green = 0.0;

	if( !get_dev_bayer_para( device, &displ_green ))
	{
		fprintf( stderr, "*** " DEMO_NAME ": get_dev_bayer_para failed (err: %ld)!!\n", IPL_GetErrorCode( ));
		return -1;
	}

	if( !set_disp_bayer_para( displ_green ))
	{
		fprintf( stderr, "*** " DEMO_NAME ": set_disp_bayer_para failed (err: %ld)!!\n", IPL_GetErrorCode( ));
		return -1;
	}

	if( !DISP_OpenWindow( image, &hwnd, 0, NULL ))
	{
		fprintf( stderr, "*** " DEMO_NAME ": open window failed (err: %ld)!!\n", IPL_GetErrorCode( ));
		return -1;
	}

	DISP_UpdateWindow( hwnd, DISP_INVALIDATE );

	memset( text, 0, 1024 );

	DISP_OoiSetAlloc( &oois );

	printf(" Press <ENTER> to stop the demo!!\n");
	while( !_kbhit( ))
	{
		if( !DEV_Grab( device, &image ))
		{
			fprintf( stderr, "*** " DEMO_NAME ": snap image failed!! (err: %ld)\n", IPL_GetErrorCode( ));
			break;
		}

		if( !BM2RGB( image, &rgbimage, width, height, comp ))
			break;

		//assert( IPL_FlExport( rgbimage, IPL_DONT_CARE, IPL_BMP, DEMO_NAME".bmp"));
		ImpactProcess( rgbimage, text );

		printf( "%s\n", text );
		disp_txt( oois, &loi, width, text );

		DISP_ControlWindow( DISP_OOISET, hwnd, &oois );
		DISP_UpdateWindow( hwnd, DISP_INVALIDATE );
		DISP_KeepRunning( NULL );

		if( loi != IPL_DONT_CARE )
		{
			DISP_OoiFree( oois, loi );
			loi = IPL_DONT_CARE;
		}

		//printf("ENTER\n");
		//getchar();
	}

	write_io( 0x00 );
	DEV_Close( device );
	DISP_OoiSetFree( oois );
	IPL_BufFree( image );
	IPL_BufFree( rgbimage );

	return 0;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
