//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>blob-afs<<
// Mit Aug 13 11:31:13 CEST 2003
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>

#include <pgutils.h>
#include <samples.h>

#include "blob-afs.h"
#include "afsapi.h"
#include "blbproc.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static STATUS_INFO si_run_mode = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_analyse_type = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_obj_nb = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_analyse_time = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_feat_cnt = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_feat_max = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_msg = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_fps = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_th = { 0, 0, NULL, -1, MVCOLOR_BLACK };

static SPIN_EDIT se_mf = { 0,{ 0, 0, NULL, -1, MVCOLOR_BLACK },0,0,0 };
#ifdef USE_TIME_VALUE
static SPIN_EDIT se_mt = { 0,{ 0, 0, NULL, -1, MVCOLOR_BLACK },0,0,0 };
#endif	// USE_TIME_VALUE
static SPIN_EDIT se_th = { 0,{ 0, 0, NULL, -1, MVCOLOR_BLACK },0,0,0 };

static pghandle wBitmap = 0, wApp = 0, wSBox = 0, wMainCanvas = 0, wBinImg = 0;
static pghandle wLive = 0, wStop = 0, wClassify = 0, wTeach = 0, wDelete = 0, wTh = 0;
static pghandle wPolarity = 0, wBlbInit = 0;

static pgcontext gcCanvas = NULL;

static COLORREF colTab[ ] =
{
	MVCOLOR_RED,
	MVCOLOR_GREEN,
	MVCOLOR_BLUE,
	MVCOLOR_MAGENTA,
	MVCOLOR_L_BLUE,
	MVCOLOR_YELLOW,
	MVCOLOR_BLACK,
	MVCOLOR_GREY,
	MVCOLOR_WHITE
};

//static unsigned int disp_col_cnt = 0;
static unsigned int redraw_flag = 0;
static char res_msg[MAX_BUF_SIZE];

//----------------------------------------------------------------------
void mk_new_context( );

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// =EVENT= fct. block
//----------------------------------------------------------------------
/*! \brief Close event.*/
int evtClose( struct pgEvent * /*evt*/ )
{
	dprintf((" ==> " TOOL_PREFIX ": evtClose\n" ));
	w_on_close( );
	pgExitEventLoop( );
	close_all( );
	return 0;
}

//----------------------------------------------------------------------
int evtLiveON( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": evtLiveON\n" ));

	set_w_color( wLive, MVCOLOR_GREY );
	set_w_color( wStop, MVCOLOR_BLACK );

	if( get_blob_init_flag( ))
	{
		set_w_color( wClassify, MVCOLOR_BLACK );
		set_w_color( wTeach, MVCOLOR_BLACK );
	}

	if( !run_flag )
	{
		run_flag = 1;
		si_run_mode.msg = START_LIVE_STR;
		repl_txt_si( si_run_mode );
		snap_img( );
	}

	run_analyse( 0 );
	si_analyse_type.msg = NO_ANALYSE_STR;
	repl_txt_si( si_analyse_type );

	dprintf((" -- " TOOL_PREFIX ": evtLiveON\n" ));
	return 0;
}

//----------------------------------------------------------------------
int evtLiveOFF( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": evtLiveOFF\n" ));

	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );

	if( run_flag )
	{
		run_flag = 0;
		si_run_mode.msg = STOP_LIVE_STR;
		repl_txt_si( si_run_mode );
	}

	run_analyse( 0 );
	si_analyse_type.msg = NO_ANALYSE_STR;
	repl_txt_si( si_analyse_type );

	dprintf((" -- " TOOL_PREFIX ": evtLiveOFF\n" ));
	return 0;
}

//----------------------------------------------------------------------
int evtClassify( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": evtClassify\n" ));

	if( !get_blob_init_flag( ))
		return 0;

	set_w_color( wClassify, MVCOLOR_GREY );
	set_w_color( wTeach, MVCOLOR_BLACK );
	set_w_color( wLive, MVCOLOR_GREY );
	set_w_color( wStop, MVCOLOR_BLACK );

	si_analyse_type.msg = CLASSIFY_STR;
	repl_txt_si( si_analyse_type );

	if( check_feat( ) >= 0 )
	{
		set_feat4classify( );
		run_analyse( 1 );

		if( !run_flag )
		{
			run_flag = 1;
			si_run_mode.msg = START_LIVE_STR;
			repl_txt_si( si_run_mode );
			snap_img( );
		}
	}
	else
		note_or_err_msg( AN_ERROR, "The automatic feature selection found no\n"
									"feature to distinguish the taught classes!\n"
									"A possible reason could be an object taught\n"
									"to the wrong class!\n\n"
									"Please check the classes and teach the objects again", 0 );

	set_w_color( wClassify, MVCOLOR_BLACK );
	redraw_img( );

	dprintf((" ++ " TOOL_PREFIX ": evtClassify\n" ));
	return 0;
}

//----------------------------------------------------------------------
static char class_list[MAX_CLASS_NB][MAX_BUF_SIZE];
int read_class_names( int cnt )
{
	dprintf((" ++ " TOOL_PREFIX ": read_class_names (cnt: %d)\n", cnt ));
	int ret = -1;
	if( cnt < MAX_CLASS_NB )
	{
		for( int i = 1; i < cnt; i++ )
		{
			memset( class_list[i-1], 0, MAX_BUF_SIZE );
			if(( ret = get_blb_class_name( i, class_list[i-1] )) < 0 )
				break;
#ifdef DEBUG
			else
				printf(" == " TOOL_PREFIX ": read_class_names - '%s'\n", class_list[i-1]);
#endif	// DEBUG
		}
	}

	dprintf((" -- " TOOL_PREFIX ": read_class_names (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int check_if_exist( char *name, int size )
{
	dprintf((" ++ " TOOL_PREFIX ": check_if_exist (%s %d)\n", name, size ));
	int ret = 0, len;

	if( size == REF_CLASS + 1 )
		ret = 0;
	else
	{
		for( int c = 0; c < size - 1; c++ )
		{
			dprintf((" == " TOOL_PREFIX ": check_if_exist - class_list[%2d]: %s, name:  %s\n",
						c, class_list[c], name ));

			len = max( strlen( name ), strlen( class_list[c] ));
			if( strncmp( class_list[c], name, len ) == 0 )
			{
				ret = 1;
				break;
			}
		}
	}

	dprintf((" -- " TOOL_PREFIX ": check_if_exist (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int add_new_class( char *name, int index )
{
	dprintf((" -> " TOOL_PREFIX ": add_new_class (%s %d)\n", name, index ));

	memset( class_list[index], 0, MAX_BUF_SIZE );
	strncpy( class_list[index], name, strlen( name ));

	return mk_new_blb_class( name );
}

//----------------------------------------------------------------------
int evtTeach( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtTeach\n"));

	if( !get_blob_init_flag( ))
		return 0;

	int kb_pid = open_keybrd( );

	IPL_RECT rect[MAX_BLB_CNT];
	unsigned int index_list[MAX_BLB_CNT];
	memset( index_list, 0, MAX_BLB_CNT );

	int cnt = 0, ret = 0;;
	int w, h, index;
	BYTE *chimg_buffer = NULL;
	char class_name[MAX_BUF_SIZE];
	memset( class_name, 0, MAX_BUF_SIZE );
	int list_size = get_class_cnt( );

	if( list_size > 1 )
	{
		if( read_class_names( list_size ) < 0 )
		{
			note_or_err_msg( AN_ERROR, "Read the class list failed", 0 );
			return 0;
		}
	}

	evtLiveOFF( evt );
	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );
	set_w_color( wTeach, MVCOLOR_GREY );
	set_w_color( wClassify, MVCOLOR_BLACK );
	set_w_color( wDelete, MVCOLOR_BLACK );

	si_analyse_type.msg = TEACH_STR;
	repl_txt_si( si_analyse_type );
	si_iupdate( &si_obj_nb, "Obj", NULL, 0, 1, -1 );
	si_fupdate( &si_analyse_time, "t[an]", "msec", 0.0, 1, -1 );
	si_iupdate( &si_feat_cnt, "feat cnt", NULL, 0, 1, -1 );

	single_snap( );
	get_bin_img( );

	//redraw_flag = 1;
	redraw_img( );

	cnt = get_blob_pos( rect, index_list );

#ifdef DEBUG
	printf(" == " TOOL_PREFIX ": evtTeach - cnt: %d\n", cnt );
	for( int i = 0; i < cnt; i++ )
		printf(" == " TOOL_PREFIX ": evtTeach - list rect[%d](l/t/r/b): %ld %ld %ld %ld, index: %d\n",
							i, rect[i].left, rect[i].top, rect[i].right, rect[i].bottom, index_list[i] );
#endif	// DEBUG

	pgEnterContext( );
	if( cnt > 0 )
	{
		for( int i = 0; i < cnt; i++ )
		{
			dprintf((" == " TOOL_PREFIX ": evtTeach - rect[%d](l/t/r/b): %ld %ld %ld %ld\n",
									i, rect[i].left, rect[i].top, rect[i].right, rect[i].bottom ));
			w = rect[i].right - rect[i].left;
			h = rect[i].bottom - rect[i].top;

			dprintf((" == " TOOL_PREFIX ": evtTeach - rect[%d](l/t/w/h): %ld %ld %d %d\n", i,
							rect[i].left, rect[i].top, w, h ));

			if( w > 0 && h > 0 )
			{
				allocprintf( chimg_buffer, i );
				if(( chimg_buffer = (BYTE*) malloc( w * h )) != NULL )
				{
					allocprintf( chimg_buffer, o );

					get_child_img( chimg_buffer, rect[i].left, rect[i].top, w, h );

					ret = get_class_name( chimg_buffer, w, h, class_name, class_list, list_size - 1, TEACH_GUI );

					if( ret == RET_CLOSE )
					{
						freeprintf( chimg_buffer, i );
							chimg_buffer = (BYTE*)clear_buf( chimg_buffer );
						freeprintf( chimg_buffer, o );
						break;
					}
					else if( ret != RET_SKIPP && ret != RET_ERROR )
					{
						dprintf((" == " TOOL_PREFIX ": evtTeach - class_name: >%s<\n", class_name ));

						if( list_size < MAX_CLASS_NB )
						{
							if( !check_if_exist( class_name, list_size ))
								list_size = add_new_class( class_name, list_size - 1 );

#ifdef DEBUG
							for( int c = 0; c < list_size - 1; c++ )
								printf(" == evtTeach - class_list[%d]: %s\n", c, class_list[c] );
#endif	// DEBUG

							if(( index = get_class_index( class_name )) >= 0 )
								teach_all_results( index, index_list[i], chimg_buffer, w, h );
						}
						else
							note_or_err_msg( AN_ERROR, "Maximal possibile number of classes reached", 0 );

						freeprintf( chimg_buffer, i );
							chimg_buffer = (BYTE*)clear_buf( chimg_buffer );
						freeprintf( chimg_buffer, o );
					}
				}
			}
			else
				fprintf( stderr, "*** wrong blob size %dx%d\n", w, h );
		}
	}
	pgLeaveContext( );

	set_w_color( wTeach, MVCOLOR_BLACK );
	close_keybrd( kb_pid );

	//redraw_flag = 1;
	redraw_img( );

	dprintf((" -- " TOOL_PREFIX ": evtTeach\n"));
	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
void redraw_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": redraw_img - redraw_flag: %d\n", redraw_flag ));

	//if( redraw_flag )
	{
		int width = 0, height = 0;
		pgSizeBitmap( &width, &height, wBitmap );
		dprintf((" == " TOOL_PREFIX ": redraw_img - will redraw the image (WxH: %dx%d)\n", width, height ));

		pgWriteCmd( wMainCanvas,
				PGCANVAS_GROP, 6,
				PG_GROP_BITMAP, 0, 0, width, height, wBitmap );

		pgFlushRequests( );
		pgContextUpdate( gcCanvas );

		redraw_flag = 0;
		pgSetIdle( 0, NULL );
	}

	dprintf((" -- " TOOL_PREFIX ": redraw_img - redraw_flag: %d\n", redraw_flag ));
}

//----------------------------------------------------------------------
void display_bounding_box( char *text, unsigned long left, unsigned long top,
					unsigned long right, unsigned long bottom, int pos )
{
	dprintf((" ++ " TOOL_PREFIX ": display_bounding_box\n" ));
	unsigned long w, h;
	pghandle color;

	if( pos >= 0 )
		 color = colTab[pos];
	else if( pos < 0 )
		color = MVCOLOR_RED;

	if( text && get_obj_cnt( ) == 1 )
	{
		memset( res_msg, 0, MAX_BUF_SIZE );
		sprintf( res_msg, "%s", text );
	}
	else
	{
		memset( res_msg, 0, MAX_BUF_SIZE );
		sprintf( res_msg, " " );
	}

	dprintf((" == " TOOL_PREFIX ": display_bounding_box - l/t: %ld/%ld, r/b: %ld/%ld, pos: %d\n",
										left, top, right, bottom, pos ));

	mk_new_context( );
	pgEnterContext( );

	w = right - left;
	h = bottom - top;

	dprintf((" == " TOOL_PREFIX ": display_bounding_box - w/h: %ld/%ld\n", w, h ));

	draw_aoi( gcCanvas, left, top, w, h, color  );
	draw_txt( gcCanvas, left, top, text, color );
	pgContextUpdate( gcCanvas );

	pgLeaveContext( );

	dprintf((" -- " TOOL_PREFIX ": display_bounding_box\n" ));
}

//----------------------------------------------------------------------
pghandle init_pg( int argc, char *argv[], char *msg )
{
	dprintf((" ++ " TOOL_PREFIX ": init_pg\n" ));

	pgInit( argc, argv );
	wApp = pgRegisterApp( PG_APP_NORMAL, msg, 0 );
	pgBind( PGDEFAULT, PG_WE_CLOSE, &evtClose, NULL );

	dprintf((" -- " TOOL_PREFIX ": init_pg\n" ));
	return wApp;
}

//----------------------------------------------------------------------
void displ_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": displ_img\n" ));
	if( wMainCanvas > 0 && wBitmap > 0 )
	{
		int width = 0, height = 0;
		pgSizeBitmap( &width, &height, wBitmap );

		dprintf((" == " TOOL_PREFIX ": displ_img - width: %d, height: %d\n", width, height ));
		pgWriteCmd( wMainCanvas,
			PGCANVAS_GROP, 6,
			PG_GROP_BITMAP, 0, 0, width, height, wBitmap );

		pgContextUpdate( gcCanvas );
		pgSubUpdate( wMainCanvas );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": displ_img - wMainCanvas: %ld, wBitmap: %ld\n", wMainCanvas, wBitmap );

	dprintf((" -- " TOOL_PREFIX ": displ_img\n" ));
}

//----------------------------------------------------------------------
void mk_new_context( )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_new_context\n" ));

	pgWriteCmd( wMainCanvas, PGCANVAS_NUKE, 0 );
	pgDeleteContext( gcCanvas );
	gcCanvas = pgNewCanvasContext( wMainCanvas, PGFX_IMMEDIATE );

	dprintf((" -- " TOOL_PREFIX ": mk_new_context\n" ));
}

//----------------------------------------------------------------------
void mk_canvas( )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_canvas\n" ));

	widget_delete( &wMainCanvas );
	wMainCanvas = pgNewWidget( PG_WIDGET_CANVAS, PG_DERIVE_INSIDE, wSBox );

	mk_new_context( );
	dprintf((" -- " TOOL_PREFIX ": mk_canvas\n" ));
}

//----------------------------------------------------------------------
void mk_disp( pghandle widget, int pos )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_disp\n"));

	widget_delete( &wSBox );
	wSBox = pgNewWidget( PG_WIDGET_SCROLLBOX, pos, widget );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_ALL,
				0 );

	mk_canvas( );
	displ_img( );

	dprintf((" -- " TOOL_PREFIX ": mk_disp\n"));
}

//----------------------------------------------------------------------
void get_img4displ( BYTE **shm_buffer, long width, long height )
{
	dprintf((" ++ " TOOL_PREFIX ": get_img4displ\n" ));

	wBitmap = get_shm_buf( shm_buffer, width, height );

	dprintf((" -- " TOOL_PREFIX ": get_img4displ\n" ));
}

//----------------------------------------------------------------------
int evtDelete( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtDelete\n"));

	if( !get_blob_init_flag( ))
		return 0;

	int ret = 0, index = -1;
	int kb_pid = open_keybrd( );


	char class_name[MAX_BUF_SIZE];
	memset( class_name, 0, MAX_BUF_SIZE );
	int list_size = get_class_cnt( );

	if( list_size > 1 )
	{
		if( read_class_names( list_size ) < 0 )
		{
			note_or_err_msg( AN_ERROR, "Read the class list failed", 0 );
			return 0;
		}
	}

	evtLiveOFF( evt );
	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );
	set_w_color( wTeach, MVCOLOR_BLACK );
	set_w_color( wClassify, MVCOLOR_BLACK );
	set_w_color( wDelete, MVCOLOR_GREY );

	si_analyse_type.msg = DELETE_STR;
	repl_txt_si( si_analyse_type );
	si_iupdate( &si_obj_nb, "Obj", NULL, 0, 1, -1 );
	si_fupdate( &si_analyse_time, "t[an]", "msec", 0.0, 1, -1 );
	si_iupdate( &si_feat_cnt, "feat cnt", NULL, 0, 1, -1 );
	si_fupdate( &si_fps, "Frames", NULL, 0.0, 1, -1 );

	pgEnterContext( );

	if(( ret = get_class_name( NULL, 0, 0, class_name, class_list, list_size - 1, DELETE_GUI )) == RET_SUCCESS )
	{
		dprintf((" == " TOOL_PREFIX ": evtDelete - class_name: >%s<\n", class_name ));

		if( check_if_exist( class_name, list_size ))
		{
			if(( index = get_class_index( class_name )) >= 0 )
			{
				dprintf((" == " TOOL_PREFIX ": evtDelete - index: %d\n", index ));
				delete_class( index );
			}
			else
				printf("***" TOOL_PREFIX ": evtDelete - index: %d\n", index );
		}
	}

	pgLeaveContext( );

	set_w_color( wDelete, MVCOLOR_BLACK );

	si_analyse_type.msg = NO_ANALYSE_STR;
	repl_txt_si( si_analyse_type );

	close_keybrd( kb_pid );
	redraw_img( );

	dprintf((" -- " TOOL_PREFIX ": evtDelete\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtChBoxPol( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": evtChBoxPol\n"));

	set_polarity( pgGetWidget( wPolarity, PG_WP_ON ));

	dprintf((" -- " TOOL_PREFIX ": evtChBoxPol\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtBinImg( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": evtBinImg\n"));
	int sflag = pgGetWidget( wBinImg, PG_WP_ON );

	displ_binimg( sflag );

	dprintf((" -- " TOOL_PREFIX ": evtBinImg\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtTh( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": evtTh\n"));

	int th_flag = pgGetWidget( wTh, PG_WP_ON );

	dprintf((" == " TOOL_PREFIX ": evtTh - th_flag: %d\n", th_flag ));

	if( th_flag )
		pgSetWidget( se_th.si.wText, PG_WP_COLOR, MVCOLOR_BLACK, 0 );
	else
		pgSetWidget( se_th.si.wText, PG_WP_COLOR, MVCOLOR_GREY, 0 );

	pgSubUpdate( se_th.si.wText );

	use_auto_threshold( th_flag );

	dprintf((" -- " TOOL_PREFIX ": evtTh\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtInit( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtInit\n"));

	evtLiveOFF( evt );

	if( !get_blob_init_flag( ))
	{
		init_blb_globals2fl( );
		pgSetWidget( wClassify, PG_WP_COLOR, MVCOLOR_BLACK, 0 );
		pgSetWidget( wTeach, PG_WP_COLOR, MVCOLOR_BLACK, 0 );
		pgSetWidget( wDelete, PG_WP_COLOR, MVCOLOR_BLACK, 0 );
		pgSetWidget( wBlbInit, PG_WP_COLOR, MVCOLOR_GREY, 0 );
	}

	dprintf((" -- " TOOL_PREFIX ": evtInit\n"));
	return 0;
}

//----------------------------------------------------------------------
void init_gui( )
{
	dprintf((" ++ " TOOL_PREFIX ": init_gui\n"));

	set_se_val( &se_mf, get_max_feat_nb( ));

#ifdef USE_TIME_VALUE
	set_se_val( &se_mt, get_max_time_ms( ));
#endif	// USE_TIME_VALUE

	pgSetWidget( wPolarity, PG_WP_ON, get_polarity(), 0 );
	pgSetWidget( wTh, PG_WP_ON, get_threshold_mode( ), 0 );
	set_se_val( &se_th, get_threshold( ));
	si_iupdate( &si_th, "Th", NULL, get_threshold( ), 1, -1 );

	dprintf((" -- " TOOL_PREFIX ": init_gui\n"));
}

//----------------------------------------------------------------------
pghandle mk_buttons( pghandle parent )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_buttons\n"));

	char msg[MAX_BUF_SIZE];
	memset( msg, 0, MAX_BUF_SIZE );

	pghandle wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_INSIDE, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIZE, 30,
				0 );

	pghandle color = MVCOLOR_BLACK;

	if( !get_blob_init_flag( ))
		color = MVCOLOR_GREY;

	wLive = mk_ctrl_btn( wBox, "Live", PG_DERIVE_INSIDE, &evtLiveON, MVCOLOR_BLACK, 10 );
	wStop = mk_ctrl_btn( PGDEFAULT, "Stop", PG_DERIVE_AFTER, &evtLiveOFF, MVCOLOR_GREY, 10 );
	wClassify = mk_ctrl_btn( PGDEFAULT, "Classify", PG_DERIVE_AFTER, &evtClassify, color, 10 );
	wTeach = mk_ctrl_btn( PGDEFAULT, "Teach", PG_DERIVE_AFTER, &evtTeach, color, 10 );
	wDelete = mk_ctrl_btn( PGDEFAULT, "Delete", PG_DERIVE_AFTER, &evtDelete, color, 10 );

	if( get_blob_init_flag( ))
		se_mf = mk_spin_edit( wDelete, get_max_feat_nb( ), "<- Max Feature" );
	else
	{
		wBlbInit = mk_ctrl_btn( wDelete, "Init", PG_DERIVE_AFTER, &evtInit, MVCOLOR_BLACK, 10 );
		se_mf = mk_spin_edit( wBlbInit, get_max_feat_nb( ), "<- Max Feature" );
	}

#ifdef USE_TIME_VALUE
	se_mt = mk_spin_edit( se_mf.wBox, get_max_time_ms( ), "<- Max Time(msec)" );
	pghandle wBox1 = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_AFTER, se_mt.wBox );
#else
	pghandle wBox1 = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_AFTER, se_mf.wBox );
#endif	// USE_TIME_VALUE

	int ts = get_x_size( "White on Black" );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIZE, ts + 30,
				PG_WP_SIDE, PG_S_LEFT,
				0 );
	wPolarity = mk_chb( PGDEFAULT, "White on Black", &evtChBoxPol, PG_DERIVE_INSIDE, 0 );

	ts = get_x_size( "Displ. bin. img." );
	pghandle wBox2 = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_AFTER, wBox1 );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIZE, ts + 30,
				PG_WP_SIDE, PG_S_LEFT,
				0 );
	wBinImg = mk_chb( PGDEFAULT, "Displ. bin. img.", &evtBinImg, PG_DERIVE_INSIDE, 0 );

	ts = get_x_size( "Threshold" );
	pghandle wBox3 = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_AFTER, wBox2 );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIZE, ts + 30,
				PG_WP_SIDE, PG_S_LEFT,
				0 );
	wTh = mk_chb( PGDEFAULT, "Threshold", &evtTh, PG_DERIVE_INSIDE, 0 );

	se_th = mk_spin_edit( wBox3, get_threshold( ), NULL );
	pgSetWidget( se_th.si.wText, PG_WP_COLOR, MVCOLOR_GREY, 0 );
	pgSubUpdate( se_th.si.wText );

	dprintf((" -- " TOOL_PREFIX ": mk_buttons\n"));

	return wBox;
}

//----------------------------------------------------------------------
void mk_blb_initdlg( )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_blb_initdlg\n"));
	pgEnterContext( );
	pgNewPopupAt( PG_POPUP_CENTER, PG_POPUP_CENTER, PGDEFAULT, PGDEFAULT );
	pgNewWidget( PG_WIDGET_LABEL, 0, 0 );
    pgSetWidget( PGDEFAULT,
		PG_WP_TEXT, pgNewString("Blob initalization! Please wait a moment..."),
		0 );

	pgUpdate();
	pgEventPoll( );
	dprintf((" -- " TOOL_PREFIX ": mk_blb_initdlg\n"));
}

//----------------------------------------------------------------------
void rm_blb_initdlg( )
{
	dprintf((" ++ " TOOL_PREFIX ": rm_blb_initdlg\n"));
	pgLeaveContext( );

	dprintf((" -- " TOOL_PREFIX ": rm_blb_initdlg\n"));
}

//----------------------------------------------------------------------
void update_status_line( int flag )
{
	dprintf((" ++ " TOOL_PREFIX ": update_status_line\n" ));

	if( flag )
	{
		si_iupdate( &si_obj_nb, "Obj", NULL, get_obj_cnt( ), 1, -1 );
		si_fupdate( &si_analyse_time, "t[an]", "msec", get_calc_time_ms( ), 1, -1 );
		si_iupdate( &si_feat_cnt, "Feat Cnt", NULL, get_feat_cnt( ), 1, -1 );

		if( !get_threshold_mode( ))
			si_iupdate( &si_th, "Th", NULL, get_threshold( ), 1, -1 );

		si_fupdate( &si_fps, "Frames", NULL, get_frame_ps( ), 1, -1 );
	}

	if( get_err_code( ) < 0 )
		si_cupdate( &si_msg, NULL, NULL, STOP_AFTER_STR( 20 ), 1, -1 );
	else
	{
		if( res_msg )
			si_cupdate( &si_msg, NULL, NULL, res_msg, 1, -1 );
		else
			si_cupdate( &si_msg, NULL, NULL, " ", 1, -1 );
	}

	dprintf((" -- " TOOL_PREFIX ": update_status_line\n" ));
}

//----------------------------------------------------------------------
void mk_statLineLabel( pghandle parent )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_statLineLabel\n" ));
	char msg[MAX_BUF_SIZE];
	memset( msg, 0, MAX_BUF_SIZE );

	widget_delete( &si_run_mode.wText );
	widget_delete( &si_run_mode.wBox );
	si_run_mode.msg = STOP_LIVE_STR;
	si_run_mode.msg_len = get_x_size( si_run_mode.msg );
	mk_stll_si( parent, PG_DERIVE_INSIDE, &si_run_mode );

	widget_delete( &si_analyse_type.wText );
	widget_delete( &si_analyse_type.wBox );
	si_analyse_type.msg = NO_ANALYSE_STR;
	si_analyse_type.msg_len = get_x_size( si_analyse_type.msg );
	mk_stll_si( si_run_mode.wBox, PG_DERIVE_AFTER, &si_analyse_type );

	widget_delete( &si_obj_nb.wText );
	widget_delete( &si_obj_nb.wBox );
	sprintf( msg, "Obj: %4ld", get_obj_cnt( ));
	si_obj_nb.msg = msg;
	mk_stll_si( si_analyse_type.wBox, PG_DERIVE_AFTER, &si_obj_nb );

	widget_delete( &si_analyse_time.wText );
	widget_delete( &si_analyse_time.wBox );
	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "t[an]: %6.3f msec", get_calc_time_ms( ));
	si_analyse_time.msg = msg;
	si_analyse_time.msg_len = get_x_size( "t[an]: 000000.000 msec" );
	mk_stll_si( si_obj_nb.wBox, PG_DERIVE_AFTER, &si_analyse_time );

	widget_delete( &si_feat_cnt.wText );
	widget_delete( &si_feat_cnt.wBox );
	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "Feat Cnt: %d", get_feat_cnt( ));
	si_feat_cnt.msg = msg;
	mk_stll_si( si_analyse_time.wBox, PG_DERIVE_AFTER, &si_feat_cnt );

	widget_delete( &si_feat_max.wText );
	widget_delete( &si_feat_max.wBox );
	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "Max Feat: %d", get_max_feat( ));
	si_feat_max.msg = msg;
	/*si_feat_max.msg_col = MVCOLOR_GREY;*/
	mk_stll_si( si_feat_cnt.wBox, PG_DERIVE_AFTER, &si_feat_max );

	widget_delete( &si_msg.wText );
	widget_delete( &si_msg.wBox );
	si_msg.msg_len = get_x_size( STOP_AFTER_STR( MAX_BLB_CNT ) );
	si_msg.msg = " ";
	mk_stll_si( si_feat_max.wBox, PG_DERIVE_AFTER, &si_msg );

	widget_delete( &si_th.wText );
	widget_delete( &si_th.wBox );
	si_th.msg_len = get_x_size( "Th: 000" );
	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "Th:  %3ld", get_threshold( ));
	si_th.msg = msg;
	mk_stll_si( si_msg.wBox, PG_DERIVE_AFTER, &si_th );

	widget_delete( &si_fps.wText );
	widget_delete( &si_fps.wBox );
	si_fps.msg_len = get_x_size( "Frames: 000.00" );
	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "Frames:  %.2f", get_frame_ps( ));
	si_fps.msg = msg;
	mk_stll_si( si_th.wBox, PG_DERIVE_AFTER, &si_fps );

	dprintf((" -- " TOOL_PREFIX ": mk_statLineLabel\n" ));
}

//----------------------------------------------------------------------
void set_val_from_se( struct pgEvent *evt, int val )
{
	dprintf((" ++ " TOOL_PREFIX ": set_val_fom_se (%d)\n", val ));

	if(( evt->from == se_mf.wBtnUp ) || ( evt->from == se_mf.wBtnDown ))
	{
		set_max_feat_nb( val );
		set_se_val( &se_mf, get_max_feat_nb( ));
	}
#ifdef USE_TIME_VALUE
	else if(( evt->from == se_mt.wBtnUp ) || ( evt->from == se_mt.wBtnDown ))
	{
		set_max_time_ms( val );
		set_se_val( &se_mt, get_max_time_ms( ));
	}
#endif	// USE_TIME_VALUE
	else if(( evt->from == se_th.wBtnUp ) || ( evt->from == se_th.wBtnDown ))
	{
		if( val > 255 )
			val = 255;
		else if( val < 0 )
			val = 0;

		if( get_threshold_mode( ))
		{
			set_threshold( val );
			set_se_val( &se_th, get_threshold( ));
		}
	}

	dprintf((" -- " TOOL_PREFIX ": set_val_fom_se (%d)\n", val ));
}

//----------------------------------------------------------------------
SPIN_EDIT get_se( struct pgEvent *evt, long *se_cnt, unsigned int *step )
{
	dprintf((" ++ " TOOL_PREFIX ": get_se_val\n"));
	SPIN_EDIT ret = { 0,{ 0, 0, NULL, -1, MVCOLOR_BLACK },0,0,0 };

	if(( evt->from == se_mf.wBtnUp ) || ( evt->from == se_mf.wBtnDown ))
	{
		dprintf((" == get_se_val - from feat\n"));
		*se_cnt = get_max_feat_nb( );
		*step = 1;
		ret = se_mf;
	}
#ifdef USE_TIME_VALUE
	else if(( evt->from == se_mt.wBtnUp ) || ( evt->from == se_mt.wBtnDown ))
	{
		dprintf((" == get_se_val - from time\n"));
		*se_cnt = get_max_time_ms( );
		*step = 5;
		ret = se_mt;
	}
#endif	// USE_TIME_VALUE
	else if(( evt->from == se_th.wBtnUp ) || ( evt->from == se_th.wBtnDown ))
	{
		dprintf((" == get_se_val - from threshold\n"));
		*se_cnt = get_threshold( );

		if( get_threshold_mode( ))
		{
			*step = 2;
			se_th.si.msg_col = MVCOLOR_BLACK;
		}
		else
		{
			*step = 0;
			se_th.si.msg_col = MVCOLOR_GREY;
		}
		ret = se_th;
	}

	dprintf((" -- " TOOL_PREFIX ": get_se_val\n"));
	return ret;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
