//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>blob-afs<<
// Tue May 20 14:35:47 CEST 2003
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include <picogui.h>
#include <system.h>
#include <pgutils.h>

#include "blob-afs.h"
#include "afsapi.h"

//-------------------------------------------------------------------
void err_msg( const char *s, long err_code );

//-------------------------------------------------------------------
//-------------------------------------------------------------------
void widget_delete( pghandle *widget )
{
	dprintf((" ++ " TOOL_PREFIX ": widget_delete\n" ));

	if( *widget != 0 )
	{
		pgDelete( *widget );
		*widget = 0;
	}

	dprintf((" -- " TOOL_PREFIX ": widget_delete\n" ));
}

//-------------------------------------------------------------------
void cp_chimg( BYTE *src, BYTE *dst, int sx, int sy, int width, int height, int spitch, int dpitch )
{
	dprintf((" ++ " TOOL_PREFIX ": cp_chimg - sx: %d, sy: %d, width: %d, height: %d, spitch: %d, dpitch: %d\n",
					sx, sy, width, height, spitch, dpitch ));

	if( src != NULL )
	{
		int x, y;
		for( y = 0; y < height; y++ )
		{
			for( x = 0; x < width; x++ )
				dst[x + y * dpitch] = src[ x + sx + ( y + sy ) * spitch];
		}
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": cp_chimg - no source image found!!\n");

	dprintf((" -- " TOOL_PREFIX ": cp_chimg\n"));
}

//-------------------------------------------------------------------
void cp_img( BYTE *src, BYTE *dst, int width, int height, int spitch, int dpitch )
{
	dprintf((" ++ " TOOL_PREFIX ": cp_img\n" ));

	if( src != NULL )
	{
		int y;
		for( y = 0; y < height; y++ )
			memcpy( dst + y * dpitch, src + y * spitch, width );
	}
	else
	{
		fprintf( stderr, "*** " TOOL_PREFIX ": cp_img - no source image found!!\n");
		err_msg( "Empty source image",  -1 );
	}

	dprintf((" -- " TOOL_PREFIX ": cp_img\n" ));
}

//-------------------------------------------------------------------
void repl_txt_si( STATUS_INFO si )
{
	dprintf((" ++ " TOOL_PREFIX ": repl_txt_si - >>%s<<\n", si.msg ));
	pghandle oldtext;

	oldtext = pgGetWidget( si.wText ,PG_WP_TEXT );
	if( si.msg_len != -1 )
		pgSetWidget( si.wText,
					PG_WP_TEXT, pgNewString( si.msg ),
					PG_WP_COLOR, si.msg_col,
					PG_WP_SIZE, si.msg_len,
					0 );
	else
		pgSetWidget( si.wText,
					PG_WP_TEXT, pgNewString( si.msg ),
					PG_WP_COLOR, si.msg_col,
					0 );

	pgSubUpdate( si.wBox );
	pgSubUpdate( si.wText );

	pgDelete( oldtext );

	dprintf((" -- " TOOL_PREFIX ": repl_txt_si\n"));
}

//-------------------------------------------------------------------
void repl_txt( pghandle widget, char *text )
{
	dprintf((" ++ " TOOL_PREFIX ": repl_txt - >>%s<<\n", text ));

	pgReplaceText( widget, text );
	pgSubUpdate( widget );

	dprintf((" -- " TOOL_PREFIX ": repl_txt\n"));
}

//-------------------------------------------------------------------
void draw_aoi( pgcontext gc, int x, int y, int w, int h, pgcolor color )
{
	dprintf((" ++ " TOOL_PREFIX ": draw_aoi - x: %d, y: %d, w: %d, h: %d, color: 0x%x\n",
								x, y, w, h, color ));

	pgSetColor( gc, color );
	pgFrame( gc, x, y, w, h );
	pgContextUpdate( gc );

	dprintf((" -- " TOOL_PREFIX ": draw_aoi\n"));
}

//-------------------------------------------------------------------
void draw_txt( pgcontext gc, int x, int y, char *text, pgcolor color )
{
	dprintf((" ++ " TOOL_PREFIX ": draw_txt - x: %d, y: %d, text: >>%s<<, color: 0x%x\n",
						x, y, text, color ));

	pgSetColor( gc, color );
	pgText( gc, x, y, pgNewString( text ));
	pgContextUpdate( gc );

	dprintf((" -- " TOOL_PREFIX ": draw_txt\n"));
}

//-------------------------------------------------------------------
void note_or_err_msg( const char *msg_type, const char *s, long err_code )
{
	dprintf((" ++ " TOOL_PREFIX ": note_or_err_msg\n" ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );

	if( err_code == 0 )
		sprintf( msg, "%s !!\n", s );
	else
		sprintf( msg, "%s -> err: %ld !!\n", s, err_code );

	pgMessageDialog( msg_type, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	dprintf((" -- " TOOL_PREFIX ": note_or_err_msg\n" ));
}

//-------------------------------------------------------------------
void err_msg( const char *s, long err_code )
{
	dprintf((" ++ " TOOL_PREFIX ": err_msg\n" ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );

	sprintf( msg, "*** An error hase occurred! ***\n"
					"Error message and code:\n"
					"\"%s -> err: %ld\"!!\n", s, err_code );
	pgMessageDialog( AN_ERROR, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	fprintf( stderr, "*** " TOOL_PREFIX ": %s - err: %ld!!\n", s, err_code );

	dprintf((" -- " TOOL_PREFIX ": err_msg\n" ));
}

//-------------------------------------------------------------------
void exit_demo( const char *s, long err_code )
{
	dprintf((" ++ " TOOL_PREFIX ": exit_demo\n" ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );

	sprintf( msg, "*** An fatal error hase occurred! ***\n"
					"Error message and code:\n"
					"\"%s -> err: %ld\"\n"
					"Application will be terminated!!", s, err_code );
	pgMessageDialog( "FATAL ERROR!!", msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	fprintf( stderr, "*** " TOOL_PREFIX ": %s - err: %ld!!\n", s, err_code );
	close_all( );

	dprintf((" -- " TOOL_PREFIX ": exit_demo\n" ));
	exit( err_code );
}

//-------------------------------------------------------------------
pghandle mk_chb( pghandle parent, char *txt, pgevthandler chbox_cb, int pos, int status )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_chb\n"));

	pghandle wChBox = pgNewWidget( PG_WIDGET_CHECKBOX, pos, parent );
	pgSetWidget( wChBox,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_EXTDEVENTS, PG_EXEV_TOGGLE,
				PG_WP_ON, status,
				0 );

	if( chbox_cb != NULL )
		pgBind( wChBox, PG_WE_ACTIVATE, chbox_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_chb\n"));

	return wChBox;
}

//-------------------------------------------------------------------
pghandle mk_btn( pghandle parent, char *txt, int pos, int size, pgevthandler btn_cb )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_btn\n"));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_SIZE, size,
				PG_WP_FONT, pgNewFont( NULL, 6, 0 ),
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, btn_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_btn\n"));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_ctrl_btn( pghandle parent, char *txt, int pos, pgevthandler btn_cb, pghandle color, int fsize )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_ctrl_btn\n"));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, fsize, 0 ),
				PG_WP_COLOR, color,
				0 );

	if( btn_cb != NULL )
		pgBind( PGDEFAULT, PG_WE_ACTIVATE, btn_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_ctrl_btn\n"));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_label( pghandle parent, char *txt, int pos, pghandle txt_col )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_label\n"));

	pghandle wLabel = pgNewWidget( PG_WIDGET_LABEL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_ALIGN, PG_A_RIGHT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_COLOR, txt_col,
				PG_WP_SIZE, 8,
				0 );

	dprintf((" -- " TOOL_PREFIX ": mk_label\n"));

	return wLabel;
}

//-------------------------------------------------------------------
pghandle mk_scroller( pghandle parent, int pos, int max_s, int def_s, pgevthandler scroll_cb )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_scroller - max_s: %d, def_s: %d\n", max_s, def_s ));

	pghandle wScroller = pgNewWidget( PG_WIDGET_SCROLL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, max_s,
				PG_WP_VALUE, def_s,
				PG_WP_SCROLL_X, 0,
				PG_WP_SCROLL_Y, def_s,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, scroll_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_scroller\n" ));

	return wScroller;
}

//-------------------------------------------------------------------
pghandle mk_wBox( pghandle parent, int pat_pos, int side, int size )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_wBox\n" ));

	pghandle wigdet = pgNewWidget( PG_WIDGET_BOX, pat_pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, side,
				PG_WP_SIZE, size,
				0 );

	dprintf((" -- " TOOL_PREFIX ": mk_wBox\n" ));

	return wigdet;
}

//-------------------------------------------------------------------
pghandle mk_statline( pghandle parent, int pos, int size )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_statline\n" ));
	pghandle wStatLine = pgNewWidget( PG_WIDGET_TOOLBAR, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, size,
				0 );

	dprintf((" -- " TOOL_PREFIX ": mk_statline\n" ));

	return wStatLine;
}

//-------------------------------------------------------------------
void get_str_size( int *x, int *y, pghandle font, char *str )
{
	dprintf((" ++ " TOOL_PREFIX ": get_str_size\n" ));
	pgEnterContext( );
	pgSizeText( x, y, font, pgNewString( str ));
	pgLeaveContext( );
	dprintf((" -- " TOOL_PREFIX ": get_str_size\n" ));
}

//-------------------------------------------------------------------
void mk_stll_si( pghandle parent, int pos, STATUS_INFO *si )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_stll_si\n" ));
	int x,y, offs = 5;

	if( si->msg != NULL )
	{
		x = si->msg_len;

		if( x < 0 )
		{
			pghandle font = pgNewFont( NULL, 8, 0 );
			get_str_size( &x, &y, font, si->msg );
		}

		si->wBox = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
		si->wText = mk_label( PGDEFAULT, si->msg, PG_DERIVE_INSIDE, si->msg_col );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": mk_stll_si - wrong msg (%s)!!\n", si->msg );

	dprintf((" -- " TOOL_PREFIX ": mk_stll_si\n" ));
}

//-------------------------------------------------------------------
pghandle mk_stll( pghandle parent, int pos, pghandle *wText, char *str, int size )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_stll\n" ));
	int x,y, offs = 5;

	x = size;

	if( size < 0 )
	{
		pghandle font = pgNewFont( NULL, 8, 0 );
		get_str_size( &x, &y, font, str );
	}

	pghandle wB = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
	*wText = mk_label( PGDEFAULT, str, PG_DERIVE_INSIDE, MVCOLOR_BLACK );

	dprintf((" -- " TOOL_PREFIX ": mk_stll\n" ));
	return wB;
}

//-------------------------------------------------------------------
void set_w_color( pghandle widget, pghandle color )
{
	dprintf((" ++ " TOOL_PREFIX ": set_w_color\n" ));

	pgSetWidget( widget, PG_WP_COLOR, color, 0 );
	pgSubUpdate( widget );

	dprintf((" -- " TOOL_PREFIX ": set_w_color\n" ));
}

//-------------------------------------------------------------------
int check_if_file_exist( char *fname )
{
	FILE *f;
	char s[MAX_CHAR_SIZE];
	memset( s, 0, MAX_CHAR_SIZE );

	dprintf(( " ++ " TOOL_PREFIX ": check_if_file_exist\n" ));

	if(( f = fopen( fname, "r" )) == NULL )
	{
		sprintf( s, "No '%s' found!!", fname );
		pgMessageDialog( KB_ERROR, s, PG_MSGBTN_OK | PG_MSGICON_ERROR );
		return -1;
	}

	fclose( f );

	dprintf(( " -- " TOOL_PREFIX ": check_if_file_exist\n" ));

	return 0;
}

//-------------------------------------------------------------------
void get_s_size( char *s, int *x, int *y )
{
	dprintf(( " ++ " TOOL_PREFIX ": get_s_size - s: >>%s<<, x: %d, y: %d\n",s, *x, *y ));
	pghandle font = pgNewFont( NULL, 8, 0 );
	get_str_size( x, y, font, s );
	dprintf(( " -- " TOOL_PREFIX ": get_s_size - x: %d, y: %d\n", *x, *y ));
}

//-------------------------------------------------------------------
int get_x_size( char *s )
{
	dprintf(( " ++ " TOOL_PREFIX ": get_x_size - s: >>%s<<\n", s ));

	int x = 0, y = 0;
	get_s_size( s, &x, &y );

	dprintf(( " -- " TOOL_PREFIX ": get_x_size - x: %d\n", y ));

	return x;
}

//-------------------------------------------------------------------
int get_y_size( char *s )
{
	dprintf(( " ++ " TOOL_PREFIX ": get_x_size - s: >>%s<<\n", s ));

	int x = 0, y = 0;
	get_s_size( s, &x, &y );

	dprintf(( " -- " TOOL_PREFIX ": get_x_size - y: %d\n", y ));
	return y;
}

//-------------------------------------------------------------------
void ni( )
{
	pgMessageDialog( NOTE_STR, "\nThis functionality is not implemented jet!\n\nBut it soon will be :o)\n", PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
}

//-------------------------------------------------------------------
void note4max_limit( char *type, int val )
{
	dprintf((" ++ " TOOL_PREFIX ": note4max_limit\n" ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "The maximal %s is limitated to %d!!\n"
					"The %s will be set to this value!\n", type, val, type );
	pgMessageDialog( NOTE_STR, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	dprintf((" -- " TOOL_PREFIX ": note4max_limit\n" ));
}

//-------------------------------------------------------------------
void note4min_limit( char *type, int val )
{
	dprintf((" ++ " TOOL_PREFIX ": note4min_limit\n" ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "The entered value is %d!! This is an illegal value!\n"
					"The %s will be set to the default value!\n", val, type );
	pgMessageDialog( NOTE_STR, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	dprintf((" -- " TOOL_PREFIX ": note4min_limit\n" ));
}

//-------------------------------------------------------------------
pghandle get_stat_color( int col_stat_mode )
{
	dprintf((" ++ " TOOL_PREFIX ": get_stat_color\n" ));
	pghandle ret;

	if( col_stat_mode == 0 )
		ret = MVCOLOR_GREY;
	else if( col_stat_mode == RET_ERROR )
		ret = MVCOLOR_RED;
	else
		ret = MVCOLOR_BLACK;

	dprintf((" -- " TOOL_PREFIX ": get_stat_color (0x%6lx)\n", ret ));
	return ret;
}

//-------------------------------------------------------------------
void si_iupdate( STATUS_INFO *si, char *msg_prefix, char *msg_sufix, int value, int color_mode, int msg_len )
{
	dprintf((" ++ " TOOL_PREFIX ": si_iupdate\n" ));
	char msg[MAX_BUF_SIZE];
	memset( msg, 0, MAX_BUF_SIZE );

	if( msg_prefix != NULL )
		sprintf( msg, "%s: %d", msg_prefix, value );
	if( msg_sufix != NULL )
		sprintf( msg, "%d %s", value, msg_sufix );
	if( msg_prefix != NULL && msg_sufix != NULL )
	{
		memset( msg, 0, MAX_BUF_SIZE );
		sprintf( msg, "%s: %d %s", msg_prefix, value, msg_sufix );
	}

	si->msg = msg;

	if( msg_len == -1 )
		si->msg_len = msg_len;
	else
		si->msg_len = get_x_size( si->msg );

	si->msg_col = get_stat_color( color_mode );

	repl_txt_si( *si );
	dprintf((" -- " TOOL_PREFIX ": si_iupdate\n" ));
}

//-------------------------------------------------------------------
void si_fupdate( STATUS_INFO *si, char *msg_prefix, char *msg_sufix, double value, int color_mode, int msg_len )
{
	dprintf((" ++ " TOOL_PREFIX ": si_fupdate\n" ));
	char msg[MAX_BUF_SIZE];
	memset( msg, 0, MAX_BUF_SIZE );

	if( msg_prefix != NULL )
		sprintf( msg, "%s: %.3f", msg_prefix, value );
	if( msg_sufix != NULL )
		sprintf( msg, "%.3f %s", value, msg_sufix );
	if( msg_prefix != NULL && msg_sufix != NULL )
	{
		memset( msg, 0, MAX_BUF_SIZE );
		sprintf( msg, "%s: %.3f %s", msg_prefix, value, msg_sufix );
	}

	si->msg = msg;

	if( msg_len == -1 )
		si->msg_len = msg_len;
	else
		si->msg_len = get_x_size( si->msg );

	si->msg_col = get_stat_color( color_mode );

	repl_txt_si( *si );
	dprintf((" -- " TOOL_PREFIX ": si_fupdate\n" ));
}

//-------------------------------------------------------------------
void si_cupdate( STATUS_INFO *si, char *msg_prefix, char *msg_sufix, char *data, int color_mode, int msg_len )
{
	dprintf((" ++ " TOOL_PREFIX ": si_cupdate - msg_prefix: >>%s<<, msg_sufix: >>%s<<, data: >>%s<<, color_mode: %d, msg_len: %d\n",
							msg_prefix, msg_sufix, data, color_mode, msg_len ));
	char msg[MAX_BUF_SIZE];
	memset( msg, 0, MAX_BUF_SIZE );

	if( data != NULL )
	{
		if( msg_prefix != NULL && msg_sufix != NULL )
			sprintf( msg, "%s: %s %s", msg_prefix, data, msg_sufix );
		else if( msg_prefix == NULL && msg_sufix == NULL )
			sprintf( msg, "%s", data );
		else if( msg_prefix != NULL )
			sprintf( msg, "%s: %s", msg_prefix, data );
		else if( msg_sufix != NULL )
			sprintf( msg, "%s %s", data, msg_sufix );


		si->msg = msg;

		if( msg_len == -1 )
			si->msg_len = msg_len;
		else
			si->msg_len = get_x_size( si->msg );

		si->msg_col = get_stat_color( color_mode );

		repl_txt_si( *si );
	}

	dprintf((" -- " TOOL_PREFIX ": si_cupdate\n" ));
}

//-------------------------------------------------------------------
int save_pgm_file( char *data_buf, FILE *f, int fsize )
{
	dprintf((" ++ " TOOL_PREFIX ": save_pgm_file\n"));
	int ret;
	char str[MAX_BUF_SIZE];

	if(( ret = fwrite (data_buf, 1, fsize, f )) != fsize )
	{
		memset( str, 0, MAX_CHAR_SIZE );
		sprintf( str,"%d BYTES written from %d", ret, fsize );
		note_or_err_msg( "Save error", str, 0 );
		ret = RET_ERROR;
	}

	dprintf((" -- " TOOL_PREFIX ": save_pgm_file (%d)\n", ret ));
	return ret;
}
//-------------------------------------------------------------------
//-------------------------------------------------------------------
