//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>

#include <pgutils.h>
#include <samples.h>

#include "crtool.h"
#include "prop.h"
#include "gui.h"
#include "ftools.h"
#include "pgtools.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static STATUS_INFO si_run_mode   = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_dec_time   = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_all_time   = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_threshold  = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_dmcode_nb  = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_dmdec_data = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_dmquality  = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_bccode_nb  = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_bcdec_data = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_bcquality  = { 0, 0, NULL, -1, MVCOLOR_BLACK };
static STATUS_INFO si_sinfo      = { 0, 0, NULL, -1, MVCOLOR_BLACK };

pghandle wApp = 0, wBox = 0, wStatLine = 0, wCTRL = 0, wSave = 0, wMainCanvas = 0;

static pghandle wDecode = 0, wScrollLab = 0, wLive = 0;
static pghandle wSBox = 0, wBitmap = 0;
static pghandle wStop = 0, wScroll = 0;
static pghandle wScrollBox = 0, wDispImg = 0;

static int ctrl_flag = 1, redraw_flag = 0, scroll_flag = 0, se_and_scroll_flag = 0;
static int disp_col_cnt = 0;
static int polarity_index = POL_UNKNOWN, ct_menu_index = CT_TYPE_UNKNOWN;
static int bc_type_menu_index = BC_TYPE_UNKNOWN, dm_type_menu_index = DM_TYPE_UNKNOWN;
static int th_mode_index = TH_MODE_AUTO;
static int disp_res_index = DISP_ONE_RESULT;
static int dm_col_index = 1, bc_col_index = 1;

int se_mode = SE_NSET;

COLORREF colTab[ ] =
{
	MVCOLOR_RED,
	MVCOLOR_GREEN,
	MVCOLOR_BLUE,
	MVCOLOR_MAGENTA,
	MVCOLOR_L_BLUE,
	MVCOLOR_YELLOW,
	MVCOLOR_BLACK,
	MVCOLOR_GREY,
	MVCOLOR_WHITE
};

static pgcontext gcCanvas = NULL;

static IPL_POINT click;
static int obj2disp_index = -1;
static CR_PROPS_ID m_dec_prop = CR_NO_PROP;

//-------------------------------------------------------------------
pgcolor get_color( );
void mk_th_scroll( );
void mk_new_context( );

//-------------------------------------------------------------------
//-------------------------------------------------------------------
// Draw & redraw image
//-------------------------------------------------------------------
#define IDLE_TIME			500			// Maximum number of milliseconds to wait between calls to handler

//-------------------------------------------------------------------
/*! \brief Redraw the image.*/
void redraw_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - redraw_flag: %d\n", __FUNCTION__, redraw_flag ));

	if( redraw_flag )
	{
		int width = 0, height = 0;
		pgSizeBitmap( &width, &height, wBitmap );
		dprintf((" == " TOOL_PREFIX ": %s - will redraw the image (WxH: %dx%d)\n", __FUNCTION__, width, height ));

		pgWriteCmd( wMainCanvas,
				PGCANVAS_GROP, 6,
				PG_GROP_BITMAP, 0, 0, width, height, wBitmap );

		pgFlushRequests( );
		pgContextUpdate( gcCanvas );

		redraw_flag = 0;
		pgSetIdle( 0, NULL );
	}

	dprintf((" -- " TOOL_PREFIX ": %s - redraw_flag: %d\n", __FUNCTION__, redraw_flag ));
}

//-------------------------------------------------------------------
/*! \brief Redraw function handler.*/
static void idle_handler( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pgSetIdle( IDLE_TIME, NULL );
	redraw_img( );
	pgSetIdle( IDLE_TIME, &idle_handler );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Redraw calback function.*/
static void request_refresh( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	pgSetIdle( IDLE_TIME, NULL );
	//pgSetIdle( 1, &idle_handler );
	pgSetIdle( IDLE_TIME, &redraw_img );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Updates the image.*/
void displ_update( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - redraw_flag: %d\n", __FUNCTION__, redraw_flag ));

	if( !redraw_flag )
	{
		redraw_flag = 1;
		pgFlushRequests( );
		request_refresh( );
		pgUpdate( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s - redraw_flag: %d\n", __FUNCTION__, redraw_flag ));
}

//-------------------------------------------------------------------
/*! \brief Display the image.*/
void displ_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	if( wMainCanvas > 0 && wBitmap > 0 )
	{
		int width = 0, height = 0;
		pgSizeBitmap( &width, &height, wBitmap );

		dprintf((" == " TOOL_PREFIX ": %s - width: %d, height: %d\n", __FUNCTION__, width, height ));
		pgWriteCmd( wMainCanvas,
			PGCANVAS_GROP, 6,
			PG_GROP_BITMAP, 0, 0, width, height, wBitmap );

		pgContextUpdate( gcCanvas );
		pgSubUpdate( wMainCanvas );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": displ_img - wMainCanvas: %ld, wBitmap: %ld\n", wMainCanvas, wBitmap );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Display the code bounding box.*/
void display_bounding_box( const IPL_RECT rect, const char *data, const int xpos, const int ypos )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int w, h;

	if( ypos == 0 )
		disp_col_cnt = 0;

	pgcolor color = get_color( );

	dprintf((" == " TOOL_PREFIX ": %s - data: >>%s<<, l/t: %ld/%ld, r/b: %ld/%ld, x/y: %d/%d\n",
								__FUNCTION__, data, rect.left, rect.top, rect.right, rect.bottom, xpos, ypos ));

	pgEnterContext( );

	w = rect.right - rect.left;
	h = rect.bottom - rect.top;

	dprintf((" == " TOOL_PREFIX ": %s - w/h: %d/%d\n", __FUNCTION__, w, h ));

	draw_aoi( gcCanvas, rect.left, rect.top, w, h, color  );
	draw_txt( gcCanvas, xpos, ypos, data, color );

	pgLeaveContext( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Display the code bounding box.*/
void display_pts_results( const IPL_POINT *pts, const char *data, const int xpos, const int ypos )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( ypos == 0 )
		disp_col_cnt = 0;

	pgcolor color = get_color( );

	dprintf((" == " TOOL_PREFIX ": %s - data: >>%s<<, x/y0: %ld/%ld, x/y1: %ld/%ld, x/y2: %ld/%ld, x/y3: %ld/%ld, x/y: %d/%d\n",
								__FUNCTION__, data, pts[0].xpos, pts[0].ypos, pts[1].xpos, pts[1].ypos,
								pts[2].xpos, pts[2].ypos, pts[3].xpos, pts[3].ypos, xpos, ypos ));

	pgEnterContext( );

	draw_line( gcCanvas, pts[0].xpos, pts[0].ypos, pts[1].xpos, pts[1].ypos, color );
	draw_line( gcCanvas, pts[1].xpos, pts[1].ypos, pts[2].xpos, pts[2].ypos, color );
	draw_line( gcCanvas, pts[2].xpos, pts[2].ypos, pts[3].xpos, pts[3].ypos, color );
	draw_line( gcCanvas, pts[3].xpos, pts[3].ypos, pts[0].xpos, pts[0].ypos, color );
	draw_txt( gcCanvas, xpos, ypos, data, color );

	pgLeaveContext( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Display the result line.*/
void display_line_results( const IPL_LINE line, const char *data, const int xpos, const int ypos )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( ypos == 0 )
		disp_col_cnt = 0;

	pgcolor color = get_color( );

	dprintf((" == " TOOL_PREFIX ": %s - data: >>%s<<, x/y0: %ld/%ld, x/y1: %ld/%ld, x/ypos: %d/%d\n",
								__FUNCTION__, data, line.x0, line.y0, line.x1, line.y1, xpos, ypos ));

	pgEnterContext( );


	draw_line( gcCanvas, line.x0, line.y0, line.x1, line.y1, color );
	draw_txt( gcCanvas, xpos, ypos, data, color );

	pgLeaveContext( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Init the menu display index.*/
void index_init( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	CR_PROP tmp_prop = prop_get_prop( CR_THRESHOLD_MODE );
	th_mode_index = tmp_prop.default_val;

	tmp_prop = prop_get_prop( CR_POLARITY );
	polarity_index = tmp_prop.default_val;

	tmp_prop = prop_get_prop( CR_CODE_TYPE );
	ct_menu_index = tmp_prop.default_val;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Delete the threshold scroller.*/
void delete_scroller( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	widget_delete( &wScrollLab );
	widget_delete( &wScroll );
	widget_delete( &wScrollBox );

	delete_se( );
	displ_update( );

	se_mode = SE_NSET;
	scroll_flag = 0;
	se_and_scroll_flag = 0;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Delete the Spin Edit and the threshold scroller.*/
void del_se_and_scrl( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( se_mode == SE_SET_TH || se_mode == SE_SET_DECSET )
		delete_scroller( );
	else if( se_mode != SE_NSET )
	{
		delete_se( );
		displ_update( );
	}
	se_mode = SE_NSET;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Selects a color.*/
pgcolor get_color( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int index = disp_col_cnt % 6;
	disp_col_cnt++;

	dprintf((" -- " TOOL_PREFIX ": %s (index: %d, color: 0x%06lx)\n", __FUNCTION__, index, colTab[index] ));

	return colTab[index];
}

//-------------------------------------------------------------------
void get_s_len( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	si_run_mode.msg = STOP_LIVE_STR;
	si_run_mode.msg_len = get_x_size( si_run_mode.msg );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Reset the symbole settings.*/
void reset_symb_set( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	prop_set_prop( CR_MAX_OBJ, DEFAULT_OBJ_CNT );

	polarity_index = POL_UNKNOWN;
	prop_set_prop( CR_POLARITY, polarity_index );

	ct_menu_index = CT_TYPE_UNKNOWN;
	prop_set_prop( CR_CODE_TYPE, ct_menu_index );
	bc_type_menu_index = BC_TYPE_UNKNOWN;
	prop_set_prop( CR_BC_TYPE, bc_type_menu_index );
	dm_type_menu_index = DM_TYPE_UNKNOWN;
	prop_set_prop( CR_DM_TYPE, dm_type_menu_index );
	set_decoder_type( ct_menu_index );

	bc_col_index = 1;
	dm_col_index = 1;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Reset the adwanced settings.*/
void reset_adw_set( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	del_se_and_scrl( );
	repl_txt_si( si_threshold );
	th_mode_index = TH_MODE_AUTO;
	prop_set_prop( CR_THRESHOLD_MODE, th_mode_index );
	prop_set_prop( CR_THRESHOLD, 0 );

	prop_set_prop( CR_SPACING,     DEFAULT_SPACING );
	prop_set_prop( CR_CONTRAST,    DEFAULT_CONTRAST );
	prop_set_prop( CR_GAP_SIZE,    DEFAULT_GAP_SIZE );
	prop_set_prop( CR_MIN_GAPS,    DEFAULT_MIN_GAPS );
	prop_set_prop( CR_NOISE_TH,     DEFAULT_NOISE_TH );
	set_cr_props( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
char *get_dm_str( )
{
	char *ret = "UNKNOWN";

	switch( dm_type_menu_index )
	{
		case DM_TYPE_PRINTED:
			ret = "PRINTED";
			break;
		case DM_TYPE_DOTTED:
			ret = "DOTTED";
			break;
		default:
			break;
	}

	return ret;
}

//-------------------------------------------------------------------
char *get_bc_str( )
{
	char *ret = "UNKNOWN";

	switch( bc_type_menu_index )
	{
		default:
			break;
	}

	return ret;
}

//-------------------------------------------------------------------
/*! \brief Get the settings infos.*/
int get_infos( char *info )
{
	int ret = RET_SUCCESS;

	if( info != NULL )
	{
		CR_PROP prop_mc = prop_get_prop( CR_MAX_OBJ );
		CR_PROP prop_ic = prop_get_prop( CR_IMG_CLEANING_MODE );
		CR_PROP prop_fs = prop_get_prop( CR_FILTER_STRENGHT );
		CR_PROP prop_ci = prop_get_prop( CR_CELL_IMPR_MODE );
		CR_PROP prop_ds = prop_get_prop( CR_DEFECT_SIZE );
		CR_PROP prop_sc = prop_get_prop( CR_SL_SHADING_CORR );
		CR_PROP prop_rs = prop_get_prop( CR_SL_REG_SELECTION );
		CR_PROP prop_cs = prop_get_prop( CR_CELL_SIZE );

		CR_PROP prop_space = prop_get_prop( CR_SPACING );
		CR_PROP prop_contr = prop_get_prop( CR_CONTRAST );
		CR_PROP prop_gs = prop_get_prop( CR_GAP_SIZE );
		CR_PROP prop_mg = prop_get_prop( CR_MIN_GAPS );
		CR_PROP prop_nth = prop_get_prop( CR_NOISE_TH );


		sprintf( info, "Symbol settings\n"
						"+++++++++++++++\n"
						"   Maximal code number: %d\n"
						"   Maximal possibile code number: %d\n"
						"   Polarity: %s\n"
						"   Code Type: %s\n"
						"         %s\n\n"
						"Advanced settings\n"
						"+++++++++++++++++\n"
						"   Threshold mode: %s\n"
						"   %s: %d\n"
						"   %s: %d\n"
						"   %s: %d\n"
						"   %s: %d\n"
						"   %s: %d\n\n"
						"Preprocessing\n"
						"+++++++++++++\n"
						"   Image cleaning\n"
						"   --------------\n"
						"      Filter type: %s\n"
						"      Filter strength: %d\n\n"
						"   Cell improvement\n"
						"   ----------------\n"
						"      Filling defect: %s\n"
						"      Defect size: %d\n\n"
						"   Symbol location\n"
						"   ---------------\n"
						"      Shading correction: %s\n"
						"      Region selection: %s\n"
						"      Cell size: %d\n",
						prop_mc.default_val, MAX_OBJ_CNT,
						polarity_index == POL_B2W ? "BLACK ON WHITE" : ( polarity_index == POL_W2B ? "WHITE ON BLACK" : "UNKNOWN" ),
						ct_menu_index == CT_TYPE_BARCODE ? "BARCODE" : ( ct_menu_index == CT_TYPE_DATAMATRIX ? "DATAMATRIX" : "UNKNOWN" ),
						ct_menu_index == CT_TYPE_BARCODE ? get_bc_str() :  ( ct_menu_index == CT_TYPE_DATAMATRIX ? get_dm_str(): "-" ),
						th_mode_index == 1 ? "AUTO" : "MANUAL",
						prop_space.name, prop_space.default_val,
						"Contrast", prop_contr.default_val,
						prop_gs.name, prop_gs.default_val,
						prop_mg.name, prop_mg.default_val,
						prop_nth.name,prop_nth.default_val,
						prop_ic.default_val == IMG_CLEAN_BEST ? "BEST" : ( prop_ic.default_val == IMG_CLEAN_FAST ? "FAST" : ( prop_ic.default_val == IMG_CLEAN_SALT ? "SALT NOISE" : ( prop_ic.default_val == IMG_CLEAN_PEPR ? "PEPPER NOISE" : "NONE" ))), prop_fs.default_val,
						prop_ci.default_val == CELL_IMPR_2THIN ? "TOO THIN" : ( prop_ci.default_val == CELL_IMPR_2THICK ? "TOO THICK" : ( prop_ci.default_val == CELL_IMPR_NU ? "NOT UNIFORM" : "NONE" )), prop_ds.default_val,
						prop_sc.default_val == SYMB_LOC_SC_BEST ? "BEST" : ( prop_sc.default_val == SYMB_LOC_SC_FAST ? "FAST" : "NONE" ),
						prop_rs.default_val == SYMB_LOC_RS_BEST ? "BEST" : ( prop_rs.default_val == SYMB_LOC_RS_FAST ? "FAST" : "NONE" ), prop_cs.default_val );
	}
	else
	{
		fprintf( stderr, "*** "  TOOL_PREFIX ": get_infos - wrong input buffer!!\n");
		ret = RET_ERROR;
	}

	return ret;
}

//-------------------------------------------------------------------
/*! \brief Get the pointer to shared memory buffer.*/
void get_img4displ( unsigned char **shm_buffer, const long width, const long height )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	wBitmap = get_shm_buf( shm_buffer, width, height );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
static char serr[MSG_SIZE];
static int serr_color = MVCOLOR_BLACK;
void set_err_str( const char *msg, const int color )
{
	memset( serr, 0, MSG_SIZE );
	serr_color = MVCOLOR_BLACK;

	if( msg )
	{
		strncpy( serr, msg, MSG_SIZE );
		serr_color = color;
	}
}

//-------------------------------------------------------------------
/*! \brief Update the status line.*/
void update_status_line( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char data[MAX_CHAR_SIZE];
	memset( data, 0, MAX_CHAR_SIZE );
	int bc_obj_cnt = 0, dm_obj_cnt = 0, disp_index = 0;

	if( disp_res_index == DISP_ALL_RESULT )
		disp_index = 0;
	else
		disp_index = obj2disp_index;

	if(( bc_obj_cnt = get_cr1d_obj_cnt( )) >= 0 )
	{
		if( obj2disp_index >= bc_obj_cnt )
			disp_index = 0;
		else
			disp_index = obj2disp_index;

		si_iupdate( &si_bccode_nb, "BC", NULL, bc_obj_cnt, bc_col_index, -1 );
		si_iupdate( &si_bcquality, "Q", NULL, get_cr1d_quality( disp_index ), bc_col_index, -1 );

		get_cr1d_data( obj2disp_index, data );
		dprintf((" == " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, data ));
		si_cupdate( &si_bcdec_data, NULL, NULL, data, disp_index >= 0 ? bc_col_index : RET_ERROR, -1 );
	}

	if(( dm_obj_cnt = get_cr2d_obj_cnt( )) >= 0 )
	{
		if( obj2disp_index > dm_obj_cnt )
			disp_index = obj2disp_index - bc_obj_cnt;
		else
			disp_index = 0;

		si_iupdate( &si_dmcode_nb, "DM", NULL, dm_obj_cnt, dm_col_index, -1 );
		si_iupdate( &si_dmquality, "Q", NULL, get_cr2d_quality( disp_index ), dm_col_index, -1 );
		get_cr2d_data( obj2disp_index, data );
		dprintf((" == " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, data ));
		si_cupdate( &si_dmdec_data, NULL, NULL, data, disp_index >= 0 ? dm_col_index : RET_ERROR, -1 );
	}

	si_iupdate( &si_dec_time, "t[dec]", "msec", get_dec_time(), 0, -1 );
	si_iupdate( &si_all_time, "t[all]", "msec", get_dec_time() + get_preproc_time(), 0, -1 );
	si_iupdate( &si_threshold, "th", NULL, get_threshold( ), 0, -1 );

	if( strlen( serr ))
		si_cupdate( &si_sinfo, PREFIX_4_MSG_DATA, NULL, serr, serr_color, -1 );
	else
		si_cupdate( &si_sinfo, PREFIX_4_MSG_DATA, NULL, " ", serr_color, -1 );

	memset( serr, 0, MSG_SIZE );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Set the property value from the Spin Edit.*/
void set_val_from_se( const int val )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - val: %d se_mode: %d\n", __FUNCTION__, val, se_mode ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	CR_PROP tmp_prop;

	switch( se_mode )
	{
	case SE_SET_TH:
		prop_set_prop( CR_THRESHOLD, val );
		tmp_prop = prop_get_prop( CR_THRESHOLD );
		set_se_val( tmp_prop.default_val );

		sprintf( msg, "th: %3d", tmp_prop.default_val );
		si_threshold.msg = msg;
		repl_txt_si( si_threshold );

		pgSetWidget( wScroll,
				PG_WP_VALUE, tmp_prop.default_val,
				PG_WP_SCROLL_Y, tmp_prop.default_val,
				0 );
		pgSubUpdate( wScroll );
		break;

	case SE_SET_DECSET:
		prop_set_prop( m_dec_prop, val );
		tmp_prop = prop_get_prop( m_dec_prop );
		set_se_val( tmp_prop.default_val );

		pgSetWidget( wScroll,
				PG_WP_VALUE, tmp_prop.default_val,
				PG_WP_SCROLL_Y, tmp_prop.default_val,
				0 );
		pgSubUpdate( wScroll );
		break;

	case SE_SET_MAXCNT:
		prop_set_prop( CR_MAX_OBJ, val );
		tmp_prop = prop_get_prop( CR_MAX_OBJ );
		set_se_val( tmp_prop.default_val );
		break;

	case SE_SET_FSTRENGTH:
		prop_set_prop( CR_FILTER_STRENGHT, val );
		tmp_prop = prop_get_prop( CR_FILTER_STRENGHT );
		set_se_val( tmp_prop.default_val );
		break;

	case SE_SET_DEFSIZE:
		prop_set_prop( CR_DEFECT_SIZE, val );
		tmp_prop = prop_get_prop( CR_DEFECT_SIZE );
		set_se_val( tmp_prop.default_val );
		break;

	case SE_SET_CSIZE:
		prop_set_prop( CR_CELL_SIZE, val );
		tmp_prop = prop_get_prop( CR_CELL_SIZE );
		set_se_val( tmp_prop.default_val );
		break;

	default:
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Set the result display menu.*/
int set_res_mode( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char *disp_res[3] =
	{
		DISP_RES_NSET_STR,
		DISP_RES_ONE_STR,
		DISP_RES_ALL_STR
	};

	static int old_index = disp_res_index;
	disp_res_index = mk_popup_menu( disp_res[disp_res_index], 2 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, disp_res_index ));

	switch( disp_res_index )
	{
	case DISP_NSET_RESULT:
		disp_res_index = old_index;
		break;

	case DISP_ONE_RESULT:
	case DISP_ALL_RESULT:
		set_disp_result_mode( disp_res_index );
		break;
	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no valid mode (%d)!!\n", __FUNCTION__, disp_res_index );
		break;
	}

	old_index = disp_res_index;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set polarity mode.*/
int set_symbset_polarity( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char *pol_menu[4] =
	{
		POL_MENU_NSET_STR,
		POL_MENU_B2W_STR,
		POL_MENU_W2B_STR,
		POL_MENU_UNKNOWN_STR
	};

	static int old_index = polarity_index;

	polarity_index = mk_popup_menu( pol_menu[polarity_index], 3 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, polarity_index ));

	switch( polarity_index )
	{
	case POL_NSET:
		polarity_index = old_index;
		break;

	case POL_B2W:
	case POL_W2B:
	case POL_UNKNOWN:
		prop_set_prop( CR_POLARITY, polarity_index );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no valid mode (%d)!!\n", __FUNCTION__, polarity_index );
		break;
	}

	old_index = polarity_index;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set datamatrix code type.*/
int set_datamatrix_type( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char *dm_type_menu[4] =
	{
		DM_MENUE_NSET_STR,
		DM_MENUE_PRINT_STR,
		DM_MENUE_DOT_STR,
		DM_MENUE_UNKNOWN_STR
	};

	static int old_index = dm_type_menu_index;

	dm_type_menu_index = mk_popup_menu( dm_type_menu[dm_type_menu_index], 3 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, dm_type_menu_index ));
	switch( dm_type_menu_index )
	{
	case DM_TYPE_NSET:
		dm_type_menu_index = old_index;
		break;

	case DM_TYPE_PRINTED:
	case DM_TYPE_DOTTED:
	case DM_TYPE_UNKNOWN:
		prop_set_prop( CR_DM_TYPE, dm_type_menu_index );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no valid mode (%d)!!\n", __FUNCTION__, dm_type_menu_index );
		break;
	}

	old_index = dm_type_menu_index;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set datamatrix code type.*/
int set_barcode_type( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char *bc_type_menu[2] =
	{
		BC_MENUE_NSET_STR,
		BC_MENUE_UNKNOWN_STR
	};

	static int old_index = bc_type_menu_index;

	bc_type_menu_index = mk_popup_menu( bc_type_menu[bc_type_menu_index], 1 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, bc_type_menu_index ));
	switch( bc_type_menu_index )
	{
	case BC_TYPE_NSET:
		bc_type_menu_index = old_index;
		break;

	case BC_TYPE_UNKNOWN:
		prop_set_prop( CR_BC_TYPE, bc_type_menu_index );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no valid mode (%d)!!\n", __FUNCTION__, bc_type_menu_index );
		break;
	}

	old_index = bc_type_menu_index;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set code type.*/
int set_code_type( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char *ct_type_menu[4] =
	{
		CT_MENUE_NSET_STR,
		CT_MENUE_BC_STR,
		CT_MENUE_DM_STR,
		CT_MENUE_UNKNOWN_STR
	};

	static int old_index = ct_menu_index;

	ct_menu_index = mk_popup_menu( ct_type_menu[ct_menu_index], 3 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, ct_menu_index ));

	switch( ct_menu_index )
	{
	case CT_TYPE_NSET:
		ct_menu_index = old_index;
		break;

	case CT_TYPE_BARCODE:
		//set_barcode_type( );
		prop_set_prop( CR_CODE_TYPE, ct_menu_index );
		bc_col_index = 1;
		dm_col_index = 0;
		break;

	case CT_TYPE_DATAMATRIX:
		set_datamatrix_type( );
		prop_set_prop( CR_CODE_TYPE, ct_menu_index );
		bc_col_index = 0;
		dm_col_index = 1;
		break;

	case CT_TYPE_UNKNOWN:
		prop_set_prop( CR_CODE_TYPE, ct_menu_index );
		bc_type_menu_index = BC_TYPE_UNKNOWN;
		prop_set_prop( CR_BC_TYPE, bc_type_menu_index );
		dm_type_menu_index = DM_TYPE_UNKNOWN;
		prop_set_prop( CR_DM_TYPE, dm_type_menu_index );
		bc_col_index = 1;
		dm_col_index = 1;
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no valid mode (%d)!!\n", __FUNCTION__, ct_menu_index );
		break;
	}

	old_index = ct_menu_index;

	set_decoder_type( ct_menu_index );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set threshold mode and value.*/
int set_adw_th_mode( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char *auto_man[3] =
	{
		AM_NSET_STR,
		AM_AUTO_STR,
		AM_MANUAL_STR
	};

	static int old_index = th_mode_index;
	CR_PROP prop_th;
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	th_mode_index = mk_popup_menu( auto_man[th_mode_index], 2 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, th_mode_index ));

	pgLeaveContext( );
	displ_update( );

	switch( th_mode_index )
	{
	case TH_MODE_NSET:
		th_mode_index = old_index;
		break;

	case TH_MODE_AUTO:
		prop_th = prop_get_prop( CR_THRESHOLD );
		prop_set_prop( CR_THRESHOLD_MODE, TH_MODE_AUTO );
		prop_set_prop( CR_THRESHOLD, 0 );
		delete_scroller( );
		sprintf( msg, "th: %3d", prop_th.default_val );
		si_threshold.msg = msg;
		repl_txt_si( si_threshold );
		break;

	case TH_MODE_MANUAL:
		prop_set_prop( CR_THRESHOLD_MODE, TH_MODE_MANUAL );
		mk_th_scroll( );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no valid mode (%d)!!\n", __FUNCTION__, th_mode_index );
		break;
	}

	old_index = th_mode_index;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
void del_widgets( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	widget_delete( &wBox );
	widget_delete( &wStatLine );
	widget_delete( &wCTRL );
	widget_delete( &wSave );
	widget_delete( &wMainCanvas );

	widget_delete( &wDecode );
	widget_delete( &wScrollLab );
	widget_delete( &wLive );
	widget_delete( &wSBox );
	widget_delete( &wBitmap );
	widget_delete( &wStop );
	widget_delete( &wScroll );
	widget_delete( &wScrollBox );
	widget_delete( &wDispImg );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Close event.*/
int evtClose( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pgWriteCmd( wMainCanvas, PGCANVAS_NUKE, 0 );
	if( gcCanvas != NULL )
	{
		dprintf(( " == %s -> pgDeleteContext - gcCanvas: %p\n", __FUNCTION__, gcCanvas ));
		pgDeleteContext( gcCanvas );
		gcCanvas = NULL;
	}

	del_widgets( );

	pgExitEventLoop( );
	close_all( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Canvas event.*/
int evtCanvas( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - evt->type: 0x%x, disp_res_index: %d\n",
					__FUNCTION__, evt->type, disp_res_index ));

	if( disp_res_index == DISP_ONE_RESULT )
	{
		switch( evt->type )
		{
		case PG_WE_PNTR_DOWN:
			dprintf((" == " TOOL_PREFIX ": %s - PG_WE_PNTR_DOWN\n", __FUNCTION__ ));
			click.xpos = evt->e.pntr.x;
			click.ypos = evt->e.pntr.y;
			break;

		case PG_WE_PNTR_UP:
		case PG_WE_PNTR_RELEASE:
			dprintf((" == " TOOL_PREFIX ": %s - PG_WE_PNTR_UP | PG_WE_PNTR_RELEASE\n", __FUNCTION__ ));
			if( check_pt( click, evt->e.pntr.x, evt->e.pntr.y ) == RET_SUCCESS )
				obj2disp_index = get_cr_index( click );
			break;

		case PG_WE_BUILD:
			dprintf((" == " TOOL_PREFIX ": %s - PG_WE_BUILD\n", __FUNCTION__ ));
			displ_update( );
			break;

		default:
			dprintf((" == " TOOL_PREFIX ": %s - default\n", __FUNCTION__ ));
			break;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Image display mode event.*/
int evtDispImg( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( se_mode != SE_SET_TH && se_mode != SE_NSET && se_mode != SE_SET_DECSET )
	{
		delete_se( );
		displ_update( );
	}

	pgEnterContext( );

	char *disp_img[5] =
	{
		DISP_IMG_NSET_STR,
		DISP_IMG_LIVE_STR,
		DISP_IMG_PP_STR,
		DISP_IMG_BIN_STR,
		DISP_IMG_RES_STR
	};

	static int old_index = SNAPED_IMAGE, disp_img_index = SNAPED_IMAGE;
	disp_img_index = mk_popup_menu( disp_img[disp_img_index], 4 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, disp_img_index ));

	switch( disp_img_index )
	{
	case NSET_IMAGE:
		disp_img_index = old_index;
		break;

	case SNAPED_IMAGE:
	case PP_IMAGE:
	case BINARY_IMAGE:
		set_disp_img_mode( disp_img_index );
		break;

	case RES_OVL_IMAGE:
		set_res_mode( );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": evtDispImg - no valid mode (%d)!!\n", disp_img_index );
		break;
	}

	old_index = disp_img_index;

	pgLeaveContext( );
	displ_update( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Start live image event.*/
int evtLiveON( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	set_decoder( 0 );
	set_w_color( wLive, MVCOLOR_GREY );
	set_w_color( wDecode, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_BLACK );

	if( !run_flag )
	{
		if( obj2disp_index < 0 )
			obj2disp_index = 0;

		run_flag = 1;
		si_run_mode.msg = START_LIVE_STR;
		repl_txt_si( si_run_mode );
		snap_img( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Stop live image event.*/
int evtLiveOFF( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	set_decoder( 0 );

	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wDecode, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );

	if( run_flag )
	{
		run_flag = 0;

		si_run_mode.msg = STOP_LIVE_STR;
		repl_txt_si( si_run_mode );
	}

	displ_update( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Decode event.*/
int evtDecode( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( se_mode != SE_NSET && se_mode != SE_SET_TH && se_mode != SE_SET_DECSET )
	{
		delete_se( );
		pgUpdate( );
	}

	set_decoder( 1 );

	set_w_color( wLive, MVCOLOR_GREY );
	set_w_color( wDecode, MVCOLOR_GREY );
	set_w_color( wStop, MVCOLOR_BLACK );

	if( !run_flag )
	{
		if( obj2disp_index < 0 )
			obj2disp_index = 0;

		run_flag = 1;
		si_run_mode.msg = START_LIVE_STR;
		repl_txt_si( si_run_mode );
		snap_img( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Symbole settings event.*/
int evtSymbSet( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	CR_PROP tmp_prop = prop_get_prop( CR_MAX_OBJ );

	evtLiveOFF( evt );

	if( se_mode != SE_SET_TH && se_mode != SE_NSET && se_mode != SE_SET_DECSET )
	{
		delete_se( );
		displ_update( );
	}
	else
		redraw_img( );

	pgEnterContext( );

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "Code Type ->|Polarity ->|Max Codes (val: %d)|Reset Settings", tmp_prop.default_val );
	int ret = mk_popup_menu( msg, 4 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, ret ));

	switch( ret )
	{
	case 0:
		/* click outside*/
		pgLeaveContext( );
		break;

	case 1:
		set_code_type( );
		pgLeaveContext( );
		displ_update( );
		break;

	case 2:
		set_symbset_polarity( );
		pgLeaveContext( );
		displ_update( );
		break;

	case 3:
		pgLeaveContext( );
		del_se_and_scrl( );
		se_mode = SE_SET_MAXCNT;
		mk_spin_edit( wSave, tmp_prop.default_val, "Max Code" );
		dprintf((" == " TOOL_PREFIX ": %s - max_obj_cnt: %d\n", __FUNCTION__, tmp_prop.default_val ));
		displ_update( );
		break;

	case 4:
		pgLeaveContext( );
		reset_symb_set( );
		displ_update( );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no valid mode (%d)!!\n", __FUNCTION__, ret );
		pgLeaveContext( );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Threshold scroller event.*/
int evtScrollThreshold( struct pgEvent *evt )
{
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	prop_set_prop( CR_THRESHOLD, evt->e.param );

	CR_PROP tmp_prop = prop_get_prop( CR_THRESHOLD );

	dprintf(( " == " TOOL_PREFIX ": %s - threshold: %d\n", __FUNCTION__, tmp_prop.default_val ));

	sprintf( msg, "th: %3d", tmp_prop.default_val );
	si_threshold.msg = msg;
	repl_txt_si( si_threshold );
	set_se_val( tmp_prop.default_val );

	return 0;
}

//-------------------------------------------------------------------
/*! \brief Decoder setting scroller event.*/
int evtScroller( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	prop_set_prop( m_dec_prop, evt->e.param );
	CR_PROP tmp_prop = prop_get_prop( m_dec_prop );

	dprintf(( " == " TOOL_PREFIX ": %s - %s: %d\n", __FUNCTION__, tmp_prop.name, tmp_prop.default_val ));
	set_se_val( tmp_prop.default_val );
	set_dec_prop( m_dec_prop );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Make the scroller for the threshold.*/
void mk_se_and_scroller( const CR_PROPS_ID prop )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - se_and_scroll_flag: %d\n", __FUNCTION__, se_and_scroll_flag ));

	if( !se_and_scroll_flag )
	{
		m_dec_prop = prop;
		CR_PROP tmp_prop = prop_get_prop( prop );

		dprintf((" == " TOOL_PREFIX ": %s - %s, %d, %d\n",
					__FUNCTION__, tmp_prop.name, tmp_prop.max_val, tmp_prop.default_val ));

		wScrollBox = mk_wBox( wSave, PG_DERIVE_AFTER, PG_S_LEFT, 180 );
		wScroll = mk_scroller( PGDEFAULT, PG_DERIVE_INSIDE, tmp_prop.max_val, tmp_prop.default_val, &evtScroller );
		wScrollLab = mk_scroller_lable( wScrollBox, PG_DERIVE_AFTER, tmp_prop.name );
		pgSubUpdate( wScrollLab );

		if( se_mode != SE_NSET )
		{
			delete_se( );
			displ_update( );
		}

		se_mode = SE_SET_DECSET;
		mk_spin_edit( wScrollLab, tmp_prop.default_val, NULL );
		se_and_scroll_flag = 1;
		displ_update( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
static void mk_dec_set( const CR_PROPS_ID prop )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pgLeaveContext( );
	displ_update( );
	del_se_and_scrl( );
	mk_se_and_scroller( prop );
	displ_update( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Adwanced settings event.*/
int evtAdwSet( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	evtLiveOFF( evt );

	if( se_mode != SE_SET_TH && se_mode != SE_NSET && se_mode != SE_SET_DECSET )
	{
		delete_se( );
		displ_update( );
	}
	else
		redraw_img( );

	pgEnterContext( );
//	int ret = mk_popup_menu( "Threshold mode ->|Spacing|Contrast|Noise size|Gap size|Min Gaps|Reset Settings", 7 );
//	int ret = mk_popup_menu( "Threshold mode ->|Spacing|Contrast|Gap size|Min Gaps|Reset Settings", 6 );
	int ret = mk_popup_menu( "Threshold mode ->|Spacing|Contrast|Gap size|Min Gaps|Noise threshold|Reset Settings", 7 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, ret ));

	switch( ret )
	{
	case 0:
		/* click outside*/
		pgLeaveContext( );
		displ_update( );
		break;

	case 1:
		set_adw_th_mode( );
		//	pgLeaveContext( );
		break;

	case 2:
		mk_dec_set( CR_SPACING );
		break;
	case 3:
		mk_dec_set( CR_CONTRAST );
		break;
	case 4:
		mk_dec_set( CR_GAP_SIZE );
		break;
	case 5:
		mk_dec_set( CR_MIN_GAPS );
		break;

	case 6:
		mk_dec_set( CR_NOISE_TH );
		break;

	case 7:
		pgLeaveContext( );
		reset_adw_set( );
		displ_update( );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": evtAdwSet - no valid mode (%d)!!\n", ret );
		pgLeaveContext( );
		displ_update( );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Controle event.*/
int evtCTRL( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	if( ctrl_flag )
	{
		ctrl_flag = 0;

		widget_delete( &wStatLine );
		wStatLine = mk_statline( wApp, PG_DERIVE_INSIDE, 20 );
		mk_statLineLabel( wStatLine );
		mk_disp( wStatLine, PG_DERIVE_AFTER );
		repl_txt( wCTRL, "SHOW CONTROL" );
		pgUpdate( );
	}
	else
	{
		ctrl_flag = 1;

		widget_delete( &wStatLine );
		wStatLine = mk_statline( wBox, PG_DERIVE_AFTER, 20 );
		mk_statLineLabel( wStatLine );
		mk_disp( wStatLine, PG_DERIVE_AFTER );
		repl_txt( wCTRL, "HIDE CONTROL" );
		pgUpdate( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Save event.*/
int evtSave( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char *ini_file_name = INI_FILE_NAME;
	FILE *f = NULL;
	evtLiveOFF( evt );

	if( se_mode != SE_SET_TH && se_mode != SE_NSET && se_mode != SE_SET_DECSET )
	{
		delete_se( );
		displ_update( );
	}

	pgEnterContext( );
	int ret = mk_popup_menu( "Save Image|Save settings to init file", 2 );
	dprintf((" == " TOOL_PREFIX ": %s - mk_popup_menu ret: %d\n", __FUNCTION__, ret ));

	pgLeaveContext( );
	displ_update( );

	switch( ret )
	{
	case 0:
		/* click outside*/
		break;

	case 1:
		save_img( );
		break;

	case 2:
		if(( f = fopen( ini_file_name, "w" )) != NULL )
		{
			save_ini_file( f );
			fclose( f );

			note_or_err_msg( NOTE_STR, "Settings have been saved into '" INI_FILE_NAME "'", 0 );
		}
		else
			note_or_err_msg( SAVE_ERROR, "Can not open the file '" INI_FILE_NAME "'", 0 );

		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": evtSave - no valid mode (%d)!!\n", ret );
		break;
	}

	displ_update( );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Display settings infos event.*/
int evtSettingsInfo( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char msg[1024];
	memset( msg, 0, 1024 );

	if( se_mode != SE_SET_TH && se_mode != SE_NSET && se_mode != SE_SET_DECSET )
	{
		delete_se( );
		displ_update( );
	}

	pgEnterContext( );
	get_infos( msg );

	pgNewPopupAt( PG_POPUP_ATEVENT, PG_POPUP_ATEVENT, PGDEFAULT, PGDEFAULT );
	pghandle wLocBox = pgNewWidget( PG_WIDGET_BOX, 0, 0 );
	pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, PGDEFAULT );
	pgSetWidget( PGDEFAULT,
			PG_WP_TEXT, pgNewString( msg ),
			PG_WP_SIDE, PG_S_LEFT,
			0 );

	pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_AFTER, wLocBox );
	pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_INSIDE, PGDEFAULT );
	pgSetWidget( PGDEFAULT,
			PG_WP_TEXT, pgNewString("OK"),
			PG_WP_EXTDEVENTS, PG_EXEV_NOCLICK | PG_EXEV_PNTR_UP,
			0 );

	pgGetPayload( pgGetEvent()->from );

	pgLeaveContext( );
	displ_update( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Make a new context.*/
void mk_new_context( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pgWriteCmd( wMainCanvas, PGCANVAS_NUKE, 0 );

	if( gcCanvas != NULL )
	{
		dprintf(( " ++ %s - pgDeleteContext - gcCanvas: %p\n", __FUNCTION__, gcCanvas ));
		pgDeleteContext( gcCanvas );
		gcCanvas = NULL;
		dprintf(( " -- %s - pgDeleteContext - gcCanvas: %p\n", __FUNCTION__, gcCanvas ));
	}

	gcCanvas = pgNewCanvasContext( wMainCanvas, PGFX_IMMEDIATE );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make canvas.*/
void mk_canvas( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	widget_delete( &wMainCanvas );
	wMainCanvas = pgNewWidget( PG_WIDGET_CANVAS, PG_DERIVE_INSIDE, wSBox );

	pgSetWidget( wMainCanvas,
			PG_WP_TRIGGERMASK, pgGetWidget( wMainCanvas, PG_WP_TRIGGERMASK ) /*| PG_TRIGGER_MOVE*/,
			0);
	pgBind( wMainCanvas, PGBIND_ANY, &evtCanvas, NULL );

	mk_new_context( );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make the display.*/
void mk_disp( const pghandle widget, const int pos )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	widget_delete( &wSBox );
	wSBox = pgNewWidget( PG_WIDGET_SCROLLBOX, pos, widget );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_ALL,
				0 );

	mk_canvas( );
	displ_img( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make the scroller for the threshold.*/
void mk_th_scroll( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - scroll_flag: %d\n", __FUNCTION__, scroll_flag ));

	if( !scroll_flag )
	{
		CR_PROP tmp_prop = prop_get_prop( CR_THRESHOLD );

		dprintf((" == " TOOL_PREFIX ": %s - %s, %d, %d\n",
					__FUNCTION__, tmp_prop.name, tmp_prop.max_val, tmp_prop.default_val ));

		wScrollBox = mk_wBox( wSave, PG_DERIVE_AFTER, PG_S_LEFT, 80 );
		wScroll = mk_scroller( PGDEFAULT, PG_DERIVE_INSIDE, MAX_THRESHOLD, tmp_prop.default_val, &evtScrollThreshold );
		wScrollLab = mk_scroller_lable( wScrollBox, PG_DERIVE_AFTER, "Threshold" );
		pgSubUpdate( wScrollLab );

		if( se_mode != SE_NSET )
		{
			delete_se( );
			displ_update( );
		}

		se_mode = SE_SET_TH;
		mk_spin_edit( wScrollLab, tmp_prop.default_val, NULL );
		scroll_flag = 1;
		displ_update( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make the status line.*/
void mk_statLineLabel( const pghandle parent )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int bc_obj_cnt = 0, dm_obj_cnt = 0, disp_index = 0;
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	char data[MAX_CHAR_SIZE];
	memset( data, 0, MAX_CHAR_SIZE );
	CR_PROP tmp_prop;

	widget_delete( &si_run_mode.wText );
	widget_delete( &si_run_mode.wBox );
	mk_stll_si( parent, PG_DERIVE_INSIDE, &si_run_mode );

	if(( bc_obj_cnt = get_cr1d_obj_cnt( )) >= 0 )
	{
		if( obj2disp_index >= bc_obj_cnt )
			disp_index = 0;
		else
			disp_index = obj2disp_index;
	}

	widget_delete( &si_bccode_nb.wText );
	widget_delete( &si_bccode_nb.wBox );
	sprintf( msg, "BC: %4d", bc_obj_cnt );
	si_bccode_nb.msg = msg;
	mk_stll_si( si_run_mode.wBox, PG_DERIVE_AFTER, &si_bccode_nb );

	widget_delete( &si_bcdec_data.wText );
	widget_delete( &si_bcdec_data.wBox );
	memset( msg, 0, MSG_SIZE );
	get_cr1d_data( disp_index, data );
	sprintf( msg, ">>%s<<", data );
	si_bcdec_data.msg = msg;
	si_bcdec_data.msg_len = get_x_size( ">>"NO_ONJ_SEL_STR"<<" );
	if( obj2disp_index >= 0 )
		si_bcdec_data.msg_col = get_stat_color( bc_obj_cnt );
	else
		si_bcdec_data.msg_col = get_stat_color( RET_ERROR );
	mk_stll_si( si_bccode_nb.wBox, PG_DERIVE_AFTER, &si_bcdec_data );

	widget_delete( &si_bcquality.wText );
	widget_delete( &si_bcquality.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Q: %04d", get_cr1d_quality( disp_index ));
	si_bcquality.msg = msg;
	si_bcquality.msg_col = get_stat_color( bc_obj_cnt );
	mk_stll_si( si_bcdec_data.wBox, PG_DERIVE_AFTER, &si_bcquality );

	if(( dm_obj_cnt = get_cr1d_obj_cnt( )) >= 0 )
	{
		if( obj2disp_index > dm_obj_cnt )
			disp_index = obj2disp_index - bc_obj_cnt;
		else
			disp_index = 0;
	}

	widget_delete( &si_dmcode_nb.wText );
	widget_delete( &si_dmcode_nb.wBox );
	sprintf( msg, "DM: %4d", dm_obj_cnt );
	si_dmcode_nb.msg = msg;
	mk_stll_si( si_bcquality.wBox, PG_DERIVE_AFTER, &si_dmcode_nb );

	widget_delete( &si_dmdec_data.wText );
	widget_delete( &si_dmdec_data.wBox );
	memset( msg, 0, MSG_SIZE );
	get_cr2d_data( disp_index, data );
	sprintf( msg, ">>%s<<", data );
	si_dmdec_data.msg = msg;
	si_dmdec_data.msg_len = get_x_size( ">>"NO_ONJ_SEL_STR"<<" );
	if( obj2disp_index >= 0 )
		si_dmdec_data.msg_col = get_stat_color( dm_obj_cnt );
	else
		si_dmdec_data.msg_col = get_stat_color( RET_ERROR );
	mk_stll_si( si_dmcode_nb.wBox, PG_DERIVE_AFTER, &si_dmdec_data );

	widget_delete( &si_dmquality.wText );
	widget_delete( &si_dmquality.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Q: %04d", get_cr2d_quality( disp_index ));
	si_dmquality.msg = msg;
	si_dmquality.msg_col = get_stat_color( dm_obj_cnt );
	mk_stll_si( si_dmdec_data.wBox, PG_DERIVE_AFTER, &si_dmquality );

	/* */
	widget_delete( &si_dec_time.wText );
	widget_delete( &si_dec_time.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "t[dec]: %6ld msec", get_dec_time( ));
	si_dec_time.msg = msg;
	si_dec_time.msg_col = get_stat_color( 0 );
	mk_stll_si( si_dmquality.wBox, PG_DERIVE_AFTER, &si_dec_time );

	widget_delete( &si_all_time.wText );
	widget_delete( &si_all_time.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "t[all]: %6ld msec", get_dec_time( ) + get_preproc_time( ));
	si_all_time.msg = msg;
	si_all_time.msg_col = get_stat_color( 0 );
	mk_stll_si( si_dec_time.wBox, PG_DERIVE_AFTER, &si_all_time );

	widget_delete( &si_threshold.wText );
	widget_delete( &si_threshold.wBox );
	memset( msg, 0, MSG_SIZE );
	tmp_prop = prop_get_prop( CR_THRESHOLD );
	sprintf( msg, "th: %4d", tmp_prop.default_val );
	si_threshold.msg = msg;
	si_threshold.msg_col = get_stat_color( 0 );
	mk_stll_si( si_all_time.wBox, PG_DERIVE_AFTER, &si_threshold );

	widget_delete( &si_sinfo.wText );
	widget_delete( &si_sinfo.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, " " );
	si_sinfo.msg = msg;
	si_sinfo.msg_len = 140;
	si_sinfo.msg_col = get_stat_color( RET_ERROR );
	mk_stll_si( si_threshold.wBox, PG_DERIVE_AFTER, &si_sinfo );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make the control menu.*/
pghandle mk_buttons( const pghandle parent )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	index_init( );

	pghandle wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_INSIDE, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIZE, 30,
				0 );

	wLive = mk_ctrl_btn( wBox, "Live", PG_DERIVE_INSIDE, &evtLiveON, MVCOLOR_BLACK, 10 );
	wDecode = mk_ctrl_btn( PGDEFAULT, "Decode", PG_DERIVE_AFTER, &evtDecode, MVCOLOR_BLACK, 10 );
	wStop = mk_ctrl_btn( PGDEFAULT, "Stop", PG_DERIVE_AFTER, &evtLiveOFF, MVCOLOR_GREY, 10 );
	wDispImg = mk_ctrl_btn( PGDEFAULT, "Disp. Img.", PG_DERIVE_AFTER, &evtDispImg, MVCOLOR_BLACK, 10 );

	mk_ctrl_btn( PGDEFAULT, "Symbol\nSettings", PG_DERIVE_AFTER, &evtSymbSet, MVCOLOR_BLACK, 8 );
	mk_ctrl_btn( PGDEFAULT, "Advanced\nSettings", PG_DERIVE_AFTER, &evtAdwSet, MVCOLOR_BLACK, 8 );
	mk_ctrl_btn( PGDEFAULT, "Preprocessing", PG_DERIVE_AFTER, &evtPreProc, MVCOLOR_BLACK, 8 );
	mk_ctrl_btn( PGDEFAULT, "Settings\nInfo", PG_DERIVE_AFTER, &evtSettingsInfo, MVCOLOR_BLACK, 8 );
	wSave = mk_ctrl_btn( PGDEFAULT, "Save...", PG_DERIVE_AFTER, &evtSave, MVCOLOR_BLACK, 10 );
	CR_PROP tmp_prop = prop_get_prop( CR_THRESHOLD_MODE );
	if( tmp_prop.default_val == TH_MODE_MANUAL )
		mk_th_scroll( );

	displ_update( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return wBox;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
