//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>

#include <pgutils.h>
#include <samples.h>

#include "crtool.h"
#include "prop.h"
#include "gui.h"
#include "ftools.h"
#include "pgtools.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \brief Preprocessing region selection.*/
int preproc_reg_sel( )
{
	dprintf((" ++ " TOOL_PREFIX ": preproc_reg_sel\n" ));

	char *bfn[4] =
	{
		BFN_NSET_STR,
		BFN_BEST_STR,
		BFN_FAST_STR,
		BFN_NONE_STR
	};

	CR_PROP tmp_prop = prop_get_prop( CR_SL_REG_SELECTION );
	static int old_index = tmp_prop.default_val, index;

	index = mk_popup_menu( bfn[tmp_prop.default_val], 3 );
	dprintf((" == " TOOL_PREFIX ": preproc_reg_sel - mk_popup_menu in: %d, out: %d\n", tmp_prop.default_val, index ));

	switch( index )
	{
	case SYMB_LOC_RS_NSET:
		/* click outside*/
		index = old_index;
		break;

	case SYMB_LOC_RS_BEST:
	case SYMB_LOC_RS_FAST:
	case SYMB_LOC_RS_NONE:
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": preproc_reg_sel - no valid mode (%d)!!\n", index );
		break;
	}

	prop_set_prop( CR_SL_REG_SELECTION, index );
	old_index = index;

	dprintf((" -- " TOOL_PREFIX ": preproc_reg_sel\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Preprocessing shading correction.*/
int preproc_shd_corr( )
{
	dprintf((" ++ " TOOL_PREFIX ": preproc_shd_corr\n" ));

	char *bfn[4] =
	{
		BFN_NSET_STR,
		BFN_BEST_STR,
		BFN_FAST_STR,
		BFN_NONE_STR
	};

	CR_PROP tmp_prop = prop_get_prop( CR_SL_SHADING_CORR );
	static int old_index = tmp_prop.default_val, index;

	index = mk_popup_menu( bfn[tmp_prop.default_val], 3 );

	dprintf((" == " TOOL_PREFIX ": preproc_shd_corr - mk_popup_menu in: %d, out: %d\n", tmp_prop.default_val, index ));

	switch( index )
	{
	case SYMB_LOC_SC_NSET:
		/* click outside*/
		index = old_index;
		break;

	case SYMB_LOC_SC_BEST:
	case SYMB_LOC_SC_FAST:
	case SYMB_LOC_SC_NONE:
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": preproc_shd_corr - no valid mode (%d)!!\n", index );
		break;
	}

	prop_set_prop( CR_SL_SHADING_CORR, index );
	old_index = index;

	dprintf((" -- " TOOL_PREFIX ": preproc_shd_corr\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Preprocessing Symbol location.*/
int preproc_symb_loc( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": preproc_symb_loc\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	CR_PROP tmp_prop = prop_get_prop( CR_CELL_SIZE );

	sprintf( msg, "Shading Correction ->|Region selection ->|Cell size (val: %d)", tmp_prop.default_val );
	int ret = mk_popup_menu( msg, 3 );

	dprintf((" == " TOOL_PREFIX ": preproc_symb_loc - mk_popup_menu ret: %d\n", ret ));

	switch( ret )
	{
	case 0:
		/* click outside*/
		pgLeaveContext( );
		displ_update( );
		break;

	case 1:
		ret = preproc_shd_corr( );
		pgLeaveContext( );
		displ_update( );
		break;

	case 2:
		ret = preproc_reg_sel( );
		pgLeaveContext( );
		displ_update( );
		break;

	case 3:
		pgLeaveContext( );
		del_se_and_scrl( );
		se_mode = SE_SET_CSIZE;
		mk_spin_edit( wSave, tmp_prop.default_val, "Cell Size" );
		displ_update( );
		break;

	default:
		pgLeaveContext( );
		displ_update( );
		fprintf( stderr, "*** " TOOL_PREFIX ": preproc_symb_loc - no valid mode (%d)!!\n", ret );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": preproc_symb_loc\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Set cell improvement filling defect filter.*/
int preproc_fl_defect( )
{
	dprintf((" ++ " TOOL_PREFIX ": preproc_fl_defect\n" ));

	char *fl_defect[5] =
	{
		FL_DEFECT_NSET_STR,
		FL_DEFECT_THIN_STR,
		FL_DEFECT_TICK_STR,
		FL_DEFECT_NUNIFORM_STR,
		FL_DEFECT_NONE_STR
	};

	CR_PROP tmp_prop = prop_get_prop( CR_CELL_IMPR_MODE );
	static int old_index = tmp_prop.default_val, index;

	index = mk_popup_menu( fl_defect[tmp_prop.default_val], 4 );
	dprintf((" == " TOOL_PREFIX ": preproc_fl_defect - mk_popup_menu in: %d. out: %d\n", tmp_prop.default_val, index ));

	switch( index )
	{
	case CELL_IMPR_NSET:
		/* click outside*/
		index = old_index;
		break;

	case CELL_IMPR_2THIN:
	case CELL_IMPR_2THICK:
	case CELL_IMPR_NU:
	case CELL_IMPR_NONE:
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": preproc_fl_defect - no valid mode (%d)!!\n", index );
		break;
	}

	prop_set_prop( CR_CELL_IMPR_MODE, index );
	old_index = index;

	dprintf((" -- " TOOL_PREFIX ": preproc_fl_defect\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Preprocessing cell improvement.*/
int preproc_cell_iprv( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": preproc_cell_iprv\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	CR_PROP tmp_prop = prop_get_prop( CR_DEFECT_SIZE );

	sprintf( msg, "Filling defect ->|Defect size (val: %d)", tmp_prop.default_val );
	int ret = mk_popup_menu( msg, 2 );
	dprintf((" == " TOOL_PREFIX ": preproc_cell_iprv - mk_popup_menu ret: %d\n", ret ));

	switch( ret )
	{
	case 0:
		/* click outside*/
		pgLeaveContext( );
		displ_update( );
		break;

	case 1:
		ret = preproc_fl_defect( );
		pgLeaveContext( );
		displ_update( );
		break;

	case 2:
		pgLeaveContext( );
		del_se_and_scrl( );
		se_mode = SE_SET_DEFSIZE;
		mk_spin_edit( wSave, tmp_prop.default_val, "Def. Size" );
		displ_update( );
		break;

	default:
		pgLeaveContext( );
		displ_update( );
		fprintf( stderr, "*** " TOOL_PREFIX ": preproc_cell_iprv - no valid mode (%d)!!\n", ret );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": preproc_cell_iprv\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Set image cleaning filter.*/
int preproc_set_img_clean_mode( )
{
	dprintf((" ++ " TOOL_PREFIX ": preproc_set_img_clean_mode\n" ));

	char *fl_type[6] =
	{
		FL_TYPE_NSET_STR,
		FL_TYPE_BEST_STR,
		FL_TYPE_FAST_STR,
		FL_TYPE_SALT_STR,
		FL_TYPE_PEPPER_STR,
		FL_TYPE_NONE_STR
	};

	CR_PROP tmp_prop = prop_get_prop( CR_IMG_CLEANING_MODE );
	static int old_index = tmp_prop.default_val, index;

	index = mk_popup_menu( fl_type[tmp_prop.default_val], 5 );
	dprintf((" == " TOOL_PREFIX ": preproc_set_img_clean_mode - mk_popup_menu in: %d, out: %d\n", tmp_prop.default_val, index ));

	switch( index )
	{
	case IMG_CLEAN_NSET:
		/* click outside*/
		index = old_index;
		break;

	case IMG_CLEAN_BEST:
	case IMG_CLEAN_FAST:
	case IMG_CLEAN_SALT:
	case IMG_CLEAN_PEPR:
	case IMG_CLEAN_NONE:
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": preproc_set_img_clean_mode - no valid mode (%d)!!\n", index );
		break;
	}

	prop_set_prop( CR_IMG_CLEANING_MODE, index );
	old_index = index;

	dprintf((" -- " TOOL_PREFIX ": preproc_set_img_clean_mode\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Preprocessing image cleaning.*/
int preproc_img_clean( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": preproc_img_clean\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	CR_PROP tmp_prop = prop_get_prop( CR_FILTER_STRENGHT );

	sprintf( msg, "Filter type ->|Filter strength (val: %d)", tmp_prop.default_val );
	int ret = mk_popup_menu( msg, 2 );
	dprintf((" == " TOOL_PREFIX ": preproc_img_clean - mk_popup_menu ret: %d\n", ret ));

	switch( ret )
	{
	case 0:
		/* click outside*/
		pgLeaveContext( );
		displ_update( );
		break;

	case 1:
		ret = preproc_set_img_clean_mode( );
		pgLeaveContext( );
		displ_update( );
		break;

	case 2:
		pgLeaveContext( );
		del_se_and_scrl( );
		se_mode = SE_SET_FSTRENGTH;
		mk_spin_edit( wSave, tmp_prop.default_val, "Fl. Strength" );
		displ_update( );
		break;

	default:
		pgLeaveContext( );
		displ_update( );
		fprintf( stderr, "*** " TOOL_PREFIX ": preproc_img_clean - no valid mode (%d)!!\n", ret );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": preproc_img_clean\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Reset the preprocessing functions.*/
void reset_pp( )
{
	dprintf((" ++ " TOOL_PREFIX ": reset_pp\n" ));
	prop_set_prop( CR_IMG_CLEANING_MODE, IMG_CLEAN_NONE );
	prop_set_prop( CR_FILTER_STRENGHT, DEFAULT_FL_STRENGTH );

	prop_set_prop( CR_CELL_IMPR_MODE, CELL_IMPR_NONE );
	prop_set_prop( CR_DEFECT_SIZE, DEFAULT_DEFECT_SIZE );

	prop_set_prop( CR_SL_SHADING_CORR, SYMB_LOC_SC_NONE );
	prop_set_prop( CR_SL_REG_SELECTION, SYMB_LOC_RS_NONE );
	prop_set_prop( CR_CELL_SIZE, DEFAULT_CELL_SIZE );

	dprintf((" -- " TOOL_PREFIX ": reset_pp\n" ));
}

//----------------------------------------------------------------------
/*! \brief Preprocessing event.*/
int evtPreProc( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtPreProc\n" ));

	evtLiveOFF( evt );

	if( se_mode != SE_SET_TH && se_mode != SE_NSET )
		delete_se( );
	else
		redraw_img( );

	pgEnterContext( );
	int ret = mk_popup_menu( "Auto|Image cleaning ->|Cell improvement ->|Symbol location ->|Reset Settings", 5 );
	dprintf((" == " TOOL_PREFIX ": evtPreProc - mk_popup_menu ret: %d\n", ret ));
	switch( ret )
	{
	case 0:
		/* click outside*/
		pgLeaveContext( );
		displ_update( );
		break;

	case 1:
		ni();
		pgLeaveContext( );
		displ_update( );
		break;

	case 2:
		preproc_img_clean( evt );
		break;

	case 3:
		preproc_cell_iprv( evt );
		break;

	case 4:
		preproc_symb_loc( evt );
		break;

	case 5:
		pgLeaveContext( );
		reset_pp( );
		displ_update( );
		break;

	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": evtPreProc - no valid mode (%d)!!\n", ret );
		pgLeaveContext( );
		displ_update( );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": evtPreProc\n" ));
	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
