//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>gmmtool<<
// Don Nov 13 11:40:07 CET 2003
//----------------------------------------------------------------------
#include <stdio.h>
#include <vector>
using namespace std;

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>

#include <mvslib.h>
#include <mvsensor.h>

#include <pgutils.h>
#include <samples.h>

#include "gmmtool.h"
#include "gmmipl.h"
#include "pgapi.h"
#include "pgdev.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static STATUS_INFO si_run_mode = { 0, 0, NULL, -1 };
static STATUS_INFO si_gmm_mode = { 0, 0, NULL, -1 };
static STATUS_INFO si_threshold = { 0, 0, NULL, -1 };
static STATUS_INFO si_quality = { 0, 0, NULL, -1 };
static STATUS_INFO si_seg_cnt = { 0, 0, NULL, -1 };
static STATUS_INFO si_max_feat = { 0, 0, NULL, -1 };
static STATUS_INFO si_corner = { 0, 0, NULL, -1 };
static STATUS_INFO si_arc = { 0, 0, NULL, -1 };
static STATUS_INFO si_fps = { 0, 0, NULL, -1 };
static STATUS_INFO si_tms = { 0, 0, NULL, -1 };

static SPIN_EDIT se_mo = { 0,{ 0, 0, NULL, -1 },0,0,0 };
#ifdef USE_MAX_FEAT
static SPIN_EDIT se_mf = { 0,{ 0, 0, NULL, -1 },0,0,0 };
#endif	// USE_MAX_FEAT

static int width = 0, height = 0, pitch = 0;

static int ctrl_flag = 1, find_flag = 0;

static BYTE *shm_buffer = NULL;
static pghandle wApp = 0, wBox = 0, wStatLine = 0, wSBox = 0;
static pghandle wLive = 0, wStop = 0, wTeach = 0, wFind = 0, wCTRL = 0, wLabelImg = 0;
static pghandle wThScroll = 0, wMainCanvas = 0, wQScroll = 0, wDisplRes = 0;
static pghandle wBitmap = 0;

#ifdef USE_CONTEXT
static pgcontext gcCanvas = NULL;
#endif	// USE_CONTEXT

//----------------------------------------------------------------------
//----------------------------------------------------------------------
COLORREF colTab[ ] =
{
	MVCOLOR_RED,
	MVCOLOR_GREEN,
	MVCOLOR_BLUE,
	MVCOLOR_MAGENTA,
	MVCOLOR_L_BLUE,
	MVCOLOR_YELLOW,
	MVCOLOR_BLACK,
	MVCOLOR_GREY,
	MVCOLOR_WHITE
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static void mk_disp( pghandle widget, int pos );
static void mk_statLineLabel( pghandle parent );
static pghandle mk_buttons( pghandle parent );
static void get_s_len( );
static void redraw_img( );
static pghandle get_color( int index );

//----------------------------------------------------------------------
// event functions
//----------------------------------------------------------------------
int evtClose( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtClose\n"));
	UNUSED( evt );

	write2file( );
	free_gmm( );
	close_dev( );
	pgExitEventLoop( );

	dprintf((" -- " TOOL_PREFIX ": evtClose\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtLiveON( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtLiveON - find_flag: %d\n", find_flag ));

	UNUSED( evt );

	if( !get_run_flag( ))
	{
		set_w_color( wLive, MVCOLOR_GREY );
		set_w_color( wStop, MVCOLOR_BLACK );

		set_run_flag( 1 );
		si_run_mode.msg = START_LIVE_STR;
		repl_txt_si( si_run_mode );
		pgUpdate( );
		snap_img( );
	}
	else
	{
		dprintf((" == " TOOL_PREFIX ": evtLiveON - gmm_mode: %d, find_flag: %d\n", get_gmm_mode( ), find_flag ));
		if( get_gmm_mode( ) != GMM_NO_MODE && !find_flag )
		{
			set_w_color( wFind, MVCOLOR_BLACK );

			if( !pgGetWidget( wLabelImg, PG_WP_ON ))
				set_gmm_mode( GMM_NO_MODE );
		}
		else
			find_flag = find_flag ? 0 : 1;
	}

	dprintf((" -- " TOOL_PREFIX ": evtLiveON - find_flag: %d\n", find_flag ));
	return 0;
}

//----------------------------------------------------------------------
int evtLiveOFF( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtLiveOFF\n"));
	UNUSED( evt );

	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );
	set_w_color( wFind, MVCOLOR_BLACK );

	si_run_mode.msg = STOP_LIVE_STR;
	repl_txt_si( si_run_mode );

	if( !pgGetWidget( wLabelImg, PG_WP_ON ))
	{
		set_gmm_mode( GMM_NO_MODE );
		si_gmm_mode.msg = NO_MODE_STR;
		repl_txt_si( si_gmm_mode );
	}

	set_run_flag( 0 );
	find_flag = 0;

	cp2disp( 1 );
	pgUpdate( );

	dprintf((" -- " TOOL_PREFIX ": evtLiveOFF\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtTeach( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtTeach\n"));

	set_gmm_mode( GMM_TEACH_MODE );

	evtLiveOFF( evt );
	set_w_color( wFind, MVCOLOR_BLACK );
	set_w_color( wTeach, MVCOLOR_GREY );

	si_gmm_mode.msg = "TEACH MODE";
	repl_txt_si( si_gmm_mode );

	teach_gmm( );

	set_w_color( wTeach, MVCOLOR_BLACK );

	si_gmm_mode.msg = NO_MODE_STR;
	repl_txt_si( si_gmm_mode );

	pgSetWidget( wLabelImg, PG_WP_ON, 0, 0 );
	update_corner_cnt( );
	update_arc_cnt( );

	pgUpdate( );

	dprintf((" -- " TOOL_PREFIX ": evtTeach\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtFind( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtFind\n"));

	set_w_color( wFind, MVCOLOR_GREY );
	set_w_color( wTeach, MVCOLOR_BLACK );

	si_gmm_mode.msg = "FIND MODE";
	repl_txt_si( si_gmm_mode );

	set_gmm_mode( GMM_FIND_MODE );
	find_flag = 1;
	evtLiveON( evt );

	pgSetWidget( wLabelImg, PG_WP_ON, 0, 0 );

	cp2disp( 1 );

	dprintf((" -- " TOOL_PREFIX ": evtFind\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtDisplRes( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtDisplRes\n"));
	UNUSED( evt );
	char *menu[2] =
	{
		"* Edge Map|  Model Results",
		"  Edge Map|* Model Results",
	};

	int old_displ_res_mode = get_displ_res_mode( );

	pgEnterContext( );
	int ret = mk_popup_menu( wMainCanvas, menu[get_displ_res_mode( )], 2 );

	if( ret - 1 < 0 )
		set_displ_res_mode( old_displ_res_mode );
	else
		set_displ_res_mode( ret -1 );

	pgLeaveContext( );

	pgUpdate( );

	dprintf((" -- " TOOL_PREFIX ": evtDisplRes\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtScrollThreshold( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtScrollThreshold\n"));
	UNUSED( evt );
	char msg[MSG_SIZE];
	int th = evt->e.param;

	dprintf(( " == " TOOL_PREFIX ": evtScrollThreshold: %d\n", th ));

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Threshold: %3d", th );
	si_threshold.msg = msg;
	repl_txt_si( si_threshold );
	set_threshold( th );

	dprintf((" -- " TOOL_PREFIX ": evtScrollThreshold\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtScrollQuality( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtScrollQuality\n"));
	UNUSED( evt );
	char msg[MSG_SIZE];
	int q = evt->e.param;

	dprintf(( " == " TOOL_PREFIX ": evtScrollQuality: %d\n", q ));

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Quality: %3d", q );
	si_quality.msg = msg;
	repl_txt_si( si_quality );
	set_quality( q );

	dprintf((" -- " TOOL_PREFIX ": evtScrollQuality\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtLabelImg( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtLabelImg\n"));
	UNUSED( evt );
	static int reset_flag = 0;
	int status = pgGetWidget( wLabelImg, PG_WP_ON );
	int gm = get_gmm_mode( );

	if( !reset_flag )
	{
		if( status )
		{
			si_gmm_mode.msg = "LABEL MODE";
			repl_txt_si( si_gmm_mode );

			if( gm != GMM_FIND_MODE )
			{
				set_gmm_mode( GMM_LABEL_MODE );
				reset_flag = 0;
			}
			else
			{
				reset_flag = 1;
				pgSetWidget( wLabelImg, PG_WP_ON, 0, 0 );
			}
		}
		else
		{
			si_gmm_mode.msg = NO_MODE_STR;
			repl_txt_si( si_gmm_mode );
			set_gmm_mode( GMM_NO_MODE );
			reset_flag = 0;
		}
	}
	else
		reset_flag = 0;

	dprintf((" -- " TOOL_PREFIX ": evtLabelImg\n"));
	return 0;
}

//----------------------------------------------------------------------
int evtCTRL( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtCTRL\n"));
	UNUSED( evt );
	if( ctrl_flag )
	{
		ctrl_flag = 0;

		widget_delete( &wStatLine );
		wStatLine = mk_statline( wApp, PG_DERIVE_INSIDE, 20 );
		mk_statLineLabel( wStatLine );
		mk_disp( wStatLine, PG_DERIVE_AFTER );
		repl_txt( wCTRL, "SHOW CONTROL" );
		pgUpdate( );
	}
	else
	{
		ctrl_flag = 1;
		widget_delete( &wStatLine );
		wStatLine = mk_statline( wBox, PG_DERIVE_AFTER, 20 );
		mk_statLineLabel( wStatLine );
		mk_disp( wStatLine, PG_DERIVE_AFTER );
		repl_txt( wCTRL, "HIDE CONTROL" );
		pgUpdate( );
	}

	dprintf((" -- " TOOL_PREFIX ": evtCTRL\n"));
	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static int mvbl_color[] =
{
	249,			// MVCOLOR_RED,
	250,			// MVCOLOR_GREEN,
	252,			// MVCOLOR_BLUE,
	253,			// MVCOLOR_MAGENTA,
	254,			// MVCOLOR_L_BLUE,
	251,			// MVCOLOR_YELLOW,
};

//----------------------------------------------------------------------
static int get_mvbl_color( int index )
{
	return mvbl_color[ index % 6 ];
}

//----------------------------------------------------------------------
static void cp_oevlay_pts2img( BYTE *img, int ipitch, vector<IPL_POINT_FLOAT>& vPoints, int col_index )
{
	dprintf((" ++ " TOOL_PREFIX ": cp_oevlay_pts2img\n" ));

	int c = get_mvbl_color( col_index );
	dprintf((" == " TOOL_PREFIX ": cp_oevlay_pts2img -  col_index: %d, c: %d\n", col_index, c ));

	if( img != NULL )
	{
		vector<IPL_POINT_FLOAT>::iterator iter;
		vector<IPL_POINT_FLOAT>::iterator end;

		iter = vPoints.begin( );
		end  = vPoints.end( );

		while (iter != end)
		{
			int px = (int)iter->x;
			int py = (int)iter->y;
			img[px + py * ipitch] = c;
			iter++;
		}
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": cp_oevlay_pts2img - no source image found!!\n");

	dprintf((" -- " TOOL_PREFIX ": cp_oevlay_pts2img\n" ));
}

//----------------------------------------------------------------------
void cp_pts2img( vector<IPL_POINT_FLOAT>& vPoints, int col_index )
{
	cp_oevlay_pts2img( shm_buffer, pitch, vPoints, col_index );
}

//----------------------------------------------------------------------
void get_s_len(  )
{
	dprintf((" ++ " TOOL_PREFIX ": get_s_len\n"));
	int y;
	pghandle font = pgNewFont( NULL, 8, 0 );

	si_run_mode.msg = STOP_LIVE_STR;
	get_str_size( &si_run_mode.msg_len, &y, font, si_run_mode.msg );

	dprintf((" -- " TOOL_PREFIX ": get_s_len\n"));
}

//----------------------------------------------------------------------
void redraw_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": redraw_img\n" ));

	int lwidth = 0, lheight = 0;
	pgSizeBitmap( &lwidth, &lheight, wBitmap );
	dprintf((" == " TOOL_PREFIX ": redraw_img - will redraw the image (WxH: %dx%d)\n", lwidth, lheight ));

	pgWriteCmd( wMainCanvas,
			PGCANVAS_GROP, 6,
			PG_GROP_BITMAP, 0, 0, lwidth, lheight, wBitmap );

#ifdef USE_CONTEXT
	pgContextUpdate( gcCanvas );
#endif	// USE_CONTEXT
	//redraw_canvas( );

	dprintf((" -- " TOOL_PREFIX ": redraw_img\n" ));
}

//----------------------------------------------------------------------
void displ_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": displ_img\n" ));

	if( wMainCanvas > 0 && wBitmap > 0 )
		redraw_img( );
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": displ_img - wMainCanvas: %ld, wBitmap: %ld\n", wMainCanvas, wBitmap );

	dprintf((" -- " TOOL_PREFIX ": displ_img\n" ));
}

//----------------------------------------------------------------------
void mk_disp( pghandle widget, int pos )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_disp\n"));

	widget_delete( &wSBox );
	wSBox = pgNewWidget( PG_WIDGET_SCROLLBOX, pos, widget );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_ALL,
				0 );

	widget_delete( &wMainCanvas );
	wMainCanvas = pgNewWidget( PG_WIDGET_CANVAS, PG_DERIVE_INSIDE, wSBox );

#ifdef USE_CONTEXT
	mk_new_context( );
#endif	// USE_CONTEXT
	cp2disp( 1 );

	dprintf((" -- " TOOL_PREFIX ": mk_disp\n"));
}

//----------------------------------------------------------------------
pghandle mk_buttons( pghandle parent )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_buttons\n"));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	pghandle wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_INSIDE, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIZE, 30,
				0 );

	wLive = mk_ctrl_btn( wBox, "LIVE", PG_DERIVE_INSIDE, &evtLiveON, MVCOLOR_BLACK );
	wStop = mk_ctrl_btn( PGDEFAULT, "STOP", PG_DERIVE_AFTER, &evtLiveOFF, MVCOLOR_BLACK );
	wFind = mk_ctrl_btn( PGDEFAULT, "FIND", PG_DERIVE_AFTER, &evtFind, MVCOLOR_BLACK );
	wTeach = mk_ctrl_btn( PGDEFAULT, "TEACH", PG_DERIVE_AFTER, &evtTeach, MVCOLOR_BLACK );
	wDisplRes = mk_ctrl_btn( PGDEFAULT, "RES DISP", PG_DERIVE_AFTER, &evtDisplRes, MVCOLOR_BLACK );

	pghandle wScrollBox = mk_wBox( wDisplRes, PG_DERIVE_AFTER, PG_S_LEFT, 130 );
	wThScroll = mk_scroller( PGDEFAULT, PG_DERIVE_INSIDE, MAX_THRESHOLD, get_threshold( ), &evtScrollThreshold );
	sprintf( msg, "%03d", get_threshold( ));
	pghandle wScrollLab = mk_scroller_label( wScrollBox, PG_DERIVE_INSIDE, "Threshold" );

	wScrollBox = mk_wBox( wScrollBox, PG_DERIVE_AFTER, PG_S_LEFT, 110 );
	wQScroll = mk_scroller( PGDEFAULT, PG_DERIVE_INSIDE, MAX_QUALITY, get_quality( ), &evtScrollQuality );
	sprintf( msg, "%03d", get_quality( ));
	wScrollLab = mk_scroller_label( wScrollBox, PG_DERIVE_INSIDE, "Quality" );

#ifdef USE_MAX_FEAT
	se_mf = mk_spin_edit( wScrollBox, get_max_feat_nb( ), "<- Max Feature" );
	se_mo = mk_spin_edit( se_mf.wBox, get_max_obj_nb( ), "<- Max Obj" );
#else
	se_mo = mk_spin_edit( wScrollBox, get_max_obj_nb( ), "<- Max Obj" );
#endif	// USE_MAX_FEAT

	pghandle wChBox = mk_wBox( se_mo.wBox, PG_DERIVE_AFTER, PG_S_LEFT, 60 );
	wLabelImg = mk_chb( wChBox, "label img", &evtLabelImg );

	dprintf((" -- " TOOL_PREFIX ": mk_buttons\n"));

	return wBox;
}

//----------------------------------------------------------------------
void mk_statLineLabel( pghandle parent )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_statLineLabel\n"));
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	widget_delete( &si_run_mode.wText );
	widget_delete( &si_run_mode.wBox );
	mk_stll_si( parent, PG_DERIVE_INSIDE, &si_run_mode );

	widget_delete( &si_gmm_mode.wText );
	widget_delete( &si_gmm_mode.wBox );
	mk_stll_si( si_run_mode.wBox, PG_DERIVE_AFTER, &si_gmm_mode );

	widget_delete( &si_threshold.wText );
	widget_delete( &si_threshold.wBox );
	sprintf( msg, "Threshold: %3d", get_threshold( ));
	si_threshold.msg = msg;
	mk_stll_si( si_gmm_mode.wBox, PG_DERIVE_AFTER, &si_threshold );

	widget_delete( &si_quality.wText );
	widget_delete( &si_quality.wBox );
	sprintf( msg, "Quality: %3d%%", get_quality( ));
	si_quality.msg = msg;
	mk_stll_si( si_threshold.wBox, PG_DERIVE_AFTER, &si_quality );

	widget_delete( &si_seg_cnt.wText );
	widget_delete( &si_seg_cnt.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Obj:  %3d/%3d", get_obj_nb( ), get_max_obj_nb( ));
	si_seg_cnt.msg = msg;
	mk_stll_si( si_quality.wBox, PG_DERIVE_AFTER, &si_seg_cnt );

	widget_delete( &si_corner.wText );
	widget_delete( &si_corner.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Edge:  %3d", get_corner_cnt( ) );
	si_corner.msg = msg;
	mk_stll_si( si_seg_cnt.wBox, PG_DERIVE_AFTER, &si_corner );

	widget_delete( &si_arc.wText );
	widget_delete( &si_arc.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Arc:  %3d", get_arc_cnt( ));
	si_arc.msg = msg;
	mk_stll_si( si_corner.wBox, PG_DERIVE_AFTER, &si_arc );

	widget_delete( &si_max_feat.wText );
	widget_delete( &si_max_feat.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Feat(max):  %3d", get_max_feat_nb( ));
	si_max_feat.msg = msg;
	mk_stll_si( si_arc.wBox, PG_DERIVE_AFTER, &si_max_feat );

	widget_delete( &si_tms.wText );
	widget_delete( &si_tms.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "t:   %.2fms", get_time_ms( ));
	si_tms.msg = msg;
	mk_stll_si( si_max_feat.wBox, PG_DERIVE_AFTER, &si_tms );

	widget_delete( &si_fps.wText );
	widget_delete( &si_fps.wBox );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Frames:   %.2f", get_frame_ps( ));
	si_fps.msg = msg;
	mk_stll_si( si_tms.wBox, PG_DERIVE_AFTER, &si_fps );

	dprintf((" -- " TOOL_PREFIX ": mk_statLineLabel\n"));
}

//----------------------------------------------------------------------
void init_gui( int argc, char *argv[], char *text )
{
	dprintf((" ++ " TOOL_PREFIX ": init_gui\n" ));

	width = get_iwidth( );
	height = get_iheight( );
	pitch = get_ipitch( );

	dprintf((" ==  " TOOL_PREFIX ": init_gui - width: %d, height: %d, pitch: %d, text: >>%s<<\n",
						width, height, pitch, text ));

	pgInit( argc, argv );
	wApp = pgRegisterApp( PG_APP_NORMAL, text, 0 );
	pgBind( PGDEFAULT, PG_WE_CLOSE, &evtClose, NULL );

	wBox = mk_buttons( wApp );
	wStatLine = mk_statline( wBox, PG_DERIVE_AFTER, 20 );

	si_gmm_mode.msg = NO_MODE_STR;
	si_run_mode.msg = STOP_LIVE_STR;
	get_s_len( );

	mk_statLineLabel( wStatLine );

	wBitmap = get_shm_buf( &shm_buffer, width, height );

	if( img_buffer && shm_buffer )
		mk_disp( wStatLine, PG_DERIVE_AFTER );

	pghandle wTL = pgGetWidget( wApp, PG_WP_PANELBAR_LABEL );
	wCTRL = mk_btn( wTL, "HIDE CONTROL", PG_DERIVE_BEFORE, 80, &evtCTRL );

	dprintf((" -- " TOOL_PREFIX ": init_gui\n" ));
}

//----------------------------------------------------------------------
void start_tool( int flag )
{
	dprintf((" ++ " TOOL_PREFIX ": start_tool (%d)\n", flag ));
	switch( flag )
	{
	case 1:
		evtLiveON( NULL );
		break;
	case 2:
		evtFind( NULL );
		break;

	default:
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": start_tool\n"));
}

//----------------------------------------------------------------------
void update_fps( )
{
	dprintf((" ++ update_fps\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Frames:  %.2f", get_frame_ps( ));
	si_fps.msg = msg;
	repl_txt_si( si_fps );

	dprintf((" -- update_fps\n" ));
}

//----------------------------------------------------------------------
void update_tms( )
{
	dprintf((" ++ update_tms\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "t:  %.2fms", get_time_ms( ));
	si_tms.msg = msg;
	repl_txt_si( si_tms );

	dprintf((" -- update_tms\n" ));
}

//----------------------------------------------------------------------
void cp2disp( int mode )
{
	dprintf((" ++ cp2disp\n"));

	cp_img( img_buffer, shm_buffer, width, height, pitch, pitch );

	if( mode )
	{
		clear_canvas( );
		displ_img( );
		redraw_canvas( );
	}

	dprintf((" -- cp2disp\n"));
}

//----------------------------------------------------------------------
SPIN_EDIT get_se( struct pgEvent *evt, long *se_cnt, unsigned int *step )
{
	dprintf((" ++ get_se_val\n"));
	SPIN_EDIT ret = { 0,{ 0, 0, NULL, -1 },0,0,0 };

#ifdef USE_MAX_FEAT
	if(( evt->from == se_mf.wBtnUp ) || ( evt->from == se_mf.wBtnDown ))
	{
		*se_cnt = get_max_feat_nb( );
		*step = 1;
		ret = se_mf;
	}
	else
#endif	// USE_MAX_FEAT
	if(( evt->from == se_mo.wBtnUp ) || ( evt->from == se_mo.wBtnDown ))
	{
		*se_cnt = get_max_obj_nb( );
		*step = 1;
		ret = se_mo;
	}

	dprintf((" -- get_se_val\n"));
	return ret;
}

//----------------------------------------------------------------------
void set_val_from_se( struct pgEvent *evt, int val )
{
	dprintf((" ++ " TOOL_PREFIX ": set_val_fom_se (%d)\n", val ));

#ifdef USE_MAX_FEAT
	if(( evt->from == se_mf.wBtnUp ) || ( evt->from == se_mf.wBtnDown ))
	{
		if( val > MAX_FEAT_NB )
			val = MAX_FEAT_NB;
		else if( val < 0 )
			val = 0;

		set_max_feat_nb( val );
		set_se_val( &se_mf, get_max_feat_nb( ));
	}
	else
#endif	// USE_MAX_FEAT
	if(( evt->from == se_mo.wBtnUp ) || ( evt->from == se_mo.wBtnDown ))
	{
		if( val > MAX_OBJ_NB )
			val = MAX_OBJ_NB;
		else if( val < 0 )
			val = 0;

		set_max_obj_nb( val );
		set_se_val( &se_mo, get_max_obj_nb( ));
	}

	dprintf((" -- " TOOL_PREFIX ": set_val_fom_se (%d)\n", val ));
}

//----------------------------------------------------------------------
int cp_bin_img( IPL_BUFHANDLE bin_image )
{
	dprintf(( " ++ " TOOL_PREFIX ": cp_bin_img\n" ));

	int ret = IPL_FALSE;
	IPL_LONG ipitch = 0;
	BYTE *ibuff = NULL;

	if(( ret = IPL_GetPtr( bin_image, IPL_LOCK_READ, 0, 0, 0, &ipitch, &ibuff )))
	{
		if( shm_buffer )
		{
			cp_img( ibuff, shm_buffer, width, height, ipitch, pitch );
			ret = IPL_TRUE;
		}
		else
			ret = IPL_FALSE;

		IPL_ReleasePtr( bin_image, IPL_LOCK_READ, 0, 0, 0, ibuff );
	}
	else
		fprintf( stderr, "*** cp_bin_img - no binary image found (%ld)!!\n", IPL_GetErrorCode( ));

	dprintf(( " -- " TOOL_PREFIX ": cp_bin_img (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
void clear_canvas( )
{
	dprintf(( " ++ " TOOL_PREFIX ": clear_canvas - wMainCanvas: %ld\n", wMainCanvas ));

	if( wMainCanvas > 0 )
	{
		pgWriteCmd( wMainCanvas, PGCANVAS_NUKE, 0 );
	}

	dprintf(( " -- " TOOL_PREFIX ": clear_canvas\n" ));
}

//----------------------------------------------------------------------
void redraw_canvas( )
{
	dprintf(( " ++ " TOOL_PREFIX ": redraw_canvas - wMainCanvas: %ld\n", wMainCanvas ));

	if( wMainCanvas > 0 )
	{
		pgWriteCmd( wMainCanvas, PGCANVAS_REDRAW, 0 );
		pgSubUpdate( wMainCanvas );
	}

	dprintf(( " -- " TOOL_PREFIX ": redraw_canvas\n" ));
}

//----------------------------------------------------------------------
int mk_new_context( )
{
	dprintf(( " ++ " TOOL_PREFIX ": mk_new_context\n" ));
	int ret = 0;

	clear_canvas( );
#ifdef USE_CONTEXT
	pgDeleteContext( gcCanvas );
	gcCanvas = pgNewCanvasContext( wMainCanvas, PGFX_IMMEDIATE );
#endif	// USE_CONTEXT

	dprintf(( " -- " TOOL_PREFIX ": mk_new_context (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
pghandle get_color( int index )
{
	dprintf((" -> " TOOL_PREFIX ": get_color - 0x%lx\n", colTab[index % 6] ));
	return colTab[index % 6];
}

//----------------------------------------------------------------------
int gui_draw_circle( int x_cent, int y_cent, int r, int col_index )
{
	dprintf((" ++ gui_draw_circle\n"));
	int ret = -1;
	if( col_index >= 0 )
	{
#ifdef USE_CONTEXT
		draw_circle( gcCanvas, x_cent, y_cent, r, get_color( col_index ));
#else
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, get_color( col_index ));
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 5, PG_GROP_ELLIPSE, x_cent - r, y_cent - r, 2 * r, 2 * r );

#endif	//USE_CONTEXT
		ret = 0;
	}

	dprintf((" -- gui_draw_circle (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int gui_draw_line( int x1, int y1, int x2, int y2, int col_index )
{
	dprintf((" ++ gui_draw_line\n" ));
	int ret = -1;
	if( col_index >= 0 )
	{
#ifdef USE_CONTEXT
		draw_line( gcCanvas, x1, y1, x2, y2, get_color( col_index ));
#else
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, get_color( col_index ));
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 5, PG_GROP_LINE, x1, y1, x2-x1, y2-y1 );
#endif	// USE_CONTEXT

		ret = 0;
	}

	dprintf((" -- gui_draw_line (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int gui_draw_text( int x, int y, char *text, int col_index )
{
	dprintf((" ++ gui_draw_text\n"));

	int ret = -1;
	if( col_index >= 0 )
	{
#ifdef USE_CONTEXT
		draw_txt( gcCanvas, x, y, text, get_color( col_index ));
#else
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, get_color( col_index ));
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 6, PG_GROP_TEXT, x, y, 1, 1, pgNewString( text ));
#endif	// USE_CONTEXT

		ret = 0;
	}

	dprintf((" -- gui_draw_text (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int gui_draw_point( int x, int y, int col_index )
{
	dprintf((" ++ gui_draw_point\n" ));

	int ret = -1;
	if( col_index >= 0 )
	{
#ifdef USE_CONTEXT
		draw_point( gcCanvas, x, y, get_color( col_index ));
#else
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, get_color( col_index ));
		pgWriteCmd( wMainCanvas, PGCANVAS_GROP, 5, PG_GROP_PIXEL, x, y, 1, 1 );
#endif	// USE_CONTEXT

		ret = 0;
	}

	dprintf((" -- gui_draw_point (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void update_obj_cnt( )
{
	dprintf((" ++ update_obj_cnt\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Obj:  %3d/%3d", get_obj_nb( ), get_max_obj_nb( ));
	si_seg_cnt.msg = msg;
	repl_txt_si( si_seg_cnt );

	dprintf((" -- update_obj_cnt\n" ));
}

//----------------------------------------------------------------------
void update_corner_cnt( )
{
	dprintf((" ++ update_corner_cnt\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Edge:  %3d", get_corner_cnt( ));
	si_corner.msg = msg;
	repl_txt_si( si_corner );

	dprintf((" -- update_corner_cnt\n" ));
}

//----------------------------------------------------------------------
void update_arc_cnt( )
{
	dprintf((" ++ update_arc_cnt\n" ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Arc:  %3d", get_arc_cnt( ));
	si_arc.msg = msg;
	repl_txt_si( si_arc );

	dprintf((" -- update_arc_cnt\n" ));
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
