//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>gmmtool<<
// Don Nov 13 11:40:07 CET 2003
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_color.h>
#include <vl_device.h>
#include <vl_display.h>

#include <mvslib.h>
#include <mvsensor.h>

#include <pgutils.h>
#include <samples.h>

#include "gmmtool.h"
#include "gmmipl.h"
#include "pgdev.h"
#include "pgapi.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
IPL_BUFHANDLE image = IPL_DONT_CARE;
static IPL_BUFHANDLE orig = IPL_DONT_CARE;
static IPL_BUFHANDLE gray = IPL_DONT_CARE;

static bool colorSensor = BASE_CheckModule( "Color" ) == IPL_TRUE;

//#define DISPLAY_GRAY_IMAGE

static int width = 0, height = 0, pitch = 0;
static DEV_HANDLE device = IPL_DONT_CARE;
static double frame_ps = 0.0;

static int run_flag = 0, gmm_mode = GMM_NO_MODE;

//----------------------------------------------------------------------
static void possiblyConvertToGray( )
{
	if( colorSensor )
	{
		IPL_LONG grayColorSpace;
		if( COL_ConvertColorSpace( orig, &gray, COL_BM2GRAY ) == IPL_FALSE ||
		    IPL_BufInquire( gray, IPL_COLOR_SPACE, &grayColorSpace ) == IPL_FALSE ||
		    grayColorSpace != COL_GRAY )
		{
			exit_demo( "demosaicing failed", IPL_GetErrorCode2( ));
		}
#ifndef DISPLAY_GRAY_IMAGE
		image = gray;
#endif
	}
#ifndef DISPLAY_GRAY_IMAGE
	else image = orig;
#endif
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
void init_dev( )
{
	dprintf((" ++ " TOOL_PREFIX ": init_dev\n"));

	static int trig_mode = -1;

	if( !DEV_Open( &device, "matrixvision.ddc" ))
		exit_demo( "open device failed", IPL_GetErrorCode( ));

    if( !DEV_Control( device, DEV_SINGLE, 0 ))
		exit_demo( "set control single failed", IPL_GetErrorCode( ));

#ifdef USE_DUBLE_BUF
	int dbuf = IPL_TRUE;
	if( !DEV_Control( device, DEV_SEQUENTIAL_MODE, &dbuf ))
		exit_demo( "set control sequential mode failed", IPL_GetErrorCode( ));
#endif	// USE_DUBLE_BUF

	if( !DEV_InquireCaps( device, DEV_TRIGGER, &trig_mode ))
		exit_demo( "get trigger mode failed", IPL_GetErrorCode( ));

	if( trig_mode != TRIG_SW && trig_mode != TRIG_AUTO )
	{
		int sw_trigmode = TRIG_SW;
		if( !DEV_ControlCaps( device, DEV_TRIGGER, &sw_trigmode ))
			exit_demo( "set SW trigger mode failed", IPL_GetErrorCode( ));
	}

	if( !DEV_Grab( device, &orig ))
		exit_demo( "grab image failed", IPL_GetErrorCode( ));

	if( colorSensor )
	{
		long lBufAttr;
		IPL_BufInquire( orig, IPL_USAGETYPE, &lBufAttr );
		colorSensor = lBufAttr == IPL_BAYERMOSAICBUF; // we have a bayer image and a color license
		possiblyConvertToGray( );
	}

	// Whatever we assign to image here will determine what
	// is displayed (original color or converted gray); the
	// actual matching is always based on the converted gray
	// image, regardless of what is displayed.
	// Note that in order to see a correct gray image, the
	// pgserver must be started with
	//   palette = mvlut_grey
	// in /etc/pgserver.conf, rather than
	//   palette = mvlut_bayer
	//
	image =
#ifdef DISPLAY_GRAY_IMAGE
		colorSensor ? gray :
#endif
		orig;

	if( trig_mode != TRIG_SW && trig_mode != TRIG_AUTO )
	{
		if( !DEV_ControlCaps( device, DEV_TRIGGER, &trig_mode ))
			exit_demo( "reset the trigger mode failed", IPL_GetErrorCode( ));
	}

	if( !IPL_BufInquire( image, IPL_XSIZE, &width ))
		exit_demo( "inquier image width failed", IPL_GetErrorCode( ));
	if( !IPL_BufInquire( image, IPL_YSIZE, &height ))
		exit_demo( "inquier image height failed", IPL_GetErrorCode( ));

	if( !IPL_GetPtr( image, IPL_LOCK_READ, 0, 0, 0, (IPL_LONG*)&pitch, &img_buffer ))
		exit_demo( "inquier image height failed", IPL_GetErrorCode( ));

	dprintf((" -- " TOOL_PREFIX ": init_dev\n"));
}

//----------------------------------------------------------------------
void close_dev( )
{
	dprintf((" ++ " TOOL_PREFIX ": close_dev\n"));
	if( img_buffer != NULL )
	{
		IPL_ReleasePtr( image, IPL_LOCK_READ, 0, 0, 0, img_buffer );
		img_buffer = NULL;
	}

	DEV_Close( device );
	IPL_BufFree( image );

	dprintf((" -- " TOOL_PREFIX ": close_dev\n"));
}

//----------------------------------------------------------------------
int get_iwidth( ){	return width; }
int get_iheight( ){	return height; }
int get_ipitch( ){	return pitch; }

//----------------------------------------------------------------------
void set_run_flag( int flag ){	run_flag = flag; }
int get_run_flag( ){	return run_flag; }

//----------------------------------------------------------------------
void set_gmm_mode( int mode ) {	gmm_mode = mode; }
int get_gmm_mode( ){	return gmm_mode; }

//----------------------------------------------------------------------
void snap_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": snap_img\n"));
	static struct timeval start_tv, diff_tv;

	while( run_flag )
	{
		pgEventPoll( );

		StartTimer( &start_tv );

		if( run_flag )
		{
			if( !DEV_Grab( device, &orig ))
				exit_demo( "grab image failed", IPL_GetErrorCode( ));
			possiblyConvertToGray( );
		}

		if( run_flag )
		{
			if( gmm_mode == GMM_FIND_MODE )
			{
				clear_label( );
				if( !find_gmm( ))
					evtLiveOFF( NULL );
			}
			else if( gmm_mode == GMM_LABEL_MODE )
				set_label_img( );
		}

		if( run_flag && gmm_mode != GMM_LABEL_MODE )
		{
			if(( get_displ_res_mode( ) != GMM_DISPL_MATCH_TABLE &&
				get_displ_res_mode( ) != GMM_DISPL_TRIGS ) ||
						get_obj_nb( ) <= 0 )
			{
				clear_canvas( );
				cp2disp( 1 );
			}
		}

		if( run_flag && gmm_mode == GMM_FIND_MODE )
			draw_results( );

		if( run_flag )
		{
			update_obj_cnt( );
			update_corner_cnt( );
			update_arc_cnt( );
		}

		ElapsedTime( &start_tv, &diff_tv, 30 );
		double time_ms = (double)(diff_tv.tv_sec * 1000 + diff_tv.tv_usec/1000. );
		frame_ps = 1000.0/time_ms;
		dprintf((" == time_ms: %.2f, frame_ps: %.2f\n", time_ms, frame_ps ));

		update_fps( );
		update_tms( );
	}

	frame_ps = 0;
	update_fps( );
	dprintf((" -- " TOOL_PREFIX ": snap_img\n"));
}

//----------------------------------------------------------------------
double get_frame_ps( ){	return frame_ps; }

//----------------------------------------------------------------------
//----------------------------------------------------------------------
