//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>gmmtool<<
// Tue Jul  8 09:54:05 CEST 2003
//----------------------------------------------------------------------
#include <pgutils.h>
#include <samples.h>

#include "gmmtool.h"
#include "pgapi.h"

//----------------------------------------------------------------------
char msg[MSG_SIZE];

//----------------------------------------------------------------------
/*! \brief Delete the spin edit.*/
void delete_se( SPIN_EDIT *se )
{
	dprintf((" ++ " TOOL_PREFIX ": delete_se\n" ));

	widget_delete( &se->wBox );
	widget_delete( &se->si.wBox );
	widget_delete( &se->si.wText );
	widget_delete( &se->wBtnUp );
	widget_delete( &se->wBtnDown );
	widget_delete( &se->wLabel );

	dprintf((" -- " TOOL_PREFIX ": delete_se\n" ));
}

//----------------------------------------------------------------------
/*! \brief Set the spin edit value and display it.*/
void set_se_val( SPIN_EDIT *se, int val )
{
	dprintf((" ++ " TOOL_PREFIX ": set_se_val\n" ));

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%d", val );
	se->si.msg = msg;
	repl_txt_si( se->si );

	dprintf((" -- " TOOL_PREFIX ": set_se_val\n" ));
}

//----------------------------------------------------------------------
/*! \brief Event button up.*/
int evtBtnUp( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtBtnUp\n" ));

	long se_cnt = 0;
	unsigned int step = 0;
	SPIN_EDIT se = get_se( evt, &se_cnt, &step );

	se_cnt += step ;
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%ld", se_cnt );
	se.si.msg = msg;
	repl_txt_si( se.si );
	set_val_from_se( evt, se_cnt );

	dprintf((" -- " TOOL_PREFIX ": evtBtnUp\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Event button down.*/
int evtBtnDown( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtBtnDown\n" ));
	long se_cnt = 0;
	unsigned int step = 0;
	SPIN_EDIT se = get_se( evt, &se_cnt, &step );

	se_cnt -= step;

	if( se_cnt < 0 )
		se_cnt = 0;

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%ld", se_cnt );
	se.si.msg = msg;
	repl_txt_si( se.si );
	set_val_from_se( evt, se_cnt );

	dprintf((" -- " TOOL_PREFIX ": evtBtnDown\n" ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Event text edit (not implemented jet).*/
int evtTextEdit( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": evtTextEdit\n" ));
	long se_cnt = 0;
	unsigned int step = 0;
	SPIN_EDIT se = get_se( evt, &se_cnt, &step );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%ld", se_cnt );
	se.si.msg = msg;
	repl_txt_si( se.si );

	dprintf((" -- " TOOL_PREFIX ": evtTextEdit\n" ));

	return 0;
}

//----------------------------------------------------------------------
/*! \brief Make the spin edit.*/
void mk_spinedit( pghandle parent, char *text, pghandle *wSEText, pghandle *wBtnUp , pghandle *wBtnDown,
						pgevthandler BtnUp, pgevthandler BtnDown, pgevthandler TextEdit, int tbox_w )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_spinedit\n" ));

	int btn_h = 9;

	*wSEText = pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_LEFT,
				PG_WP_SIZE, tbox_w,
				PG_WP_MARGIN, 0,
				PG_WP_TEXT, pgNewString( text ),
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, TextEdit, NULL );

	*wBtnUp = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_AFTER, *wSEText );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_MARGIN, 0,
				PG_WP_SIZE, btn_h,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, BtnUp, NULL );

	*wBtnDown = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_AFTER, *wBtnUp );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_BOTTOM,
				PG_WP_MARGIN, 0,
				PG_WP_SIZE, btn_h,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, BtnDown, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_spinedit\n" ));
}

//----------------------------------------------------------------------
/*! \brief Make a spin edit.*/
SPIN_EDIT mk_spin_edit( pghandle parent, long value, char *label )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_spin_edit\n" ));

	SPIN_EDIT se = { 0,{ 0, 0, NULL, -1 },0,0,0 };

	int lab_len = 0, size = 0;
	memset( msg, 0, MSG_SIZE );
	int len = get_x_size( "88888" );

	if( label != NULL )
	{
		lab_len = get_x_size( label );
		size = len + lab_len + 30;
	}
	else
		size = len + 20;

	dprintf((" ++ " TOOL_PREFIX ": mk_spin_edit - val: %ld, label: >>%s<<, len: %d, lab_len: %d, size: %d\n",
					value, label, len, lab_len, size ));

	se.wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_AFTER, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_LEFT,
				PG_WP_MARGIN, 0,
				PG_WP_SIZE, size,
				0 );

	se.si.wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_INSIDE, se.wBox );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_LEFT,
				PG_WP_SIZE, size,
				0 );

	sprintf( msg, "%4ld", value );
	mk_spinedit( se.si.wBox, msg, &se.si.wText, &se.wBtnUp, &se.wBtnDown, &evtBtnUp, &evtBtnDown, &evtTextEdit, len );

	if( label != NULL )
	{
		se.wLabel = pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, se.wBox );
		pgSetWidget( PGDEFAULT,
					PG_WP_SIDE, PG_S_RIGHT,
					PG_WP_SIZE, lab_len + 10,
					PG_WP_TEXT, pgNewString( label ),
					PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
					0 );
	}

	dprintf((" -- " TOOL_PREFIX ": mk_spin_edit\n" ));
	return se;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
