//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \file spinedit.h
 *  \brief MATRIX VISION GmbH
 *
 * $Id: spinedit.cpp,v 1.1 2004/10/20 08:18:19 td Exp $
 *
 *
 */

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <pgtools.h>
#include <spinedit.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
int CSpinEdit :: se_mkSEBox( const pghandle wParent, const int position, const int size, PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( size > 0 )
	{
		if(( se.wBox = pgNewWidget( PG_WIDGET_BOX, position, wParent )) > 0 )
		{
			pgSetWidget( se.wBox,
						PG_WP_SIDE, PG_S_LEFT,
						PG_WP_MARGIN, 0,
						PG_WP_SIZE, size,
						0 );

			if(( se.si.wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_INSIDE, se.wBox )) > 0 )
			{
				pgSetWidget( se.si.wBox,
							PG_WP_SIDE, PG_S_LEFT,
							PG_WP_SIZE, size,
							0 );
				ret = 0;
			}
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
int CSpinEdit :: se_mkSEBtns( PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1, BtnH = 9;

	if(( se.wBtnUp = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_AFTER, se.si.wText )) > 0 )
	{
		pgSetWidget( se.wBtnUp,
					PG_WP_SIDE, PG_S_TOP,
					PG_WP_MARGIN, 0,
					PG_WP_SIZE, BtnH,
					0 );
		pgBind( PGDEFAULT, PG_WE_ACTIVATE, &evtSEBtnUp, NULL );

		if(( se.wBtnDown = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_AFTER, se.wBtnUp )) > 0 )
		{
			pgSetWidget( se.wBtnDown,
						PG_WP_SIDE, PG_S_BOTTOM,
						PG_WP_MARGIN, 0,
						PG_WP_SIZE, BtnH,
						0 );
			pgBind( PGDEFAULT, PG_WE_ACTIVATE, &evtSEBtnDown, NULL );
			ret = 0;
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
int CSpinEdit :: se_mkSELabel( const char *pLabel, PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );
	int ret = 0;

	if( pLabel != NULL )
	{
		int LabLen = PGT_GetStrXSize( pLabel );
		if(( se.wLabel = pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, se.wBox )) > 0 )
		{
			pgSetWidget( PGDEFAULT,
						PG_WP_SIDE, PG_S_RIGHT,
						PG_WP_SIZE, LabLen + 15,
						PG_WP_TEXT, pgNewString( pLabel ),
						0 );
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
int CSpinEdit :: SE_mkSpinEdit( const pghandle wParent, const int position, const int size,
						const char *pLabel, const int TBoxW, PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if(( ret = se_mkSEBox( wParent, position, size, se )) >= 0 )
	{
		ret = -1;
		se_CleanMsgBuf( );
		sprintf( m_se_msg, "%4ld", se.DefaultValue );

		if(( se.si.wText = pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, se.si.wBox )) > 0 )
		{
			pgSetWidget( se.si.wText,
						PG_WP_SIDE, PG_S_LEFT,
						PG_WP_SIZE, TBoxW,
						PG_WP_MARGIN, 0,
						PG_WP_TEXT, pgNewString( m_se_msg ),
						0 );

			if(( ret = se_mkSEBtns( se )) >= 0 )
				ret = se_mkSELabel( pLabel, se );
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
int CSpinEdit :: SE_mkSpinEdit( const pghandle wParent, const long value, const char *pLabel, PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	int LabLen = 0, size = 0;
	int len = PGT_GetStrXSize( "88888" );

	if( pLabel != NULL )
	{
		LabLen = PGT_GetStrXSize( pLabel );
		size = len + LabLen + 30;
	}
	else
		size = len + 20;

	dprintf((" == %s - val: %ld, label: >>%s<<, len: %d, LabLen: %d, size: %d\n",
										__FUNCTION__, value, pLabel, len, LabLen, size ));

	if(( ret = SE_ChechValue( value, se )) >= 0 )
		ret = SE_mkSpinEdit( wParent, PG_DERIVE_AFTER, size, pLabel, len, se );
	else
		PGT_GUIErrMsg( "out of range", ret );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
void CSpinEdit :: SE_Init( PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );

	se.wBox       = 0;
	se.si.wBox    = 0;
	se.si.wText   = 0;
	se.si.pMsg    = NULL;
	se.si.MsgLen  = -1;
	se.si.wMsgCol = PGT_COLOR_BLACK;
	se.wBtnUp     = 0;
	se.wBtnDown   = 0;
	se.wLabel     = 0;

	se.DefaultValue = 0;
	se.MinValue     = 0;
	se.MaxValue     = 0;

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
void CSpinEdit :: SE_Delete( PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );

	PGT_WDelete( se.wBox );
	PGT_WDelete( se.si.wBox );
	PGT_WDelete( se.si.wText );
	PGT_WDelete( se.wBtnUp );
	PGT_WDelete( se.wBtnDown );
	PGT_WDelete( se.wLabel );

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
int CSpinEdit :: SE_SetVal( const long value, PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if(( ret = SE_ChechValue( value, se )) >= 0 )
	{
		se_CleanMsgBuf( );
		sprintf( m_se_msg, "%ld", se.DefaultValue );
		se.si.pMsg = m_se_msg;
		PGT_ReplaceSIText( se.si );
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
void CSpinEdit :: SE_IncreaseValue( PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );

	long value = se.DefaultValue + m_se_step;
	SE_SetVal( value, se );
	//set_val_from_se( se_cnt );

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
void CSpinEdit :: SE_DecreaseValue( PG_SPIN_EDIT& se )
{
	PGT_FCT_START( DBG_START_END );

	long value = se.DefaultValue - m_se_step;
	SE_SetVal( value, se );
	//set_val_from_se( se_cnt );

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
