//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo
// Tue May 20 14:35:47 CEST 2003
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include <picogui.h>
#include <system.h>
#include <pgutils.h>

#include "pgtools.h"

//-------------------------------------------------------------------
//-------------------------------------------------------------------
void widget_delete( pghandle *widget )
{
	if( *widget != 0 )
	{
		pgDelete( *widget );
		*widget = 0;
	}
}

//-------------------------------------------------------------------
void cp_chimg( unsigned char *src, unsigned char *dst, int sx, int sy, int width, int height, int spitch, int dpitch )
{
	dprintf((" ++ %s - sx: %d, sy: %d, width: %d, height: %d, spitch: %d, dpitch: %d\n",
		__FUNCTION__, sx, sy, width, height, spitch, dpitch ));

	if( src != NULL )
	{
		int x, y;
		for( y = 0; y < height; y++ )
		{
			for( x = 0; x < width; x++ )
				dst[x + y * dpitch] = src[ x + sx + ( y + sy ) * spitch];
		}
	}
	else
		fprintf( stderr, "*** %s - no source image found!!\n", __FUNCTION__ );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void cp_img( unsigned char *src, unsigned char *dst, int width, int height, int spitch, int dpitch )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	if( src != NULL )
	{
		int y;
		for( y = 0; y < height; y++ )
			memcpy( dst + y * dpitch, src + y * spitch, width );
	}
	else
		fprintf( stderr, "*** %s - no source image found!!\n", __FUNCTION__ );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void repl_txt_si( STATUS_INFO si )
{
	dprintf((" ++ %s - >>%s<<\n", __FUNCTION__, si.msg ));

	pgReplaceText( si.wText, si.msg );
	pgSubUpdate( si.wBox );
	pgSubUpdate( si.wText );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void repl_txt( pghandle widget, char *text )
{
	dprintf((" ++ %s - >>%s<<\n", __FUNCTION__, text ));

	pgReplaceText( widget, text );
	pgSubUpdate( widget );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void draw_aoi( pgcontext gc, int x, int y, int w, int h, pgcolor color )
{
	dprintf((" ++ %s - x: %d, y: %d, w: %d, h: %d, color: 0x%x\n",
								__FUNCTION__, x, y, w, h, color ));

	pgSetColor( gc, color );
	pgFrame( gc, x, y, w, h );
	pgContextUpdate( gc );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void draw_txt( pgcontext gc, int x, int y, char *text, pgcolor color )
{
	dprintf((" ++ %s - x: %d, y: %d, text: >>%s<<, color: 0x%x\n",
						__FUNCTION__, x, y, text, color ));

	pgSetColor( gc, color );
	pgText( gc, x, y, pgNewString( text ));
	pgContextUpdate( gc );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
pghandle mk_chb( pghandle parent, char *txt, pgevthandler chbox_cb )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	pghandle wChBox = pgNewWidget( PG_WIDGET_CHECKBOX, PG_DERIVE_INSIDE, parent );
	pgSetWidget( wChBox,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_EXTDEVENTS, PG_EXEV_TOGGLE,
				0 );
	pgBind( wChBox, PG_WE_ACTIVATE, chbox_cb, NULL );

	dprintf((" -- %s\n", __FUNCTION__ ));

	return wChBox;
}

//-------------------------------------------------------------------
pghandle mk_btn( pghandle parent, char *txt, int pos, int size, pgevthandler btn_cb )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_SIZE, size,
				PG_WP_FONT, pgNewFont( NULL, 6, 0 ),
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, btn_cb, NULL );

	dprintf((" -- %s\n", __FUNCTION__ ));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_ctrl_btn( pghandle parent, char *txt, int pos, pgevthandler btn_cb, pghandle color )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_COLOR, color,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, btn_cb, NULL );

	dprintf((" -- %s\n", __FUNCTION__ ));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_label( pghandle parent, char *txt, int pos )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	pghandle wLabel = pgNewWidget( PG_WIDGET_LABEL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_ALIGN, PG_A_RIGHT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_SIZE, 8,
				0 );

	dprintf((" -- %s\n", __FUNCTION__ ));

	return wLabel;
}

//-------------------------------------------------------------------
pghandle mk_scroller( pghandle parent, int pos, int max_s, int def_s, pgevthandler scroll_cb )
{
	dprintf((" ++ %s - max_s: %d, def_s: %d\n", __FUNCTION__, max_s, def_s ));

	pghandle wScroller = pgNewWidget( PG_WIDGET_SCROLL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, max_s,
				PG_WP_VALUE, def_s,
				PG_WP_SCROLL_X, 0,
				PG_WP_SCROLL_Y, def_s,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, scroll_cb, NULL );

	dprintf((" -- %s\n", __FUNCTION__ ));

	return wScroller;
}

//-------------------------------------------------------------------
pghandle mk_wBox( pghandle parent, int pat_pos, int side, int size )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	pghandle wigdet = pgNewWidget( PG_WIDGET_BOX, pat_pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, side,
				PG_WP_SIZE, size,
				0 );

	dprintf((" -- %s\n", __FUNCTION__ ));

	return wigdet;
}

//-------------------------------------------------------------------
pghandle mk_scroller_lable( pghandle parent, int pos, char *txt )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	int x,y;
	pghandle font = pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT );

	get_str_size( &x, &y, font, txt );

	dprintf((" == %s - txt: >>%s<< (x/y: %d/%d)\n", __FUNCTION__, txt, x, y ));

	pghandle wScLab = mk_wBox( parent, pos, PG_S_LEFT, x + 5 );

	pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, PGDEFAULT );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, font,
				PG_WP_SIZE, x,
				PG_WP_SIDE, PG_S_LEFT,
				0 );

	dprintf((" -- %s\n", __FUNCTION__ ));

	return wScLab;
}

//-------------------------------------------------------------------
int set_min_max( char *name, int val, int min_val, int max_val )
{
	int new_val = val;
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	pgEnterContext( );
	pghandle wToolbar, wScrlLab, wScroller, wOK, wCancel;
	struct pgEvent *evt;

	sprintf( msg, "Set segment: %s", name );
	pgDialogBox( msg );
	wToolbar = pgNewWidget( PG_WIDGET_TOOLBAR, 0, 0 );
	pgSetWidget( PGDEFAULT,
			PG_WP_SIDE, PG_S_LEFT,
			0 );

	wOK = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_INSIDE, wToolbar );
	pgSetWidget( PGDEFAULT,
			PG_WP_TEXT, pgNewString( "OK" ),
			0 );
	wCancel = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_INSIDE, wToolbar );
	pgSetWidget( PGDEFAULT,
			PG_WP_TEXT, pgNewString( "Cancel" ),
			0 );

	wScroller = pgNewWidget( PG_WIDGET_SCROLL, PG_DERIVE_AFTER, wToolbar );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, max_val,
				PG_WP_VALUE, val,
				PG_WP_SCROLL_X, 0,
				PG_WP_SCROLL_Y, val,
				0 );

	pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_AFTER, PGDEFAULT );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( name ),
				PG_WP_FONT, pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT ),
				PG_WP_SIDE, PG_S_LEFT,
				0 );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "old value: %d", val );

	pgNewWidget( PG_WIDGET_LABEL, 0,0 );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( msg ),
				PG_WP_FONT, pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT ),
				PG_WP_SIDE, PG_S_LEFT,
				0 );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "new value: %d", val );

	wScrlLab = pgNewWidget( PG_WIDGET_LABEL, 0,0 );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( msg ),
				PG_WP_FONT, pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT ),
				PG_WP_SIDE, PG_S_RIGHT,
				PG_WP_ALIGN, PG_A_RIGHT,
				0 );

	for (;;)
	{
		evt = pgGetEvent();

		if( evt->from == wOK )
		{
			if( new_val < min_val )
				new_val = min_val;

			pgReplaceTextFmt( wScrlLab, "new value: %d", new_val );
			pgSubUpdate( wScrlLab );

			dprintf((" == %s - new_val: %d\n", __FUNCTION__, new_val ));
			pgLeaveContext( );
			return new_val;
		}
		else if( evt->from == wCancel )
		{
			pgReplaceTextFmt( wScrlLab, "new value: %d", val );
			pgSubUpdate( wScrlLab );

			dprintf((" == %s - new_val: %d\n", __FUNCTION__, val ));
			pgLeaveContext( );
			return val;
		}
		else if( evt->from == wScroller )
		{
			dprintf((" == %s - wScroller: evt->e.param: %d\n", __FUNCTION__, evt->e.param ));

			if( evt->e.param != 0 )
				new_val = evt->e.param;
			else
				new_val = min_val;

			pgReplaceTextFmt( wScrlLab, "new value: %d", new_val );
			pgSubUpdate( wScrlLab );
		}
	}

	pgLeaveContext( );

	return -1;
}

//-------------------------------------------------------------------
pghandle mk_statline( pghandle parent, int pos, int size )
{
	pghandle wStatLine = pgNewWidget( PG_WIDGET_TOOLBAR, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, size,
				0 );

	return wStatLine;
}

//-------------------------------------------------------------------
void get_str_size( int *x, int *y, pghandle font, char *str )
{
	pgEnterContext( );
	pgSizeText( x, y, font, pgNewString( str ));
	pgLeaveContext( );
}

//-------------------------------------------------------------------
void mk_stll_si( pghandle parent, int pos, STATUS_INFO *si )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));
	int x,y, offs = 5;

	x = si->msg_len;

	if( x < 0 )
	{
		pghandle font = pgNewFont( NULL, 8, 0 );
		get_str_size( &x, &y, font, si->msg );
	}

	si->wBox  = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
	si->wText = mk_label( PGDEFAULT, si->msg, PG_DERIVE_INSIDE );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
pghandle mk_stll( pghandle parent, int pos, pghandle *wText, char *str, int size )
{
	int x,y, offs = 5;

	x = size;

	if( size < 0 )
	{
		pghandle font = pgNewFont( NULL, 8, 0 );
		get_str_size( &x, &y, font, str );
	}

	pghandle wB = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
	*wText = mk_label( PGDEFAULT, str, PG_DERIVE_INSIDE );

	return wB;
}

//-------------------------------------------------------------------
void set_w_color( pghandle widget, pghandle color )
{
	pgSetWidget( widget, PG_WP_COLOR, color, 0 );
	pgSubUpdate( widget );

}

//-------------------------------------------------------------------
int check_if_file_exist( char *fname )
{
	FILE *f;
	char s[MSG_SIZE];
	memset( s, 0, MSG_SIZE );

	dprintf(( " ++ %s\n", __FUNCTION__ ));

	if(( f = fopen( fname, "r" )) == NULL )
	{
		sprintf( s, "No '%s' found!!", fname );
		pgMessageDialog( KB_ERROR, s, PG_MSGBTN_OK | PG_MSGICON_ERROR );
		return -1;
	}

	fclose( f );

	dprintf(( " -- %s\n", __FUNCTION__ ));

	return 0;
}

//----------------------------------------------------------------------
static int bcMenuFromArray( const pghandle wCanvas, const pghandle *pItems, const unsigned int NumItems )
{
	dprintf(( " ++ %s\n", __FUNCTION__ ));

	int ret = 0;
	pgNewPopupAt( PG_POPUP_ATEVENT, PG_POPUP_ATEVENT, 0, 0 );

	for( unsigned int i = 0; i < NumItems; i++ )
	{
		pgNewWidget( PG_WIDGET_MENUITEM, 0, 0 );
		pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pItems[i],
				0 );

		pgSetPayload( PGDEFAULT, i+1 );
	}

	pghandle evtFrom = pgGetEvent()->from;
	while( evtFrom == wCanvas )
		evtFrom = pgGetEvent()->from;

	dprintf((" == %s - event from: %ld\n", __FUNCTION__, evtFrom ));

	/* Return event */
	ret = pgGetPayload( evtFrom );

	dprintf(( " -- %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//----------------------------------------------------------------------
int mkPopupMenu( const pghandle wCanvas, const char* pMenuEntrys, const unsigned int EntryCnt )
{
	dprintf((" ++ %s - menu entrys: >%s<, cnt: %d\n", __FUNCTION__, pMenuEntrys, EntryCnt ));

	char *p, menu[MSG_SIZE];
	int ret = 0;

	if( pMenuEntrys && *pMenuEntrys && EntryCnt > 0 )
	{

		pghandle *pItems = NULL;

		if(( pItems = new pghandle[ EntryCnt] ) != NULL )
		{
			for( unsigned int i = 0; i < EntryCnt; i++ )
			{
				if( !( p = strchr( pMenuEntrys,'|' )))
					p = (char*)(pMenuEntrys + strlen( pMenuEntrys ));

				memset( menu, 0, MSG_SIZE );
				strncpy( menu, pMenuEntrys, strlen( pMenuEntrys ) - strlen( p ));

				pgFlushRequests( );
				pMenuEntrys = p + 1;
				pItems[i] = pgNewString( menu );
			}

			ret = bcMenuFromArray( wCanvas, pItems, EntryCnt );
		}

		if( pItems )	delete [] pItems;
	}

	dprintf(( " -- %s\n", __FUNCTION__ ));
	return ret;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
