#ifndef MVIMPACT_IMPL_H_
#define MVIMPACT_IMPL_H_

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY

#include <iterator>
#include <algorithm>

namespace mvIMPACT {

///////////////////////////////////////////////////////////////
// former base module

template< class T, class O >
inline DynArray<T, O>::DynArray()
	: m_array(NULL), m_size(0)
{ }

template< class T, class O >
inline DynArray<T, O>::DynArray(size_t size)
	: m_array(NULL), m_size(0)
{
	allocate(size);
}

template< class T, class O >
inline DynArray<T, O>::DynArray(size_t size, T init)
	: m_array(NULL), m_size(0)
{
	allocate(size, init);
}

template< class T, class O >
inline DynArray<T, O>::DynArray(const O* pO, size_t size)
	: m_array(NULL), m_size(0)
{
	allocate(pO, size);
}

template< class T, class O >
inline DynArray<T, O>::DynArray(T* pT, size_t size, bool doCopy)
	: m_array(NULL), m_size(0)
{
	allocate(pT, size, doCopy);
}


template< class T, class O >
inline DynArray<T, O>::DynArray(const std::vector<O> &v)
	: m_array(NULL), m_size(0)
{
	allocate(v);
}

template< class T , class O >
inline T* DynArray<T, O>::getPointer() const
{
	return m_array;
}

template< class T , class O >
inline T* DynArray<T, O>::getPointer()
{
	return m_array;
}

template< class T , class O >
inline DynArray<T, O>::operator const T*() const
{
	return getPointer();
}

template< class T , class O >
inline DynArray<T, O>::operator T*()
{
	return getPointer();
}

template<class T, class O >
inline void DynArray<T, O>::copy(const DynArray& src)
{
	assert((0 == src.m_size) == (NULL == src.m_array));

	m_size = src.m_size;
	if (src.m_array)
	{
		m_array = new T[m_size];
		for (size_t i = 0; i < m_size; i++) 
		{ 
			m_array[i] = src.m_array[i]; 
		}
	}
	else
	{
		m_array = NULL;
	}

	assert((0 == m_size) == (NULL == m_array));
}


template<class T, class O >
inline DynArray<T, O>::~DynArray()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

template<class T, class O >
inline DynArray<T, O>::DynArray(const DynArray& src)
{
	copy(src);
}

template<class T, class O >
inline DynArray<T, O>& DynArray<T, O>::operator=(const DynArray<T, O>& src)
{
	if (this != &src)
	{
		dispose();
		copy(src);
	}
	return *this;
}

template<class T, class O >
inline std::vector<O> DynArray<T, O>::asVector()
{
	std::vector<O> result(m_size);
	typename std::vector<O>::iterator it;
	size_t i;
	for (it = result.begin(), i = 0; i < m_size; i++, it++) {
		(*it) = m_array[i];
	}
	return result;
}

template<class T, class O >
inline size_t DynArray<T, O>::getSize() const
{
	return m_size;
}

template<class T, class O >
inline void DynArray<T, O>::allocate(const std::vector<O> &v)
{
	dispose();
	if(v.size() > 0)
	{
		m_size = v.size();
		m_array = new T[m_size];
		typename std::vector<O>::const_iterator i; size_t j;
		for (i = v.begin(), j = 0; i != v.end(); i++, j++)
		{
			m_array[j] = *i;
		}
	}
}

template<class T, class O >
inline void DynArray<T, O>::allocate(const O* pO, size_t size)
{
	dispose();
	if(size > 0)
	{
		m_array = new T[size];
		m_size = size;
		for (size_t i = 0; i < size; i++)
		{
			m_array[i] = pO[i];
		}
	}
}

template<class T, class O >
inline void DynArray<T, O>::allocate(T* pT, size_t size, bool doCopy)
{
	dispose();
	if(size > 0)
	{
		m_size = size;
		if (doCopy)
		{
			m_array = new T[size];
			for (size_t i = 0; i < size; i++)
			{
				m_array[i] = pT[i];
			}
		}
		else
		{
			m_array = pT;
		}
	}
}

template<class T, class O >
inline void DynArray<T, O>::allocate(size_t size)
{
	dispose();
	if(size > 0)
	{
		m_array = new T[size];
		m_size = size;
	}
}

template<class T, class O >
inline void DynArray<T, O>::allocate(size_t size, T init)
{
	dispose();
	if(size > 0)
	{
		m_array = new T[size];
		m_size = size;
		for (size_t i = 0; i < m_size; i++)
		{
			m_array[i] = init;
		}
	}
}

template<class T, class O >
inline void DynArray<T, O>::dispose() {
	assert((NULL == m_array) == (0 == m_size));

	if (m_array)
	{
		delete[] m_array;
		m_array = NULL;	
		m_size = 0;
	}
}

inline void throwImpactError()
{
	throw ImpactException(mvIMPACT_C::IPL_GetErrorCode());
}

inline void throwImpactError(const std::string& msg)
{
	throw ImpactException(msg);
}

inline ImpactException::ImpactException(int errorCode)
		: std::runtime_error(mvIMPACT_C::IPL_GetErrorString(errorCode)), m_origin(mvIMPACT_C::IPL_GetErrorOrigin(errorCode)), m_error(errorCode)
	{
	}

inline ImpactException::ImpactException( const std::string& errorString, int errorCode /* = 0 */)
		: std::runtime_error(errorString), m_origin(""), m_error(errorCode)
	{}

inline ImpactException::ImpactException( const std::string& errorString, const std::string& errorOrigin, int errorCode /* = 0 */)
		: std::runtime_error(errorString), m_origin(errorOrigin), m_error(errorCode)
	{}

inline std::string ImpactException::getErrorString() const
	{ return what(); }

inline std::string ImpactException::getErrorOrigin() const
	{ return m_origin; }

inline int ImpactException::getErrorCode() const
	{ return m_error; }

inline int ImpactException::getError() const
	{ return m_error; }

inline std::string ImpactException::getErrorCodeAsString() const
	{ return m_error ? mvIMPACT_C::IPL_StringifyErrorCode(m_error) : "MVIMPACT_CPP_ERROR"; }
   // do not look up 0 at the C level : 0 is the default error code in the ImpactException ctor

inline MatrixDouble::MatrixDouble()
	{
		m_iWidth  = 0;
		m_iHeight = 0;
		m_pData   = NULL;
	}

inline MatrixDouble::MatrixDouble(int w, int h)
	{
		if (w<=0 || h<=0)
		{
			throwImpactException("Dimension < 0");
		}

		m_iWidth  = w;
		m_iHeight = h;
		m_pData   = new double[w*h];

		::memset(m_pData,0,sizeof(double)*w*h);
	}

inline MatrixDouble::MatrixDouble(int w, int h, const double* data)
	{
		if (w<=0 || h<=0)
		{
			throwImpactException("Dimension < 0");
		}

		m_iWidth  = w;
		m_iHeight = h;
		m_pData   = new double[w*h];

		::memcpy(m_pData, data, sizeof(double)*w*h);
	}

inline MatrixDouble::MatrixDouble(int w, int h, const std::vector<double> & data)
	{
		if (w<=0 || h<=0)
		{
			throw ImpactException(mvIMPACT_C::IPL_DIM_ERROR);
		}

		if ((int)data.size() != w*h)
		{
			throw ImpactException(mvIMPACT_C::IPL_DIM_ERROR);
		}


		m_iWidth  = w;
		m_iHeight = h;
		m_pData   = new double[w*h];

		int j;
		std::vector<double>::const_iterator i;
		for(i = data.begin(), j = 0;
			data.end() != i;
			i++, j++)
		{
			m_pData[j] = *i;
		}
	}

inline MatrixDouble::~MatrixDouble()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline double& MatrixDouble::operator [] ( int index ) const
	{
		if (index < 0 || index>= m_iWidth*m_iHeight)
		{
			throwImpactException("Index out of range");
		}

		return m_pData[index];
	}

inline MatrixDouble& MatrixDouble::set (int row, int col, double val)
	{
		if (col<0 || col>=m_iWidth || row<0 || row>=m_iHeight)
		{
			throwImpactException("Position out of range");
		}

		m_pData[row*m_iWidth+col] = val;
		return *this;
	}

inline double MatrixDouble::get(int row, int col) const
	{
		if (col<0 || col>=m_iWidth || row<0 || row>=m_iHeight)
		{
			throwImpactException("Position out of range");
		}

		return m_pData[row*m_iWidth+col];
	}

inline double* MatrixDouble::data()
	{
		return m_pData;
	}

inline const double* MatrixDouble::data() const
	{
		return m_pData;
	}

inline int MatrixDouble::getWidth() const
	{
		return m_iWidth;
	}

inline int MatrixDouble::getHeight() const
	{
		return m_iHeight;
	}

inline char* MatrixDouble::toString() const
	{
		static std::string result;

		std::ostringstream stream;
		stream << std::setprecision(2) << std::fixed;

		int row,col;
		for (row=0;row<getHeight();++row)
		{
			for (col=0;col<getWidth();++col)
			{
				stream << std::setw(6) << get(row,col) << " ";
			}
			stream << std::endl;
		}

		result = stream.str();
		return (char*)result.c_str();
	}

inline MatrixDouble::MatrixDouble(const MatrixDouble& m)
	{
		copy(m);
	}

inline MatrixDouble& MatrixDouble::operator=(const MatrixDouble& rhs)
	{
		if(&rhs != this) {
			dispose();
			copy(rhs);
		}
		return *this;
	}

inline void MatrixDouble::copy(const MatrixDouble& m)
	{

		m_iWidth  = m.getWidth();
		m_iHeight = m.getHeight();
		m_pData   = new double[m_iWidth*m_iHeight];

		::memcpy(m_pData,m.data(),sizeof(double)*m_iWidth*m_iHeight);
	}

inline void MatrixDouble::dispose()
{
		delete[] m_pData;
}

inline WindowHandle::WindowHandle()
{
	m_handle = 0;
}

inline WindowHandle::WindowHandle(mvIMPACT_C::IPL_HWND handle)
{
	m_handle = handle;
}

inline WindowHandle::WindowHandle(int handle)
{
	m_handle = (mvIMPACT_C::IPL_HWND)handle;
}

inline WindowHandle::operator mvIMPACT_C::IPL_HWND() const
{
	return m_handle;
}

inline WindowHandle::operator int() const
{
	return (int)m_handle;
}

inline mvIMPACT_C::IPL_HWND WindowHandle::ConvertToWindowHandle() const
{
	return m_handle;
}

inline int WindowHandle::ConvertToInteger() const
{
	return (int)m_handle;
}

// 64bit support makes sense for 64bit capable compilers only
#ifdef _M_X64
inline long long WindowHandle::ConvertToInteger64() const
{
	return (long long)m_handle;
}
#endif // #ifdef _M_X64

inline RGBColor::RGBColor ()
	{ }

inline RGBColor::RGBColor (int color)
	{
		fromDWord(color);
	}

inline RGBColor::RGBColor (unsigned char r, unsigned char g, unsigned char b, unsigned char a)
		: m_r(r), m_g(g), m_b(b), m_a(a)
	{
	}

inline unsigned char RGBColor::getRed() const { return m_r; }
inline unsigned char & RGBColor::getRed() { return m_r; }
inline RGBColor& RGBColor::setRed(unsigned char r) { m_r = r; return *this; }

inline unsigned char RGBColor::getGreen() const { return m_g; }
inline unsigned char & RGBColor::getGreen() { return m_g; }
inline RGBColor& RGBColor::setGreen(unsigned char g) { m_g = g; return *this; }

inline unsigned char RGBColor::getBlue() const { return m_b; }
inline unsigned char & RGBColor::getBlue() { return m_b; }
inline RGBColor& RGBColor::setBlue(unsigned char b) { m_b = b; return *this; }

inline unsigned char RGBColor::getAlpha() const { return m_a; }
inline unsigned char & RGBColor::getAlpha() { return m_a; }
inline RGBColor& RGBColor::setAlpha(unsigned char a) { m_a = a; return *this; }


inline RGBColor& RGBColor::setRGB (unsigned char r, unsigned char g, unsigned char b, unsigned char a)
	{
		m_r = r;
		m_g = g;
		m_b = b;
		m_a = a;
		return *this;
	}

inline int RGBColor::asDWord () const
	{
#ifdef __BIG_ENDIAN__
		 return m_b*0x00000001 + m_g*0x00000100 + m_r*0x00010000 + m_a*01000000;
#else // #ifdef __BIG_ENDIAN__
		 return m_r*0x00000001 + m_g*0x00000100 + m_b*0x00010000 + m_a*01000000;
#endif // #ifdef __BIG_ENDIAN__
	}

inline void RGBColor::fromDWord (int c)
	{
#ifdef __BIG_ENDIAN__
		 m_b = (c / 0x00000001) % 0x00000100;
		 m_g = (c / 0x00000100) % 0x00000100;
		 m_r = (c / 0x00010000) % 0x00000100;
		 m_a = (c / 0x01000000) % 0x00000100;
#else // #ifdef __BIG_ENDIAN__
		 m_r = (c / 0x00000001) % 0x00000100;
		 m_g = (c / 0x00000100) % 0x00000100;
		 m_b = (c / 0x00010000) % 0x00000100;
		 m_a = (c / 0x01000000) % 0x00000100;
#endif // #ifdef __BIG_ENDIAN__
	}


inline RGBColor::operator mvIMPACT_C::IPL_COLOR() const {
	return this->asDWord();
}

inline PointDouble::PointDouble( double x, double y):
		m_x(x), m_y(y) {}

inline bool PointDouble::operator== (const PointDouble& p) const
	{
		return (m_x==p.m_x && m_y==p.m_y);
	}

inline bool PointDouble::operator!=(const PointDouble& p) const
	{
		return !operator==(p);
	}

inline PointDouble::operator const mvIMPACT_C::IPL_POINT_DOUBLE() const
	{
		mvIMPACT_C::IPL_POINT_DOUBLE result;

		result.xpos = m_x;
		result.ypos = m_y;

		return result;
	}

inline PointDouble::PointDouble(const mvIMPACT_C::IPL_POINT_DOUBLE &p)
		: m_x(p.xpos), m_y(p.ypos)
	{ }

inline double PointDouble::getX() const
{
	return m_x;
}

inline double & PointDouble::getX()
{
	return m_x;
}

inline PointDouble& PointDouble::setX(double x)
{
	m_x = x;
	return *this;
}

inline double PointDouble::getY() const
{
	return m_y;
}

inline double & PointDouble::getY()
{
	return m_y;
}

inline PointDouble& PointDouble::setY(double y)
{
	m_y = y;
	return *this;
}


inline Point3DDouble::Point3DDouble(double x, double y, double z)
	: m_x(x), m_y(y), m_z(z)
{
}

inline double Point3DDouble::getX() const
{
	return m_x;
}

inline double & Point3DDouble::getX()
{
	return m_x;
}

inline Point3DDouble& Point3DDouble::setX(double x)
{
	m_x = x;
	return *this;
}

inline double Point3DDouble::getY() const
{
	return m_y;
}

inline double & Point3DDouble::getY()
{
	return m_y;
}

inline Point3DDouble& Point3DDouble::setY(double y)
{
	m_y = y;
	return *this;
}

inline double Point3DDouble::getZ() const
{
	return m_z;
}

inline double & Point3DDouble::getZ()
{
	return m_z;
}

inline Point3DDouble& Point3DDouble::setZ(double z)
{
	m_z = z;
	return *this;
}

inline Point3DDouble::operator const mvIMPACT_C::IPL_POINT3D_DOUBLE() const
{
	mvIMPACT_C::IPL_POINT3D_DOUBLE result;

	result.xpos = m_x;
	result.ypos = m_y;
	result.zpos = m_z;

	return result;
}

inline Point3DDouble::Point3DDouble(const mvIMPACT_C::IPL_POINT3D_DOUBLE &p)
		: m_x(p.xpos), m_y(p.ypos), m_z(p.zpos)
{
}


inline Point::Point( int x, int y)
	:	m_x(x), m_y(y) {}

inline bool Point::operator== (const Point& p) const
	{
		return (m_x==p.m_x && m_y==p.m_y);
	}

inline bool Point::operator!=(const Point p) const
	{
		return !operator==(p);
	}

inline Point::operator const mvIMPACT_C::IPL_POINT() const
	{
		mvIMPACT_C::IPL_POINT result;

		result.xpos = m_x;
		result.ypos = m_y;

		return result;
	}

inline Point::Point(const mvIMPACT_C::IPL_POINT &p)
		: m_x(p.xpos), m_y(p.ypos)
	{ }

inline int Point::getX() const
{
	return m_x;
}

inline int & Point::getX()
{
	return m_x;
}

inline Point& Point::setX(int x)
{
	m_x = x;
	return *this;
}


inline int Point::getY() const
{
	return m_y;
}

inline int & Point::getY()
{
	return m_y;
}

inline Point& Point::setY(int y)
{
	m_y = y;
	return *this;
}


inline Size::Size
	(
		int width,
		int height
	):
		m_width(width),m_height(height) {}

inline int Size::getWidth() const
{
	return m_width;
}

inline int & Size::getWidth()
{
	return m_width;
}

inline Size& Size::setWidth(int width)
{
	m_width = width;
	return *this;
}


inline int Size::getHeight() const
{
	return m_height;
}

inline int & Size::getHeight()
{
	return m_height;
}

inline Size& Size::setHeight(int height)
{
	m_height = height;
	return *this;
}


inline Line::Line() {}

inline Line::Line
	(
		int x0,
		int y0,
		int x1,
		int y1
	)
		:m_p0(x0,y0), m_p1(x1,y1) {}

inline Line::Line
	(
		const Point& p0,
		const Point& p1
	)
		:m_p0(p0), m_p1(p1) {}

inline bool Line::operator== (const Line& l) const
	{
		return m_p0 == l.m_p0 && m_p1 == l.m_p1;
	}

inline bool Line::operator!= (const Line& l) const
	{
		return !operator==(l);
	}


inline Point Line::getP0() const
{
	return m_p0;
}

inline Point & Line::getP0()
{
	return m_p0;
}

inline Line& Line::setP0(const Point &p0)
{
	m_p0 = p0;
	return *this;
}

inline Point Line::getP1() const
{
	return m_p1;
}

inline Point & Line::getP1()
{
	return m_p1;
}

inline Line& Line::setP1(const Point &p1)
{
	m_p1 = p1;
	return *this;
}

inline Line::operator mvIMPACT_C::IPL_LINE() const
{
	mvIMPACT_C::IPL_LINE result;

	result.x0 = m_p0.getX();
	result.y0 = m_p0.getY();
	result.x1 = m_p1.getX();
	result.y1 = m_p1.getY();

	return result;
}


inline Line::Line(const mvIMPACT_C::IPL_LINE & l)
	: m_p0(l.x0, l.y0), m_p1(l.x1, l.y1)
{ }

inline Rectangle::Rectangle()
	: m_left(0), m_right(0), m_top(0), m_bottom(0)
{ }

inline Rectangle::Rectangle( int left, int top, int right, int bottom )
	: m_left(left), m_right(right), m_top(top), m_bottom(bottom) 
{ }

inline Rectangle::Rectangle( const Point& p0, const Point& p1 )
	: m_left(p0.getX()), m_right(p1.getX()), m_top(p0.getY()), m_bottom(p1.getY()) 
{ }

inline int Rectangle::getWidth() const
{
	return m_right - m_left + 1;
}

inline int Rectangle::getHeight() const
{
	return m_bottom - m_top + 1;
}

inline bool Rectangle::operator== (const Rectangle& r) const
{
	return (m_left  == r.m_left  && m_top    == r.m_top &&
		m_right == r.m_right && m_bottom == r.m_bottom);
}

inline bool Rectangle::operator!= (const Rectangle& r) const
{
	return !operator==(r);
}

inline Rectangle::operator mvIMPACT_C::IPL_RECT() const
{
	mvIMPACT_C::IPL_RECT result;

	result.left = m_left;
	result.right = m_right;
	result.top = m_top;
	result.bottom = m_bottom;

	return result;
}


inline Rectangle::Rectangle(const mvIMPACT_C::IPL_RECT& r)
	: m_left(r.left), m_right(r.right), m_top(r.top), m_bottom(r.bottom)
{ }


inline int Rectangle::getLeft() const
{
	return m_left;
}

inline int & Rectangle::getLeft()
{
	return m_left;
}

inline Rectangle& Rectangle::setLeft(int left)
{
	m_left = left;
	return *this;
}


inline int Rectangle::getRight() const
{
	return m_right;
}

inline int & Rectangle::getRight()
{
	return m_right;
}

inline Rectangle& Rectangle::setRight(int right)
{
	m_right = right;
	return *this;
}


inline int Rectangle::getTop() const
{
	return m_top;
}

inline int & Rectangle::getTop() 
{
	return m_top;
}

inline Rectangle& Rectangle::setTop(int top)
{
	m_top = top;
	return *this;
}


inline int Rectangle::getBottom() const
{
	return m_bottom;
}

inline int & Rectangle::getBottom()
{
	return m_bottom;
}

inline Rectangle& Rectangle::setBottom(int bottom)
{
	m_bottom = bottom;
	return *this;
}


inline LineDouble::LineDouble() {}

inline LineDouble::LineDouble (double x0, double y0, double x1, double y1)
		:m_p0(x0,y0), m_p1(x1,y1)
{}


inline LineDouble::LineDouble (PointDouble p0, PointDouble p1)
		: m_p0(p0), m_p1(p1)
{}

inline PointDouble LineDouble::getP0() const
{
	return m_p0;
}

inline PointDouble & LineDouble::getP0()
{
	return m_p0;
}

inline LineDouble& LineDouble::setP0(PointDouble p0)
{
	m_p0 = p0;
	return *this;
}


inline PointDouble LineDouble::getP1() const
{
	return m_p1;
}

inline PointDouble & LineDouble::getP1()
{
	return m_p1;
}

inline LineDouble& LineDouble::setP1(PointDouble p1)
{
	m_p1 = p1;
	return *this;
}

inline LineDouble::operator mvIMPACT_C::IPL_LINE_DOUBLE() const
{
	mvIMPACT_C::IPL_LINE_DOUBLE result;

	result.x0 = m_p0.getX();
	result.y0 = m_p0.getY();
	result.x1 = m_p1.getX();
	result.y1 = m_p1.getY();

	return result;
}


inline LineDouble::LineDouble(const mvIMPACT_C::IPL_LINE_DOUBLE & l)
	: m_p0(l.x0, l.y0), m_p1(l.x1, l.y1)
{ }


DEFINE_USAGE_COUNTER(Calibrator);

#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(Unit, std::string, Calibrator)
PROXY_CLASS_SIMPLE_IMPL(Power, int, Calibrator)

#pragma warning(pop)

inline std::string Calibrator::getUnit() const
{
	return m_->chUnit;
}

inline Calibrator::UnitProxy Calibrator::getUnit()
{
	return UnitProxy(*this);
}

inline Calibrator& Calibrator::setUnit(const std::string& strUnit)
{
	strncpy(m_->chUnit, strUnit.c_str(), sizeof(m_->chUnit)-1);
	m_->chUnit[sizeof(m_->chUnit)-1] = 0;
	return *this;
}

inline int Calibrator::getPower() const
{
	return m_->iPower;
}

inline Calibrator::PowerProxy Calibrator::getPower()
{
	return PowerProxy(*this);
}

inline Calibrator& Calibrator::setPower(int p)
{
	m_->iPower = p;
	return *this;
}

inline double Calibrator::getOffset() const
{
	return m_->dfOffset;
}

inline double & Calibrator::getOffset()
{
	return m_->dfOffset;
}

inline Calibrator& Calibrator::setOffset(double o)
{
	m_->dfOffset = o;
	return *this;
}

inline int Calibrator::getSize() const
{
	return m_->iSize;
}

inline double Calibrator::operator[](int index) const
{
	return m_->dValues[index];
}

inline Calibrator::Calibrator
	(
		int size
	): m_(0)
	{
	  INC_USAGE_COUNTER;

		m_ = (mvIMPACT_C::IPL_CALIBRATE*)new char[sizeof(mvIMPACT_C::IPL_CALIBRATE)+size*sizeof(mvIMPACT_C::IPL_DOUBLE)];

		strncpy(m_->chUnit, "", sizeof(m_->chUnit)-1);
		m_->chUnit[sizeof(m_->chUnit)-1] = 0;
		m_->iPower = 0;
		m_->dfOffset = 0.0;
		m_->iSize = size;

		for(int i=0; i<size; ++i)
			m_->dValues[i] = i;
	}

template<class InIt>
inline Calibrator::Calibrator(InIt begin, InIt end, std::string unit, int power, double offset)
		: m_(0)
	{
	  INC_USAGE_COUNTER;

		typename std::iterator_traits<InIt>::difference_type size = std::distance(begin, end);

		m_ = (mvIMPACT_C::IPL_CALIBRATE*)new char[sizeof(mvIMPACT_C::IPL_CALIBRATE)+size*sizeof(mvIMPACT_C::IPL_DOUBLE)];

		strncpy(m_->chUnit, unit.c_str(), sizeof(m_->chUnit)-1);
		m_->chUnit[sizeof(m_->chUnit)-1] = 0;
		m_->iPower   = power;
		m_->dfOffset = offset;
		m_->iSize    = size;

		for(typename std::iterator_traits<InIt>::difference_type i=0; i<size; ++i)
			m_->dValues[i] = *begin++;
	}

inline Calibrator::~Calibrator()
	{
		DEC_USAGE_COUNTER

		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}

inline Calibrator::Calibrator(const Calibrator& r)
	{
	  INC_USAGE_COUNTER;

	  copy(r);
	}

inline Calibrator& Calibrator::operator=(const Calibrator& r)
	{
		if (this != &r)
		{
			dispose();
			copy(r);
		}
		return *this;
	}

	inline bool Calibrator::operator==(const Calibrator& r) const
	{
		return
			getUnit()   == r.getUnit() &&
			getPower()  == r.getPower() &&
			getOffset() == r.getOffset() &&
			getSize()   == r.getSize() &&
			0 == memcmp(m_->dValues, r.m_->dValues, sizeof(mvIMPACT_C::IPL_DOUBLE)*getSize());
	}

inline bool Calibrator::operator!=(const Calibrator& r) const
	{
		return !operator==(r);
	}

inline double Calibrator::getValue(int index) const
{
		return m_->dValues[index];
}

inline Calibrator& Calibrator::setValue(int index, double value)
{
		m_->dValues[index] = value;
		return *this;
}

inline double& Calibrator::operator[](int index)
	{
		return m_->dValues[index];
	}

inline void Calibrator::copy(const Calibrator& src)
{
	m_ = (mvIMPACT_C::IPL_CALIBRATE*)new char[sizeof(mvIMPACT_C::IPL_CALIBRATE)+src.m_->iSize*sizeof(mvIMPACT_C::IPL_DOUBLE)];

	strncpy(m_->chUnit, src.m_->chUnit, sizeof(m_->chUnit)-1);
	m_->chUnit[sizeof(m_->chUnit)-1] = 0;

	m_->iPower = src.m_->iPower;
	m_->dfOffset = src.m_->dfOffset;
	m_->iSize = src.m_->iSize;
	memcpy(m_->dValues, src.m_->dValues, sizeof(mvIMPACT_C::IPL_DOUBLE)*src.m_->iSize);
}

inline void Calibrator::dispose()
{
		delete [] m_;
		m_ = 0;
}


inline Coord::Coord()
		: m_x(0), m_y(0), m_band(0), m_slice(0), m_frame(0)
{}


inline Coord::Coord(int x, int y, int band, int slice, int frame)
		: m_x(x), m_y(y), m_band(band), m_slice(slice), m_frame(frame)
{}


inline int Coord::getX() const
{
	return m_x;
}

inline int & Coord::getX()
{
	return m_x;
}

inline Coord& Coord::setX(int x)
{
	m_x = x;
	return *this;
}


inline int Coord::getY() const
{
	return m_y;
}

inline int & Coord::getY()
{
	return m_y;
}

inline Coord& Coord::setY(int y)
{
	m_y = y;
	return *this;
}

inline int Coord::getBand() const
{
	return m_band;
}

inline int & Coord::getBand()
{
	return m_band;
}

inline Coord& Coord::setBand(int band)
{
	m_band = band;
	return *this;
}

inline int Coord::getSlice() const
{
	return m_slice;
}

inline int & Coord::getSlice()
{
	return m_slice;
}

inline Coord& Coord::setSlice(int slice)
{
	m_slice = slice;
	return *this;
}

inline int Coord::getFrame() const
{
	return m_frame;
}

inline int & Coord::getFrame()
{
	return m_frame;
}

inline Coord& Coord::setFrame(int frame)
{
	m_frame = frame;
	return *this;
}


inline Position::Position()
		: m_x(0), m_y(0), m_scaleX(1), m_scaleY(1), m_angle(0)
{ }

inline double Position::getX() const
{
	return m_x;
}

inline double & Position::getX()
{
	return m_x;
}

inline Position& Position::setX(double x)
{
	m_x = x;
	return *this;
}

inline double Position::getY() const
{
	return m_y;
}

inline double & Position::getY()
{
	return m_y;
}

inline Position& Position::setY(double y)
{
	m_y = y;
	return *this;
}

inline double Position::getScaleX() const
{
	return m_scaleX;
}

inline double & Position::getScaleX()
{
	return m_scaleX;
}

inline Position& Position::setScaleX(double scaleX)
{
	m_scaleX = scaleX;
	return *this;
}


inline double Position::getScaleY() const
{
	return m_scaleY;
}

inline double & Position::getScaleY()
{
	return m_scaleY;
}

inline Position& Position::setScaleY(double scaleY)
{
	m_scaleY = scaleY;
	return *this;
}


inline double Position::getAngle() const
{
	return m_angle;
}

inline double & Position::getAngle()
{
	return m_angle;
}

inline Position& Position::setAngle(double angle)
{
	m_angle = angle;
	return *this;
}

inline Position::Position(double x, double y, double scaleX, double scaleY, double angle)
	: m_x(x), m_y(y), m_scaleX(scaleX), m_scaleY(scaleY), m_angle(angle)
{ }

inline MyBufferAccess::MyBufferAccess()
	: m_width(0), m_height(0), m_bands(0), m_slices(0), m_frames(0),
	  m_currentBand(0), m_currentSlice(0), m_currentFrame(0),
	  m_isValid(false), m_hasWriteAccess(false), m_pitch(0), m_getPixelFunc(NULL), m_setPixelFunc(NULL)
{ }

inline void MyBufferAccess::init(mvIMPACT_C::IPL_BUFHANDLE handle, int band, int slice, int frame, int width, int height, int bands, int slices, int frames, BufferDatatype datatype, bool hasWriteAccess)
{
	if (m_isValid)
		dispose();

	m_handle = handle;

	m_currentBand = band;
	m_currentSlice = slice;
	m_currentFrame = frame;

	m_width = width;
	m_height = height;

	m_bands = bands;
	m_slices = slices;
	m_frames = frames;

	int index = band +
		slice*m_bands +
		frame*m_slices*m_bands;


	m_hasWriteAccess = hasWriteAccess;

	switch (datatype)
	{
		case bdChar:
			m_getPixelFunc = getPixelChar;
			m_setPixelFunc = setPixelChar;
			break;
		case bdUChar:
			m_getPixelFunc = getPixelUChar;
			m_setPixelFunc = setPixelUChar;
			break;
		case bdShort:
			m_getPixelFunc = getPixelShort;
			m_setPixelFunc = setPixelShort;
			break;
		case bdUShort:
			m_getPixelFunc = getPixelUShort;
			m_setPixelFunc = setPixelUShort;
			break;
		case bdLong:
			m_getPixelFunc = getPixelLong;
			m_setPixelFunc = setPixelLong;
			break;
		case bdULong:
			m_getPixelFunc = getPixelULong;
			m_setPixelFunc = setPixelULong;
			break;
		case bdFloat:
			m_getPixelFunc = getPixelFloat;
			m_setPixelFunc = setPixelFloat;
			break;
		case bdDouble:
			m_getPixelFunc = getPixelDouble;
			m_setPixelFunc = setPixelDouble;
			break;
		case bdNone:
			m_getPixelFunc = getPixelNone;
			m_setPixelFunc = setPixelNone;
			break;
		default:
			throwImpactException("Unknown buffer datatype");
	}

	if (!m_pointers.size())
		m_pointers = std::vector<mvIMPACT_C::IPL_UCHAR*>(
			m_bands*m_slices*m_frames, (mvIMPACT_C::IPL_UCHAR*)0);

	getPointer(band, slice, frame);

	m_planeIndexStack.push_back(index);
}

inline unsigned char* MyBufferAccess::getPointer() const
{
	return getPointer(m_currentBand, m_currentSlice, m_currentFrame);
}

inline unsigned char* MyBufferAccess::getPointer(int band, int slice, int frame) const
{
	int index = band +
		slice*m_bands +
		frame*m_slices*m_bands;

	if (0 == m_pointers[index])
	{
		if (!mvIMPACT_C::IPL_GetPtr(
			m_handle, mvIMPACT_C::IPL_NO_LOCK,
			band, slice, frame,
			(mvIMPACT_C::IPL_LONG*)&m_pitch,
			(mvIMPACT_C::IPL_UCHAR**)&m_pointers[index]))
		{
			throwImpactError();
		}
	}

	return m_pointers[index];
}

inline int MyBufferAccess::getPitch() const
{
	return m_pitch;
}

inline void MyBufferAccess::releaseAllPointers()
{
	for (int b=0; b<m_bands; ++b)
	{
		for (int s=0; s<m_slices; ++s)
		{
			for (int f=0; f<m_frames; ++f)
			{
				releasePointer(b, s, f);
			}
		}
	}
}

inline void MyBufferAccess::releasePointer(int band, int slice, int frame)
{
	int index = band +
		slice*m_bands +
		frame*m_slices*m_bands;

	if ((size_t)index < m_pointers.size() && m_pointers[index])
	{
		if (!mvIMPACT_C::IPL_ReleasePtr(
			m_handle,
			mvIMPACT_C::IPL_NO_LOCK,
			band, slice, frame,
			m_pointers[index]))
		{
			throwImpactError();
		}
		m_pointers[index] = 0;
	}
	//remove the index from the planeIndexStack and set m_currentBand, m_currentSlice and m_currentFrame
	//accordingly	
	std::vector<int>::iterator it = find(m_planeIndexStack.begin(), m_planeIndexStack.end(),index);
	while ( it != m_planeIndexStack.end()) {
		m_planeIndexStack.erase(it);
		it = find(m_planeIndexStack.begin(), m_planeIndexStack.end(),index);
	}

	//get access to the plane that was locked just before the one that was released
	if (!m_planeIndexStack.empty()) {
		index = m_planeIndexStack.back();
		m_currentFrame = index / (m_bands*m_slices);
		int temp = index % (m_bands*m_slices);
		m_currentSlice = temp / m_bands;
		temp = temp / m_bands;
		m_currentBand = index%m_bands;

	} else {
		m_currentBand = 0;
		m_currentSlice = 0;
		m_currentFrame = 0;
	}



}

inline void MyBufferAccess::dispose()
{
	releaseAllPointers();
	m_pointers = std::vector<mvIMPACT_C::IPL_UCHAR*>();
}

inline MyBufferAccess::~MyBufferAccess()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}


inline int MyBufferAccess::getPixel(int x, int y, int band, int slice, int frame) const
{
	if (!(0 <= x && x <= m_width &&
		  0 <= y && y <= m_height))
	{
		throwImpactException("Index out of range");
	}
	return m_getPixelFunc(this, x, y, band, slice, frame);
}



inline void MyBufferAccess::setPixel(int x, int y, int band, int slice, int frame, int value)
{
	if (!(m_hasWriteAccess &&
		0 <= x && x <= m_width &&
		0 <= y && y <= m_height))
	{
		throwImpactException("Index out of range");
	}
	if(!m_hasWriteAccess)
	{
		throwImpactException("Buffer has no direct access lock for writing to specific plane.");
	}
	else
	{
		m_setPixelFunc(this, x, y, band, slice, frame, value);
	}
}

inline int MyBufferAccess::getCurrentBand() const
{
	return m_currentBand;
}

inline int MyBufferAccess::getCurrentSlice() const
{
	return m_currentSlice;
}

inline int MyBufferAccess::getCurrentFrame() const
{
	return m_currentFrame;
}


inline int MyBufferAccess::getHasWriteAccess() const
{
	return m_hasWriteAccess;
}

inline bool MyBufferAccess::getHasPointer(int band, int slice, int frame) const
{
	int index = band +
		slice*m_bands +
		frame*m_slices*m_bands;
	return (size_t)index < m_pointers.size() &&
		0 != m_pointers[index];
}

inline bool MyBufferAccess::getHasPointer() const
{
	return getHasPointer(m_currentBand, m_currentSlice, m_currentFrame);
}

inline bool MyBufferAccess::getHasAnyPointer() const
{
	for (int b=0; b<m_bands; ++b)
	{
		for (int s=0; s<m_slices; ++s)
		{
			for (int f=0; f<m_frames; ++f)
			{
				if (getHasPointer(b, s, f))
					return true;
			}
		}
	}

	return false;
}

inline int MyBufferAccess::getPixelChar(const MyBufferAccess* t, int x, int y, int band, int slice, int frame)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	return *((mvIMPACT_C::IPL_CHAR*)(t->m_pointers[index]+y*t->m_pitch)+x);
}

inline int MyBufferAccess::getPixelUChar(const MyBufferAccess* t, int x, int y, int band, int slice, int frame)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	return *((mvIMPACT_C::IPL_UCHAR*)(t->m_pointers[index]+y*t->m_pitch)+x);
}

inline int MyBufferAccess::getPixelShort(const MyBufferAccess* t, int x, int y, int band, int slice, int frame)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	return *((mvIMPACT_C::IPL_SHORT*)(t->m_pointers[index]+y*t->m_pitch)+x);
}

inline int MyBufferAccess::getPixelUShort(const MyBufferAccess* t, int x, int y, int band, int slice, int frame)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	return *((mvIMPACT_C::IPL_USHORT*)(t->m_pointers[index]+y*t->m_pitch)+x);
}

inline int MyBufferAccess::getPixelLong(const MyBufferAccess* t, int x, int y, int band, int slice, int frame)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	return *((mvIMPACT_C::IPL_LONG*)(t->m_pointers[index]+y*t->m_pitch)+x);
}

inline int MyBufferAccess::getPixelULong(const MyBufferAccess* t, int x, int y, int band, int slice, int frame)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	return *((mvIMPACT_C::IPL_ULONG*)(t->m_pointers[index]+y*t->m_pitch)+x);
}

inline int MyBufferAccess::getPixelFloat(const MyBufferAccess* /*t*/, int /*x*/, int /*y*/, int /*band*/, int /*slice*/, int /*frame*/)
{
	throwImpactException("getPixel() for buffer datatype float currently not supported");
}

inline int MyBufferAccess::getPixelDouble(const MyBufferAccess* /*t*/, int /*x*/, int /*y*/, int /*band*/, int /*slice*/, int /*frame*/)
{
	throwImpactException("getPixel() for buffer datatype double currently not supported");
}

inline int MyBufferAccess::getPixelNone(const MyBufferAccess* /*t*/, int /*x*/, int /*y*/, int /*band*/, int /*slice*/, int /*frame*/)
{
	throwImpactException("getPixel() for buffer datatype bdNone not supported");
}

inline void MyBufferAccess::setPixelChar(MyBufferAccess* t, int x, int y, int band, int slice, int frame, int value)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	*((mvIMPACT_C::IPL_CHAR*)(t->m_pointers[index]+y*t->m_pitch)+x) =
		(mvIMPACT_C::IPL_CHAR)value;
}

inline void MyBufferAccess::setPixelUChar(MyBufferAccess* t, int x, int y, int band, int slice, int frame, int value)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	*((mvIMPACT_C::IPL_UCHAR*)(t->m_pointers[index]+y*t->m_pitch)+x) =
		(mvIMPACT_C::IPL_UCHAR)value;
}

inline void MyBufferAccess::setPixelShort(MyBufferAccess* t, int x, int y, int band, int slice, int frame, int value)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	*((mvIMPACT_C::IPL_SHORT*)(t->m_pointers[index]+y*t->m_pitch)+x) =
		(mvIMPACT_C::IPL_SHORT)value;
}

inline void MyBufferAccess::setPixelUShort(MyBufferAccess* t, int x, int y, int band, int slice, int frame, int value)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	*((mvIMPACT_C::IPL_USHORT*)(t->m_pointers[index]+y*t->m_pitch)+x) =
		(mvIMPACT_C::IPL_USHORT)value;
}

inline void MyBufferAccess::setPixelLong(MyBufferAccess* t, int x, int y, int band, int slice, int frame, int value)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	*((mvIMPACT_C::IPL_LONG*)(t->m_pointers[index]+y*t->m_pitch)+x) =
		(mvIMPACT_C::IPL_LONG)value;
}

inline void MyBufferAccess::setPixelULong(MyBufferAccess* t, int x, int y, int band, int slice, int frame, int value)
{
	int index = band + t->m_bands * (slice + frame*t->m_slices);
	*((mvIMPACT_C::IPL_ULONG*)(t->m_pointers[index]+y*t->m_pitch)+x) =
		(mvIMPACT_C::IPL_ULONG)value;
}

inline void MyBufferAccess::setPixelFloat(MyBufferAccess* /*t*/, int /*x*/, int /*y*/, int /*band*/, int /*slice*/, int /*frame*/, int /*value*/)
{
	throwImpactException("setPixel() for buffer datatype bdFloat currently not supported");
}

inline void MyBufferAccess::setPixelDouble(MyBufferAccess* /*t*/, int /*x*/, int /*y*/, int /*band*/, int /*slice*/, int /*frame*/, int /*value*/)
{
	throwImpactException("setPixel() for buffer datatype bdDouble currently not supported");
}

inline void MyBufferAccess::setPixelNone(MyBufferAccess* /*t*/, int /*x*/, int /*y*/, int /*band*/, int /*slice*/, int /*frame*/, int /*value*/)
{
	throwImpactException("setPixel() for buffer datatype bdNone not supported");
}

DEFINE_USAGE_COUNTER(BufferLock);


inline bool BufferLock::getIsValid() const
{
	return
		( getUseCount()>0 ) &&
		( lmRead == lock_ || lmReadWrite == lock_ );
}

inline int BufferLock::getUseCount() const
{
	return *counter_;
}

inline BufferLock::BufferLock()
	: buffer_(0), counter_(new int(1)), lock_(lmRead)
{
	INC_USAGE_COUNTER;
	assert(getIsValid());
}

inline BufferLock::BufferLock(const BufferBase* buffer, LockMode lock)
	: buffer_(buffer->makeShallowCopy()), counter_(new int(1)), lock_(lock)
{
	INC_USAGE_COUNTER
    mvIMPACT_C::IPL_LOCK_MODE lm = static_cast<mvIMPACT_C::IPL_LOCK_MODE>(lock_);
	if (!mvIMPACT_C::IPL_BufLock(
			buffer_->getHandle(),
			lm))
		throwImpactError();
	assert(getIsValid());
}

inline BufferLock::BufferLock(const BufferLock& r)
{
	INC_USAGE_COUNTER;
	copy(r);
}

inline BufferLock& BufferLock::operator=(const BufferLock& r)
{
	if (&r != this)
	{
		dispose();
		copy(r);
	}
	return *this;
}

inline BufferLock::~BufferLock()
{
	DEC_USAGE_COUNTER

	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline void BufferLock::copy(const BufferLock& src)
{
	buffer_ = src.buffer_;
	lock_ = src.lock_;
	counter_ = src.counter_;

	(*counter_)++;
}


inline void BufferLock::dispose()
{
	(*counter_)--;
	if (0 == *counter_)
	{
		if (buffer_)
		{
			if (!mvIMPACT_C::IPL_BufUnlock(
					buffer_->getHandle(),
					mvIMPACT_C::IPL_LOCK_MODE(lock_)))
				throwImpactError();
		}
		delete buffer_;
		buffer_ = 0;
		delete counter_;
	}
}

inline int BufferBase::getPixel(
	int x, int y, int band, int slice, int frame) const
{
	if (access_->getHasPointer(band, slice, frame))
	{
		return access_->getPixel(x, y, band, slice, frame);
	}
	else
	{
		return getValue32s(x, y, band, slice, frame);
	}
}

inline int BufferBase::getPixel(int x, int y) const
{
	if (access_->getHasPointer())
	{
		return access_->getPixel(x, y, access_->getCurrentBand(), access_->getCurrentSlice(), access_->getCurrentFrame());
	}
	else
	{
		throwImpactError("Buffer has no direct access lock for reading from specific plane.");
		return 0; // prevent compiler warning
	}
}

inline void BufferBase::setPixel(
	int x, int y, int band, int slice, int frame, int value)
{
	if (access_->getHasPointer(band, slice, frame) &&
		access_->getHasWriteAccess())
	{
		access_->setPixel(x, y, band, slice, frame, value);
	}
	else
	{
		setValue32s(value, x, y, band, slice, frame);
	}
}

inline void BufferBase::setPixel(int x, int y, int value)
{
	if (access_->getHasPointer() &&
		access_->getHasWriteAccess())
	{
		access_->setPixel(x, y, access_->getCurrentBand(), access_->getCurrentSlice(), access_->getCurrentFrame(), value);
	}
	else
	{
		throwImpactError("Buffer has no direct access lock for writing to specific plane.");
	}
}

inline void BufferBase::prepareReadAccess(int band, int slice, int frame) const
{
	if (!access_->getHasWriteAccess() &&
		access_->getHasPointer(band, slice, frame))
	{
		// if the buffer has exactly the requested access lock, do nothing
	}
	else
	{
		access_->init(getHandle(),
			band, slice, frame,
			getWidth(), getHeight(), getBands(), getSlices(), getFrames(),
			getDatatype(),
			false);
	}
}

inline void BufferBase::prepareReadWriteAccess(int band, int slice, int frame)
{
	if (access_->getHasWriteAccess() &&
		access_->getHasPointer(band, slice, frame))
	{
		// if the buffer has exactly the requested access lock, do nothing
	}
	else
	{
		access_->init(getHandle(),
			band, slice, frame,
			getWidth(), getHeight(), getBands(), getSlices(), getFrames(),
			getDatatype(),
			true);
	}
}

inline unsigned char* BufferBase::getPointer() const
{
	if (!access_->getHasPointer())
	{
		throwImpactException("Buffer has no direct access lock for specific plane.");
	}
	else
	{
		return access_->getPointer();
	}
}

inline unsigned char* BufferBase::getPointer(int band, int slice, int frame) const
{
	if (!access_->getHasPointer(band, slice, frame))
	{
		throwImpactException("Buffer has no direct access lock for specific plane.");
	}
	else
	{
		return access_->getPointer(band, slice, frame);
	}
}

inline unsigned char* BufferBase::getPointer()
{
	if (!access_->getHasPointer())
	{
		throwImpactException("Buffer has no direct access lock for specific plane.");
	}
	else
	{
		return access_->getPointer();
	}
}

inline unsigned char* BufferBase::getPointer(int band, int slice, int frame)
{
	if (!access_->getHasPointer(band, slice, frame))
	{
		throwImpactException("Buffer has no direct access lock for specific plane.");
	}
	else
	{
		return access_->getPointer(band, slice, frame);
	}
}

inline int BufferBase::getPitch() const
{
	if (!access_->getHasAnyPointer())
	{
		throwImpactException("Buffer has no direct access lock for any plane.");
	}
	else
	{
		return access_->getPitch();
	}
}

inline void BufferBase::releaseAccess(int band, int slice, int frame) const
{
	access_->releasePointer(band, slice, frame);
}

inline void BufferBase::releaseAccess() const
{
	access_->releaseAllPointers();
}

inline double BufferBase::getValue(
	int x, int y, int band, int slice, int frame) const
{
	double result = 0.0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &result,
		mvIMPACT_C::IPL_DTDOUBLE))
	{ throwImpactError(); }

	return result;
}

inline std::vector<double> BufferBase::getValues(
	int x, int y, int width, int height, int band, int slice, int frame) const
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	std::vector<double> result;
	result.resize(width*height);

	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*result.begin(),
		mvIMPACT_C::IPL_DTDOUBLE))
	{ throwImpactError(); }

	return result;
}

inline int BufferBase::getValue32s(
	int x, int y, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_LONG result = 0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &result,
		mvIMPACT_C::IPL_DTLONG))
	{ throwImpactError(); }

	return result;
}

inline std::vector<int> BufferBase::getValues32s(
	int x, int y, int width, int height, int band, int slice, int frame) const
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	std::vector<int> result;
	result.resize(width*height);

	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*result.begin(),
		mvIMPACT_C::IPL_DTLONG))
	{ throwImpactError(); }

	return result;
}

inline unsigned int BufferBase::getValue32(
	int x, int y, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_ULONG result = 0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &result,
		mvIMPACT_C::IPL_DTULONG))
	{ throwImpactError(); }

	return result;
}

inline std::vector<unsigned int> BufferBase::getValues32(
	int x, int y, int width, int height, int band, int slice, int frame) const
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	std::vector<unsigned int> result;
	result.resize(width*height);

	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*result.begin(),
		mvIMPACT_C::IPL_DTULONG))
	{ throwImpactError(); }

	return result;
}

inline short BufferBase::getValue16s(
	int x, int y, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_SHORT result = 0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &result,
		mvIMPACT_C::IPL_DTSHORT))
	{ throwImpactError(); }

	return result;
}

inline std::vector<short> BufferBase::getValues16s(
	int x, int y, int width, int height, int band, int slice, int frame) const
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	std::vector<short> result;
	result.resize(width*height);

	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*result.begin(),
		mvIMPACT_C::IPL_DTSHORT))
	{ throwImpactError(); }

	return result;
}

inline unsigned short BufferBase::getValue16(
	int x, int y, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_USHORT result = 0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &result,
		mvIMPACT_C::IPL_DTUSHORT))
	{ throwImpactError(); }

	return result;
}

inline std::vector<unsigned short> BufferBase::getValues16(
	int x, int y, int width, int height, int band, int slice, int frame) const
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	std::vector<unsigned short> result;
	result.resize(width*height);

	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*result.begin(),
		mvIMPACT_C::IPL_DTUSHORT))
	{ throwImpactError(); }

	return result;
}

inline char BufferBase::getValue8s(
	int x, int y, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_CHAR result = 0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &result,
		mvIMPACT_C::IPL_DTCHAR))
	{ throwImpactError(); }

	return result;
}

inline std::vector<char> BufferBase::getValues8s(
	int x, int y, int width, int height, int band, int slice, int frame) const
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	std::vector<char> result;
	result.resize(width*height);

	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*result.begin(),
		mvIMPACT_C::IPL_DTCHAR))
	{ throwImpactError(); }

	return result;
}

inline unsigned char BufferBase::getValue8(
	int x, int y, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_UCHAR result = 0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &result,
		mvIMPACT_C::IPL_DTUCHAR))
	{ throwImpactError(); }

	return result;
}

inline std::vector<unsigned char> BufferBase::getValues8(
	int x, int y, int width, int height, int band, int slice, int frame) const
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	std::vector<unsigned char> result;
	result.resize(width*height);

	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*result.begin(),
		mvIMPACT_C::IPL_DTUCHAR))
	{ throwImpactError(); }

	return result;
}

inline BufferBase& BufferBase::setValue(
	double value, int x, int y, int band, int slice, int frame)
{
	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &value,
		mvIMPACT_C::IPL_DTDOUBLE))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValues(
	const std::vector<double>& values,
	int x, int y, int width, int height, int band, int slice, int frame)
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	if (values.size() < (size_t)(width*height))
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*values.begin(),
		mvIMPACT_C::IPL_DTDOUBLE))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValue32(
	unsigned int value, int x, int y, int band, int slice, int frame)
{
	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &value,
		mvIMPACT_C::IPL_DTULONG))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValues32(
	const std::vector<unsigned int>& values,
	int x, int y, int width, int height, int band, int slice, int frame)
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	if (values.size() < (size_t)(width*height))
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*values.begin(),
		mvIMPACT_C::IPL_DTULONG))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValue32s(
	int value, int x, int y, int band, int slice, int frame)
{
	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &value,
		mvIMPACT_C::IPL_DTLONG))
	{ throwImpactError(); }
	return *this;
}

inline BufferBase& BufferBase::setValues32s(
	const std::vector<int>& values,
	int x, int y, int width, int height, int band, int slice, int frame)
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	if (values.size() < (size_t)(width*height))
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*values.begin(),
		mvIMPACT_C::IPL_DTLONG))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValue16(
	unsigned short value, int x, int y, int band, int slice, int frame)
{
	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &value,
		mvIMPACT_C::IPL_DTUSHORT))
	{ throwImpactError(); }
	return *this;
}

inline BufferBase& BufferBase::setValues16(
	const std::vector<unsigned short>& values,
	int x, int y, int width, int height, int band, int slice, int frame)
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	if (values.size() < (size_t)(width*height))
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*values.begin(),
		mvIMPACT_C::IPL_DTUSHORT))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValue16s(
	short value, int x, int y, int band, int slice, int frame)
{
	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &value,
		mvIMPACT_C::IPL_DTSHORT))
	{ throwImpactError(); }
	return *this;
}

inline BufferBase& BufferBase::setValues16s(
	const std::vector<short>& values,
	int x, int y, int width, int height, int band, int slice, int frame)
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	if (values.size() < (size_t)(width*height))
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*values.begin(),
		mvIMPACT_C::IPL_DTSHORT))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValue8(
	unsigned char value, int x, int y, int band, int slice, int frame)
{
	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &value,
		mvIMPACT_C::IPL_DTUCHAR))
	{ throwImpactError(); }
	return *this;
}

inline BufferBase& BufferBase::setValues8(
	const std::vector<unsigned char>& values,
	int x, int y, int width, int height, int band, int slice, int frame)
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	if (values.size() < (size_t)(width*height))
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*values.begin(),
		mvIMPACT_C::IPL_DTUCHAR))
	{ throwImpactError(); }

	return *this;
}

inline BufferBase& BufferBase::setValue8s(
	char value, int x, int y, int band, int slice, int frame)
{
	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		1, 1, 0, 1, &value,
		mvIMPACT_C::IPL_DTCHAR))
	{ throwImpactError(); }
	return *this;
}

inline BufferBase& BufferBase::setValues8s(
	const std::vector<char>& values,
	int x, int y, int width, int height, int band, int slice, int frame)
{
	if (width <= 0)
		width = getWidth();
	if (height <= 0)
		height = getHeight();

	if (values.size() < (size_t)(width*height))
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);

	if(!mvIMPACT_C::IPL_PutBufDataEx (
		getHandle(),
		x, y, band, slice, frame,
		width, height, 0, width, &*values.begin(),
		mvIMPACT_C::IPL_DTCHAR))
	{ throwImpactError(); }

	return *this;
}

inline bool BufferBase::getHasReadAccess() const
{
	return access_->getHasPointer();
}

inline bool BufferBase::getHasReadWriteAccess() const
{
	return getHasReadAccess() && access_->getHasWriteAccess();
}

inline Size BufferBase::getSize() const
	{
		mvIMPACT_C::IPL_LONG size[2];
		if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_SIZE, size))
			throwImpactError();
		return Size(size[0], size[1]);
	}

inline int BufferBase::getWidth() const
	{
		mvIMPACT_C::IPL_LONG width = -1;
		if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_XSIZE, &width))
			throwImpactError();
		return width;
	}

inline int BufferBase::getHeight() const
	{
		mvIMPACT_C::IPL_LONG height = -1;
		if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_YSIZE, &height))
			throwImpactError();
		return height;
	}

inline double BufferBase::getDatatypeMinValue() const
	{
		mvIMPACT_C::IPL_DOUBLE min = 0;
		if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_MIN_GREY_VAL, &min))
			throwImpactError();
		return min;
	}

inline double BufferBase::getDatatypeMaxValue() const
	{
		mvIMPACT_C::IPL_DOUBLE max = 0;
		if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_MAX_GREY_VAL, &max))
			throwImpactError();
		return max;
	}


inline int BufferBase::getBands() const
{
	int b;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_NB_BANDS, &b))
		throwImpactError();
	return b;
}

inline BufferBase::BandsProxy BufferBase::getBands()
{
	return BufferBase::BandsProxy(*this);
}

inline BufferBase& BufferBase::setBands(int b)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_NB_BANDS, &b))
		throwImpactError();
	return *this;
}


inline int BufferBase::getSlices() const
{
	int s;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_NB_ZSLICES, &s))
		throwImpactError();
	return s;
}

inline BufferBase::SlicesProxy BufferBase::getSlices()
{
	return BufferBase::SlicesProxy(*this);
}

inline BufferBase& BufferBase::setSlices(int s)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_NB_ZSLICES, &s))
		throwImpactError();
	return *this;
}


inline int BufferBase::getFrames() const
{
	int f;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_NB_TSLICES, &f))
		throwImpactError();
	return f;
}

inline BufferBase::FramesProxy BufferBase::getFrames()
{
	return BufferBase::FramesProxy(*this);
}

inline BufferBase& BufferBase::setFrames(int f)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_NB_TSLICES, &f))
		throwImpactError();
	return *this;
}


inline Point BufferBase::getXYOffset() const
{
	mvIMPACT_C::IPL_LONG offset[2];
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_XYOFF, offset))
		throwImpactError();
	return Point(offset[0], offset[1]);
}

inline BufferBase::XYOffsetProxy BufferBase::getXYOffset()
{
	return BufferBase::XYOffsetProxy(*this);
}

inline BufferBase& BufferBase::setXYOffset(const Point& xyOffset)
{
	releaseAccess();
	mvIMPACT_C::IPL_LONG offset[2] = { xyOffset.getX(), xyOffset.getY() };
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_XYOFF, offset))
		throwImpactError();
	return *this;
}


inline int BufferBase::getXOffset() const
{
	int x;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_XOFF, &x))
		throwImpactError();
	return x;
}

inline BufferBase::XOffsetProxy BufferBase::getXOffset()
{
	return BufferBase::XOffsetProxy(*this);
}

inline BufferBase& BufferBase::setXOffset(int x)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_XOFF, &x))
		throwImpactError();
	return *this;
}


inline int BufferBase::getYOffset() const
{
	int y;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_YOFF, &y))
		throwImpactError();
	return y;
}

inline BufferBase::YOffsetProxy BufferBase::getYOffset() 
{
	return BufferBase::YOffsetProxy(*this);
}

inline BufferBase& BufferBase::setYOffset(int y)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_YOFF, &y))
		throwImpactError();
	return *this;
}


inline int BufferBase::getBandsOffset() const
{
	int b;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BDOFF, &b))
		throwImpactError();
	return b;
}

inline BufferBase::BandsOffsetProxy BufferBase::getBandsOffset()
{
	return BufferBase::BandsOffsetProxy(*this);
}

inline BufferBase& BufferBase::setBandsOffset(int b)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BDOFF, &b))
		throwImpactError();
	return *this;
}


inline int BufferBase::getSlicesOffset() const
{
	int s;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_ZOFF, &s))
		throwImpactError();
	return s;
}

inline BufferBase::SlicesOffsetProxy BufferBase::getSlicesOffset()
{
	return BufferBase::SlicesOffsetProxy(*this);
}

inline BufferBase& BufferBase::setSlicesOffset(int s)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_ZOFF, &s))
		throwImpactError();
	return *this;
}


inline int BufferBase::getFramesOffset() const
{
	int f;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_TOFF, &f))
		throwImpactError();
	return f;
}

inline BufferBase::FramesOffsetProxy BufferBase::getFramesOffset()
{
	return BufferBase::FramesOffsetProxy(*this);
}

inline BufferBase& BufferBase::setFramesOffset(int f)
{
	releaseAccess();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_TOFF, &f))
		throwImpactError();
	return *this;
}


inline ColorSpace BufferBase::getColorSpace() const
{
	ColorSpace f;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_COLOR_SPACE, (mvIMPACT_C::IPL_COL_SPACE*)&f))
		throwImpactError();
	return f;
}

inline BufferBase::ColorSpaceProxy BufferBase::getColorSpace()
{
	return BufferBase::ColorSpaceProxy(*this);
}

inline BufferBase& BufferBase::setColorSpace(ColorSpace f)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_COLOR_SPACE, (mvIMPACT_C::IPL_COL_SPACE*)&f))
		throwImpactError();
	return *this;
}


inline void BufferBase::whiteBalance()
{
	if (!mvIMPACT_C::IPL_BufWhiteBalance(getHandle()))
		throwImpactError();
}


inline double BufferBase::getBayerMosaicRedGain() const
{
	double v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_RED_GAIN, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicRedGainProxy BufferBase::getBayerMosaicRedGain()
{
	return BufferBase::BayerMosaicRedGainProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicRedGain(double v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_RED_GAIN, &v))
		throwImpactError();
	return *this;
}


inline double BufferBase::getBayerMosaicRedOffset() const
{
	double v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_RED_OFFSET, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicRedOffsetProxy BufferBase::getBayerMosaicRedOffset()
{
	return BufferBase::BayerMosaicRedOffsetProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicRedOffset(double v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_RED_OFFSET, &v))
		throwImpactError();
	return *this;
}


inline double BufferBase::getBayerMosaicGreenGain() const
{
	double v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_GREEN_GAIN, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicGreenGainProxy BufferBase::getBayerMosaicGreenGain()
{
	return BufferBase::BayerMosaicGreenGainProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicGreenGain(double v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_GREEN_GAIN, &v))
		throwImpactError();
	return *this;
}


inline double BufferBase::getBayerMosaicGreenOffset() const
{
	double v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_GREEN_OFFSET, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicGreenOffsetProxy BufferBase::getBayerMosaicGreenOffset()
{
	return BufferBase::BayerMosaicGreenOffsetProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicGreenOffset(double v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_GREEN_OFFSET, &v))
		throwImpactError();
	return *this;
}


inline double BufferBase::getBayerMosaicBlueGain() const
{
	double v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_BLUE_GAIN, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicBlueGainProxy BufferBase::getBayerMosaicBlueGain()
{
	return BufferBase::BayerMosaicBlueGainProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicBlueGain(double v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_BLUE_GAIN, &v))
		throwImpactError();
	return *this;
}


inline double BufferBase::getBayerMosaicBlueOffset() const
{
	double v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_BLUE_OFFSET, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicBlueOffsetProxy BufferBase::getBayerMosaicBlueOffset()
{
	return BufferBase::BayerMosaicBlueOffsetProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicBlueOffset(double v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_BLUE_OFFSET, &v))
		throwImpactError();
	return *this;
}


inline double BufferBase::getBayerMosaicDigitalGain() const
{
	double v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_DIGITAL_GAIN, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicDigitalGainProxy BufferBase::getBayerMosaicDigitalGain()
{
	return BufferBase::BayerMosaicDigitalGainProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicDigitalGain(double v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_DIGITAL_GAIN, &v))
		throwImpactError();
	return *this;
}


inline int BufferBase::getBayerMosaicParity() const
{
	int v;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BAYER_PARITY, &v))
		throwImpactError();
	return v;
}

inline BufferBase::BayerMosaicParityProxy BufferBase::getBayerMosaicParity()
{
	return BufferBase::BayerMosaicParityProxy(*this);
}

inline BufferBase& BufferBase::setBayerMosaicParity(int v)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BAYER_PARITY, &v))
		throwImpactError();
	return *this;
}


inline bool BufferBase::getIsThreadSafe() const
{
	mvIMPACT_C::IPL_LONG thread_safe = 0;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_THREAD_SAFE, &thread_safe))
		throwImpactError();
	return thread_safe ? true : false;
}

inline BufferBase::IsThreadSafeProxy BufferBase::getIsThreadSafe()
{
	return BufferBase::IsThreadSafeProxy(*this);
}

inline BufferBase& BufferBase::setIsThreadSafe(bool ts)
{
	mvIMPACT_C::IPL_LONG thread_safe = ts ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_THREAD_SAFE, &thread_safe))
		throwImpactError();
	return *this;
}


inline BufferDatatype BufferBase::getDatatype() const
{
	BufferDatatype dt;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_DATATYPE, &dt))
		throwImpactError();
	return dt;
}

inline BufferBase::DatatypeProxy BufferBase::getDatatype()
{
	return BufferBase::DatatypeProxy(*this);
}

inline BufferBase& BufferBase::setDatatype(BufferDatatype dt)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_DATATYPE, &dt))
		throwImpactError();
	return *this;
}


inline int BufferBase::getBitsPerSample() const
{
	int bps;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_BITSPERSAMPLE, &bps))
		throwImpactError();
	return bps;
}

inline BufferBase::BitsPerSampleProxy BufferBase::getBitsPerSample()
{
	return BufferBase::BitsPerSampleProxy(*this);
}

inline BufferBase& BufferBase::setBitsPerSample(int bps)
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_BITSPERSAMPLE, &bps))
		throwImpactError();
	return *this;
}


inline Calibrator BufferBase::getXCalibrator() const
{
	Calibrator cal = Calibrator(getWidth());
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_CALIBRATE_X_SIZE, cal.m_))
		throwImpactError();
	return cal;
}

inline BufferBase::XCalibratorProxy BufferBase::getXCalibrator()
{
	return BufferBase::XCalibratorProxy(*this);
}

inline BufferBase& BufferBase::setXCalibrator( const Calibrator& cal )
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_CALIBRATE_X_SIZE, cal.m_))
		throwImpactError();
	return *this;
}


inline Calibrator BufferBase::getYCalibrator() const
{
	Calibrator cal = Calibrator(getHeight());
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_CALIBRATE_Y_SIZE, cal.m_))
		throwImpactError();
	return cal;
}

inline BufferBase::YCalibratorProxy BufferBase::getYCalibrator()
{
	return BufferBase::YCalibratorProxy(*this);
}

inline BufferBase& BufferBase::setYCalibrator( const Calibrator& cal )
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_CALIBRATE_Y_SIZE, cal.m_))
		throwImpactError();
	return *this;
}


inline Calibrator BufferBase::getSlicesCalibrator() const
{
	Calibrator cal = Calibrator(getSlices());
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_CALIBRATE_Z_SLICES, cal.m_))
		throwImpactError();
	return cal;
}

inline BufferBase::SlicesCalibratorProxy BufferBase::getSlicesCalibrator()
{
	return BufferBase::SlicesCalibratorProxy(*this);
}

inline BufferBase& BufferBase::setSlicesCalibrator( const Calibrator& cal )
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_CALIBRATE_Z_SLICES, cal.m_))
		throwImpactError();
	return *this;
}


inline Calibrator BufferBase::getFramesCalibrator() const
{
	Calibrator cal = Calibrator(getFrames());
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_CALIBRATE_TIME_SLICES, cal.m_))
		throwImpactError();
	return cal;
}

inline BufferBase::FramesCalibratorProxy BufferBase::getFramesCalibrator()
{
	return BufferBase::FramesCalibratorProxy(*this);
}

inline BufferBase& BufferBase::setFramesCalibrator( const Calibrator& cal )
{
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_CALIBRATE_TIME_SLICES, cal.m_))
		throwImpactError();
	return *this;
}


inline mvIMPACT_C::IPL_BUFHANDLE BufferBase::getHandle  () const { return handle_; }

inline int BufferBase::getUseCount() const { return *counter_; }

inline bool BufferBase::getIsUnique   () const {	return 1 == *counter_; 	}

inline bool BufferBase::getIsAssigned() const { return (-1 != getHandle()); }

inline const BufferBase* BufferBase::getParentBuffer() const
{
	return parent_;
}

inline bool BufferBase::getIsChild() const
{
	// assert that the buffer handle is a child, too:
	assert((parent_ != NULL) == getChildID());

	return parent_ != NULL;
}

inline bool BufferBase::getIsParent() const
{
	return !getIsChild();
}

inline BufferLock BufferBase::readLock() const
{
	return BufferLock(this, BufferLock::lmRead);
}

inline BufferLock BufferBase::readWriteLock()
{
	return BufferLock(this, BufferLock::lmReadWrite);
}

inline BufferAttribute BufferBase::getAttribute() const
{
	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	return BufferAttribute(att);
}


inline bool BufferBase::getChildID() const
{
	mvIMPACT_C::IPL_LONG child_id = -1;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_CHILDID, &child_id))
		throwImpactError();
	return child_id ? true : false ;
}

inline int BufferBase::getDatatypeSize() const
{
	mvIMPACT_C::IPL_LONG datatype_size = -1;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_DATATYPESIZE, &datatype_size))
		throwImpactError();
	return (int)datatype_size;
}

inline bool BufferBase::getIsHorizontalCalibrated() const
{
	mvIMPACT_C::IPL_BOOL calibrated = IPL_FALSE;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_ISCALIBRATED_X_SIZE, &calibrated))
		throwImpactError();
	return calibrated ? true : false;
}

inline bool BufferBase::getIsVerticalCalibrated() const
{
	mvIMPACT_C::IPL_BOOL calibrated = IPL_FALSE;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_ISCALIBRATED_Y_SIZE, &calibrated))
		throwImpactError();
	return calibrated ? true : false;
}

inline bool BufferBase::getIsSlicesCalibrated() const
{
	mvIMPACT_C::IPL_BOOL calibrated = IPL_FALSE;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_ISCALIBRATED_Z_SLICES, &calibrated))
		throwImpactError();
	return calibrated ? true : false;
}

inline bool BufferBase::getIsFramesCalibrated() const
{
	mvIMPACT_C::IPL_BOOL calibrated = IPL_FALSE;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_ISCALIBRATED_TIME_SLICES, &calibrated))
		throwImpactError();
	return calibrated ? true : false;
}

inline double BufferBase::getSequenceDuration() const
{
	mvIMPACT_C::IPL_DOUBLE duration = 0;
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_SEQUENCE_DURATION, &duration))
		throwImpactError();
	return (double)duration;
}

#pragma warning(push)
#pragma warning(disable : 4717)

	PROXY_CLASS_SIMPLE_IMPL(Bands, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(Slices, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(Frames, int, BufferBase)
	PROXY_CLASS_POINT_IMPL(XYOffset, Point, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(XOffset, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(YOffset, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BandsOffset, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(SlicesOffset, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(FramesOffset, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(ColorSpace, ColorSpace, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicRedGain, double, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicRedOffset, double, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicGreenGain, double, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicGreenOffset, double, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicBlueGain, double, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicBlueOffset, double, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicDigitalGain, double, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BayerMosaicParity, int, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(IsThreadSafe, bool, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(Datatype, BufferDatatype, BufferBase)
	PROXY_CLASS_SIMPLE_IMPL(BitsPerSample, int, BufferBase)
	PROXY_CLASS_CALIBRATOR_IMPL(XCalibrator, Calibrator, BufferBase)
	PROXY_CLASS_CALIBRATOR_IMPL(YCalibrator, Calibrator, BufferBase)
	PROXY_CLASS_CALIBRATOR_IMPL(SlicesCalibrator, Calibrator, BufferBase)
	PROXY_CLASS_CALIBRATOR_IMPL(FramesCalibrator, Calibrator, BufferBase)

#pragma warning(pop)

inline BufferBase::BufferBase()
	: handle_(unassigned), counter_(new int(1)), parent_(0), access_(new MyBufferAccess()), mustCleanup_(true)
{
}

inline BufferBase::BufferBase(
	int width,
	int height,
	int bands,
	int slices,
	int frames,
	BufferDatatype dt,
	BufferAttribute bt)
	: handle_(unassigned), counter_(new int(1)), parent_(0), access_(new MyBufferAccess()), mustCleanup_(true)
{
	assert(unassigned == getHandle());
	if (!mvIMPACT_C::IPL_BufAlloc(
			&handle_,
			width, height, bands, slices, frames,
			mvIMPACT_C::IPL_BUFFER_DATATYPE(dt),
			mvIMPACT_C::IPL_BUFFER_ATTRIBUTE(bt),
			NULL))
		throwImpactError();
	assert(unassigned != getHandle());
}

inline BufferBase::BufferBase(
	int width,
	int height,
	int bands,
	int slices,
	int frames,
	BufferDatatype dt,
	BufferAttribute bt,
	void * ptr)
	: handle_(unassigned), counter_(new int(1)), parent_(0), access_(new MyBufferAccess()), mustCleanup_(true)
{
	assert(unassigned == getHandle());

	mvIMPACT_C::IPL_DEVICEINFO info;
	info.DevId = IPL_DONT_CARE;
	info.Flags = mvIMPACT_C::IPL_USER_MEM;
	info.IdentStr = (mvIMPACT_C::IPL_CHAR *)ptr;

	if (!mvIMPACT_C::IPL_BufAlloc(
		&handle_,
		width, height, bands, slices, frames,
		mvIMPACT_C::IPL_BUFFER_DATATYPE(dt),
		mvIMPACT_C::IPL_BUFFER_ATTRIBUTE(bt),
		&info))
		throwImpactError();

	assert(unassigned != getHandle());
}

inline BufferBase::BufferBase( const std::string& xml )
	: handle_(unassigned), counter_(new int(1)), parent_(0), access_(new MyBufferAccess()), mustCleanup_(true)
{
	assert(unassigned == getHandle());
	if (!mvIMPACT_C::IPL_BufAllocXML(&handle_, xml.c_str(), -1, 0, -1))
		throwImpactError();
	assert(unassigned != getHandle());
}

inline BufferBase::BufferBase(const BufferBase& parent, const std::string& xml)
	: handle_(unassigned), counter_(new int(1)), parent_(parent.makeShallowCopy()),
	  access_(new MyBufferAccess()), mustCleanup_(true)
{
	assert(unassigned == getHandle());
	if (!mvIMPACT_C::IPL_BufAllocXML(&handle_, xml.c_str(), parent.getHandle(), 0, -1))
		throwImpactError();
	assert(unassigned != getHandle());
}

inline BufferBase::BufferBase(const BufferBase& parent,
	int xOffset, int yOffset, int bOffset, int sOffset, int fOffset,
	int width, int height, int bands, int slices, int frames)
	: handle_(unassigned), counter_(new int(1)), parent_(parent.makeShallowCopy()),
	  access_(new MyBufferAccess()), mustCleanup_(true)
{
	assert(unassigned == getHandle());
	if (!mvIMPACT_C::IPL_BufAllocChild(
			&handle_, parent.getHandle(),
			xOffset, yOffset, bOffset, sOffset, fOffset,
			width, height, bands, slices, frames))
		throwImpactError();
	assert(unassigned != getHandle());
}

inline BufferBase::BufferBase(const BufferBase& parent,
	const Coord &offset,
	int width, int height, int bands, int slices, int frames)
	: handle_(unassigned), counter_(new int(1)), parent_(parent.makeShallowCopy()),
	  access_(new MyBufferAccess()), mustCleanup_(true)
{
	assert(unassigned == getHandle());
	if (!mvIMPACT_C::IPL_BufAllocChild(
			&handle_, parent.getHandle(),
			offset.getX(), offset.getY(), offset.getBand(),
			offset.getSlice(), offset.getFrame(),
			width, height, bands, slices, frames))
		throwImpactError();
	assert(unassigned != getHandle());
}


inline BufferBase::BufferBase(const BufferBase& r)
{
	copy(r);
}

inline BufferBase::BufferBase(mvIMPACT_C::IPL_BUFHANDLE bufhandle, bool mustCleanup)
	: handle_(bufhandle), counter_(new int(1)), parent_(0), access_(new MyBufferAccess()), mustCleanup_(mustCleanup)
{
}

inline BufferBase& BufferBase::operator=(const BufferBase& r)
{
	if (&r != this)
	{
		dispose();
		copy(r);
	}
	return *this;
}

inline BufferBase::~BufferBase()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		assert(!"Caught exception in BufferBase::~BufferBase() destructor");
	}
}


inline void BufferBase::translate(int x, int y, int width, int height)
	{
		if (!mvIMPACT_C::IPL_BufTranslateChild(
				getHandle(), x, y, width, height))
		{ throwImpactError(); }
	}

inline std::string BufferBase::asXml(bool as_parent , bool compressed ) const
	{
		char * xml = 0;
		if (!mvIMPACT_C::IPL_StoreXML(
			getHandle(),
			compressed ? mvIMPACT_C::IPL_LZW_COMPRESSION : mvIMPACT_C::IPL_NO_COMPRESSION,
			as_parent ? mvIMPACT_C::IPL_SAVE_DATA : mvIMPACT_C::IPL_DONT_SAVE_DATA,
			&xml))
			throwImpactError();
		std::string xmls(xml);
		if (!mvIMPACT_C::IPL_FreeXML(xml))
			throwImpactError();
		return xmls;
	}

inline void BufferBase::freeDim(
		DimensionSpec dim,
		const std::vector<int>& offsets,
		LocationSpec loc)
	{
		DynArray<mvIMPACT_C::IPL_ULONG, int> OffsetArray(offsets);
		if (!mvIMPACT_C::IPL_BufFreeDim(
				&handle_,
				mvIMPACT_C::IPL_DIMENSION(dim),
				OffsetArray.getPointer(),
				static_cast<mvIMPACT_C::IPL_ULONG>(offsets.size()),
				mvIMPACT_C::IPL_LOCATION(loc)))
		{ throwImpactError(); }
	}


inline void BufferBase::dispose()
{
	if (0 == --*counter_)
	{
		delete access_;
		if (unassigned != getHandle())
		{
			if(mustCleanup_) {
				 if (!mvIMPACT_C::IPL_BufFree(getHandle()))	 { throwImpactError(); }
			}
			handle_= unassigned;
		}
		delete counter_;
		delete parent_; // delete on a (potential) NULL pointer is OK
	}
}


inline void BufferBase::copy(const BufferBase& src)
{
	handle_ = src.handle_;
	counter_ = src.counter_;
	parent_ = src.parent_;
	access_ = src.access_;
	mustCleanup_ = src.mustCleanup_;

	(*counter_)++;
}


inline int BufferBase::getMin(int band, int slice, int frame) const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdULong);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MIN_VALUE))
	{ throwImpactError(); }

	return (int)resultBuf.getIntValue(0, band, slice, frame);
}

inline std::vector<int> BufferBase::getAllMinValues() const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdULong);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MIN_VALUE))
	{ throwImpactError(); }

	std::vector<int> results;

	for (int frame=0; frame<getFrames(); ++frame)
		for (int slice=0; slice<getSlices(); ++slice)
			for (int band=0; band<getBands(); ++band)
				results.push_back((int)resultBuf.getIntValue(0, band, slice, frame));

	return results;
}

inline int BufferBase::getMax(int band, int slice, int frame) const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdULong);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MAX_VALUE))
	{ throwImpactError(); }

	return (int)resultBuf.getIntValue(0, band, slice, frame);
}

inline std::vector<int> BufferBase::getAllMaxValues() const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdULong);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MAX_VALUE))
	{ throwImpactError(); }

	std::vector<int> results;

	for (int frame=0; frame<getFrames(); ++frame)
		for (int slice=0; slice<getSlices(); ++slice)
			for (int band=0; band<getBands(); ++band)
				results.push_back((int)resultBuf.getIntValue(0, band, slice, frame));

	return results;
}

inline std::pair<int,int> BufferBase::getMinMax(int band, int slice, int frame) const
{
	ResultBuf resultBuf(2, getBands(), getSlices(), getFrames(), bdULong);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MIN_AND_MAX_VALUE))
	{ throwImpactError(); }

	return std::pair<int,int>(
		(int)resultBuf.getIntValue(0, band, slice, frame),
		(int)resultBuf.getIntValue(1, band, slice, frame)
		);
}

inline std::vector<int> BufferBase::getAllMinMaxValues() const
{
	ResultBuf resultBuf(2, getBands(), getSlices(), getFrames(), bdULong);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MIN_AND_MAX_VALUE))
	{ throwImpactError(); }

	std::vector<int> results;

	for (int frame=0; frame<getFrames(); ++frame)
		for (int slice=0; slice<getSlices(); ++slice)
			for (int band=0; band<getBands(); ++band)
			{
				results.push_back((int)resultBuf.getIntValue(0, band, slice, frame));
				results.push_back((int)resultBuf.getIntValue(1, band, slice, frame));
			}

	return results;
}

inline double BufferBase::getMean(int band, int slice, int frame) const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdDouble);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MEAN_GRAY_VALUE))
	{ throwImpactError(); }

	return resultBuf.getDoubleValue(0, band, slice, frame);
}

inline std::vector<double> BufferBase::getAllMeanValues() const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdDouble);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_MEAN_GRAY_VALUE))
	{ throwImpactError(); }

	std::vector<double> results;

	for (int frame=0; frame<getFrames(); ++frame)
		for (int slice=0; slice<getSlices(); ++slice)
			for (int band=0; band<getBands(); ++band)
				results.push_back(resultBuf.getDoubleValue(0, band, slice, frame));

	return results;
}

inline double BufferBase::getStdDev(int band, int slice, int frame) const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdDouble);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_STANDARD_DEVIATION))
	{ throwImpactError(); }

	return resultBuf.getDoubleValue(0, band, slice, frame);
}

inline std::vector<double> BufferBase::getAllStdDevValues() const
{
	ResultBuf resultBuf(1, getBands(), getSlices(), getFrames(), bdDouble);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		getHandle(),
		IPL_DONT_CARE,
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_STANDARD_DEVIATION))
	{ throwImpactError(); }

	std::vector<double> results;

	for (int frame=0; frame<getFrames(); ++frame)
		for (int slice=0; slice<getSlices(); ++slice)
			for (int band=0; band<getBands(); ++band)
				results.push_back(resultBuf.getDoubleValue(0, band, slice, frame));

	return results;
}

DEFINE_USAGE_COUNTER(ResultBuf);

inline ResultBuf::ResultBuf(int width, int bands, int slices, int frames, BufferDatatype dt)
: BufferBase(width, 1, bands, slices, frames, dt, baResult)
{
	INC_USAGE_COUNTER;
}

inline ResultBuf::~ResultBuf()
{
	DEC_USAGE_COUNTER
}

inline double ResultBuf::getDoubleValue(int index, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_DOUBLE result = 0.;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		index,0,band,slice,frame,
		1,1,
		0,1,
		&result,
		mvIMPACT_C::IPL_DTDOUBLE))
	{ throwImpactError(); }

	return result;
}


inline int ResultBuf::getIntValue(int index, int band, int slice, int frame) const
{
	mvIMPACT_C::IPL_ULONG result = 0;
	if(!mvIMPACT_C::IPL_GetBufDataEx(
		getHandle(),
		index,0,band,slice,frame,
		1,1,
		0,1,
		&result,
		mvIMPACT_C::IPL_DTULONG))
	{ throwImpactError(); }

	return result;
}


inline BufferBase* ResultBuf::makeShallowCopy() const
{
	return new ResultBuf(*this);
}

inline ResultBuf::ResultBuf(const ResultBuf &resultBuf)
	: BufferBase(resultBuf)
{
	INC_USAGE_COUNTER;
}

DEFINE_USAGE_COUNTER(Image);

inline Image::Image()
	: BufferBase()
{
	INC_USAGE_COUNTER;
}

inline Image::Image(
	int width,
	int height,
	int bands,
	BufferDatatype dt)
	: BufferBase(width, height, bands, 1, 1, dt, baImage)
{
	INC_USAGE_COUNTER;
}

inline Image::Image(
	int width,
	int height,
	int bands,
	int slices,
	int frames,
	BufferDatatype dt)
	: BufferBase(width, height, bands, slices, frames, dt, baImage)
{
	INC_USAGE_COUNTER;
}

inline Image::Image(
	int width,
	int height,
	int bands,
	int slices,
	int frames,
	BufferDatatype dt,
	void * ptr)
	: BufferBase(width, height, bands, slices, frames, dt, baImage, ptr)
{
	INC_USAGE_COUNTER;
}



inline Image::Image(mvIMPACT_C::IPL_BUFHANDLE bufhandle, bool mustCleanup)
		: BufferBase(bufhandle, mustCleanup)
{
	INC_USAGE_COUNTER;
	if( getAttribute() != baImage && getAttribute() != baBayerMosaic && getAttribute() != baRle )
	{
		throwImpactException("Cannot create an Image object from a BufferBase not being of type Image");
	}
}

inline Image::Image(const std::string& xml)
		: BufferBase(xml)
{
	INC_USAGE_COUNTER;
}

inline Image::Image
	(
		const Image& parent,
		const std::string& xml
	)
		: BufferBase(parent, xml)
{
	INC_USAGE_COUNTER;
}


inline Image::Image(const Image& image)
	: BufferBase(image)
{
	INC_USAGE_COUNTER;
}

inline const Image* Image::getParentBuffer() const
	{
		// the parent buffer must be an Image:
		assert( ( baImage == BufferBase::getParentBuffer()->getAttribute() ) ||
				( baBayerMosaic == BufferBase::getParentBuffer()->getAttribute() ) ||
				( baRle == BufferBase::getParentBuffer()->getAttribute() ) );

		return reinterpret_cast< const Image* >(BufferBase::getParentBuffer());
	}


inline Image::~Image()
{
	DEC_USAGE_COUNTER
}

inline Image::Image
	(
		const Image& parent,
		int xOffset,
		int yOffset,
		int width,
		int height
	)
		: BufferBase(parent, xOffset, yOffset, 0, 0, 0, width, height, 1, 1, 1)
	{ INC_USAGE_COUNTER;}

inline Image::Image
	(
		const Image& parent,
		int xOffset,
		int yOffset,
		int bOffset,
		int width,
		int height,
		int bands
	)
		: BufferBase(parent, xOffset, yOffset, bOffset, 0, 0, width, height, bands, 1, 1)
	{ INC_USAGE_COUNTER;}

inline Image::Image
	(
		const Image& parent,
		int xOffset,
		int yOffset,
		int bOffset,
		int sOffset,
		int fOffset,
		int width,
		int height,
		int bands,
		int slices,
		int frames
	)
		: BufferBase(parent, xOffset, yOffset, bOffset, sOffset, fOffset,
		width, height, bands, slices, frames)
	{ INC_USAGE_COUNTER;}

inline Image::Image
	(
		const Image& parent,
		const Coord &offset,
		int width,
		int height,
		int bands,
		int slices,
		int frames
	)
		: BufferBase(parent, offset, width, height, bands, slices, frames)
	{ INC_USAGE_COUNTER;}


inline Image& Image::operator=(const Image& r)
	{
		if (this != &r)	{ BufferBase::operator=(r); }
		return *this;
	}

inline Image Image::clone() const
	{
		if(getIsChild()) {
			return Image(
				*(getParentBuffer()),
				getXOffset(), getYOffset(),
				getBandsOffset(), getSlicesOffset(), getFramesOffset(),
				getWidth(), getHeight(),
				getBands(), getSlices(), getFrames());
		} else {
			return mvIMPACT::copy(*this);
		}
	}

inline Image Image::copyAsImage() const
	{
		return mvIMPACT::copy(*this);
	}

inline BufferBase* Image::makeShallowCopy() const
{
	return new Image(*this);
}

inline void Image::drawPolygon
(
	const std::vector<Point>& polygon,
	int colorIndex
)
{
	DynArray<mvIMPACT_C::IPL_POINT, Point> iplPolygon(polygon);

	if (!mvIMPACT_C::IPL_GrDrawPolygon(
			getHandle(),
			mvIMPACT_C::IPL_OUTLINE,
			colorIndex,
			iplPolygon.getPointer(),
			static_cast<mvIMPACT_C::IPL_ULONG>(polygon.size())))
		throwImpactError();
}

inline void Image::drawFilledPolygon
(
	const std::vector<Point>& polygon,
	int colorIndex
)
{
	DynArray<mvIMPACT_C::IPL_POINT, Point> iplPolygon(polygon);
	if (!mvIMPACT_C::IPL_GrDrawPolygon(
			getHandle(),
			mvIMPACT_C::IPL_FILL,
			colorIndex,
			iplPolygon.getPointer(),
			static_cast<mvIMPACT_C::IPL_ULONG>(polygon.size())))
		throwImpactError();
}


inline void Image::drawEllipse
(
    const Point& center,
	int major,
	int minor,
	int colorIndex
	)
{
	if (!mvIMPACT_C::IPL_GrDrawEllipse(
			getHandle(),
			center.getX(),
			center.getY(),
			major,
			minor,
			colorIndex,
			IPL_DONT_CARE))
		throwImpactError();
}


inline void Image::drawFilledEllipse
(
	const Point& center,
	int major,
	int minor,
	int colorIndex
	)
{
	if (!mvIMPACT_C::IPL_GrDrawEllipse(
			getHandle(),
			center.getX(),
			center.getY(),
			major,
			minor,
			colorIndex,
			colorIndex))
		throwImpactError();
}


inline void Image::drawRectangle
(
	const Point& topleft,
	const Point& bottomright,
	int colorIndex
)
{
	std::vector<Point> points(5);
	points[0]   = topleft;
	points[1].setX( bottomright.getX() ); // topright
	points[1].setY( topleft.getY() );
	points[2]   = bottomright;
	points[3].setX( topleft.getX() );
	points[3].setY( bottomright.getY() );  // bottomleft
	points[4]   = topleft;

	drawPolygon(
				points,
				colorIndex);
}

inline void Image::drawRectangle(const Rectangle& rectangle, int colorIndex)
{
	std::vector<Point> points;
	points.reserve(5);

	points.push_back(Point(rectangle.getLeft(),rectangle.getTop()));
	points.push_back(Point(rectangle.getRight(),rectangle.getTop()));
	points.push_back(Point(rectangle.getRight(),rectangle.getBottom()));
	points.push_back(Point(rectangle.getLeft(),rectangle.getBottom()));
	points.push_back(points.at(0));

	drawPolygon(points, colorIndex);
}

inline void Image::drawLine(const Line& line,int colorIndex)
{
	std::vector<Point> points;
	points.reserve(2);

	points.push_back(line.getP0());
	points.push_back(line.getP1());

	drawPolygon(points, colorIndex);
}

inline void Image::drawLine(const Point& first,const Point& second,int colorIndex)
{
	std::vector<Point> points;
	points.reserve(2);

	points.push_back(first);
	points.push_back(second);

	drawPolygon(points, colorIndex);
}

inline void Image::drawPoint(const Point& point, int colorIndex)
{
	// do the same as IPL_GrDrawPolygon: draw in all planes
	for(int b = 0; b < getBands(); b++) {
		for(int s = 0; s < getSlices(); s++) {
			for(int f = 0; f < getFrames(); f++) {
				setPixel(point.getX(), point.getY(), b, s, f, colorIndex);
			}
		}
	}
}

inline int Image::getMin(int band, int slice, int frame) const
{
	return BufferBase::getMin(band, slice, frame);
}

inline std::vector<int> Image::getAllMinValues() const
{
	return BufferBase::getAllMinValues();
}

inline int Image::getMax(int band, int slice, int frame) const
{
	return BufferBase::getMax(band, slice, frame);
}

inline std::vector<int> Image::getAllMaxValues() const
{
	return BufferBase::getAllMaxValues();
}

inline std::pair<int,int> Image::getMinMax(int band, int slice, int frame) const
{
	return BufferBase::getMinMax(band, slice, frame);
}

inline std::vector<int> Image::getAllMinMaxValues() const
{
	return BufferBase::getAllMinMaxValues();
}

inline double Image::getMean(int band, int slice, int frame) const
{
	return BufferBase::getMean(band, slice, frame);
}

inline std::vector<double> Image::getAllMeanValues() const
{
	return BufferBase::getAllMeanValues();
}

inline double Image::getStdDev(int band, int slice, int frame) const
{
	return BufferBase::getStdDev(band, slice, frame);
}

inline std::vector<double> Image::getAllStdDevValues() const
{
	return BufferBase::getAllStdDevValues();
}

inline int Image::getPixel(int x, int y, int band, int slice, int frame) const
{
	return BufferBase::getPixel(x,y,band,slice,frame);
}

inline int Image::getPixel(int x, int y) const
{
	return BufferBase::getPixel(x,y);
}

inline void Image::setPixel(int x, int y, int band, int slice, int frame, int value)
{
	BufferBase::setPixel(x,y,band,slice,frame,value);
}

inline void Image::setPixel(int x, int y, int value)
{
	BufferBase::setPixel(x,y,value);
}

inline void Image::convertToBitmap(
	void * bitmap,
	unsigned int shift) const
{
	if (!mvIMPACT_C::IPL_GetDispData(
			getHandle(),
			IPL_DONT_CARE,
			0, 0, getWidth(), getHeight(),
			0, 0,
			3 == getBands() ? mvIMPACT_C::IPL_RGB : mvIMPACT_C::IPL_MONO,
			mvIMPACT_C::IPL_NOT_INVERTED,
			0, 1, 2, 0, 1, 2,
			IPL_TRUE, IPL_TRUE, IPL_TRUE,
			shift,
			(mvIMPACT_C::IPL_VOID_PTR)bitmap))
		throwImpactError();
}

inline void Image::convertToBitmap(
	void * bitmap,
	const Palette& palette) const
{
	if (!mvIMPACT_C::IPL_GetDispData(
			getHandle(),
			palette.getHandle(),
			0, 0, getWidth(), getHeight(),
			0, 0,
			3 == getBands() ? mvIMPACT_C::IPL_RGB : mvIMPACT_C::IPL_MONO,
			mvIMPACT_C::IPL_NOT_INVERTED,
			0, 1, 2, 0, 1, 2,
			IPL_TRUE, IPL_TRUE, IPL_TRUE,
			0,
			(mvIMPACT_C::IPL_VOID_PTR)bitmap))
		throwImpactError();
}


DEFINE_USAGE_COUNTER(Palette);

inline Palette::Palette()
	: BufferBase()
{ INC_USAGE_COUNTER; }

inline Palette::Palette(int width, int bands, BufferDatatype dt)
	: BufferBase(width, 1, bands, 1, 1, dt, baLut)
{ INC_USAGE_COUNTER;}

inline Palette::Palette(int width, int bands, int slices, int frames, BufferDatatype dt)
	: BufferBase(width, 1, bands, slices, frames, dt, baLut)
{ INC_USAGE_COUNTER; }

inline Palette::Palette(int width, int bands, int slices, int frames, BufferDatatype dt, void * ptr)
	: BufferBase(width, 1, bands, slices, frames, dt, baLut, ptr)
{ INC_USAGE_COUNTER; }

inline Palette::Palette(const Palette& parent, int xOffset, int bOffset, int sOffset, int fOffset, int width, int bands, int slices, int frames)
	: BufferBase(parent, xOffset, 0, bOffset, sOffset, fOffset, width, 1, bands, slices, frames)
{ INC_USAGE_COUNTER; }

inline Palette::Palette(const Palette& parent, const Coord &offset, int width, int bands, int slices, int frames)
	: BufferBase(parent, offset.getX(), 0, offset.getBand(), offset.getSlice(), offset.getFrame(), width, 1, bands, slices, frames)
{ INC_USAGE_COUNTER; }

inline const Palette* Palette::getParentBuffer() const
	{ return reinterpret_cast< const Palette* >(BufferBase::getParentBuffer()); }

inline Palette::Palette(mvIMPACT_C::IPL_BUFHANDLE bufhandle)
	: BufferBase(bufhandle)
{
	INC_USAGE_COUNTER;
	if (getAttribute() != baLut)
	{
		throwImpactException("Cannot create a Palette object from a BufferBase not being of type Palette");
	}
}

inline Palette::Palette(const Palette& parent, const std::string& xml)
	: BufferBase(parent, xml)
{ INC_USAGE_COUNTER; }




inline Palette::Palette(const std::string& xml)
	: BufferBase(xml)
{ INC_USAGE_COUNTER;}


inline Palette::Palette(const Palette& palette)
	: BufferBase(palette)
{ INC_USAGE_COUNTER;}


inline Palette& Palette::operator=(const Palette& r)
{
	if (this != &r)
		BufferBase::operator=(r);
	return *this;
}


inline Palette Palette::clone() const
{
	if(getIsChild()) {
		return Palette(
			*(getParentBuffer()),
			getXOffset(),
			getBandsOffset(), getSlicesOffset(), getFramesOffset(),
			getWidth(),
			getBands(), getSlices(), getFrames());
	} else {
		return mvIMPACT::copy(*this);
	}
}

inline Palette::~Palette()
{ DEC_USAGE_COUNTER }

inline Palette::Palette(const Palette& parent, int bOffset, int bands)
	: BufferBase(parent, 0, 0, bOffset, 0, 0, parent.getWidth(), 1, bands, 1, 1)
{ INC_USAGE_COUNTER;}



inline Palette Palette::copyAsPalette() const
{
	return mvIMPACT::copy(*this);
}


inline BufferBase* Palette::makeShallowCopy() const
{
	return new Palette(*this);
}


inline int Palette::getEntry(int index, int band, int slice, int frame) const
{
	return BufferBase::getPixel(index,0,band,slice,frame);
}

inline int Palette::getEntry(int index) const
{
	return BufferBase::getPixel(index,0);
}

inline void Palette::setEntry(int index, int band, int slice, int frame, int value)
{
	BufferBase::setPixel(index,0,band,slice,frame,value);
}

inline void Palette::setEntry(int index, int value)
{
	BufferBase::setPixel(index,0,value);
}


DEFINE_USAGE_COUNTER(Profile);


inline Profile::Profile()
	: BufferBase()
{ INC_USAGE_COUNTER;}


inline Profile::Profile(int width, int bands, int slices, int frames, BufferDatatype dt)
	: BufferBase(width, 1, bands, slices, frames, dt, baProfile)
{ INC_USAGE_COUNTER;}


inline Profile::Profile(int width, int bands, int slices, int frames, BufferDatatype dt, void * ptr)
	: BufferBase(width, 1, bands, slices, frames, dt, baProfile, ptr)
{ INC_USAGE_COUNTER;}


inline Profile::Profile(const Profile& parent, int xOffset, int bOffset, int sOffset, int fOffset, int width, int bands, int slices, int frames)
	: BufferBase(parent, xOffset, 0, bOffset, sOffset, fOffset, width, 1, bands, slices, frames)
{ INC_USAGE_COUNTER;}


inline Profile::Profile(const Profile& parent, const Coord &offset, int width, int bands, int slices, int frames)
	: BufferBase(parent, offset.getX(), 0, offset.getBand(), offset.getSlice(), offset.getFrame(), width, 1, bands, slices, frames)
{ INC_USAGE_COUNTER;}


inline const Profile* Profile::getParentBuffer() const
{
	return reinterpret_cast< const Profile* >(BufferBase::getParentBuffer());
}


inline Profile& Profile::operator=(const Profile& r)
{
	if (this != &r)
	{
		BufferBase::operator=(r);
	}
	return *this;
}


inline Profile::Profile(const std::string& xml)
	: BufferBase(xml)
{ INC_USAGE_COUNTER;}


inline Profile::Profile(const Profile &profile)
	: BufferBase(profile)
{ INC_USAGE_COUNTER;}


inline Profile::~Profile()
{ DEC_USAGE_COUNTER }


inline BufferBase* Profile::makeShallowCopy() const
{
	return new Profile(*this);
}


inline Profile::Profile(mvIMPACT_C::IPL_BUFHANDLE bufhandle) : BufferBase(bufhandle)
{
	INC_USAGE_COUNTER;
	if (getAttribute() != baProfile)
	{
		throwImpactException("Cannot create a Profile object from a BufferBase not being of type Profile");
	}
}


inline Profile::Profile
	(
		const Profile& parent,
		const std::string& xml
	)
		: BufferBase(parent, xml)
	{ INC_USAGE_COUNTER;}


inline Profile Profile::clone() const
	{
		if(getIsChild()) {
			return Profile(
				*(getParentBuffer()),
				getXOffset(),
				getBandsOffset(), getSlicesOffset(), getFramesOffset(),
				getWidth(),
				getBands(), getSlices(), getFrames());
		} else {
			return mvIMPACT::copy(*this);
		}
	}


inline Profile Profile::copyAsProfile() const
	{
		return mvIMPACT::copy(*this);
	}


inline int Profile::getPixel(int x, int band, int slice, int frame) const
{
	return BufferBase::getPixel(x,0,band,slice,frame);
}


inline int Profile::getPixel(int x) const
{
	return BufferBase::getPixel(x,0);
}


inline void Profile::setPixel(int x, int band, int slice, int frame, int value)
{
	BufferBase::setPixel(x,0,band,slice,frame,value);
}


inline void Profile::setPixel(int x, int value)
{
	BufferBase::setPixel(x,0,value);
}


inline int Profile::getMin(int band, int slice, int frame) const
{
	return BufferBase::getMin(band, slice, frame);
}


inline std::vector<int> Profile::getAllMinValues() const
{
	return BufferBase::getAllMinValues();
}


inline int Profile::getMax(int band, int slice, int frame) const
{
	return BufferBase::getMax(band, slice, frame);
}


inline std::vector<int> Profile::getAllMaxValues() const
{
	return BufferBase::getAllMaxValues();
}


inline std::pair<int,int> Profile::getMinMax(int band, int slice, int frame) const
{
	return BufferBase::getMinMax(band, slice, frame);
}


inline std::vector<int> Profile::getAllMinMaxValues() const
{
	return BufferBase::getAllMinMaxValues();
}


inline double Profile::getMean(int band, int slice, int frame) const
{
	return BufferBase::getMean(band, slice, frame);
}


inline std::vector<double> Profile::getAllMeanValues() const
{
	return BufferBase::getAllMeanValues();
}


inline double Profile::getStdDev(int band, int slice, int frame) const
{
	return BufferBase::getStdDev(band, slice, frame);
}


inline std::vector<double> Profile::getAllStdDevValues() const
{
	return BufferBase::getAllStdDevValues();
}



DEFINE_USAGE_COUNTER(Histogram);


inline Histogram::Histogram()
	: BufferBase()
{ INC_USAGE_COUNTER;}


inline Histogram::Histogram(int width, int bands, int slices, int frames, BufferDatatype dt)
	: BufferBase(width, 1, bands, slices, frames, dt, baHistogram)
{ INC_USAGE_COUNTER;}


inline Histogram::Histogram(int width, int bands, int slices, int frames, BufferDatatype dt, void * ptr)
	: BufferBase(width, 1, bands, slices, frames, dt, baHistogram, ptr)
{ INC_USAGE_COUNTER;}


inline Histogram::Histogram(const Histogram& parent, int xOffset, int bOffset, int sOffset, int fOffset, int width, int bands, int slices, int frames)
	: BufferBase(parent, xOffset, 0, bOffset, sOffset, fOffset, width, 1, bands, slices, frames)
{ INC_USAGE_COUNTER;}


inline Histogram::Histogram(const Histogram& parent, const Coord &offset, int width, int bands, int slices, int frames)
	: BufferBase(parent, offset, width, 1, bands, slices, frames)
{ INC_USAGE_COUNTER;}


inline const Histogram* Histogram::getParentBuffer() const
{
	return reinterpret_cast< const Histogram* >(BufferBase::getParentBuffer());
}


inline Histogram& Histogram::operator=(const Histogram& r)
{
	if (this != &r)	BufferBase::operator=(r);
	return *this;
}


inline Histogram::Histogram(int width, int bands, int slices, int frames)
	: BufferBase(width, 1, bands, slices, frames, bdULong, baHistogram)
{ INC_USAGE_COUNTER;}


inline Histogram::Histogram(const std::string& xml)
	: BufferBase(xml)
{ INC_USAGE_COUNTER;}


inline Histogram::Histogram(const Histogram &histogram)
	: BufferBase(histogram)
{ INC_USAGE_COUNTER;}


inline Histogram Histogram::clone() const
{
	if(getIsChild()) {
		return Histogram(
			*(getParentBuffer()),
			getXOffset(),
			getBandsOffset(), getSlicesOffset(), getFramesOffset(),
			getWidth(),
			getBands(), getSlices(), getFrames());
	} else {
		return mvIMPACT::copy(*this);
	}
}


inline Histogram::~Histogram()
{ DEC_USAGE_COUNTER }


inline BufferBase* Histogram::makeShallowCopy() const
{
	return new Histogram(*this);
}


inline Histogram::Histogram(mvIMPACT_C::IPL_BUFHANDLE bufhandle)
	: BufferBase(bufhandle)
{
	INC_USAGE_COUNTER;
	if (getAttribute() != baHistogram)
	{
		throwImpactException("Cannot create a Histogram object from a BufferBase not being of type Histogram");
	}
}


inline Histogram::Histogram(const Histogram& parent, const std::string& xml)
	: BufferBase(parent, xml)
{ INC_USAGE_COUNTER;}


inline Histogram Histogram::copyAsHistogram() const
{
	return mvIMPACT::copy(*this);
}


inline int Histogram::getFrequency(int index, int band, int slice, int frame) const
{
	return BufferBase::getPixel(index,0,band,slice,frame);
}


inline int Histogram::getFrequency(int index) const
{
	return BufferBase::getPixel(index,0);
}


inline void Histogram::setFrequency(int index, int band, int slice, int frame, int value)
{
	BufferBase::setPixel(index,0,band,slice,frame,value);
}


inline void Histogram::setFrequency(int index, int value)
{
	BufferBase::setPixel(index,0,value);
}


inline int Histogram::getMin(int band, int slice, int frame) const
{
	return BufferBase::getMin(band, slice, frame);
}


inline std::vector<int> Histogram::getAllMinValues() const
{
	return BufferBase::getAllMinValues();
}


inline int Histogram::getMax(int band, int slice, int frame) const
{
	return BufferBase::getMax(band, slice, frame);
}


inline std::vector<int> Histogram::getAllMaxValues() const
{
	return BufferBase::getAllMaxValues();
}


inline std::pair<int,int> Histogram::getMinMax(int band, int slice, int frame) const
{
	return BufferBase::getMinMax(band, slice, frame);
}


inline std::vector<int> Histogram::getAllMinMaxValues() const
{
	return BufferBase::getAllMinMaxValues();
}


inline double Histogram::getMean(int band, int slice, int frame) const
{
	return BufferBase::getMean(band, slice, frame);
}


inline std::vector<double> Histogram::getAllMeanValues() const
{
	return BufferBase::getAllMeanValues();
}


inline double Histogram::getStdDev(int band, int slice, int frame) const
{
	return BufferBase::getStdDev(band, slice, frame);
}


inline std::vector<double> Histogram::getAllStdDevValues() const
{
	return BufferBase::getAllStdDevValues();
}



DEFINE_USAGE_COUNTER(Kernel);


inline Point Kernel::getCenter() const
{
	int x;
	int y;

	if (!mvIMPACT_C::IPL_BufInquire(getHandle(),mvIMPACT_C::IPL_XCENTER, &x))
		throwImpactError();
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_YCENTER, &y))
		throwImpactError();

	return Point(x,y);
}


inline Kernel& Kernel::setCenter(const Point& c)
{
	int x = c.getX();
	int y = c.getY();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_XCENTER, &x))
		throwImpactError();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_YCENTER, &y))
		throwImpactError();
	return *this;
}


inline Kernel::Kernel()
	: BufferBase()
{ INC_USAGE_COUNTER; }


inline Kernel::Kernel(StockKernel sk)
	: BufferBase(sk, false)
{ INC_USAGE_COUNTER; }


inline Kernel::Kernel(int width, int height, int bands, int slices, int frames)
	: BufferBase(width, height, bands, slices, frames, bdChar, baKernel)
{INC_USAGE_COUNTER; }


inline Kernel::Kernel(int width, int height, int bands, int slices, int frames, void * ptr)
	: BufferBase(width, height, bands, slices, frames, bdChar, baKernel, ptr)
{INC_USAGE_COUNTER; }


inline Kernel::Kernel(const Kernel& parent, int xOffset, int yOffset, int bOffset, int sOffset, int fOffset, int width, int height, int bands, int slices, int frames)
	: BufferBase(parent, xOffset, yOffset, bOffset, sOffset, fOffset, width, height, bands, slices, frames)
{INC_USAGE_COUNTER; }


inline Kernel::Kernel(const Kernel& parent, const Coord &offset, int width, int height, int bands, int slices, int frames)
	: BufferBase(parent, offset, width, height, bands, slices, frames)
{INC_USAGE_COUNTER; }


inline const Kernel* Kernel::getParentBuffer() const
{
	return reinterpret_cast< const Kernel* >(BufferBase::getParentBuffer());
}


inline Kernel::~Kernel()
{ DEC_USAGE_COUNTER; }


inline Kernel::Kernel(mvIMPACT_C::IPL_BUFHANDLE bufhandle)
	: BufferBase(bufhandle)
{
	INC_USAGE_COUNTER;
	if (getAttribute() != baKernel)
	{
		throwImpactException("Cannot create a Kernel object from a BufferBase not being of type Kernel");
	}
}


inline Kernel::Kernel(const std::string& xml)
	: BufferBase(xml)
{INC_USAGE_COUNTER; }


inline Kernel::Kernel(const Kernel& parent, const std::string& xml)
	: BufferBase(parent, xml)
{INC_USAGE_COUNTER; }


inline Kernel& Kernel::operator=(const Kernel& r)
{
	if (this != &r) BufferBase::operator=(r);
	return *this;
}


inline Kernel::Kernel(const Kernel &kernel)
	: BufferBase(kernel)
{ INC_USAGE_COUNTER; }


inline Kernel Kernel::clone() const
{
	if(getIsChild()) {
		return Kernel(
			*(getParentBuffer()),
			getXOffset(), getYOffset(),
			getBandsOffset(), getSlicesOffset(), getFramesOffset(),
			getWidth(), getHeight(),
			getBands(), getSlices(), getFrames());
	} else {
		return mvIMPACT::copy(*this);
	}
}


inline Kernel Kernel::copyAsKernel() const
{
	return mvIMPACT::copy(*this);
}


inline BufferBase* Kernel::makeShallowCopy() const
{
	return new Kernel(*this);
}


inline int Kernel::getCoefficient(int x, int y, int band, int slice, int frame) const
{
	return BufferBase::getPixel(x,y,band,slice,frame);
}


inline int Kernel::getCoefficient(int x, int y) const
{
	return BufferBase::getPixel(x,y);
}


inline void Kernel::setCoefficient(int x, int y, int band, int slice, int frame, int value)
{
	BufferBase::setPixel(x,y,band,slice,frame,value);
}


inline void Kernel::setCoefficient(int x, int y, int value)
{
	BufferBase::setPixel(x,y,value);
}


inline Kernel& Kernel::setClippingMode(KernelClippingMode cm)
{
	if(!mvIMPACT_C::IPL_BufSetupNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_CLIPFLAG,
		(mvIMPACT_C::IPL_LONG)cm))
	{ throwImpactError(); }
	return *this;
}


inline KernelClippingMode Kernel::getClippingMode() const
{
	mvIMPACT_C::IPL_LONG cm = 0;
	if(!mvIMPACT_C::IPL_BufGetNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_CLIPFLAG,
		&cm))
	{ throwImpactError(); }
	return (mvIMPACT::KernelClippingMode)cm;
}


inline Kernel& Kernel::setOverscanMode(OverscanMode om)
{
	if(!mvIMPACT_C::IPL_BufSetupNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN,
		(mvIMPACT_C::IPL_LONG)om))
	{ throwImpactError(); }
	return *this;
}


inline OverscanMode Kernel::getOverscanMode() const
{
	mvIMPACT_C::IPL_LONG om = 0;
	if(!mvIMPACT_C::IPL_BufGetNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN,
		&om))
	{ throwImpactError(); }
	return (mvIMPACT::OverscanMode)om;
}


inline Kernel& Kernel::setOverscanConstValue(int val)
{
	if(!mvIMPACT_C::IPL_BufSetupNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN_REPLACE,
		(mvIMPACT_C::IPL_LONG)val))
	{ throwImpactError(); }
	return *this;
}


inline int Kernel::getOverscanConstValue() const
{
	mvIMPACT_C::IPL_LONG val = 0;
	if(!mvIMPACT_C::IPL_BufGetNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN_REPLACE,
		&val))
	{ throwImpactError(); }
	return val;
}


inline Kernel& Kernel::setNormalizationFactor(int n)
{
	if(!mvIMPACT_C::IPL_BufSetupNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_NORMFACTOR,
		n))
	{ throwImpactError(); }
	return *this;
}

inline int Kernel::getNormalizationFactor() const
{
	mvIMPACT_C::IPL_LONG n = 0;
	if(!mvIMPACT_C::IPL_BufGetNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_NORMFACTOR,
		&n))
	{ throwImpactError(); }
	return (int)n;
}



DEFINE_USAGE_COUNTER(StructuringElement);


inline Point StructuringElement::getCenter() const
{
	int x;
	int y;

	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_YCENTER, &x))
		throwImpactError();
	if (!mvIMPACT_C::IPL_BufInquire(getHandle(), mvIMPACT_C::IPL_YCENTER, &y))
		throwImpactError();
	return Point(x,y);
}


inline StructuringElement& StructuringElement::setCenter( const Point& c )
{
	int x = c.getX();
	int y = c.getY();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_XCENTER, &x))
		throwImpactError();
	if (!mvIMPACT_C::IPL_BufControl(getHandle(), mvIMPACT_C::IPL_YCENTER, &y))
		throwImpactError();
	return *this;
}


inline StructuringElement::StructuringElement()
	: BufferBase()
{ INC_USAGE_COUNTER; }


inline StructuringElement::StructuringElement(StockStructuringElement sse)
	: BufferBase(sse, false)
{ INC_USAGE_COUNTER; }


inline StructuringElement::StructuringElement(int width, int height, int bands, int slices, int frames)
	: BufferBase(width, height, bands, slices, frames, bdLong, baStructuringElement)
{ INC_USAGE_COUNTER; }


inline StructuringElement::StructuringElement(int width, int height, int bands, int slices, int frames, void * ptr)
	: BufferBase(width, height, bands, slices, frames, bdLong, baStructuringElement, ptr)
{ INC_USAGE_COUNTER; }


inline StructuringElement::StructuringElement(const StructuringElement& parent, int xOffset, int yOffset, int bOffset, int sOffset, int fOffset, int width, int height, int bands, int slices, int frames)
	: BufferBase(parent, xOffset, yOffset, bOffset, sOffset, fOffset, width, height, bands, slices, frames)
{ INC_USAGE_COUNTER; }


inline StructuringElement::StructuringElement(const StructuringElement& parent, const Coord &offset, int width, int height, int bands, int slices, int frames)
	: BufferBase(parent, offset, width, height, bands, slices, frames)
{ INC_USAGE_COUNTER; }


inline const StructuringElement* StructuringElement::getParentBuffer() const
{ return reinterpret_cast< const StructuringElement* >(BufferBase::getParentBuffer()); }


inline StructuringElement::~StructuringElement()
{ DEC_USAGE_COUNTER; }


inline StructuringElement::StructuringElement(mvIMPACT_C::IPL_BUFHANDLE bufhandle)
	: BufferBase(bufhandle)
{
	INC_USAGE_COUNTER;
	if (getAttribute() != baStructuringElement)
	{
		throwImpactException("Cannot create a StructuringElement object from a BufferBase not being of type StructuringElement");
	}
}


inline StructuringElement::StructuringElement(const std::string& xml)
	: BufferBase(xml)
{ INC_USAGE_COUNTER; }


inline StructuringElement::StructuringElement(const StructuringElement& parent, const std::string& xml)
	: BufferBase(parent, xml)
{ INC_USAGE_COUNTER; }


inline StructuringElement& StructuringElement::operator=(const StructuringElement& r)
{
	if (this != &r)
		BufferBase::operator=(r);
	return *this;
}


inline StructuringElement::StructuringElement(const StructuringElement &structuringElement)
	: BufferBase(structuringElement)
{ INC_USAGE_COUNTER;}


inline StructuringElement StructuringElement::clone() const
{
	if(getIsChild()) {
		return StructuringElement(
			*(getParentBuffer()),
			getXOffset(), getYOffset(),
			getBandsOffset(), getSlicesOffset(), getFramesOffset(),
			getWidth(), getHeight(),
			getBands(), getSlices(), getFrames());
	} else {
		return mvIMPACT::copy(*this);
	}
}

inline BufferBase* StructuringElement::makeShallowCopy() const
{
	return new StructuringElement(*this);
}


inline StructuringElement StructuringElement::copyAsStructuringElement() const
{
	return mvIMPACT::copy(*this);
}


inline int StructuringElement::getCoefficient(int x, int y, int band, int slice, int frame) const
{
	return BufferBase::getPixel(x,y,band,slice,frame);
}


inline int StructuringElement::getCoefficient(int x, int y) const
{
	return BufferBase::getPixel(x,y);
}


inline void StructuringElement::setCoefficient(int x, int y, int band, int slice, int frame, int value)
{
	BufferBase::setPixel(x,y,band,slice,frame,value);
}


inline void StructuringElement::setCoefficient(int x, int y, int value)
{
	BufferBase::setPixel(x,y,value);
}


inline StructuringElement& StructuringElement::setOverscanMode(OverscanMode om)
{
	if(!mvIMPACT_C::IPL_BufSetupNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN,
		(mvIMPACT_C::IPL_LONG)om))
	{ throwImpactError(); }
	return *this;
}


inline OverscanMode StructuringElement::getOverscanMode() const
{
	mvIMPACT_C::IPL_LONG om = 0;
	if(!mvIMPACT_C::IPL_BufGetNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN,
		&om))
	{ throwImpactError(); }
	return (mvIMPACT::OverscanMode)om;
}


inline StructuringElement& StructuringElement::setOverscanConstValue(int val)
{
	if(!mvIMPACT_C::IPL_BufSetupNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN_REPLACE,
		(mvIMPACT_C::IPL_LONG)val))
	{ throwImpactError(); }
	return *this;
}


inline int StructuringElement::getOverscanConstValue() const
{
	mvIMPACT_C::IPL_LONG val = 0;
	if(!mvIMPACT_C::IPL_BufGetNeighborhood (
                getHandle(),
		mvIMPACT_C::IPL_CTLID_OVERSCAN_REPLACE,
		&val))
	{ throwImpactError(); }
	return val;
}


inline int getCountElementsOfOne(StockStructuringElement se)
{
	mvIMPACT_C::IPL_LONG cnt = 0;
	if(!mvIMPACT_C::IPL_BufInquire( se, mvIMPACT_C::IPL_STRUCT_COUNT_1, &cnt))
	{ throwImpactError(); }
	return cnt;
}


inline int getCountElementsOfOne(const StructuringElement &se)
{
	mvIMPACT_C::IPL_BUFHANDLE hSE = se.getHandle();
	mvIMPACT_C::IPL_LONG cnt = 0;
	if(!mvIMPACT_C::IPL_BufInquire( hSE, mvIMPACT_C::IPL_STRUCT_COUNT_1, &cnt))
	{ throwImpactError(); }
	return cnt;
}


inline OnProgress::OnProgress()
{
	enable();
}

inline OnProgress::~OnProgress()
{
	disable();
}

// for backward compatibility reasons
inline bool OnProgress::update(int /*total*/, int /*done*/)
{
	return true;
}

inline bool OnProgress::onUpdate(int total, int done)
{
	// try the old overload, in case this one was not overloaded
	return update(total, done);
}

inline void OnProgress::enable()
	{
		if (!mvIMPACT_C::IPL_PushProgressFunction((mvIMPACT_C::IPL_VOID_PTR)progress_function, this))
			throwImpactError();
		enabled_ = true;
	}

inline void OnProgress::disable()
	{
		std::vector<OnProgress*> tempStack;
		// temporary stack to hold the function pointers "above" the current
		// one on the internal stack of the C-API

		// Pop functions from the internal stack of the C-API until the
		// current object is found or the stack is empty:
		while (1) {
			mvIMPACT_C::IPL_BOOL (*currProgFunc)(
						mvIMPACT_C::IPL_LONG,
						mvIMPACT_C::IPL_LONG,
						mvIMPACT_C::IPL_VOID_PTR
						);
			OnProgress* currCookie;
			if (!mvIMPACT_C::IPL_PopProgressFunction(
				(mvIMPACT_C::IPL_VOID_PTR*)&currProgFunc,
				(mvIMPACT_C::IPL_VOID_PTR*)&currCookie))
			{ throwImpactError(); }

			if(NULL == currProgFunc || this == currCookie) { break; }
			// the current object is pushed on the temporary
			// stack and the NULL pointer indicating an empty stack
			// is not either.

			tempStack.push_back(currCookie);
		}

		enabled_ = false;
		// now, the current object is not enabled any more (if
		// it ever was)

		// push the functions above the current one again on the stack:
		while(!tempStack.empty()) {
			if (!mvIMPACT_C::IPL_PushProgressFunction(
				(mvIMPACT_C::IPL_VOID_PTR)progress_function,
				tempStack.back()))
			{ throwImpactError(); }

			tempStack.pop_back();
		}
	}

inline bool OnProgress::getIsEnabled() const
{
	return enabled_;
}


inline mvIMPACT_C::IPL_BOOL IPL_C_CALLTYPE OnProgress::progress_function(mvIMPACT_C::IPL_LONG Total, mvIMPACT_C::IPL_LONG Done, mvIMPACT_C::IPL_VOID_PTR Cookie)
{
		OnProgress* f = reinterpret_cast<OnProgress*>(Cookie);
		if (!f)
			return IPL_TRUE;

		mvIMPACT_C::IPL_BOOL result = IPL_FALSE;

		try
		{
			result = f->onUpdate(Total, Done) ? IPL_TRUE : IPL_FALSE;
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception callback");
			result = IPL_FALSE;
		}

		return result;
}


inline Image Image::imageFactory(mvIMPACT_C::IPL_BUFHANDLE handle)
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == handle)
		return Image();

	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(handle, mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	BufferAttribute attribute(static_cast<BufferAttribute>(att));

	switch (attribute)
	{
	case baImage:
	case baBayerMosaic:
	case baRle:
		break;
	default:
		throwImpactException("Not Image type");
	}
	return Image(handle);
}


inline Image Image::imageDeviceFactory(mvIMPACT_C::IPL_BUFHANDLE handle)
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == handle)
		return Image();

	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(handle, mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	BufferAttribute attribute(static_cast<BufferAttribute>(att));

	switch (attribute)
	{
	case baImage:
	case baBayerMosaic:
	case baRle:
		break;
	default:
		throwImpactException("Not Image type");
	}
#ifdef PLATFORM_MVBL
// #ifdef __ppc_6xx__
	// under mvBL the image handle is freed by DEV_Close(), so the destructor must not
	// cleanup it again. Thus we return a deep copy here.
	return Image(handle, false).copyAsImage();
#else
	return Image(handle, true);
#endif	// __ppc_6xx__
}

inline Palette Palette::paletteFactory(mvIMPACT_C::IPL_BUFHANDLE handle)
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == handle)
		return Palette();

	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(handle, mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	BufferAttribute attribute(static_cast<BufferAttribute>(att));

	switch (attribute)
	{
	case baLut:
		break;
	default:
		throwImpactException("Not Palette type");
	}
	return Palette(handle);
}


inline Profile Profile::profileFactory(mvIMPACT_C::IPL_BUFHANDLE handle)
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == handle)
		return Profile();

	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(handle, mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	BufferAttribute attribute(static_cast<BufferAttribute>(att));

	switch (attribute)
	{
	case baProfile:
		break;
	default:
		throwImpactException("Not Profile type");
	}
	return Profile(handle);
}


inline Histogram Histogram::histogramFactory(mvIMPACT_C::IPL_BUFHANDLE handle)
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == handle)
		return Histogram();

	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(handle, mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	BufferAttribute attribute(static_cast<BufferAttribute>(att));

	switch (attribute)
	{
	case baHistogram:
		break;
	default:
		throwImpactException("Not Histogram type");
	}
	return Histogram(handle);
}





inline Kernel Kernel::kernelFactory(mvIMPACT_C::IPL_BUFHANDLE handle)
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == handle)
		return Kernel();

	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(handle, mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	BufferAttribute attribute(static_cast<BufferAttribute>(att));

	switch (attribute)
	{
	case baKernel:
		break;
	default:
		throwImpactException("Not Kernel type");
	}
	return Kernel(handle);
}





inline StructuringElement StructuringElement::structuringElementFactory(mvIMPACT_C::IPL_BUFHANDLE handle)
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == handle)
		return StructuringElement();

	mvIMPACT_C::IPL_BUFFER_ATTRIBUTE att = mvIMPACT_C::IPL_NOTALLOCATED;
	if (!mvIMPACT_C::IPL_BufInquire(handle, mvIMPACT_C::IPL_USAGETYPE, &att))
		throwImpactError();
	BufferAttribute attribute(static_cast<BufferAttribute>(att));

	switch (attribute)
	{
	case baStructuringElement:
		break;
	default:
		throwImpactException("Not StructuringElement type");
	}
	return StructuringElement(handle);
}


inline Image combineImages(
	const std::vector<Image>& buffers,
	DimensionSpec dimension,
	int width,
	int height,
	GeometricInterpolation geometricMode
	)
{
	return combineBuffers(buffers, dimension, width, height, geometricMode);
}

inline Image combineBuffers(
	const std::vector<Image>& buffers,
	DimensionSpec dimension,
	int width,
	int height,
	GeometricInterpolation geometricMode
	)
{
	std::vector<mvIMPACT_C::IPL_BUFHANDLE> vHandles;
	for (std::vector<Image>::const_iterator i = buffers.begin(); i != buffers.end(); ++i)
	{
		vHandles.push_back(i->getHandle());
	}

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplHandles(vHandles);

	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_BufCombine(
			iplHandles.getPointer(),
			iplHandles.getSize(),
			&hD,
			mvIMPACT_C::IPL_DIMENSION(dimension),
			width, height,
			mvIMPACT_C::IPL_GEOTRANS(geometricMode),
			NULL))
		throwImpactError();

	return Image::imageFactory(hD);
}


inline Palette combinePalettes(
	const std::vector<Palette>& buffers,
	DimensionSpec dimension
	)
{
	return combineBuffers(buffers, dimension);
}

inline Palette combineBuffers(
	const std::vector<Palette>& buffers,
	DimensionSpec dimension
	)
{
	int width = buffers.begin()->getWidth();
	std::vector<mvIMPACT_C::IPL_BUFHANDLE> vHandles;
	for (std::vector<Palette>::const_iterator i = buffers.begin(); i != buffers.end(); ++i)
	{
		if(width != i->getWidth()) {
			throwImpactException("All Palettes must have the same width");
		}
		vHandles.push_back(i->getHandle());
	}

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplHandles(vHandles);

	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_BufCombine(
			iplHandles.getPointer(),
			iplHandles.getSize(),
			&hD,
			mvIMPACT_C::IPL_DIMENSION(dimension),
			width, 1,
			mvIMPACT_C::IPL_NEAREST_NEIGHBOR,
			NULL))
		throwImpactError();

	return Palette::paletteFactory(hD);
}


inline Profile combineProfiles
(
	const std::vector<Profile>& buffers,
	DimensionSpec dimension,
	int width,
	GeometricInterpolation geometricMode
	)
{
	return combineBuffers(buffers, dimension, width, geometricMode);
}

inline Profile combineBuffers
(
	const std::vector<Profile>& buffers,
	DimensionSpec dimension,
	int width,
	GeometricInterpolation geometricMode
	)
{
	std::vector<mvIMPACT_C::IPL_BUFHANDLE> vHandles;
	for (std::vector<Profile>::const_iterator i = buffers.begin(); i != buffers.end(); ++i)
	{
		vHandles.push_back(i->getHandle());
	}

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplHandles(vHandles);

	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_BufCombine(
			iplHandles.getPointer(),
			iplHandles.getSize(),
			&hD,
			mvIMPACT_C::IPL_DIMENSION(dimension),
			width, 1,
			mvIMPACT_C::IPL_GEOTRANS(geometricMode),
			NULL))
		throwImpactError();

	return Profile::profileFactory(hD);
}


inline Histogram combineHistograms
(
	const std::vector<Histogram>& buffers,
	DimensionSpec dimension
	)
{
	return combineBuffers(buffers, dimension);
}

inline Histogram combineBuffers
(
	const std::vector<Histogram>& buffers,
	DimensionSpec dimension
	)
{
	int width = buffers.begin()->getWidth();
	std::vector<mvIMPACT_C::IPL_BUFHANDLE> vHandles;
	for (std::vector<Histogram>::const_iterator i = buffers.begin(); i != buffers.end(); ++i)
	{
		if(width != i->getWidth()) {
			throwImpactException("All Histograms must have the same width");
		}
		vHandles.push_back(i->getHandle());
	}

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplHandles(vHandles);

	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_BufCombine(
			iplHandles.getPointer(),
			iplHandles.getSize(),
			&hD,
			mvIMPACT_C::IPL_DIMENSION(dimension),
			width, 1,
			mvIMPACT_C::IPL_NEAREST_NEIGHBOR,
			NULL))
		throwImpactError();

	return Histogram::histogramFactory(hD);
}


inline Kernel combineKernels
(
	const std::vector<Kernel>& buffers,
	DimensionSpec dimension
	)
{
	return combineBuffers(buffers, dimension);
}

inline Kernel combineBuffers
(
	const std::vector<Kernel>& buffers,
	DimensionSpec dimension
	)
{
	int width = buffers.begin()->getWidth();
	int height = buffers.begin()->getHeight();
	std::vector<mvIMPACT_C::IPL_BUFHANDLE> vHandles;
	for (std::vector<Kernel>::const_iterator i = buffers.begin(); i != buffers.end(); ++i)
	{
		if(width != i->getWidth() || height != i->getHeight()) {
			throwImpactException("All Kernels must have the same width and height");
		}
		vHandles.push_back(i->getHandle());
	}

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplHandles(vHandles);

	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_BufCombine(
			iplHandles.getPointer(),
			iplHandles.getSize(),
			&hD,
			mvIMPACT_C::IPL_DIMENSION(dimension),
			width, height,
			mvIMPACT_C::IPL_NEAREST_NEIGHBOR,
			NULL))
		throwImpactError();

	return Kernel::kernelFactory(hD);
}


inline StructuringElement combineStructuringElements
(
	const std::vector<StructuringElement>& buffers,
	DimensionSpec dimension
	)
{
	return combineBuffers(buffers, dimension);
}

inline StructuringElement combineBuffers
(
	const std::vector<StructuringElement>& buffers,
	DimensionSpec dimension
	)
{
	int width = buffers.begin()->getWidth();
	int height = buffers.begin()->getHeight();
	std::vector<mvIMPACT_C::IPL_BUFHANDLE> vHandles;
	for (std::vector<StructuringElement>::const_iterator i = buffers.begin(); i != buffers.end(); ++i)
	{
		if(width != i->getWidth() || height != i->getHeight()) {
			throwImpactException("All Kernels must have the same width and height");
		}
		vHandles.push_back(i->getHandle());
	}

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplHandles(vHandles);

	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_BufCombine(
			iplHandles.getPointer(),
			iplHandles.getSize(),
			&hD,
			mvIMPACT_C::IPL_DIMENSION(dimension),
			width, height,
			mvIMPACT_C::IPL_NEAREST_NEIGHBOR,
			NULL))
		throwImpactError();

	return StructuringElement::structuringElementFactory(hD);
}


inline StructuringElement copy
(
	const StructuringElement& S,
	int smask,
	int dmask,
	int shift
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtBufCopy(
			S.getHandle(),
			&D,
			static_cast<mvIMPACT_C::IPL_ULONG>(smask),
			static_cast<mvIMPACT_C::IPL_ULONG>(dmask),
			shift))
		throwImpactError();
	return StructuringElement::structuringElementFactory(D);
}

inline Profile copy
(
	const Profile& S,
	int smask,
	int dmask,
	int shift
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtBufCopy(
			S.getHandle(),
			&D,
			static_cast<mvIMPACT_C::IPL_ULONG>(smask),
			static_cast<mvIMPACT_C::IPL_ULONG>(dmask),
			shift))
		throwImpactError();
	return Profile::profileFactory(D);
}

inline Palette copy
(
	const Palette& S,
	int smask,
	int dmask,
	int shift
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtBufCopy(
			S.getHandle(),
			&D,
			static_cast<mvIMPACT_C::IPL_ULONG>(smask),
			static_cast<mvIMPACT_C::IPL_ULONG>(dmask),
			shift))
		throwImpactError();
	return Palette::paletteFactory(D);
}

inline Kernel copy
(
	const Kernel& S,
	int smask,
	int dmask,
	int shift
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtBufCopy(
			S.getHandle(),
			&D,
			static_cast<mvIMPACT_C::IPL_ULONG>(smask),
			static_cast<mvIMPACT_C::IPL_ULONG>(dmask),
			shift))
		throwImpactError();
	return Kernel::kernelFactory(D);
}

inline Image copy
(
	const Image& S,
	int smask,
	int dmask,
	int shift
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtBufCopy(
			S.getHandle(),
			&D,
			static_cast<mvIMPACT_C::IPL_ULONG>(smask),
			static_cast<mvIMPACT_C::IPL_ULONG>(dmask),
			shift))
		throwImpactError();
	return Image::imageFactory(D);
}

inline Histogram copy
(
	const Histogram& S,
	int smask,
	int dmask,
	int shift
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtBufCopy(
			S.getHandle(),
			&D,
			static_cast<mvIMPACT_C::IPL_ULONG>(smask),
			static_cast<mvIMPACT_C::IPL_ULONG>(dmask),
			shift))
		throwImpactError();
	return Histogram::histogramFactory(D);
}

inline void copy
(
	BufferBase& D,
	const BufferBase& S,
	int smask,  // the default is listed in a previous declaration
	int dmask,  // the default is listed in a previous declaration
	int shift
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtBufCopy(
			S.getHandle(),
			&hD,
			static_cast<mvIMPACT_C::IPL_ULONG>(smask),
			static_cast<mvIMPACT_C::IPL_ULONG>(dmask),
			shift))
		throwImpactError();
	if(hD != D.getHandle()) {
		BufferAttribute at = D.getAttribute();

		// BufferBase is an abstract class, so we cannot instantiate it directly
		// but have to switch over the buffer types known by the C++ wrapper:
		switch (at) {
			case baImage:
			case baBayerMosaic:
			case baRle:
				D = Image::imageFactory(hD);
				break;
			case baLut:
				D = Palette::paletteFactory(hD);
				break;
			case baHistogram:
				D = Histogram::histogramFactory(hD);
				break;
			case baProfile:
				D = Profile::profileFactory(hD);
				break;
			case baKernel:
				D = Kernel::kernelFactory(hD);
				break;
			case baStructuringElement:
				D = StructuringElement::structuringElementFactory(hD);
			default:
				throwImpactException("Buffer attribute not supported by C++ wrapper");
		}
	}
}




inline Histogram createCompatibleHistogram
(
	const Histogram& S
)
//-----------------------------------------------------------------------------
{
	return Histogram(S.getWidth(), S.getBands(), S.getSlices(), S.getFrames());
}




inline Image createCompatibleImage
(
	const Image& S
)
//-----------------------------------------------------------------------------
{
	return Image(S.getWidth(),  S.getHeight(), S.getBands(),
		S.getSlices(), S.getFrames(), S.getDatatype());
}




inline Kernel createCompatibleKernel
(
	const Kernel& S
)
//-----------------------------------------------------------------------------
{
	return Kernel(S.getWidth(),  S.getHeight(), S.getBands(),
		S.getSlices(), S.getFrames());
}




inline Palette createCompatiblePalette
(
	const Palette& S
)
//-----------------------------------------------------------------------------
{
	return Palette(S.getWidth(),  S.getBands(),
		S.getSlices(), S.getFrames(), S.getDatatype());
}




inline Profile createCompatibleProfile
(
	const Profile& S
)
//-----------------------------------------------------------------------------
{
	return Profile(S.getWidth(),  S.getBands(),
		S.getSlices(), S.getFrames(), S.getDatatype());
}




inline StructuringElement createCompatibleStructuringElement
(
	const StructuringElement& S
)
//-----------------------------------------------------------------------------
{
	return StructuringElement(S.getWidth(),  S.getHeight(), S.getBands(),
		S.getSlices(), S.getFrames());
}





inline void trace(const std::string& message)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::IPL_Trace(message.c_str()))
		throwImpactError();
}





inline std::string serialNumber
(
	const std::string& authentificator,
	const std::string& public_key
)
//-----------------------------------------------------------------------------
{
	const int serial_number_size = 256;
	DynArray<char> serial(serial_number_size);

	if (!mvIMPACT_C::IPL_LicenseSerialNumber(
			authentificator.c_str(),
			public_key.c_str(),
			static_cast<mvIMPACT_C::IPL_ULONG>(public_key.length()),
			serial.getPointer(),
			serial_number_size))
		throwImpactError();

	return std::string(serial.getPointer());
}



inline std::string getTimeStamp()
//-----------------------------------------------------------------------------
{
	const int stamp_size = 256;
	DynArray<char> stamp(stamp_size);

	if (!mvIMPACT_C::IPL_Timestamp(
			stamp.getPointer()))
		throwImpactError();

	return std::string(stamp.getPointer());
}

inline bool checkLicenseBase()
//-----------------------------------------------------------------------------
{
	return (IPL_TRUE == mvIMPACT_C::BASE_CheckLic());
}

inline std::string getVersionBase()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::BASE_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

inline std::string getVersionFile()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::FL_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

inline std::string getVersionFileFilter(const std::string& filter)
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::FL_GetVersionFilter(filter.c_str(), sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}


inline std::string getVersionIPL()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::IPL_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

inline std::string getVersionSlice()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::SL_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

#ifndef I386_LINUX
inline std::string getVersionDisplay()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::DISP_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}
#endif // I386_LINUX

inline std::string getVersionAutotrace()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::ATR_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}


inline void checkModule
(
	const std::string& module
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::BASE_CheckModule(
			module.c_str()))
		throwImpactError();
}


inline void checkLicense
(
 const std::string& authentificator,
 const std::vector<unsigned char>& public_key,
 const std::string& module
 )
 //-----------------------------------------------------------------------------
{
  DynArray<unsigned char> _publicKey(public_key);
  if (!mvIMPACT_C::IPL_CheckLicense(
    authentificator.c_str(),
    (const mvIMPACT_C::IPL_CHAR*)_publicKey.getPointer(),
    static_cast<mvIMPACT_C::IPL_ULONG>(public_key.size()),
    module.c_str(),
    0,
    0,
    0,
    0))
    throwImpactError();
}


inline bool keyHit()
//-----------------------------------------------------------------------------
{
   // ASSERTs in IPL_CheckForInput require that when 'Block' is false, 'Echo'
   // should be false too, and 'Character' should be NULL.
   // So the only way to call IPL_CheckForInput non-blocking is as below.
	mvIMPACT_C::IPL_BOOL valid = IPL_FALSE; // What we assign here is irrelevant, the value is never read.
	if (!mvIMPACT_C::IPL_CheckForInput(
			&valid, IPL_FALSE, IPL_FALSE, NULL))
		throwImpactError();
	return valid == IPL_TRUE ? true : false;
}

inline void keyHitBlock
(
	bool echo
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::IPL_CheckForInput( NULL, IPL_TRUE,
			echo ? IPL_TRUE : IPL_FALSE , NULL))
		throwImpactError();
}

inline std::string getChar
(
	bool echo
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_CHAR ch = 0;
	if (!mvIMPACT_C::IPL_CheckForInput(
			NULL, IPL_TRUE, echo ? IPL_TRUE : IPL_FALSE, &ch))
		throwImpactError();
	return std::string(&ch, 1);
}





inline mvIMPACT_C::IPL_BUFHANDLE bufferToHandle(const BufferBase& buffer)
{
	return buffer.getHandle();
}


inline int getBufferLockTimeout() 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG timeout = 0;
	if (!mvIMPACT_C::BASE_GetBufferLockTimeout(&timeout))
		throwImpactError();
	return (int)timeout;
}

inline void setBufferLockTimeout(int timeoutMs) 
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::BASE_SetBufferLockTimeout(timeoutMs))
		throwImpactError();
}

///////////////////////////////////////////////////////////////
// former ipl module

inline Threshold::Threshold(
		int value,
		PointMode type
		)
		: m_value(value), m_type(type) {}


inline Threshold::operator mvIMPACT_C::IPL_THRESHOLD() const {
	mvIMPACT_C::IPL_THRESHOLD result;

	result.threshold_value = m_value;
	result.threshold_type =
		mvIMPACT_C::IPL_POINT_MODE(m_type);

	return result;
}


inline Threshold::Threshold(const mvIMPACT_C::IPL_THRESHOLD& r)
	: m_value(r.threshold_value),
	  m_type(PointMode(r.threshold_type))
{ }


inline int Threshold::getValue() const
{
	return m_value;
}

inline Threshold& Threshold::setValue(int value)
{
	m_value = value;
	return *this;
}

inline PointMode Threshold::getType() const
{
	return m_type;
}

inline Threshold& Threshold::setType(PointMode type)
{
	m_type = type;
	return *this;
}


inline Shuffle::operator mvIMPACT_C::IPL_SHUFFLE() const {
	mvIMPACT_C::IPL_SHUFFLE result;

	result.from = m_fromIndex;
	result.to = m_toIndex;

	return result;
}


inline Shuffle::Shuffle(const mvIMPACT_C::IPL_SHUFFLE& s)
	: m_fromIndex(s.from), m_toIndex(s.to)
{ }

inline Shuffle::Shuffle(int fromIndex, int toIndex)
	: m_fromIndex(fromIndex), m_toIndex(toIndex)
{ }

inline Shuffle::Shuffle() { }


inline int Shuffle::getFromIndex() const
{
	return m_fromIndex;
}

inline Shuffle& Shuffle::setFromIndex(int fromIndex)
{
	m_fromIndex = fromIndex;
	return *this;
}

inline int Shuffle::getToIndex() const
{
	return m_toIndex;
}

inline Shuffle& Shuffle::setToIndex(int toIndex)
{
	m_toIndex = toIndex;
	return *this;
}


inline PresetTransformation::PresetTransformation(
	double sx, double sy, double tx, double ty, double phi)
	: m_sx(sx), m_sy(sy), m_tx(tx), m_ty(ty), m_phi(phi)
{ }

inline PresetTransformation::operator const mvIMPACT_C::IPL_PRESET_GEOTRANS() const
{
	mvIMPACT_C::IPL_PRESET_GEOTRANS result;

	result.sx = m_sx;
	result.sy = m_sy;
	result.tx = m_tx;
	result.ty = m_ty;
	result.phi = m_phi;
	return result;
}

inline double PresetTransformation::getScaleX() const
{
	return m_sx;
}

inline PresetTransformation& PresetTransformation::setScaleX(double sx)
{
	m_sx = sx;
	return *this;
}


inline double PresetTransformation::getScaleY() const
{
	return m_sy;
}

inline PresetTransformation& PresetTransformation::setScaleY(double sy)
{
	m_sy = sy;
	return *this;
}

inline double PresetTransformation::getTranslationX() const
{
	return m_tx;
}

inline PresetTransformation& PresetTransformation::setTranslationX(double tx)
{
	m_tx = tx;
	return *this;
}


inline double PresetTransformation::getTranslationY() const
{
	return m_ty;
}

inline PresetTransformation& PresetTransformation::setTranslationY(double ty)
{
	m_ty = ty;
	return *this;
}

inline double PresetTransformation::getRotationAngle() const
{
	return m_phi;
}

inline PresetTransformation& PresetTransformation::setRotationAngle(double phi)
{
	m_phi = phi;
	return *this;
}


inline PresetColors::PresetColors(
	double black, double white)
	: m_black(black), m_white(white)
{ }

inline double PresetColors::getBlack() const
{
	return m_black;
}

inline PresetColors& PresetColors::setBlack(double black)
{
	m_black = black;
	return *this;
}


inline double PresetColors::getWhite() const
{
	return m_white;
}

inline PresetColors& PresetColors::setWhite(double white)
{
	m_white = white;
	return *this;
}



inline  void forwardFFT
(
	Image&       destReal,
	Image&       destImag,
	const Image& S,
	FFTMode          mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hDestReal = destReal.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hDestImag = destImag.getHandle();
	if (!mvIMPACT_C::IPL_TrFFT(
			S.getHandle(),
			-1,
			&hDestReal,
			&hDestImag,
			mvIMPACT_C::IPL_FORWARD_FFT,
			(mvIMPACT_C::IPL_FFT_MODE)mode))
		throwImpactError();
	if(hDestReal != destReal.getHandle()) {
		destReal = Image::imageFactory(hDestReal);
	}
	if(hDestImag != destImag.getHandle()) {
		destImag = Image::imageFactory(hDestImag);
	}

}

inline std::pair<Image,Image> forwardFFT
(
	const Image& S,
	FFTMode          mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE destReal = -1;
	mvIMPACT_C::IPL_BUFHANDLE destImag = -1;
	if (!mvIMPACT_C::IPL_TrFFT(
			S.getHandle(),
			-1,
			&destReal,
			&destImag,
			mvIMPACT_C::IPL_FORWARD_FFT,
			(mvIMPACT_C::IPL_FFT_MODE)mode))
		throwImpactError();
	return std::make_pair(Image::imageFactory(destReal),Image::imageFactory(destImag));
}





inline  void inverseFFT
(
	Image&       D,
	const Image& srcReal,
	const Image& srcImag,
	FFTMode          mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD      = D.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hUnused = -1;
	if (!mvIMPACT_C::IPL_TrFFT(
			srcReal.getHandle(),
			srcImag.getHandle(),
			&hD,
			&hUnused,
			mvIMPACT_C::IPL_INVERSE_FFT,
			(mvIMPACT_C::IPL_FFT_MODE)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image inverseFFT
(
	const Image& srcReal,
	const Image& srcImag,
	FFTMode          mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD      = -1;
	mvIMPACT_C::IPL_BUFHANDLE hUnused = -1;
	if (!mvIMPACT_C::IPL_TrFFT(
			srcReal.getHandle(),
			srcImag.getHandle(),
			&hD,
			&hUnused,
			mvIMPACT_C::IPL_INVERSE_FFT,
			(mvIMPACT_C::IPL_FFT_MODE)mode))
		throwImpactError();
	return Image::imageFactory(hD);
}





inline  void getSpectrum
(
	Image&       D,
	const Image& srcReal,
	const Image& srcImag,
	FFTSpectrum      spectrum,
	double           gamma ,
	FFTMode          mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_TrGetSpectrum(
			srcReal.getHandle(),
			srcImag.getHandle(),
			&hD,
			(mvIMPACT_C::IPL_FFT_SPECTRUM)spectrum,
			(mvIMPACT_C::IPL_FFT_MODE)mode,
			gamma))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image getSpectrum
(
	const Image& srcReal,
	const Image& srcImag,
	FFTSpectrum      spectrum,
	double           gamma ,
	FFTMode          mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrGetSpectrum(
			srcReal.getHandle(),
			srcImag.getHandle(),
			&D,
			(mvIMPACT_C::IPL_FFT_SPECTRUM)spectrum,
			(mvIMPACT_C::IPL_FFT_MODE)mode,
			gamma))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void dataWindow
(
	Image&  srcReal,
	Image&  srcImag,
	int        xPosWin,
	int        yPosWin,
	int        xSizeWin,
	int        ySizeWin,
	FFTData     data,
	FFTFunction function,
	FFTWindow   window,
	double      scale
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::IPL_TrDataWindow(
			srcReal.getHandle(),
			srcImag.getHandle(),
			xPosWin,
			yPosWin,
			xSizeWin,
			ySizeWin,
			(mvIMPACT_C::IPL_FFT_DATA)data,
			(mvIMPACT_C::IPL_FFT_FUNCTION)function,
			(mvIMPACT_C::IPL_FFT_WINDOW)window,
			scale))
		throwImpactError();
}





inline  void closing(
	Image& D,
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoClose(
			S.getHandle(),
			&hD,
			K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image closing(
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoClose(
			S.getHandle(),
			&D,
			K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void closing(
	Image& D,
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoClose(
			S.getHandle(),
			&hD,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image closing
(
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoClose(
			S.getHandle(),
			&D,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void connectivity(Image& D, const Image& S, const Palette& P)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoConnectivity(
			S.getHandle(),
			&hD,
			P.getHandle()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image connectivity(const Image& S, const Palette& P)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoConnectivity(
			S.getHandle(),
			&D,
			P.getHandle()))
		throwImpactError();
	return Image::imageFactory(D);
}


inline  void connectivity(Image& D, const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoConnectivity(
			S.getHandle(),
			&hD,
			IPL_DONT_CARE))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image connectivity(const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoConnectivity(
			S.getHandle(),
			&D,
			IPL_DONT_CARE))
		throwImpactError();
	return Image::imageFactory(D);
}




inline  void dilate
(
	Image& D,
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoDilate(
			S.getHandle(),
			&hD,
			K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image dilate
(
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoDilate(
			S.getHandle(),
			&D,
			K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void dilate
(
	Image& D,
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoDilate(
			S.getHandle(),
			&hD,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image dilate
(
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoDilate(
			S.getHandle(),
			&D,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void distance(Image& D, const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoDistance(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_DISTANCE))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image distance(const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoDistance(
			S.getHandle(),
			&D,
			mvIMPACT_C::IPL_DISTANCE))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void erode
(
	Image& D,
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoErode(
			S.getHandle(),
			&hD,
			K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image erode
(
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoErode(
			S.getHandle(),
			&D,
			K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void erode
(
	Image& D,
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoErode(
			S.getHandle(),
			&hD,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image erode
(
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoErode(
			S.getHandle(),
			&D,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void hitOrMiss(
	Image& D,
	const Image& S,
	const StructuringElement& K,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoHitOrMiss(
			S.getHandle(),
			&hD,
			K.getHandle(),
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image hitOrMiss(
	const Image& S,
	const StructuringElement& K,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoHitOrMiss(
			S.getHandle(),
			&D,
			K.getHandle(),
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void hitOrMiss(
	Image& D,
	const Image& S,
	StockStructuringElement K,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoHitOrMiss(
			S.getHandle(),
			&hD,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image hitOrMiss
(
	const Image& S,
	StockStructuringElement K,
	MorphOpMode mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoHitOrMiss(
			S.getHandle(),
			&D,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void label(
	Image& D,
	const Image& S,
	int* count,
	NeighborhoodMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG ucount = 0;

	if (S.getBands() > 1 || S.getSlices() > 1 || S.getFrames() > 1)
		throwImpactException("Parameter mismatch. "
			"Bands, slices and frames must be 1");
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();

	mvIMPACT_C::IPL_LABEL_MODE labelMode;
	switch(mode)
	{
		case nm4Connected:
			labelMode = mvIMPACT_C::IPL_RECT4_3x3;
			break;
		case nm8Connected:
			labelMode = mvIMPACT_C::IPL_RECT8_3x3;
			break;
	    default:
			 throwImpactException("Unknown value for mode");
	}

	if (!mvIMPACT_C::IPL_MoLabel(
			S.getHandle(),
			&hD,
			1,
			labelMode,
			&ucount))
		throwImpactError();

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}

	if(ucount > INT_MAX) { throwImpactException("Count out of range for type int"); }
	else if (count) { *count = (int)ucount; }
}

inline  Image label(
	const Image& S,
	int* count,
	NeighborhoodMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG ucount = 0;

	if (S.getBands() > 1 || S.getSlices() > 1 || S.getFrames() > 1)
		throwImpactException("Parameter mismatch. "
			"Bands, slices and frames must be 1");
	mvIMPACT_C::IPL_BUFHANDLE D = -1;

	mvIMPACT_C::IPL_LABEL_MODE labelMode;
	switch(mode)
	{
		case nm4Connected:
			labelMode = mvIMPACT_C::IPL_RECT4_3x3;
			break;
		case nm8Connected:
			labelMode = mvIMPACT_C::IPL_RECT8_3x3;
			break;
	    default:
			throwImpactException("Unknown value for mode");
	}

	if (!mvIMPACT_C::IPL_MoLabel(
			S.getHandle(),
			&D,
			1,
			labelMode,
			&ucount))
		throwImpactError();

	if(ucount > INT_MAX) { throwImpactException("Count out of range for type int"); }
	else if (count) { *count = (int)ucount; }

	return Image::imageFactory(D);
}





inline  void medialAxis(Image& D, const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoDistance(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_MEDIAL_AXIS))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image medialAxis(const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoDistance(
			S.getHandle(),
			&D,
			mvIMPACT_C::IPL_MEDIAL_AXIS))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void opening(
	Image& D,
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoOpen(
			S.getHandle(),
			&hD,
			K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image opening(
	const Image& S,
	const StructuringElement& K,
	int iterations,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoOpen(
			S.getHandle(),
			&D, K.getHandle(),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void opening(
	Image& D,
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoOpen(
			S.getHandle(),
			&hD,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image opening(
	const Image& S,
	StockStructuringElement K,
	int iterations,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoOpen(
			S.getHandle(),
			&D,
			static_cast< mvIMPACT_C::IPL_BUFHANDLE >(K),
			iterations,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}


inline  void sizeFilter(
	Image& D,
	const Image& S,
	int size,
	SizeMode mode,
	ObjectState state)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoSizeFilter(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_SIZE_MODE(mode),
			size,
			0,
			mvIMPACT_C::IPL_OBJECT_STATES(state),
			mvIMPACT_C::IPL_RECT_3x3,
			NULL))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image sizeFilter(
	const Image& S,
	int size,
	SizeMode mode,
	ObjectState state)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoSizeFilter(
			S.getHandle(),
			&D,
			mvIMPACT_C::IPL_SIZE_MODE(mode),
			size,
			0,
			mvIMPACT_C::IPL_OBJECT_STATES(state),
			mvIMPACT_C::IPL_RECT_3x3,
			NULL))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void ultimate(Image& D, const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoUltimate(
			S.getHandle(),
			&hD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image ultimate(const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoUltimate(
			S.getHandle(),
			&D))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void watershed(
	Image& D,
	const Image& S,
	WatershedMode mode,
	int color)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoWaterShed(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_WATERSHED_MODE(mode),
			color,
			0))		// the threshold parameter is bogus, fix in IPL
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image watershed(
	const Image& S,
	WatershedMode mode,
	int color)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoWaterShed(
			S.getHandle(),
			&D,
			mvIMPACT_C::IPL_WATERSHED_MODE(mode),
			color,
			0))		// the threshold parameter is bogus, fix in IPL
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void tophat(
	Image& D,
	const Image& S,
	int size,
	PolarityMode mode,
	int threshold)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_TOPHAT_MODE iplMode;
	switch (mode) {
	case pmWhiteOnBlack:
		iplMode = mvIMPACT_C::IPL_WHITE_OBJECTS;
		break;
	case pmBlackOnWhite:
		iplMode = mvIMPACT_C::IPL_BLACK_OBJECTS;
		break;
	case pmAuto:
	default:
		throwImpactException("Unknown value for mode");
	}

	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoTophat(
			S.getHandle(),
			&hD,
			size,
			iplMode,
			threshold))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image tophat(
	const Image& S,
	int size,
	PolarityMode mode,
	int threshold)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_TOPHAT_MODE iplMode;
	switch (mode) {
	case pmWhiteOnBlack:
		iplMode = mvIMPACT_C::IPL_WHITE_OBJECTS;
		break;
	case pmBlackOnWhite:
		iplMode = mvIMPACT_C::IPL_BLACK_OBJECTS;
		break;
	case pmAuto:
	default:
		throwImpactException("Unknown value for mode");
	}
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoTophat(
			S.getHandle(),
			&D,
			size,
			iplMode,
			threshold))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void reconstruct(
	Image& D,
	const Image& S,
	const Image& M,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&hD,
			M.getHandle(),
			mvIMPACT_C::IPL_RECONSTRUCT_MARKER,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image reconstruct(
	const Image& S,
	const Image& M,
	MorphOpMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&D,
			M.getHandle(),
			mvIMPACT_C::IPL_RECONSTRUCT_MARKER,
			mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void fillHoles(
	Image& D,
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&hD,
			IPL_DONT_CARE,
			mvIMPACT_C::IPL_FILL_HOLES_BLOBS,
			mvIMPACT_C::IPL_BINARY))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image fillHoles(
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&D,
			IPL_DONT_CARE,
			mvIMPACT_C::IPL_FILL_HOLES_BLOBS,
			mvIMPACT_C::IPL_BINARY))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void extractHoles(
	Image& D,
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&hD,
			IPL_DONT_CARE,
			mvIMPACT_C::IPL_EXTRACT_HOLES_BLOBS,
			mvIMPACT_C::IPL_BINARY))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image extractHoles(
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&D,
			IPL_DONT_CARE,
			mvIMPACT_C::IPL_EXTRACT_HOLES_BLOBS,
			mvIMPACT_C::IPL_BINARY))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void killBorderObjects(
	Image& D,
	const Image& S,
	bool top,
	bool left,
	bool bottom,
	bool right)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&hD,
			IPL_DONT_CARE,
			mvIMPACT_C::IPL_KILL_BORDER_BLOBS,
			static_cast<mvIMPACT_C::IPL_MORPH_OPMODE>(
				mvIMPACT_C::IPL_BINARY |
				top ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_TOP : 0 |
					left ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_LEFT : 0 |
					bottom ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_BOTTOM : 0 |
					right ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_RIGHT : 0 )))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image killBorderObjects(
	const Image& S,
	bool top,
	bool left,
	bool bottom,
	bool right)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_MoReconstruct(
			S.getHandle(),
			&D,
			IPL_DONT_CARE,
			mvIMPACT_C::IPL_KILL_BORDER_BLOBS,
			static_cast<mvIMPACT_C::IPL_MORPH_OPMODE>(
				mvIMPACT_C::IPL_BINARY |
					top ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_TOP : 0 |
					left ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_LEFT : 0 |
					bottom ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_BOTTOM : 0 |
					right ? (mvIMPACT_C::IPL_LONG)mvIMPACT_C::IPL_RIGHT : 0 )))
		throwImpactError();
	return Image::imageFactory(D);
}

inline Image rankFilter(
	const Image& S,
	const StructuringElement &K,
	int rankIndex,
	MorphOpMode mode
	)
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	if(IPL_TRUE != mvIMPACT_C::IPL_Rank(
		S.getHandle(),
		&D,
		K.getHandle(),
		rankIndex,
		mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
	{ throwImpactError(); }

	return Image::imageFactory(D);
}

inline Image rankFilter(
	const Image& S,
	StockStructuringElement K,
	MorphOpMode mode
	)
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	if(IPL_TRUE != mvIMPACT_C::IPL_Rank(
		S.getHandle(),
		&D,
		K,
		(mvIMPACT_C::IPL_ULONG)mvIMPACT_C::IPL_MEDIAN,
		mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
	{ throwImpactError(); }

	return Image::imageFactory(D);
}

inline void rankFilter(
	Image& D,
	const Image& S,
	const StructuringElement &K,
	int rankIndex,
	MorphOpMode mode
	)
 {
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(IPL_TRUE != mvIMPACT_C::IPL_Rank(
		S.getHandle(),
		&hD,
		K.getHandle(),
		rankIndex,
		mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
	{ throwImpactError(); }

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline void rankFilter(
	Image& D,
	const Image& S,
	StockStructuringElement K,
	MorphOpMode mode
	)
 {
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(IPL_TRUE != mvIMPACT_C::IPL_Rank(
		S.getHandle(),
		&hD,
		K,
		(mvIMPACT_C::IPL_ULONG)mvIMPACT_C::IPL_MEDIAN,
		mvIMPACT_C::IPL_MORPH_OPMODE(mode)))
	{ throwImpactError(); }

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline Image thick(
	    const Image& S,
	    const StructuringElement& K,
	    int iterations,
            MorphOpMode morphOpMode,
	    RotationMode rotateMode)
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	if(IPL_TRUE != mvIMPACT_C::IPL_MoThick(
		S.getHandle(),
		&D,
		K.getHandle(),
		(mvIMPACT_C::IPL_ULONG)iterations,
		mvIMPACT_C::IPL_MORPH_OPMODE(morphOpMode),
		mvIMPACT_C::IPL_KERNEL_ROTATE(rotateMode)))
	{ throwImpactError(); }

	return Image::imageFactory(D);
}


inline void thick(
	    Image& D,
	    const Image& S,
	    const StructuringElement& K,
	    int iterations,
            MorphOpMode morphOpMode,
	    RotationMode rotateMode)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(IPL_TRUE != mvIMPACT_C::IPL_MoThick(
		S.getHandle(),
		&hD,
		K.getHandle(),
		(mvIMPACT_C::IPL_ULONG)iterations,
		mvIMPACT_C::IPL_MORPH_OPMODE(morphOpMode),
		mvIMPACT_C::IPL_KERNEL_ROTATE(rotateMode)))
	{ throwImpactError(); }

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline Image thin(
	    const Image& S,
	    const StructuringElement& K,
	    int iterations,
            MorphOpMode morphOpMode,
	    RotationMode rotateMode)
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	if(IPL_TRUE != mvIMPACT_C::IPL_MoThin(
		S.getHandle(),
		&D,
		K.getHandle(),
		(mvIMPACT_C::IPL_ULONG)iterations,
		mvIMPACT_C::IPL_MORPH_OPMODE(morphOpMode),
		mvIMPACT_C::IPL_KERNEL_ROTATE(rotateMode)))
	{ throwImpactError(); }

	return Image::imageFactory(D);
}


inline void thin(
	    Image& D,
	    const Image& S,
	    const StructuringElement& K,
	    int iterations,
            MorphOpMode morphOpMode,
	    RotationMode rotateMode)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(IPL_TRUE != mvIMPACT_C::IPL_MoThin(
		S.getHandle(),
		&hD,
		K.getHandle(),
		(mvIMPACT_C::IPL_ULONG)iterations,
		mvIMPACT_C::IPL_MORPH_OPMODE(morphOpMode),
		mvIMPACT_C::IPL_KERNEL_ROTATE(rotateMode)))
	{ throwImpactError(); }

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline Image scaleLuminance(
	const Image& S,
	double offset,
	double gain,
	SaturationMode mode)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if(!mvIMPACT_C::IPL_PtScale (
		S.getHandle(),
		&hD,
		offset,
		gain,
		(mvIMPACT_C::IPL_LONG)mode))
	{ throwImpactError(); }

	return Image::imageFactory(hD);
}


inline void scaleLuminance(
	Image& D,
	const Image& S,
	double offset,
	double gain,
	SaturationMode mode)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(!mvIMPACT_C::IPL_PtScale (
		S.getHandle(),
		&hD,
		offset,
		gain,
		(mvIMPACT_C::IPL_LONG)mode))
	{ throwImpactError(); }
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}



inline  void generatePalette(
	Palette& D,
	int width,
	int height,
	double luminance,
	double contrast,
	double gamma )
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  hD = D.getHandle();
	if (!mvIMPACT_C::IPL_LtGenerateLGC(&hD, width, height, luminance, contrast, gamma))
		throwImpactError();

	if (hD != D.getHandle())
		D = Palette::paletteFactory(hD);
}

inline  Palette generatePalette(
	int width,
	int height,
	double luminance,
	double contrast,
	double gamma )
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  D = -1;
	if (!mvIMPACT_C::IPL_LtGenerateLGC(&D, width, height, luminance, contrast, gamma))
		throwImpactError();
	return Palette::paletteFactory(D);
}

inline  void generateRampPalette(
	Palette& D,
	Segment function,
	const Point& from,
	const Point& to)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  hD = D.getHandle();
	if (!mvIMPACT_C::IPL_LtGenerate(
			&hD,
			mvIMPACT_C::IPL_LUTSEGMENT(function),
			from.getX(),
			from.getY(),
			to.getX(),
			to.getY()))
		throwImpactError();

	if (hD != D.getHandle())
		D = Palette::paletteFactory(hD);
}

inline  Palette generateRampPalette(
	Segment function,
	const Point& from,
	const Point& to)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  D = -1;
	if (!mvIMPACT_C::IPL_LtGenerate(
			&D,
			mvIMPACT_C::IPL_LUTSEGMENT(function),
			from.getX(),
			from.getY(),
			to.getX(),
			to.getY()))
		throwImpactError();
	return Palette::paletteFactory(D);
}

inline  void generateRampPalette(
	Palette& D,
	const Point& from,
	const Point& to)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  hD = D.getHandle();
	if (!mvIMPACT_C::IPL_LtRamp(
			&hD,
			from.getX(),
			from.getY(),
			to.getX(),
			to.getY()))
		throwImpactError();

	if (hD != D.getHandle())
		D = Palette::paletteFactory(hD);
}

inline  Palette generateRampPalette(
	const Point& from,
	const Point& to)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  D = -1;
	if (!mvIMPACT_C::IPL_LtRamp(
			&D,
			from.getX(),
			from.getY(),
			to.getX(),
			to.getY()))
		throwImpactError();
	return Palette::paletteFactory(D);
}


inline  void generateShufflePalette
(
	Palette& D,
	const std::vector<Shuffle>& shuffle
)
//-----------------------------------------------------------------------------
{
	DynArray<mvIMPACT_C::IPL_SHUFFLE, Shuffle> iplShuffle(shuffle);

	mvIMPACT_C::IPL_BUFHANDLE  hD = D.getHandle();
	if (!mvIMPACT_C::IPL_ShuffleBitplanePalette(
			iplShuffle.getPointer(),
			static_cast<mvIMPACT_C::IPL_ULONG>(shuffle.size()),
			&hD))
		throwImpactError();

	if (hD != D.getHandle())
		D = Palette::paletteFactory(hD);
}

inline  Palette generateShufflePalette
(
	const std::vector<Shuffle>& shuffle
)
//-----------------------------------------------------------------------------
{
	DynArray<mvIMPACT_C::IPL_SHUFFLE, Shuffle> iplShuffle(shuffle);

	mvIMPACT_C::IPL_BUFHANDLE  D = -1;
	if (!mvIMPACT_C::IPL_ShuffleBitplanePalette(
			iplShuffle.getPointer(),
			static_cast<mvIMPACT_C::IPL_ULONG>(shuffle.size()),
			&D))
		throwImpactError();
	return Palette::paletteFactory(D);
}

inline  void generateThresholdPalette
(
	Palette& D,
	const std::vector<Threshold>& thresholds,
	BufferDatatype datatype
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  hD = D.getHandle();
	DynArray<mvIMPACT_C::IPL_THRESHOLD, Threshold> iplThresholds(thresholds);
	if (!mvIMPACT_C::IPL_ThresholdPalette(
			iplThresholds.getPointer(),
			static_cast<mvIMPACT_C::IPL_ULONG>(thresholds.size()),
			&hD,
			mvIMPACT_C::IPL_BUFFER_DATATYPE(datatype)))
		throwImpactError();

	if (hD != D.getHandle())
		D = Palette::paletteFactory(hD);
}

inline  Palette generateThresholdPalette
(
	const std::vector<Threshold>& thresholds,
	BufferDatatype datatype
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	DynArray<mvIMPACT_C::IPL_THRESHOLD, Threshold> iplThresholds(thresholds);

	if (!mvIMPACT_C::IPL_ThresholdPalette
	(
		iplThresholds.getPointer(),
		static_cast<mvIMPACT_C::IPL_ULONG>(thresholds.size()),
		&D,
		mvIMPACT_C::IPL_BUFFER_DATATYPE(datatype))
	) throwImpactError();
	return Palette::paletteFactory(D);
}




inline Image absolute(
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &D, mvIMPACT_C::IPL_ABS))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void absolute(
	Image& D,
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &hD, mvIMPACT_C::IPL_ABS))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline Image add(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(
			S.getHandle(),
			value,
			&D,
			mvIMPACT_C::IPL_ADD_CST))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void add(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(
			S1.getHandle(),
			S2.getHandle(),
			&hD,
			mvIMPACT_C::IPL_ADD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline void add(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(
			S.getHandle(),
			value,
			&hD,
			mvIMPACT_C::IPL_ADD_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline Image add(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(
			S1.getHandle(),
			S2.getHandle(),
			&D,
			mvIMPACT_C::IPL_ADD))
		throwImpactError();
	return Image::imageFactory(D);
}


inline Image andImage(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_AND_CST))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void andImage(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_AND))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline void andImage(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_AND_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline Image andImage(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_AND))
		throwImpactError();
	return Image::imageFactory(D);
}





inline Image clip(
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &D, mvIMPACT_C::IPL_CLIP))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void clip(
	Image& D,
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &hD, mvIMPACT_C::IPL_CLIP))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}



inline Image compareEqual(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_EQUAL_CST))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void compareEqual(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_EQUAL))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void compareEqual(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_EQUAL_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image compareEqual(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_EQUAL))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void compareGreater(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_GREATER))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void compareGreater(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_GREATER_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image compareGreater(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_GREATER))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image compareGreater(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_GREATER_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline Image compareGreaterOrEqual(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_GREATER_OR_EQUAL_CST))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void compareGreaterOrEqual(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_GREATER_OR_EQUAL))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void compareGreaterOrEqual(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_GREATER_OR_EQUAL_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image compareGreaterOrEqual(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_GREATER_OR_EQUAL))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void compareLess(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_LESS))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void compareLess(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_LESS_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image compareLess(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_LESS))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image compareLess(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_LESS_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void compareLessOrEqual(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_LESS_OR_EQUAL))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void compareLessOrEqual(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_LESS_OR_EQUAL_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image compareLessOrEqual(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_LESS_OR_EQUAL))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image compareLessOrEqual(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_LESS_OR_EQUAL_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void compareNotEqual(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_NOT_EQUAL))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void compareNotEqual(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_NOT_EQUAL_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image compareNotEqual(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_NOT_EQUAL))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image compareNotEqual(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_NOT_EQUAL_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void divide(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_DIV))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void divide(
	Image& D,
	int value,
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(value, S.getHandle(), &hD, mvIMPACT_C::IPL_DIV_CST_SRC2))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void divide(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_DIV_SRC1_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image divide(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_DIV))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image divide(
	int value,
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(value, S.getHandle(), &D, mvIMPACT_C::IPL_DIV_CST_SRC2))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image divide(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_DIV_SRC1_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void map2(
	Image& D,
	const Image& S1,
	const Image& S2,
	const Palette& P)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtLutMap(
			S1.getHandle(),
			S2.getHandle(),
			&hD,
			P.getHandle()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image map2
(
	const Image& S1,
	const Image& S2,
	const Palette& P
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtLutMap(
			S1.getHandle(),
			S2.getHandle(),
			&D,
			P.getHandle()))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void mapLut
(
	Image& D,
	const Image& S,
	const Palette& P
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtLutMap(
			S.getHandle(),
			-1,
			&hD,
			P.getHandle()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image mapLut
(
	const Image& S,
	const Palette& P
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtLutMap(
			S.getHandle(),
			-1,
			&D,
			P.getHandle()))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void multiply(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_MUL))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void multiply(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_MUL_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image multiply(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_MUL))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image multiply(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_MUL_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void nand(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_NAND))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void nand(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_NAND_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image nand(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_NAND))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image nand(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_NAND_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void neg(
	Image& D,
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &hD, mvIMPACT_C::IPL_NEG))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image neg(
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &D, mvIMPACT_C::IPL_NEG))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void nor(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_NOR))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void nor(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_NOR_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image nor(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_NOR))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image nor(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_NOR_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void notImage(
	Image& D,
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &hD, mvIMPACT_C::IPL_NOT))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image notImage(
	const Image& S1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), -1, &D, mvIMPACT_C::IPL_NOT))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void orImage(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_OR))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void orImage(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_OR_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image orImage(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_OR))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image orImage(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_OR_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline Image overlay
(
	const std::vector<Image>&   Images,
	const std::vector<Palette>& MappingLUTs,
	const std::vector<Palette>& DisplayLUTs
)
//-----------------------------------------------------------------------------
{
	size_t image_count = Images.size();
	size_t map_count = MappingLUTs.size();
	size_t disp_count = DisplayLUTs.size();

	if (image_count < 2)
		throw ImpactException(mvIMPACT_C::IPL_INVALID_PARAM_ERROR);

	if (map_count != 0 && map_count != image_count)
		throw ImpactException(mvIMPACT_C::IPL_DIM_SIZEMISMATCH_ERROR);

	if (disp_count != 0 && disp_count != image_count)
		throw ImpactException(mvIMPACT_C::IPL_DIM_SIZEMISMATCH_ERROR);

	std::vector<mvIMPACT_C::IPL_BUFHANDLE> image_ids(Images.size());
	std::transform(Images.begin(), Images.end(), std::back_inserter(image_ids), bufferToHandle);

	std::vector<mvIMPACT_C::IPL_BUFHANDLE> map_ids;
	if (map_count > 0)
	{
		std::transform(MappingLUTs.begin(), MappingLUTs.end(), std::back_inserter(map_ids), bufferToHandle);
	}
	else
	{
		map_ids.resize(image_count);
		std::fill(map_ids.begin(), map_ids.end(), IPL_DONT_CARE);
	}

	std::vector<mvIMPACT_C::IPL_BUFHANDLE> disp_ids;
	if (disp_count > 0)
	{
		std::transform(DisplayLUTs.begin(), DisplayLUTs.end(), std::back_inserter(disp_ids), bufferToHandle);
	}
	else
	{
		disp_ids.resize(image_count);
		std::fill(disp_ids.begin(), disp_ids.end(), IPL_DONT_CARE);
	}

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplImageIds(image_ids);
	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplMapIds(map_ids);
	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplDispIds(disp_ids);

	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_BufOverlay(
			iplImageIds.getPointer(),
			iplMapIds.getPointer(),
			iplDispIds.getPointer(),
			(mvIMPACT_C::IPL_ULONG)image_count,
			&D))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void setConstant(
	BufferBase& D,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(value, -1, &hD, mvIMPACT_C::IPL_PASS_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline  void shadingCorrection
(
	Image&       D,
	const Image& S,
	const Image& whiteReference,
	int             blackMeanGrayValue,
	ShadingMode      mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_SHADING_MODE m;
	m = mvIMPACT_C::IPL_MULTIPLICATIVE_BLACK_CST;
	if (mode == smAdditive)	m = mvIMPACT_C::IPL_ADDITIVE_BLACK_CST;

	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtShadingCorrection(
			S.getHandle(),
			&hD,
			whiteReference.getHandle(),
			blackMeanGrayValue,
			m))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image shadingCorrection
(
	const Image& S,
	const Image& whiteReference,
	int             blackMeanGrayValue,
	ShadingMode      mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_SHADING_MODE m;
	m = mvIMPACT_C::IPL_MULTIPLICATIVE_BLACK_CST;
	if (mode == smAdditive)	m = mvIMPACT_C::IPL_ADDITIVE_BLACK_CST;

	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	if (!mvIMPACT_C::IPL_PtShadingCorrection(
			S.getHandle(),
			&hD,
			whiteReference.getHandle(),
			blackMeanGrayValue,
			m))
		throwImpactError();
	return Image::imageFactory(hD);
}

inline  void shadingCorrection
(
	Image&       D,
	const Image& S,
	const Image& whiteReference,
	const Image& blackReference,
	ShadingMode      mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtShadingCorrection(
			S.getHandle(),
			&hD,
			whiteReference.getHandle(),
			blackReference.getHandle(),
			(mvIMPACT_C::IPL_SHADING_MODE)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image shadingCorrection
(
	const Image& S,
	const Image& whiteReference,
	const Image& blackReference,
	ShadingMode      mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	if (!mvIMPACT_C::IPL_PtShadingCorrection(
			S.getHandle(),
			&hD,
			whiteReference.getHandle(),
			blackReference.getHandle(),
			(mvIMPACT_C::IPL_SHADING_MODE)mode))
		throwImpactError();
	return Image::imageFactory(hD);
}





inline  void subtract(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_SUB))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void subtract(
	Image& D,
	int value,
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(value, S.getHandle(), &hD, mvIMPACT_C::IPL_SUB_CST_SRC2))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void subtract(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_SUB_SRC1_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image subtract(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_SUB))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image subtract(
	int value,
	const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(value, S.getHandle(), &D, mvIMPACT_C::IPL_SUB_CST_SRC2))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image subtract(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_SUB_SRC1_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void subtractAbsolute(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_SUB_ABS))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void subtractAbsolute(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_SUB_ABS_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image subtractAbsolute(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_SUB_ABS))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image subtractAbsolute(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_SUB_ABS_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void subtractClipped(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_SUB_CLIP))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void subtractClipped(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_SUB_CLIP_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image subtractClipped(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_SUB_CLIP))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image subtractClipped(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_SUB_CLIP_CST))
		throwImpactError();
	return Image::imageFactory(D);
}


inline Image weightedAdd(
		     const Image& S1,
		     const Image& S2,
		     const Image& Weight
		     )
{
	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_PtToPointMix(
		S1.getHandle(),
		S2.getHandle(),
		Weight.getHandle(),
		&hD,
		mvIMPACT_C::IPL_ADD))
	{ throwImpactError(); }
	return Image::imageFactory(hD);
}


inline void weightedAdd(
		     Image& D,
		     const Image& S1,
		     const Image& S2,
		     const Image& Weight
		     )
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPointMix(
		S1.getHandle(),
		S2.getHandle(),
		Weight.getHandle(),
		&hD,
		mvIMPACT_C::IPL_ADD))
	{ throwImpactError(); }
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline Image weightedAdd(
		     const Image& S1,
		     const Image& S2,
		     int weight
		     )
{
	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_PtToPointMix(
		S1.getHandle(),
		S2.getHandle(),
		(mvIMPACT_C::IPL_BUFHANDLE)weight,
		&hD,
		mvIMPACT_C::IPL_ADD_CST))
	{ throwImpactError(); }
	return Image::imageFactory(hD);
}


inline void weightedAdd(
		     Image& D,
		     const Image& S1,
		     const Image& S2,
		     int weight
		     )
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPointMix(
		S1.getHandle(),
		S2.getHandle(),
		(mvIMPACT_C::IPL_BUFHANDLE)weight,
		&hD,
		mvIMPACT_C::IPL_ADD_CST))
	{ throwImpactError(); }
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}






inline  void xnor(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_XNOR))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void xnor(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_XNOR_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image xnor(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_XNOR))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image xnor(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_XNOR_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void xorImage(
	Image& D,
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &hD, mvIMPACT_C::IPL_XOR))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  void xorImage(
	Image& D,
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &hD, mvIMPACT_C::IPL_XOR_CST))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image xorImage(
	const Image& S1,
	const Image& S2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S1.getHandle(), S2.getHandle(), &D, mvIMPACT_C::IPL_XOR))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image xorImage(
	const Image& S,
	int value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtToPoint(S.getHandle(), value, &D, mvIMPACT_C::IPL_XOR_CST))
		throwImpactError();
	return Image::imageFactory(D);
}





inline int bestThreshold(const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_LONG threshold = 0;
	if (!mvIMPACT_C::IPL_StatBestThreshold(
			S.getHandle(),
			&threshold))
		throwImpactError();
	return threshold;
}


inline int getSingleLocalMinimum(const Histogram& S)
{
	mvIMPACT_C::IPL_BUFHANDLE ResultHandle = IPL_DONT_CARE;
	if(!mvIMPACT_C::IPL_StatHistogramLocalMin(
		S.getHandle(),
		&ResultHandle,
		IPL_DONT_CARE,
		IPL_DONT_CARE,
		mvIMPACT_C::IPL_SINGLE_THRES))
	{ throwImpactError(); }

	mvIMPACT_C::IPL_LONG result;
	if(!mvIMPACT_C::IPL_GetBufData(
		ResultHandle,
		0,0,0,0,0,
		1,1,
		&result))
	{ throwImpactError(); }

	if(!mvIMPACT_C::IPL_BufFree(ResultHandle))
	{ throwImpactError(); }

	return (int)result;
}

inline std::vector<int> getLocalMinima
	(
		const Histogram& S,
		double range1,
		double range2
	)
{
	mvIMPACT_C::IPL_BUFHANDLE ResultHandle = IPL_DONT_CARE;
	if(!mvIMPACT_C::IPL_StatHistogramLocalMin(
		S.getHandle(),
		&ResultHandle,
		range1,
		range2,
		mvIMPACT_C::IPL_MULT_THRES))
	{ throwImpactError(); }

	mvIMPACT_C::IPL_LONG count;
	if(!mvIMPACT_C::IPL_BufInquire(ResultHandle, mvIMPACT_C::IPL_XSIZE, &count))
	{ throwImpactError(); }

	DynArray<mvIMPACT_C::IPL_LONG, int> resultArray(count);
	if(!mvIMPACT_C::IPL_GetBufData(
		ResultHandle,
		0,0,0,0,0,
		count,1,
		resultArray.getPointer()))
	{ throwImpactError(); }

	if(!mvIMPACT_C::IPL_BufFree(ResultHandle))
	{ throwImpactError(); }

	return resultArray.asVector();
}


inline Histogram calcHistogram(const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE H = -1;
	if (!mvIMPACT_C::IPL_StatHistogram(S.getHandle(), &H))
		throwImpactError();
	return Histogram::histogramFactory(H);
}

inline void calcHistogram(Histogram& D, const Image& S)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_StatHistogram(S.getHandle(), &hD))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Histogram::histogramFactory(hD);
	}
}





inline Profile calcLineProfile(const Image& S, ProfileMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE P = -1;
	if (!mvIMPACT_C::IPL_StatProfile(S.getHandle(), &P, mvIMPACT_C::IPL_PROFILE_MODE(mode)))
		throwImpactError();
	return Profile::profileFactory(P);
}

inline  void calcLineProfile(Profile& D, const Image& S, const Line& line)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_StatLineProfile(
			S.getHandle(),
			&hD,
			line.getP0().getX(),
			line.getP0().getY(),
			line.getP1().getX(),
			line.getP1().getY()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Profile::profileFactory(hD);
	}
}

inline  Profile calcLineProfile(const Image& S, const Line& line)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE P = -1;
	if (!mvIMPACT_C::IPL_StatLineProfile(
			S.getHandle(),
			&P,
			line.getP0().getX(),
			line.getP0().getY(),
			line.getP1().getX(),
			line.getP1().getY()))
		throwImpactError();
	return Profile::profileFactory(P);
}

inline  void calcLineProfile(Profile& D, const Image& S, ProfileMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_StatProfile(S.getHandle(), &hD, (mvIMPACT_C::IPL_PROFILE_MODE)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Profile::profileFactory(hD);
	}
}


inline Profile calcPolygonProfile(
		    const Image& S,
		    const std::vector<Point> & polygonPoints)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	DynArray<mvIMPACT_C::IPL_POINT, Point> iplPolygonPoints(polygonPoints);

	if(!mvIMPACT_C::IPL_StatCurveProfile (
		S.getHandle(),
		&hD,
		iplPolygonPoints.getPointer(),
		iplPolygonPoints.getSize()))
	{ throwImpactError(); }

	return Profile::profileFactory(hD);
}



inline Image calcProfileImage(const Image& S,
                       const Point& from,
                       const Point& to,
		       int size,
		       GeometricInterpolation mode
		       )
{
	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;

	if(!mvIMPACT_C::IPL_StatLineProfileImage (
		S.getHandle(),
		&hD,
		from.getX(),
		from.getY(),
		to.getX(),
		to.getY(),
		size,
		mvIMPACT_C::IPL_GEOTRANS(mode)))
	{ throwImpactError(); }

	return Image::imageFactory(hD);
}


inline void calcProfileImage(Image& D,
		       const Image& S,
                       const Point& from,
                       const Point& to,
		       int size,
		       GeometricInterpolation mode
		       )
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();

	if(!mvIMPACT_C::IPL_StatLineProfileImage (
		S.getHandle(),
		&hD,
		from.getX(),
		from.getY(),
		to.getX(),
		to.getY(),
		size,
		mvIMPACT_C::IPL_GEOTRANS(mode)))
	{ throwImpactError(); }
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}



inline  void convolve(
	Image& D,
	const Image& S,
	StockKernel K)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_Convolve(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_BUFHANDLE(K)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image convolve(
	const Image& S,
	StockKernel K)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_Convolve(
			S.getHandle(),
			&D,
			mvIMPACT_C::IPL_BUFHANDLE(K)))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void convolve(
	Image& D,
	const Image& S,
	const Kernel& K)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_Convolve(
			S.getHandle(),
			&hD,
			K.getHandle()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image convolve(
	const Image& S,
	const Kernel& K)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_Convolve(
			S.getHandle(),
			&D,
			K.getHandle()))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  double difference(const Image& S1, const Image& S2)
//-----------------------------------------------------------------------------
{
	double diff = 0.0;
	if (!mvIMPACT_C::IPL_Difference(
			S1.getHandle(),
			S2.getHandle(),
			&diff))
		throwImpactError();
	return diff;
}


inline int compare(const Image& S1, const Image& S2)
{
	ResultBuf resultBuf(1, S1.getBands(), S1.getSlices(), S1.getFrames(), bdULong);

	if(!mvIMPACT_C::IPL_StatGetBufferStatistics (
		S1.getHandle(),
		S2.getHandle(),
		resultBuf.getHandle(),
		mvIMPACT_C::IPL_COUNT_DIFFERENCES))
	{ throwImpactError(); }

	int differences=0;

	for (int frame=0; frame<S1.getFrames(); ++frame)
		for (int slice=0; slice<S1.getSlices(); ++slice)
			for (int band=0; band<S1.getBands(); ++band)
				differences += (int)resultBuf.getIntValue(0, band, slice, frame);

	return differences;
}

inline int calcLineProfileLength(const Line& line)
{
	mvIMPACT_C::IPL_ULONG length;
	if(!mvIMPACT_C::IPL_StatGetLineProfileLength (
		(mvIMPACT_C::IPL_ULONG)line.getP0().getX(),
		(mvIMPACT_C::IPL_ULONG)line.getP0().getY(),
		(mvIMPACT_C::IPL_ULONG)line.getP1().getX(),
		(mvIMPACT_C::IPL_ULONG)line.getP1().getY(),
		&length))
	{ throwImpactError(); }
	return (int)length;
}


inline Image averageFrames(const Image& S)
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	if(!mvIMPACT_C::IPL_PtAverage(
		S.getHandle(),
		&D,
		(mvIMPACT_C::IPL_ULONG)IPL_DONT_CARE))
	{ throwImpactError(); }

	return Image::imageFactory(D);
}

inline void averageFrames(Image& D, const Image& S)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if(!mvIMPACT_C::IPL_PtAverage(
		S.getHandle(),
		&hD,
		(mvIMPACT_C::IPL_ULONG)IPL_DONT_CARE))
	{ throwImpactError(); }

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}



inline  void dynamicThreshold(
	Image& D,
	const Image& S,
	int width,
	int height,
	int contrast,
	AdaptiveMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtDynamicThreshold(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_ADAPTIVE_MODE(mode),
			width,
			height,
			contrast))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image dynamicThreshold(
	const Image& S,
	int width,
	int height,
	int contrast,
	AdaptiveMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_PtDynamicThreshold(
			S.getHandle(),
			&D,
			mvIMPACT_C::IPL_ADAPTIVE_MODE(mode),
			width,
			height,
			contrast))
		throwImpactError();
	return Image::imageFactory(D);
}

inline Image dynamicLineThreshold(
				const Image &S,
				PolarityMode polarityMode,
				int operatorWidth,
				int minDistance)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	if (!mvIMPACT_C::IPL_PtDynamicLineThreshold(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_POLARITY(polarityMode),
			operatorWidth,
			minDistance))
		throwImpactError();
	return Image::imageFactory(hD);
}

inline void dynamicLineThreshold(
				Image &D,
				const Image &S,
				PolarityMode polarityMode,
				int operatorWidth,
				int minDistance)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_PtDynamicLineThreshold(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_POLARITY(polarityMode),
			operatorWidth,
			minDistance))
		throwImpactError();

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline  void equalize(Palette& D, const Histogram& S, EqualizeMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_StatHistogramEqualize(S.getHandle(), &hD, (mvIMPACT_C::IPL_EQUALIZE_MODE)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Palette::paletteFactory(hD);
	}
}

inline  Palette equalize(const Histogram& S, EqualizeMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE H = -1;
	if (!mvIMPACT_C::IPL_StatHistogramEqualize(S.getHandle(), &H, (mvIMPACT_C::IPL_EQUALIZE_MODE)mode))
		throwImpactError();
	return Palette::paletteFactory(H);
}


inline  void presetCheckerImage
(
	Image&                  D,
	const PresetColors&         colors,
	const PresetTransformation& transformation
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_PRESET_DATA data;

	data.checker.val.black = colors.getBlack();
	data.checker.val.white = colors.getWhite();
	data.checker.val.type  = (mvIMPACT_C::IPL_PRESET_CURVETYPE )pctPresetLinear;
	data.checker.geo       = transformation;

	if (!mvIMPACT_C::IPL_BufPreset(
			D.getHandle(),
			mvIMPACT_C::IPL_PRESET_CHECKER,
			&data))
		throwImpactError();
}





inline  void presetConstantImage(Image& D, int color)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_PRESET_DATA data;

	data.constant.val.black = color;

	if (!mvIMPACT_C::IPL_BufPreset(
			D.getHandle(),
			mvIMPACT_C::IPL_PRESET_CONSTANT,
			&data))
		throwImpactError();
}




inline void presetNoiseImage
(
	Image&             D,
	const PresetColors& colors,
	int                seed ,
	PresetCurveType     type
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_PRESET_DATA data;

	data.noise.val.black = colors.getBlack();
	data.noise.val.white = colors.getWhite();
	data.noise.val.type  = (mvIMPACT_C::IPL_PRESET_CURVETYPE )type;
	data.noise.seed      = seed;

	if (!mvIMPACT_C::IPL_BufPreset(
			D.getHandle(),
			mvIMPACT_C::IPL_PRESET_NOISE,
			&data))
		throwImpactError();
}



inline  void presetRingImage
(
	Image& D,
	const PresetColors&         colors,
	const PresetTransformation& transformation,
	PresetCurveType             type
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_PRESET_DATA data;

	data.ring.val.black = colors.getBlack();
	data.ring.val.white = colors.getWhite();
	data.ring.val.type  = (mvIMPACT_C::IPL_PRESET_CURVETYPE )type;
	data.ring.geo       = transformation;

	if (!mvIMPACT_C::IPL_BufPreset(
			D.getHandle(),
			mvIMPACT_C::IPL_PRESET_RING,
			&data))
		throwImpactError();
}


inline  void presetWedgeImage
(
	Image& D,
	const PresetColors&         colors,
	const PresetTransformation& transformation,
	PresetCurveType             type
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_PRESET_DATA data;

	data.wedge.val.black = colors.getBlack();
	data.wedge.val.white = colors.getWhite();
	data.wedge.val.type  = (mvIMPACT_C::IPL_PRESET_CURVETYPE )type;
	data.wedge.geo       = transformation;

	if (!mvIMPACT_C::IPL_BufPreset(
			D.getHandle(),
			mvIMPACT_C::IPL_PRESET_WEDGE,
			&data))
		throwImpactError();
}

inline void presetPalette
(
	Palette& D,
	PalettePresetType type,
	double gamma
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_PRESET_DATA data;

	data.lut.type  = (mvIMPACT_C::IPL_PRESET_LUT_TYPE )type;
	data.lut.gamma = gamma;

	if (!mvIMPACT_C::IPL_BufPreset(
			D.getHandle(),
			mvIMPACT_C::IPL_PRESET_LUT,
			&data))
		throwImpactError();
}



inline  void quantize
(
	Image&         D,
	const Image&   S,
	Palette& palette,
	int               colors,
	DitherMode         dither
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hP = palette.getHandle();

	if (!mvIMPACT_C::IPL_Quantize(S.getHandle(), &hD, &hP, colors, (mvIMPACT_C::IPL_DITHER_MODE)dither))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Palette quantize
(
	Image&         D,
	const Image&   S,
	int               colors,
	DitherMode         dither
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hP = -1;

	if (!mvIMPACT_C::IPL_Quantize(S.getHandle(), &hD, &hP, colors, (mvIMPACT_C::IPL_DITHER_MODE)dither))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}

	return Palette::paletteFactory(hP);
}

inline  Image quantize
(
	const Image&   S,
	Palette& palette,
	int               colors,
	DitherMode         dither
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	mvIMPACT_C::IPL_BUFHANDLE hP = palette.getHandle();

	if (!mvIMPACT_C::IPL_Quantize(S.getHandle(), &hD, &hP, colors, (mvIMPACT_C::IPL_DITHER_MODE)dither))
		throwImpactError();

	return Image::imageFactory(hD);
}

inline std::pair<Image,Palette> quantize
(
	const Image& S,
	int             colors,
	DitherMode       dither
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = -1;
	mvIMPACT_C::IPL_BUFHANDLE hP = -1;

	if (!mvIMPACT_C::IPL_Quantize(S.getHandle(), &hD, &hP, colors, (mvIMPACT_C::IPL_DITHER_MODE)dither))
		throwImpactError();

	return std::make_pair(Image::imageFactory(hD),Palette::paletteFactory(hP));
}


inline  void smooth(Histogram& D, const Histogram& S, int iterations)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_StatHistogramSmooth(S.getHandle(), &hD, iterations))
		throwImpactError();
}

inline  Histogram smooth(const Histogram& S, int iterations)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE H = -1;
	if (!mvIMPACT_C::IPL_StatHistogramSmooth(S.getHandle(), &H, iterations))
		throwImpactError();
	return Histogram::histogramFactory(H);
}


inline  void flip
(
	Image& D,
	const Image& S,
	FlipMode flip
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_TrFlip(
			S.getHandle(),
			&hD,
			mvIMPACT_C::IPL_FLIPMODE(flip)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image flip
(
	const Image& S,
	FlipMode flip
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrFlip(
			S.getHandle(),
			&D,
			mvIMPACT_C::IPL_FLIPMODE(flip)))
		throwImpactError();
	return Image::imageFactory(D);
}


inline Image convertToRle (
	const Image& source,
	int threshold,
	PolarityMode polarityMode,
	bool deleteBorderPixel)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_GetRLEData(
		source.getHandle(),
		&hD,
		threshold,
		mvIMPACT_C::IPL_POLARITY(polarityMode),
		deleteBorderPixel ? IPL_TRUE : IPL_FALSE))
		throwImpactError();
	return Image::imageFactory(hD);
}

inline void convertToRle (
	Image& dest,
	const Image& source,
	int threshold,
	PolarityMode polarityMode,
	bool deleteBorderPixel)
{
	mvIMPACT_C::IPL_BUFHANDLE hD = dest.getHandle();
	if (!mvIMPACT_C::IPL_GetRLEData(
		source.getHandle(),
		&hD,
		threshold,
		mvIMPACT_C::IPL_POLARITY(polarityMode),
		deleteBorderPixel ? IPL_TRUE : IPL_FALSE))
		throwImpactError();
	if(hD != dest.getHandle())
		dest = Image::imageFactory(hD);
}




inline MatrixDouble getMatrixPassPoint
(
	const std::vector<PointDouble>& vReference,
	const std::vector<PointDouble>& vDistorted
)
//-----------------------------------------------------------------------------
{
	if (vReference.size() != vDistorted.size())
	{
		throw ImpactException(mvIMPACT_C::IPL_SIZEMISMATCH_ERROR);
	}

	MatrixDouble m(3,3);

	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplReference(vReference);
	DynArray<mvIMPACT_C::IPL_POINT_DOUBLE, PointDouble> iplDistorted(vDistorted);

	if (!mvIMPACT_C::IPL_TrGetMatrixPassPoint(
		iplReference.getPointer(),
		iplDistorted.getPointer(),
		(mvIMPACT_C::IPL_ULONG)vReference.size(),
		m.data())
		)
		throwImpactError();

	return m;
}





inline  void matrix
(
	Image& D,
	const Image& S,
	const MatrixDouble& m,
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	if (m.getWidth()!=3 || m.getHeight()!=3)
	{
		throwImpactException("Matrix is not a 3x3 matrix");
	}

	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_TrMatrix(S.getHandle(), &hD, m.data(), (mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image matrix
(
	Image& S,
	const MatrixDouble& m,
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	if (m.getWidth()!=3 || m.getHeight()!=3)
	{
		throwImpactException("Matrix is not a 3x3 matrix");
	}

	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrMatrix(S.getHandle(), &D, m.data(), (mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	return Image::imageFactory(D);
}





inline  void rotate
(
	Image&             D,              //!< The destination image.
	const Image&       S,              //!< The source image.
 	double                 sourceCenterX,  //!< Specifies the x-coordinate of the source center of rotation.
	double                 sourceCenterY,  //!< Specifies the y-coordinate of the source center of rotation.
	double                 destCenterX,    //!< Specifies the x-coordinate of the destination center.
	double                 destCenterY,    //!< Specifies the y-coordinate of the destination center.
	double                 angle,          //!< Specified the rotation angle.
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_TrRotate(S.getHandle(), &hD, sourceCenterX, sourceCenterY,
		destCenterX, destCenterY, angle, (mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image rotate
(
	const Image&             S,              //!< The source image.
	double                 sourceCenterX,  //!< Specifies the x-coordinate of the source center of rotation.
	double                 sourceCenterY,  //!< Specifies the y-coordinate of the source center of rotation.
	double                 destCenterX,    //!< Specifies the x-coordinate of the destination center.
	double                 destCenterY,    //!< Specifies the y-coordinate of the destination center.
	double                 angle,          //!< Specified the rotation angle.
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrRotate(S.getHandle(), &D, sourceCenterX, sourceCenterY,
		destCenterX, destCenterY, angle, (mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  Image rotatefit
(
	const Image&             S,              //!< The source image.
	double                 angle,          //!< Specified the rotation angle.
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrRotate(S.getHandle(), &D, IPL_DONT_CARE, IPL_DONT_CARE,
		IPL_DONT_CARE, IPL_DONT_CARE, angle, (mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	return Image::imageFactory(D);
}




inline  void scale
(
	Image&                 D,              //!< The destination image.
	const Image&       S,              //!< The source image.
	double                 scaleX,         //!< Specifies the scale factor in x-direction.
	double                 scaleY,         //!< Specifies the scale factor in y-direction.
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_TrScale(S.getHandle(), &hD, scaleX, scaleY,
			(mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image scale
(
	Image&             S,              //!< The source image.
	double                 scaleX,         //!< Specifies the scale factor in x-direction.
	double                 scaleY,         //!< Specifies the scale factor in y-direction.
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrScale(S.getHandle(), &D, scaleX, scaleY,
			(mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	return Image::imageFactory(D);
}

inline  void translate
(
	Image& D,
	const Image& S,
	double dX,
	double dY,
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_TrTranslate(S.getHandle(), &hD, dX, dY, (mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image translate
(
	const Image& S,
	double dX,
	double dY,
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrTranslate(S.getHandle(), &D, dX, dY, (mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	return Image::imageFactory(D);
}


inline  void unwind
(
	Image& D,
	const Image& S,
	int xExtend,
	int yExtend,
	double xCenter,
	double yCenter,
	double fromAngle,
	double toAngle,
	int fromSize,
	int toSize,
	double angleStep,
	double rotationAngle ,
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_TrUnwind(S.getHandle(), &hD, xExtend, yExtend, xCenter, yCenter,
		fromAngle, toAngle, fromSize, toSize, angleStep, rotationAngle,	(mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image unwind
(
	const Image& S,
	int xExtend,
	int yExtend,
	double xCenter,
	double yCenter,
	double fromAngle,
	double toAngle,
	int fromSize,
	int toSize,
	double angleStep,
	double rotationAngle ,
	GeometricInterpolation mode
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::IPL_TrUnwind(S.getHandle(), &D, xExtend, yExtend, xCenter, yCenter,
		fromAngle, toAngle, fromSize, toSize, angleStep, rotationAngle,	(mvIMPACT_C::IPL_GEOTRANS)mode))
		throwImpactError();
	return Image::imageFactory(D);
}


///////////////////////////////////////////////////////////////
// former file module


inline Image loadImage(
	const std::string& fileName,
	FileFormat format)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_FlImport(
			&img,
			NULL,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();

	return Image::imageFactory(img);
}


inline void loadImage
(
	Image& D,
	const std::string& fileName,
	FileFormat format
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_FlImport(
			&hD,
			NULL,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline std::pair<Image, Palette> loadImageWithPalette
(
	const std::string& fileName,
	FileFormat         format
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE i = IPL_DONT_CARE;
	mvIMPACT_C::IPL_BUFHANDLE p = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_FlImport(
			&i,
			&p,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
	return std::pair<Image, Palette>(Image::imageFactory(i), Palette::paletteFactory(p));
}

inline void loadImageWithPalette
(
	Image& I,
	Palette& P,
	const std::string& fileName,
	FileFormat format
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hI = I.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hP = P.getHandle();
	if (!mvIMPACT_C::IPL_FlImport(
			&hI,
			&hP,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
	if(hI != I.getHandle()) {
		I = Image::imageFactory(hI);
	}
	if(hP != P.getHandle()) {
		P = Palette::paletteFactory(hP);
	}
}



inline Image loadRawImage
(
	const std::string& fileName,
	int width,
	int height,
	int bands,
	FileDatatype datatype,
	FileOrganization organization,
	int pitch,
	int offset
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE i = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_FlImportUser(
			&i,
			width,
			height,
			bands,
			mvIMPACT_C::FL_DATATYPE(datatype),
			mvIMPACT_C::FL_PIXELFORMAT(organization),
			pitch,
			offset,
			fileName.c_str()))
		throwImpactError();
	return Image::imageFactory(i);
}

inline void loadRawImage
(
	Image& D,
	const std::string& fileName,
	int width,
	int height,
	int bands,
	FileDatatype datatype,
	FileOrganization organization,
	int pitch,
	int offset
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_FlImportUser(
			&hD,
			width,
			height,
			bands,
			mvIMPACT_C::FL_DATATYPE(datatype),
			mvIMPACT_C::FL_PIXELFORMAT(organization),
			pitch,
			offset,
			fileName.c_str()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}



inline Image loadImageSequence
(
	const std::string& fileName,
	FileFormat format
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE i = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_FlImportSequence(
			&i,
			NULL,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
	return Image::imageFactory(i);
}

inline void loadImageSequence
(
	Image& D,
	const std::string& fileName,
	FileFormat format
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::IPL_FlImportSequence(
			&hD,
			NULL,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline std::pair<Image, Palette> loadImageSequenceWithPalette
(
	const std::string& fileName,
	FileFormat format
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE i = IPL_DONT_CARE;
	mvIMPACT_C::IPL_BUFHANDLE p = IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_FlImportSequence(
			&i,
			&p,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
	return std::pair<Image, Palette>(Image::imageFactory(i), Palette::paletteFactory(p));
}

inline void loadImageSequenceWithPalette
(
	Image& I,
	Palette& P,
	const std::string& fileName,
	FileFormat format
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hI = I.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE hP = P.getHandle();
	if (!mvIMPACT_C::IPL_FlImportSequence(
			&hI,
			&hP,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
	if(hI != I.getHandle()) {
		I = Image::imageFactory(hI);
	}
	if(hP != P.getHandle()) {
		P = Palette::paletteFactory(hP);
	}
}


inline void saveImage
(
	const Image& img,
	const std::string& fileName,
	FileFormat format,
	Palette* pal
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE p = pal ? pal->getHandle() : IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_FlExport(
			img.getHandle(),
			p,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
}





inline void saveImageSequence
(
	const Image& img,
	const std::string& fileName,
	FileFormat format,
	Palette* pal
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE p = pal ? pal->getHandle() : IPL_DONT_CARE;
	if (!mvIMPACT_C::IPL_FlExportSequence(
			img.getHandle(),
			p,
			mvIMPACT_C::FL_FILEFORMAT(format),
			fileName.c_str()))
		throwImpactError();
}

inline ImageInfo::ImageInfo()
{
	fileName_ = "";
	fileFormat_ = ffAny;
}


inline ImageInfo::ImageInfo(
		const std::string& fileName,
		FileFormat format)
	{
		fileName_ = fileName;
		fileFormat_  = format;
	}

inline int ImageInfo::getWidth() const
	{
		int w=0;
		if (!mvIMPACT_C::IPL_FlInquire(
				mvIMPACT_C::FL_FILEFORMAT(fileFormat_),
				mvIMPACT_C::IPL_WIDTH,
				&w,
				fileName_.c_str()))
			throwImpactError();
		return w;
	}

inline int ImageInfo::getHeight() const
	{
		int h=0;
		if (!mvIMPACT_C::IPL_FlInquire(
				mvIMPACT_C::FL_FILEFORMAT(fileFormat_),
				mvIMPACT_C::IPL_WIDTH,
				&h,
				fileName_.c_str()))
			throwImpactError();
		return h;
	}

inline int ImageInfo::getSamplesPerPixel() const
	{
		int samples=0;
		if (!mvIMPACT_C::IPL_FlInquire(
				mvIMPACT_C::FL_FILEFORMAT(fileFormat_),
				mvIMPACT_C::IPL_SAMPLES_PER_PIXEL,
				&samples,
				fileName_.c_str()))
			throwImpactError();
		return samples;
	}

inline int ImageInfo::getBitsPerSample() const
	{
		int bits=0;
		if (!mvIMPACT_C::IPL_FlInquire(
				mvIMPACT_C::FL_FILEFORMAT(fileFormat_),
				mvIMPACT_C::IPL_BITS_PER_SAMPLE,
				&bits,
				fileName_.c_str()))
			throwImpactError();
		return bits;
	}

inline bool ImageInfo::getIsCompressed() const
	{
		int compressed=0;
		if (!mvIMPACT_C::IPL_FlInquire(
				mvIMPACT_C::FL_FILEFORMAT(fileFormat_),
				mvIMPACT_C::IPL_COMPRESSED,
				&compressed,
				fileName_.c_str()))
			throwImpactError();
		return compressed != 0;
	}

inline bool ImageInfo::getHasPalette() const
	{
		int palette=0;
		if (!mvIMPACT_C::IPL_FlInquire(
				mvIMPACT_C::FL_FILEFORMAT(fileFormat_),
				mvIMPACT_C::IPL_PALETTE,
				&palette,
				fileName_.c_str()))
			throwImpactError();
		return palette != 0;
	}

inline int ImageInfo::getSequenceLength() const
	{
		int length=0;
		if (!mvIMPACT_C::IPL_FlInquire(
				mvIMPACT_C::FL_FILEFORMAT(fileFormat_),
				mvIMPACT_C::IPL_SEQ_LENGTH,
				&length,
				fileName_.c_str()))
			throwImpactError();
		return length;
	}


///////////////////////////////////////////////////////////////
// former slice module


inline  void lineProfile3d(
	Profile& D,
	const Image& S,
	const Coord &p0,
	const Coord &p1)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::SL_SlcLineProfile(
			S.getHandle(),
			&hD,
			p0.getX(),p0.getY(),p0.getBand(),p0.getSlice(),	p0.getFrame(),
			p1.getX(),p1.getY(),p1.getBand(),p1.getSlice(),	p1.getFrame()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Profile::profileFactory(hD);
	}
}



inline  Profile lineProfile3d(
	const Image& S,
	const Coord &p0,
	const Coord &p1
	)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::SL_SlcLineProfile(
			S.getHandle(),
			&D,
			p0.getX(),
			p0.getY(),
			p0.getBand(),
			p0.getSlice(),
			p0.getFrame(),
			p1.getX(),
			p1.getY(),
			p1.getBand(),
			p1.getSlice(),
			p1.getFrame()))
		throwImpactError();
	return Profile::profileFactory(D);
}


inline  Image orthogonalSlice(
	const Image& S,
	OrthogonalProjection projection,
	int cut,
	int band,
	int slice,
	int frame)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::SL_SlcGetSliceOrt(
			S.getHandle(),
			&D,
			mvIMPACT_C::SL_ORTHOSLICE(projection),
			(opXTopToBottom == projection || opXBottomToTop == projection) ? cut : 0,
			(opYLeftToRight == projection || opYRightToLeft == projection) ? cut : 0,
			band,
			slice,
			frame))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void orthogonalSlice(
	Image& D,
	const Image& S,
	OrthogonalProjection projection,
	int cut,
	int band,
	int slice,
	int frame)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::SL_SlcGetSliceOrt(
			S.getHandle(),
			&hD,
			mvIMPACT_C::SL_ORTHOSLICE(projection),
			(opXTopToBottom == projection || opXBottomToTop == projection) ? cut : 0,
			(opYLeftToRight == projection || opYRightToLeft == projection) ? cut : 0,
			band,
			slice,
			frame))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}


inline  void obliqueSlice(
	Image& D,
	const Image& S,
	ObliqueProjection projection,
	const Point& from,
	const Point& to,
	int band,
	int slice,
	int frame)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::SL_SlcGetSliceObl(
			S.getHandle(),
			&hD,
			mvIMPACT_C::SL_OBLSLICE(projection),
			from.getX(),
			from.getY(),
			to.getX(),
			to.getY(),
			band,
			slice,
			frame))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline  Image obliqueSlice(
	const Image& S,
	ObliqueProjection projection,
	const Point& from,
	const Point& to,
	int band,
	int slice,
	int frame)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = -1;
	if (!mvIMPACT_C::SL_SlcGetSliceObl(
			S.getHandle(),
			&D,
			mvIMPACT_C::SL_OBLSLICE(projection),
			from.getX(),
			from.getY(),
			to.getX(),
			to.getY(),
			band,
			slice,
			frame))
		throwImpactError();
	return Image::imageFactory(D);
}

#if !defined(I386_LINUX) && !defined(__ppc_6xx__)
inline void Window::onOoiCreate
	(
		const ObjectOfInterest * /*pOoi*/
	)
	{
	}

inline void Window::onOoiDestroy
	(
		const ObjectOfInterest * /*pOoi*/
	)
	{
	}

inline bool Window::onOoiSelect
	(
		const ObjectOfInterest * /*pOoi*/,
		bool /*bSelected*/
	)
	{
		return true;
	}

inline bool Window::onOoiStartTrack
	(
		const ObjectOfInterest* /*pOoi*/,
		int /*handle*/
	)
	{
		return true;
	}

inline void Window::onOoiStopTrack
	(
		const ObjectOfInterest* /*pOoi*/
	)
	{
	}

inline bool Window::onOoiTrackTo
	(
		const ObjectOfInterest* /*pOoi*/,
		int /*handle*/,
		const Point* /*pPoint*/
	)
	{
		return true;
	}

inline void Window::onOoiTracked
	(
		const ObjectOfInterest* /*pOoi*/
	)
	{
	}



inline void Window::onDraw
	(
		DeviceContext* /*pDc*/,
		const Rectangle* /*pSourceRect*/,
		const Rectangle* /*pDestRect*/
	)
	{}


inline bool Window::onBeforeDraw
	(
		DeviceContext* /*pDc*/,
		const Rectangle* /*pSourceRect*/,
		const Rectangle* /*pDestRect*/
	)
	{
		return true;
	}


inline bool Window::onMouseButtonDownWindow
	(
		MouseButton /*button*/,
		const Point* /*pPoint */
	)
	{
		return true;
	}

inline bool Window::onMouseButtonUpWindow
	(
		MouseButton /*button*/,
		const Point* /*pPoint */
	)
	{
		return true;
	}

inline bool Window::onMouseDoubleClickWindow
	(
		MouseButton /*button*/,
		const Point* /*pPoint */
	)
	{
		return true;
	}

inline bool Window::onMouseMoveWindow
	(
		MouseButton /*button*/,
		const Point* /*pPoint*/
	)
	{
		return true;
	}


inline bool Window::onMouseButtonDown
	(
		MouseButton /*button*/,
		const PointDouble* /*pPoint */
	)
	{
		return true;
	}

inline bool Window::onMouseButtonUp
	(
		MouseButton /*button*/,
		const PointDouble* /*pPoint */
	)
	{
		return true;
	}

inline bool Window::onMouseDoubleClick
	(
		MouseButton /*button*/,
		const PointDouble* /*pPoint */
	)
	{
		return true;
	}


inline bool Window::onMouseMove
	(
		MouseButton /*button*/,
		const PointDouble* /*pPoint*/
	)
	{
		return true;
	}

inline bool Window::onKeyDown
	(
		int /*character*/
	)
	{
		return true;
	}


inline void Window::onScrollPosChange
	(
		int /*x*/,
		int /*y*/
	)
	{
	}


inline void Window::onZoomFactorsChange
	(
		double /*x*/,
		double /*y*/
	)
	{
	}


inline ObjectOfInterest::ObjectOfInterest()
	: counter_(new int(1))
	, handle_(new mvIMPACT_C::DISP_OOIHANDLE(unassigned))
	, window_(new Window())
{
	assert(unassigned == getHandle());
	assert(0 == (mvIMPACT_C::IPL_HWND)getWindow()->getHandle());
	window_->addOoi(this);
}


inline ObjectOfInterest::ObjectOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: counter_(new int(1))
	, handle_(new mvIMPACT_C::DISP_OOIHANDLE(unassigned))
	, window_(new Window(window))
{
	assert(unassigned == getHandle());
	*handle_ = h;
	assert(0 != (mvIMPACT_C::IPL_HWND)getWindow()->getHandle());
	window_->addOoi(this);
}

inline ObjectOfInterest::ObjectOfInterest(
	const Window & window)
	: counter_(new int(1))
	, handle_(new mvIMPACT_C::DISP_OOIHANDLE(unassigned))
	, window_(window.makeShallowCopy())
{
	assert(unassigned == getHandle());
	assert(0 != (mvIMPACT_C::IPL_HWND)getWindow()->getHandle());
	window_->addOoi(this);
}

inline ObjectOfInterest::ObjectOfInterest(
	const ObjectOfInterest & r)
	: counter_(r.counter_)
	, handle_(r.handle_)
	, window_(r.window_->makeShallowCopy())
{
	(*counter_)++;
	window_->addOoi(this);
}

inline ObjectOfInterest & ObjectOfInterest::operator=(
	const ObjectOfInterest & r)
{
	if (&r != this)
	{
		dispose();

		counter_ = r.counter_;
		(*counter_)++;
		handle_ = r.handle_;
		window_ = r.window_->makeShallowCopy();

		window_->addOoi(this);
	}
	return *this;
}

inline ObjectOfInterest::~ObjectOfInterest()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline void ObjectOfInterest::dispose()
{
	window_->removeOoi(this);
	if (0 == --*counter_)
	{
		if (unassigned != getHandle())
		{
 			if (!mvIMPACT_C::DISP_OoiFree(getWindow() ? getWindow()->getOoiSetHandle() : -1 , getHandle()))
 				throwImpactError();
			*handle_ = unassigned;
		}

		delete handle_;
		handle_ = 0;

		delete counter_;
		counter_ = 0;
	}

	delete window_;
	window_ = 0;
}

inline int ObjectOfInterest::getObjectType(const Window& window, mvIMPACT_C::DISP_OOIHANDLE handle)
{
	mvIMPACT_C::IPL_LONG length=0;
	if (!mvIMPACT_C::DISP_InquireOoi(window.getOoiSetHandle(), handle, mvIMPACT_C::DISP_OOI_TYPE_LENGTH, &length))
		throwImpactError();

	DynArray<char> type(length+1);
	type.getPointer()[type.getSize()-1] = '\0';

	if (!mvIMPACT_C::DISP_InquireOoi(window.getOoiSetHandle(), handle, mvIMPACT_C::DISP_OOI_TYPE, type.getPointer()))
		throwImpactError();

	if (0 == strncmp(DISP_POI_TOOL, type.getPointer(), strlen(DISP_POI_TOOL)+1))
		return otPoi;
	else if (0 == strncmp(DISP_LOI_TOOL, type.getPointer(), strlen(DISP_LOI_TOOL)+1))
		return otLoi;
	else if (0 == strncmp(DISP_AOI_TOOL, type.getPointer(), strlen(DISP_AOI_TOOL)+1))
		return otAoi;
	else if (0 == strncmp(DISP_EOI_TOOL, type.getPointer(), strlen(DISP_EOI_TOOL)+1))
		return otEoi;
	else if (0 == strncmp(DISP_PLOI_TOOL, type.getPointer(), strlen(DISP_PLOI_TOOL)+1))
		return otPloi;
	else if (0 == strncmp(DISP_PGOI_TOOL, type.getPointer(), strlen(DISP_PGOI_TOOL)+1))
		return otPgoi;
	else if (0 == strncmp(DISP_ALOI_TOOL, type.getPointer(), strlen(DISP_ALOI_TOOL)+1))
		return otAloi;
	else if (0 == strncmp(DISP_CR1D_TOOL, type.getPointer(), strlen(DISP_CR1D_TOOL)+1))
		return otCr1d;
	else if (0 == strncmp(DISP_CR2D_DM_TOOL, type.getPointer(), strlen(DISP_CR2D_DM_TOOL)+1))
		return otCr2dDm;
	else
		return otNone;
}

inline int ObjectOfInterest::getObjectType() const
{
	if (unassigned == *handle_)
		return otNone;
	else
		return getObjectType(*window_, *handle_);
}

inline ObjectOfInterest * ObjectOfInterest::constructOoi(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE hOoi)
{
	switch (getObjectType(window, hOoi))
	{
	case otPoi:
		return new PointOfInterest(window, hOoi);
		break;
	case otLoi:
		return new LineOfInterest(window, hOoi);
		break;
	case otAoi:
		return new AreaOfInterest(window, hOoi);
		break;
	case otEoi:
		return new EllipseOfInterest(window, hOoi);
		break;
	case otPloi:
		return new PolylineOfInterest(window, hOoi);
		break;
	case otPgoi:
		return new PolygonOfInterest(window, hOoi);
		break;
	case otAloi:
		return new LineArrayOfInterest(window, hOoi);
		break;
	case otCr1d:
		return new BarcodeVisualizer(window, hOoi);
		break;
	case otCr2dDm:
		return new DatamatrixVisualizer(window, hOoi);
		break;
	default:
		break;
	}
}


#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(Visible, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(LinesVisible, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(LinesSolid, bool, ObjectOfInterest)
PROXY_CLASS_RGBCOLOR_IMPL(LineColor, RGBColor, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(HandleSize, int, ObjectOfInterest)
PROXY_CLASS_RGBCOLOR_IMPL(HandleLineColor, RGBColor, ObjectOfInterest)
PROXY_CLASS_RGBCOLOR_IMPL(HandleFillColor, RGBColor, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(HandlesVisible, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(HandleDrawingStyle, unsigned int, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Active, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(UIActive, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Selected, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(FillVisible, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(FillTransparent, bool, ObjectOfInterest)
PROXY_CLASS_RGBCOLOR_IMPL(FillColor, RGBColor, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Text, std::string, ObjectOfInterest)
PROXY_CLASS_RGBCOLOR_IMPL(TextColor, RGBColor, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(TextVisible, bool, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Angle, double, ObjectOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Width, int, ObjectOfInterest)

#pragma warning(pop)


inline Window * ObjectOfInterest::getWindow() const
{
	return window_;
}

inline mvIMPACT_C::DISP_OOIHANDLE ObjectOfInterest::getHandle() const
{
	return *handle_;
}

inline int ObjectOfInterest::getUseCount() const
{
	return *counter_;
}

inline bool ObjectOfInterest::getIsUnique() const
{
	return 1 == *counter_;
}


inline RGBColor ObjectOfInterest::getColor() const
{
	return getLineColor();
}

inline ObjectOfInterest::LineColorProxy ObjectOfInterest::getColor()
{
	return getLineColor();
}

inline ObjectOfInterest& ObjectOfInterest::setColor(const RGBColor& color)
{
	return setLineColor(color);
}


inline RGBColor ObjectOfInterest::getLineColor() const
{
	mvIMPACT_C::IPL_COLOR iplColor(0);
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_LINE_COLOR, &iplColor))
		throwImpactError();
	return iplColor;
}

inline ObjectOfInterest::LineColorProxy ObjectOfInterest::getLineColor()
{
	return ObjectOfInterest::LineColorProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setLineColor(const RGBColor& color)
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_LINE_COLOR, &v))
		throwImpactError();
	return *this;
}


inline RGBColor ObjectOfInterest::getFillColor() const
{
	mvIMPACT_C::IPL_COLOR iplColor(0);
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_FILL_COLOR, &iplColor))
		throwImpactError();
	return iplColor;
}

inline ObjectOfInterest::FillColorProxy ObjectOfInterest::getFillColor() 
{
	return ObjectOfInterest::FillColorProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setFillColor(const RGBColor& color)
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_FILL_COLOR, &v))
		throwImpactError();
	return *this;
}


inline RGBColor ObjectOfInterest::getTextColor() const
{
	mvIMPACT_C::IPL_COLOR iplColor(0);
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_TEXT_COLOR, &iplColor))
		throwImpactError();
	return iplColor;
}

inline ObjectOfInterest::TextColorProxy ObjectOfInterest::getTextColor() 
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setTextColor(const RGBColor& color)
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_TEXT_COLOR, &v))
		throwImpactError();
	return *this;
}


inline RGBColor ObjectOfInterest::getHandleLineColor() const
{
	mvIMPACT_C::IPL_COLOR iplColor(0);
	// initalized only because of compiler warning
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLE_LINE_COLOR, (mvIMPACT_C::IPL_VOID_PTR)&iplColor))
		throwImpactError();
	return iplColor;
}

inline ObjectOfInterest::HandleLineColorProxy ObjectOfInterest::getHandleLineColor()
{
	return ObjectOfInterest::HandleLineColorProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setHandleLineColor(const RGBColor& color)
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLE_LINE_COLOR, &v))
		throwImpactError();
	return *this;
}


inline RGBColor ObjectOfInterest::getHandleFillColor() const
{
	mvIMPACT_C::IPL_COLOR iplColor(0);
	// initalized only because of compiler warning
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLE_FILL_COLOR, (mvIMPACT_C::IPL_VOID_PTR)&iplColor))
		throwImpactError();
	return iplColor;
}

inline ObjectOfInterest::HandleFillColorProxy ObjectOfInterest::getHandleFillColor()
{
	return ObjectOfInterest::HandleFillColorProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setHandleFillColor(const RGBColor& color)
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLE_FILL_COLOR, &v))
		throwImpactError();
	return *this;
}


inline std::string ObjectOfInterest::getText() const
{
	int length=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_TEXT_LENGTH, &length))
		throwImpactError();
	DynArray<mvIMPACT_C::IPL_CHAR> text(length+1);
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_TEXT, text.getPointer()))
		throwImpactError();

	return std::string(text.getPointer());
}

inline ObjectOfInterest::TextProxy ObjectOfInterest::getText() 
{
	return ObjectOfInterest::TextProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setText(const std::string& text)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_TEXT, text.c_str()))
		throwImpactError();
	return *this;
}


inline int ObjectOfInterest::getHandleSize() const
{
	int size=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLE_SIZE, &size))
		throwImpactError();
	return size;
}

inline ObjectOfInterest::HandleSizeProxy ObjectOfInterest::getHandleSize() 
{
	return ObjectOfInterest::HandleSizeProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setHandleSize(int size)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLE_SIZE, &size))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getSelected() const
{
	mvIMPACT_C::IPL_BOOL selected=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_SELECTED, &selected))
		throwImpactError();
	return selected ? true : false;
}

inline ObjectOfInterest::SelectedProxy ObjectOfInterest::getSelected() 
{
	return ObjectOfInterest::SelectedProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setSelected(bool bSelected)
{
	mvIMPACT_C::IPL_BOOL selected = bSelected ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_SELECTED, &selected))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getActive() const
{
	mvIMPACT_C::IPL_BOOL active=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_ACTIVE, &active))
		throwImpactError();
	return active ? true : false;
}

inline ObjectOfInterest::ActiveProxy ObjectOfInterest::getActive()
{
	return ObjectOfInterest::ActiveProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setActive(bool bActive)
{
	mvIMPACT_C::IPL_BOOL active = bActive ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_ACTIVE, &active))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getUIActive() const
{
	mvIMPACT_C::IPL_BOOL ui_active=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_UI_ACTIVE, &ui_active))
		throwImpactError();
	return ui_active ? true : false;
}

inline ObjectOfInterest::UIActiveProxy ObjectOfInterest::getUIActive()
{
	return ObjectOfInterest::UIActiveProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setUIActive(bool bActive)
{
	mvIMPACT_C::IPL_BOOL ui_active = bActive ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_UI_ACTIVE, &ui_active))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getVisible() const
{
	mvIMPACT_C::IPL_BOOL visible=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_VISIBLE, &visible))
		throwImpactError();
	return visible ? true : false;
}

inline ObjectOfInterest::VisibleProxy ObjectOfInterest::getVisible()
{
	return ObjectOfInterest::VisibleProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setVisible(bool bVisible)
{
	mvIMPACT_C::IPL_BOOL visible = bVisible ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_VISIBLE, &visible))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getLinesVisible() const
{
	mvIMPACT_C::IPL_BOOL lines_visible=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_LINES_VISIBLE, &lines_visible))
		throwImpactError();
	return lines_visible ? true : false;
}

inline ObjectOfInterest::LinesVisibleProxy ObjectOfInterest::getLinesVisible()
{
	return ObjectOfInterest::LinesVisibleProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setLinesVisible(bool bLinesVisible)
{
	mvIMPACT_C::IPL_BOOL lines_visible = bLinesVisible ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_LINES_VISIBLE, &lines_visible))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getLinesSolid() const
{
	mvIMPACT_C::IPL_BOOL lines_solid=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_LINES_SOLID, &lines_solid))
		throwImpactError();
	return lines_solid ? true : false;
}

inline ObjectOfInterest::LinesSolidProxy ObjectOfInterest::getLinesSolid()
{
	return ObjectOfInterest::LinesSolidProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setLinesSolid(bool bSolid)
{
	mvIMPACT_C::IPL_BOOL lines_solid = bSolid ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_LINES_SOLID, &lines_solid))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getFillVisible() const
{
	mvIMPACT_C::IPL_BOOL fill_visible=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_FILL_VISIBLE, &fill_visible))
		throwImpactError();
	return fill_visible ? true : false;
}

inline ObjectOfInterest::FillVisibleProxy ObjectOfInterest::getFillVisible() 
{
	return ObjectOfInterest::FillVisibleProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setFillVisible(bool bFillVisible)
{
	mvIMPACT_C::IPL_BOOL fill_visible = bFillVisible ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_FILL_VISIBLE, &fill_visible))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getFillTransparent() const
{
	mvIMPACT_C::IPL_BOOL fill_transparent=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_FILL_TRANSPARENT, &fill_transparent))
		throwImpactError();
	return fill_transparent ? true : false;
}

inline ObjectOfInterest::FillTransparentProxy ObjectOfInterest::getFillTransparent()
{
	return ObjectOfInterest::FillTransparentProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setFillTransparent(bool bTransparent)
{
	mvIMPACT_C::IPL_BOOL fill_transparent = bTransparent ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_FILL_TRANSPARENT, &fill_transparent))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getTextVisible() const
{
	mvIMPACT_C::IPL_BOOL text_visible=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_TEXT_VISIBLE, &text_visible))
		throwImpactError();
	return text_visible ? true : false;
}

inline ObjectOfInterest::TextVisibleProxy ObjectOfInterest::getTextVisible() 
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setTextVisible(bool bTextVisible)
{
	mvIMPACT_C::IPL_BOOL text_visible = bTextVisible ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_TEXT_VISIBLE, &text_visible))
		throwImpactError();
	return *this;
}


inline bool ObjectOfInterest::getHandlesVisible() const
{
	mvIMPACT_C::IPL_BOOL handles_visible=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLES_VISIBLE, &handles_visible))
		throwImpactError();
	return handles_visible ? true : false;
}

inline ObjectOfInterest::HandlesVisibleProxy ObjectOfInterest::getHandlesVisible()
{
	return ObjectOfInterest::HandlesVisibleProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setHandlesVisible(bool bHandlesVisible)
{
	mvIMPACT_C::IPL_BOOL handles_visible = bHandlesVisible ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_HANDLES_VISIBLE, &handles_visible))
		throwImpactError();
	return *this;
}


inline double ObjectOfInterest::getAngle() const
{
	double angle=0.0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_ANGLE, &angle))
		throwImpactError();
	return angle;
}

inline ObjectOfInterest::AngleProxy ObjectOfInterest::getAngle()
{
	return ObjectOfInterest::AngleProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setAngle(double Angle)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_ANGLE, &Angle))
		throwImpactError();
	return *this;
}


inline int ObjectOfInterest::getWidth() const
{
	int width=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_WIDTH, &width))
		throwImpactError();
	return width;
}

inline ObjectOfInterest::WidthProxy ObjectOfInterest::getWidth() 
{
	return ObjectOfInterest::WidthProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setWidth(int Width)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_WIDTH, &Width))
		throwImpactError();
	return *this;
}


inline unsigned int ObjectOfInterest::getHandleDrawingStyle() const
{
	unsigned int style=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_HANDLE_DRAWING_STYLE, &style))
		throwImpactError();
	return style;
}

inline ObjectOfInterest::HandleDrawingStyleProxy ObjectOfInterest::getHandleDrawingStyle() 
{
	return ObjectOfInterest::HandleDrawingStyleProxy(*this);
}

inline ObjectOfInterest& ObjectOfInterest::setHandleDrawingStyle(unsigned int Style)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_HANDLE_DRAWING_STYLE, &Style))
		throwImpactError();
	return *this;
}


inline int ObjectOfInterest::getPos(int index) const
{
	mvIMPACT_C::IPL_LONG pos=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), (mvIMPACT_C::DISP_OOICONTROLTYPE)(mvIMPACT_C::DISP_OOI_POSINDEX+index), &pos))
		throwImpactError();
	return (int)pos;
}

inline ObjectOfInterest& ObjectOfInterest::setPos(int index, const int& pos)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), (mvIMPACT_C::DISP_OOICONTROLTYPE)(mvIMPACT_C::DISP_OOI_POSINDEX+index), &pos))
		throwImpactError();
	return *this;
}

inline std::vector<Point> ObjectOfInterest::getPoints() const
{
	int n;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_NPOINTS, &n))
		throwImpactError();

	std::vector<Point> points;

	for (int i=0; i<n; ++i)
		points.push_back(Point(getPos(i*2), getPos(i*2+1)));

	return points;
}

inline ObjectOfInterest& ObjectOfInterest::setPoints(std::vector<Point> const& points)
{
	int n;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_NPOINTS, &n))
		throwImpactError();

	if (n != (int)points.size())
		throwImpactException("Parameter mismatch. Point count incorrect.");

	for (int i=0; i<n; ++i)
	{
		setPos(i*2, points[i].getX());
		setPos(i*2+1, points[i].getY());
	}

	return *this;
}

inline PointOfInterest::PointOfInterest()
	: ObjectOfInterest()
{ }

inline PointOfInterest::PointOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline PointOfInterest::PointOfInterest(
	const Window & window,
	const Point & point)
	: ObjectOfInterest(window)
{
	DynArray<mvIMPACT_C::IPL_POINT, Point> iplPoints(&point, 1);

	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_POI_TOOL, iplPoints.getPointer(), 1, window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline PointOfInterest::PointOfInterest(
	const PointOfInterest &s)
	: ObjectOfInterest(s)
{ }

inline PointOfInterest::~PointOfInterest()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const PointOfInterest* PointOfInterest::castFromObjectOfInterest(
	const ObjectOfInterest* pOoi)
{
	if (pOoi->getObjectType() != otPoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to PointOfInterest");

	return static_cast<const PointOfInterest*>(pOoi);
}

inline PointOfInterest* PointOfInterest::castFromObjectOfInterest(
	ObjectOfInterest* pOoi)
{
	if (pOoi->getObjectType() != otPoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to PointOfInterest");

	return static_cast<PointOfInterest*>(pOoi);
}

inline ObjectOfInterest* PointOfInterest::makeShallowCopy() const
{
	return new PointOfInterest(*this);
}


#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(PoiStyle, PointDrawingStyle, PointOfInterest)
PROXY_CLASS_SIMPLE_IMPL(X, int, PointOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Y, int, PointOfInterest)
PROXY_CLASS_POINT_IMPL(Point, Point, PointOfInterest)

#pragma warning(pop)


inline std::string PointOfInterest::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy PointOfInterest::getText()
{
	return ObjectOfInterest::TextProxy(*this);
}

inline PointOfInterest& PointOfInterest::setText(
	const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor PointOfInterest::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy PointOfInterest::getTextColor()
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline PointOfInterest& PointOfInterest::setTextColor(
	const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool PointOfInterest::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy PointOfInterest::getTextVisible()
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline PointOfInterest& PointOfInterest::setTextVisible(
	bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline double PointOfInterest::getAngle() const
{
	return ObjectOfInterest::getAngle();
}

inline ObjectOfInterest::AngleProxy PointOfInterest::getAngle()
{
	return ObjectOfInterest::AngleProxy(*this);
}

inline PointOfInterest& PointOfInterest::setAngle(
	double Angle)
{
	ObjectOfInterest::setAngle(Angle);
	return *this;
}


inline int PointOfInterest::getWidth() const
{
	return ObjectOfInterest::getWidth();
}

inline ObjectOfInterest::WidthProxy PointOfInterest::getWidth()
{
	return ObjectOfInterest::WidthProxy(*this);
}

inline PointOfInterest& PointOfInterest::setWidth(
	int Width)
{
	ObjectOfInterest::setWidth(Width);
	return *this;
}


inline PointDrawingStyle PointOfInterest::getPoiStyle() const
{
	int Style=0;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_POI_STYLE, &Style))
		throwImpactError();
	return (PointDrawingStyle)Style;
}

inline PointOfInterest::PoiStyleProxy PointOfInterest::getPoiStyle()
{
	return PointOfInterest::PoiStyleProxy(*this);
}

inline PointOfInterest& PointOfInterest::setPoiStyle(
	PointDrawingStyle Style)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_POI_STYLE, &Style))
		throwImpactError();
	return *this;
}


inline int PointOfInterest::getX() const
{
	return ObjectOfInterest::getPos(0);
}

inline PointOfInterest::XProxy PointOfInterest::getX()
{
	return PointOfInterest::XProxy(*this);
}

inline PointOfInterest& PointOfInterest::setX(
	int x)
{
	ObjectOfInterest::setPos(0, x);
	return *this;
}


inline int PointOfInterest::getY() const
{
	return ObjectOfInterest::getPos(1);
}

inline PointOfInterest::YProxy PointOfInterest::getY()
{
	return PointOfInterest::YProxy(*this);
}

inline PointOfInterest& PointOfInterest::setY(
	int y)
{
	ObjectOfInterest::setPos(1, y);
	return *this;
}


inline Point PointOfInterest::getPoint() const
{
	return Point(ObjectOfInterest::getPos(0), ObjectOfInterest::getPos(1));
}

inline PointOfInterest::PointProxy PointOfInterest::getPoint()
{
	return PointOfInterest::PointProxy(*this);
}

inline PointOfInterest& PointOfInterest::setPoint(
	const Point & point)
{
	ObjectOfInterest::setPos(0, point.getX());
	ObjectOfInterest::setPos(1, point.getY());
	return *this;
}


inline LineOfInterest::LineOfInterest()
	: ObjectOfInterest()
{ }

inline LineOfInterest::LineOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline LineOfInterest::LineOfInterest(
	const Window & window,
	const Line & line)
	: ObjectOfInterest(window)
{
	std::vector<Point> points;
	points.push_back(line.getP0());
	points.push_back(line.getP1());

	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_LOI_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline LineOfInterest::LineOfInterest(
	const LineOfInterest &s)
	: ObjectOfInterest(s)
{ }

inline LineOfInterest::~LineOfInterest()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const LineOfInterest* LineOfInterest::castFromObjectOfInterest(
	const ObjectOfInterest* pOoi)
{
	if (pOoi->getObjectType() != otLoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to LineOfInterest");

	return static_cast<const LineOfInterest*>(pOoi);
}

inline LineOfInterest* LineOfInterest::castFromObjectOfInterest(
	ObjectOfInterest* pOoi)
{
	if (pOoi->getObjectType() != otLoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to LineOfInterest");

	return static_cast<LineOfInterest*>(pOoi);
}

inline ObjectOfInterest* LineOfInterest::makeShallowCopy() const
{
	return new LineOfInterest(*this);
}


#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(ShowArrow, bool, LineOfInterest)
PROXY_CLASS_SIMPLE_IMPL(DragWidth, bool, LineOfInterest)
PROXY_CLASS_SIMPLE_IMPL(X0, int, LineOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Y0, int, LineOfInterest)
PROXY_CLASS_SIMPLE_IMPL(X1, int, LineOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Y1, int, LineOfInterest)
PROXY_CLASS_LINE_IMPL(Line, Line, LineOfInterest)

#pragma warning(pop)


inline bool LineOfInterest::getFillVisible() const
{
	return ObjectOfInterest::getFillVisible();
}

inline ObjectOfInterest::FillVisibleProxy LineOfInterest::getFillVisible()
{
	return ObjectOfInterest::FillVisibleProxy(*this);
}

inline LineOfInterest& LineOfInterest::setFillVisible(bool bVisible)
{
	ObjectOfInterest::setFillVisible(bVisible);
	return *this;
}


inline bool LineOfInterest::getFillTransparent() const
{
	return ObjectOfInterest::getFillTransparent();
}

inline ObjectOfInterest::FillTransparentProxy LineOfInterest::getFillTransparent()
{
	return ObjectOfInterest::FillTransparentProxy(*this);
}

inline LineOfInterest& LineOfInterest::setFillTransparent(bool bTransparent)
{
	ObjectOfInterest::setFillTransparent(bTransparent);
	return *this;
}


inline RGBColor LineOfInterest::getFillColor() const
{
	return ObjectOfInterest::getFillColor();
}

inline ObjectOfInterest::FillColorProxy LineOfInterest::getFillColor()
{
	return ObjectOfInterest::FillColorProxy(*this);
}

inline LineOfInterest& LineOfInterest::setFillColor(const RGBColor& color)
{
	ObjectOfInterest::setFillColor(color);
	return *this;
}


inline std::string LineOfInterest::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy LineOfInterest::getText()
{
	return ObjectOfInterest::TextProxy(*this);
}

inline LineOfInterest& LineOfInterest::setText(const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor LineOfInterest::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy LineOfInterest::getTextColor()
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline LineOfInterest& LineOfInterest::setTextColor(const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool LineOfInterest::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy LineOfInterest::getTextVisible()
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline LineOfInterest& LineOfInterest::setTextVisible(bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline double LineOfInterest::getAngle() const
{
	return ObjectOfInterest::getAngle();
}


inline int LineOfInterest::getWidth() const
{
	return ObjectOfInterest::getWidth();
}

inline ObjectOfInterest::WidthProxy LineOfInterest::getWidth()
{
	return ObjectOfInterest::WidthProxy(*this);
}

inline LineOfInterest& LineOfInterest::setWidth(int Width)
{
	ObjectOfInterest::setWidth(Width);
	return *this;
}


inline bool LineOfInterest::getShowArrow() const
{
	mvIMPACT_C::IPL_BOOL show_arrow;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_LOI_SHOW_ARROW, &show_arrow))
		throwImpactError();
	return show_arrow ? true : false;
}

inline LineOfInterest::ShowArrowProxy LineOfInterest::getShowArrow()
{
	return LineOfInterest::ShowArrowProxy(*this);
}

inline LineOfInterest& LineOfInterest::setShowArrow(bool bShowArrow)
{
	mvIMPACT_C::IPL_BOOL show_arrow = bShowArrow ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_LOI_SHOW_ARROW, &show_arrow))
		throwImpactError();
	return *this;
}


inline bool LineOfInterest::getDragWidth() const
{
	mvIMPACT_C::IPL_BOOL drag_width;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_LOI_DRAG_THICKNESS, &drag_width))
		throwImpactError();
	return drag_width ? true : false;
}

inline LineOfInterest::DragWidthProxy LineOfInterest::getDragWidth()
{
	return LineOfInterest::DragWidthProxy(*this);
}

inline LineOfInterest& LineOfInterest::setDragWidth(bool bDragWidth)
{
	mvIMPACT_C::IPL_BOOL drag_width = bDragWidth ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_LOI_DRAG_THICKNESS, &drag_width))
		throwImpactError();
	return *this;
}


inline int LineOfInterest::getX0() const
{
	return ObjectOfInterest::getPos(0);
}

inline LineOfInterest::X0Proxy LineOfInterest::getX0()
{
	return LineOfInterest::X0Proxy(*this);
}

inline LineOfInterest& LineOfInterest::setX0(
	int x0)
{
	ObjectOfInterest::setPos(0, x0);
	return *this;
}


inline int LineOfInterest::getY0() const
{
	return ObjectOfInterest::getPos(2);
}

inline LineOfInterest::Y0Proxy LineOfInterest::getY0()
{
	return LineOfInterest::Y0Proxy(*this);
}

inline LineOfInterest& LineOfInterest::setY0(
	int y0)
{
	ObjectOfInterest::setPos(2, y0);
	return *this;
}


inline int LineOfInterest::getX1() const
{
	return ObjectOfInterest::getPos(1);
}

inline LineOfInterest::X1Proxy LineOfInterest::getX1()
{
	return LineOfInterest::X1Proxy(*this);
}

inline LineOfInterest& LineOfInterest::setX1(
	int x1)
{
	ObjectOfInterest::setPos(1, x1);
	return *this;
}


inline int LineOfInterest::getY1() const
{
	return ObjectOfInterest::getPos(3);
}

inline LineOfInterest::Y1Proxy LineOfInterest::getY1()
{
	return LineOfInterest::Y1Proxy(*this);
}

inline LineOfInterest& LineOfInterest::setY1(
	int y1)
{
	ObjectOfInterest::setPos(3, y1);
	return *this;
}


inline int LineOfInterest::getCenterX() const
{
	return ObjectOfInterest::getPos(4);
}


inline int LineOfInterest::getCenterY() const
{
	return ObjectOfInterest::getPos(5);
}

inline Line LineOfInterest::getLine() const
{
	return Line(getPos(0), getPos(2), getPos(1), getPos(3));
	// the order of coordinates is a little bit weird :
	// x1, x2, y1, y2
	// for the line going from point (x1,y1) to (x2, y2)
}

inline LineOfInterest::LineProxy LineOfInterest::getLine()
{
	return LineOfInterest::LineProxy(*this);
}

inline LineOfInterest& LineOfInterest::setLine(Line line)
{
	setPos(0, line.getP0().getX());
	setPos(1, line.getP1().getX());
	setPos(2, line.getP0().getY());
	setPos(3, line.getP1().getY());
	return *this;
}


inline AreaOfInterest::AreaOfInterest()
	: ObjectOfInterest()
{ }

inline AreaOfInterest::AreaOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline AreaOfInterest::AreaOfInterest(
	const Window & window,
	const Rectangle & rectangle)
	: ObjectOfInterest(window)
{
	std::vector<Point> points;
	points.push_back(Point(rectangle.getLeft(), rectangle.getTop()));
	points.push_back(Point(rectangle.getRight(), rectangle.getTop()));
	points.push_back(Point(rectangle.getRight(), rectangle.getBottom()));
	points.push_back(Point(rectangle.getLeft(), rectangle.getBottom()));

	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_AOI_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline AreaOfInterest::AreaOfInterest(
	const AreaOfInterest &s)
	: ObjectOfInterest(s)
{ }

inline AreaOfInterest::~AreaOfInterest()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const AreaOfInterest* AreaOfInterest::castFromObjectOfInterest (const ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otAoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to AreaOfInterest");

	return static_cast<const AreaOfInterest*>(pOoi);
}

inline AreaOfInterest* AreaOfInterest::castFromObjectOfInterest (ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otAoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to AreaOfInterest");

	return static_cast<AreaOfInterest*>(pOoi);
}

inline ObjectOfInterest* AreaOfInterest::makeShallowCopy() const
{
	return new AreaOfInterest(*this);
}


#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(ConstrainSquare, bool, AreaOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Left, int, AreaOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Top, int, AreaOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Right, int, AreaOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Bottom, int, AreaOfInterest)
PROXY_CLASS_RECTANGLE_IMPL(Rectangle, Rectangle, AreaOfInterest)

#pragma warning(push)


inline bool AreaOfInterest::getFillVisible() const
{
	return ObjectOfInterest::getFillVisible();
}

inline ObjectOfInterest::FillVisibleProxy AreaOfInterest::getFillVisible()
{
	return ObjectOfInterest::FillVisibleProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setFillVisible(bool bVisible)
{
	ObjectOfInterest::setFillVisible(bVisible);
	return *this;
}


inline bool AreaOfInterest::getFillTransparent() const
{
	return ObjectOfInterest::getFillTransparent();
}

inline ObjectOfInterest::FillTransparentProxy AreaOfInterest::getFillTransparent()
{
	return ObjectOfInterest::FillTransparentProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setFillTransparent(bool bTransparent)
{
	ObjectOfInterest::setFillTransparent(bTransparent);
	return *this;
}


inline RGBColor AreaOfInterest::getFillColor() const
{
	return ObjectOfInterest::getFillColor();
}

inline ObjectOfInterest::FillColorProxy AreaOfInterest::getFillColor()
{
	return ObjectOfInterest::FillColorProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setFillColor(const RGBColor& color)
{
	ObjectOfInterest::setFillColor(color);
	return *this;
}


inline std::string AreaOfInterest::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy AreaOfInterest::getText()
{
	return ObjectOfInterest::TextProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setText(const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor AreaOfInterest::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy AreaOfInterest::getTextColor()
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setTextColor(const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool AreaOfInterest::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy AreaOfInterest::getTextVisible()
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setTextVisible(bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline bool AreaOfInterest::getConstrainSquare() const
{
	mvIMPACT_C::IPL_BOOL constrain;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_AOI_CONSTRAIN_SQUARE, &constrain))
		throwImpactError();
	return constrain ? true : false;
}

inline AreaOfInterest::ConstrainSquareProxy AreaOfInterest::getConstrainSquare()
{
	return AreaOfInterest::ConstrainSquareProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setConstrainSquare(bool bConstrain)
{
	mvIMPACT_C::IPL_BOOL constrain = bConstrain ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_AOI_CONSTRAIN_SQUARE, &constrain))
		throwImpactError();
	return *this;
}


inline int AreaOfInterest::getLeft() const
{
	return ObjectOfInterest::getPos(0);
}

inline AreaOfInterest::LeftProxy AreaOfInterest::getLeft()
{
	return AreaOfInterest::LeftProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setLeft(
	int left)
{
	ObjectOfInterest::setPos(0, left);
	return *this;
}


inline int AreaOfInterest::getRight() const
{
	return ObjectOfInterest::getPos(1);
}

inline AreaOfInterest::RightProxy AreaOfInterest::getRight()
{
	return AreaOfInterest::RightProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setRight(
	int right)
{
	ObjectOfInterest::setPos(1, right);
	return *this;
}


inline int AreaOfInterest::getTop() const
{
	return ObjectOfInterest::getPos(2);
}

inline AreaOfInterest::TopProxy AreaOfInterest::getTop()
{
	return AreaOfInterest::TopProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setTop(
	int top)
{
	ObjectOfInterest::setPos(2, top);
	return *this;
}


inline int AreaOfInterest::getBottom() const
{
	return ObjectOfInterest::getPos(3);
}

inline AreaOfInterest::BottomProxy AreaOfInterest::getBottom()
{
	return AreaOfInterest::BottomProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setBottom(
	int bottom)
{
	ObjectOfInterest::setPos(3, bottom);
	return *this;
}


inline int AreaOfInterest::getCenterX() const
{
	return ObjectOfInterest::getPos(4);
}


inline int AreaOfInterest::getCenterY() const
{
	return ObjectOfInterest::getPos(5);
}


inline Rectangle AreaOfInterest::getRectangle() const
{
	// get unnormalized (raw) coordinates of the rectangle:
	int rawleft = getLeft();
	int rawright = getRight();
	int rawtop = getTop();
	int rawbottom = getBottom();

	// normalize coordinates:
	int normleft =  rawleft < rawright ?  rawleft : rawright;
	int normright = rawleft > rawright ?  rawleft : rawright;
	int normtop =  rawtop < rawbottom ? rawtop : rawbottom;
	int normbottom = rawtop > rawbottom ? rawtop : rawbottom;

	return Rectangle(normleft, normtop, normright, normbottom);
}

inline AreaOfInterest::RectangleProxy AreaOfInterest::getRectangle() 
{
	return AreaOfInterest::RectangleProxy(*this);
}

inline AreaOfInterest& AreaOfInterest::setRectangle(Rectangle Rect)
{
	setLeft(Rect.getLeft());
	setRight(Rect.getRight());
	setTop(Rect.getTop());
	setBottom(Rect.getBottom());

	return *this;
}


inline EllipseOfInterest::EllipseOfInterest()
	: ObjectOfInterest()
{ }

inline EllipseOfInterest::EllipseOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline EllipseOfInterest::EllipseOfInterest(
	const Window & window,
	const Rectangle & rectangle)
	: ObjectOfInterest(window)
{
	std::vector<Point> points;
	points.push_back(Point(rectangle.getLeft(), rectangle.getTop()));
	points.push_back(Point(rectangle.getRight(), rectangle.getTop()));
	points.push_back(Point(rectangle.getRight(), rectangle.getBottom()));
	points.push_back(Point(rectangle.getLeft(), rectangle.getBottom()));

	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_EOI_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline EllipseOfInterest::EllipseOfInterest(
	const EllipseOfInterest &s)
	: ObjectOfInterest(s)
{ }

inline EllipseOfInterest::~EllipseOfInterest()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const EllipseOfInterest* EllipseOfInterest::castFromObjectOfInterest (const ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otEoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to EllipseOfInterest");

	return static_cast<const EllipseOfInterest*>(pOoi);
}

inline EllipseOfInterest* EllipseOfInterest::castFromObjectOfInterest (ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otEoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to EllipseOfInterest");

	return static_cast<EllipseOfInterest*>(pOoi);
}

inline ObjectOfInterest* EllipseOfInterest::makeShallowCopy() const
{
	return new EllipseOfInterest(*this);
}


#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(ConstrainCircle, bool, EllipseOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Left, int, EllipseOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Top, int, EllipseOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Right, int, EllipseOfInterest)
PROXY_CLASS_SIMPLE_IMPL(Bottom, int, EllipseOfInterest)
PROXY_CLASS_RECTANGLE_IMPL(Rectangle, Rectangle, EllipseOfInterest)

#pragma warning(push)


inline bool EllipseOfInterest::getFillVisible() const
{
	return ObjectOfInterest::getFillVisible();
}

inline ObjectOfInterest::FillVisibleProxy EllipseOfInterest::getFillVisible()
{
	return ObjectOfInterest::FillVisibleProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setFillVisible(bool bVisible)
{
	ObjectOfInterest::setFillVisible(bVisible);
	return *this;
}


inline bool EllipseOfInterest::getFillTransparent() const
{
	return ObjectOfInterest::getFillTransparent();
}

inline ObjectOfInterest::FillTransparentProxy EllipseOfInterest::getFillTransparent()
{
	return ObjectOfInterest::FillTransparentProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setFillTransparent(bool bTransparent)
{
	ObjectOfInterest::setFillTransparent(bTransparent);
	return *this;
}


inline RGBColor EllipseOfInterest::getFillColor() const
{
	return ObjectOfInterest::getFillColor();
}

inline ObjectOfInterest::FillColorProxy EllipseOfInterest::getFillColor()
{
	return ObjectOfInterest::FillColorProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setFillColor(const RGBColor& color)
{
	ObjectOfInterest::setFillColor(color);
	return *this;
}


inline std::string EllipseOfInterest::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy EllipseOfInterest::getText()
{
	return ObjectOfInterest::TextProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setText(const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor EllipseOfInterest::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy EllipseOfInterest::getTextColor()
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setTextColor(const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool EllipseOfInterest::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy EllipseOfInterest::getTextVisible()
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setTextVisible(bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline bool EllipseOfInterest::getConstrainCircle() const
{
	mvIMPACT_C::IPL_BOOL constrain;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_EOI_CONSTRAIN_CIRCLE, &constrain))
		throwImpactError();
	return constrain ? true : false;
}

inline EllipseOfInterest::ConstrainCircleProxy EllipseOfInterest::getConstrainCircle()
{
	return EllipseOfInterest::ConstrainCircleProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setConstrainCircle(bool bConstrain)
{
	mvIMPACT_C::IPL_BOOL constrain = bConstrain ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_EOI_CONSTRAIN_CIRCLE, &constrain))
		throwImpactError();
	return *this;
}


inline int EllipseOfInterest::getLeft() const
{
	return ObjectOfInterest::getPos(0);
}

inline EllipseOfInterest::LeftProxy EllipseOfInterest::getLeft()
{
	return EllipseOfInterest::LeftProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setLeft(
	int left)
{
	ObjectOfInterest::setPos(0, left);
	return *this;
}


inline int EllipseOfInterest::getRight() const
{
	return ObjectOfInterest::getPos(1);
}

inline EllipseOfInterest::RightProxy EllipseOfInterest::getRight() 
{
	return EllipseOfInterest::RightProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setRight(
	int right)
{
	ObjectOfInterest::setPos(1, right);
	return *this;
}


inline int EllipseOfInterest::getTop() const
{
	return ObjectOfInterest::getPos(2);
}

inline EllipseOfInterest::TopProxy EllipseOfInterest::getTop() 
{
	return EllipseOfInterest::TopProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setTop(
	int top)
{
	ObjectOfInterest::setPos(2, top);
	return *this;
}


inline int EllipseOfInterest::getBottom() const
{
	return ObjectOfInterest::getPos(3);
}

inline EllipseOfInterest::BottomProxy EllipseOfInterest::getBottom() 
{
	return EllipseOfInterest::BottomProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setBottom(
	int bottom)
{
	ObjectOfInterest::setPos(3, bottom);
	return *this;
}


inline int EllipseOfInterest::getCenterX() const
{
	return ObjectOfInterest::getPos(4);
}


inline int EllipseOfInterest::getCenterY() const
{
	return ObjectOfInterest::getPos(5);
}


inline Rectangle EllipseOfInterest::getRectangle() const
{
	// get unnormalized (raw) coordinates of the rectangle:
	int rawleft = getLeft();
	int rawright = getRight();
	int rawtop = getTop();
	int rawbottom = getBottom();

	// normalize coordinates:
	int normleft =  rawleft < rawright ?  rawleft : rawright;
	int normright = rawleft > rawright ?  rawleft : rawright;
	int normtop =  rawtop < rawbottom ? rawtop : rawbottom;
	int normbottom = rawtop > rawbottom ? rawtop : rawbottom;

	return Rectangle(normleft, normtop, normright, normbottom);
}

inline EllipseOfInterest::RectangleProxy EllipseOfInterest::getRectangle() 
{
	return EllipseOfInterest::RectangleProxy(*this);
}

inline EllipseOfInterest& EllipseOfInterest::setRectangle(Rectangle Rect)
{
	setLeft(Rect.getLeft());
	setRight(Rect.getRight());
	setTop(Rect.getTop());
	setBottom(Rect.getBottom());

	return *this;
}


inline PolylineOfInterest::PolylineOfInterest()
	: ObjectOfInterest()
{ }

inline PolylineOfInterest::PolylineOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline PolylineOfInterest::PolylineOfInterest(
	const Window & window,
	const std::vector<Point> & points)
	: ObjectOfInterest(window)
{
	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_PLOI_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline PolylineOfInterest::PolylineOfInterest(
	const PolylineOfInterest &s)
	: ObjectOfInterest(s)
{ }

inline PolylineOfInterest::~PolylineOfInterest()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const PolylineOfInterest* PolylineOfInterest::castFromObjectOfInterest (const ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otPloi)
		throwImpactException("Tried to cast wrong subclass of Ooi to PolylineOfInterest");

	return static_cast<const PolylineOfInterest*>(pOoi);
}

inline PolylineOfInterest* PolylineOfInterest::castFromObjectOfInterest (ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otPloi)
		throwImpactException("Tried to cast wrong subclass of Ooi to PolylineOfInterest");

	return static_cast<PolylineOfInterest*>(pOoi);
}

inline ObjectOfInterest* PolylineOfInterest::makeShallowCopy() const
{
	return new PolylineOfInterest(*this);
}


inline std::string PolylineOfInterest::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy PolylineOfInterest::getText()
{
	return ObjectOfInterest::TextProxy(*this);
}

inline PolylineOfInterest& PolylineOfInterest::setText(const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor PolylineOfInterest::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy PolylineOfInterest::getTextColor()
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline PolylineOfInterest& PolylineOfInterest::setTextColor(const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool PolylineOfInterest::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy PolylineOfInterest::getTextVisible() 
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline PolylineOfInterest& PolylineOfInterest::setTextVisible(bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline int PolylineOfInterest::getX(int i) const
{
	return ObjectOfInterest::getPos(i*2);
}

inline PolylineOfInterest& PolylineOfInterest::setX(
	int i,
	int x)
{
	ObjectOfInterest::setPos(i*2, x);
	return *this;
}


inline int PolylineOfInterest::getY(int i) const
{
	return ObjectOfInterest::getPos(i*2+1);
}

inline PolylineOfInterest& PolylineOfInterest::setY(
	int i,
	int y)
{
	ObjectOfInterest::setPos(i*2+1, y);
	return *this;
}


inline std::vector<Point> PolylineOfInterest::getPoints() const
{
	return ObjectOfInterest::getPoints();
}

inline PolylineOfInterest& PolylineOfInterest::setPoints(std::vector<Point> const& points)
{
	ObjectOfInterest::setPoints(points);
	return *this;
}

inline PolygonOfInterest::PolygonOfInterest()
	: ObjectOfInterest()
{ }

inline PolygonOfInterest::PolygonOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline PolygonOfInterest::PolygonOfInterest(
	const Window & window,
	const std::vector<Point> & points)
	: ObjectOfInterest(window)
{
	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_PGOI_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline PolygonOfInterest::PolygonOfInterest(
	const PolygonOfInterest &s)
	: ObjectOfInterest(s)
{ }

inline PolygonOfInterest::~PolygonOfInterest()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const PolygonOfInterest* PolygonOfInterest::castFromObjectOfInterest (const ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otPgoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to PolygonOfInterest");

	return static_cast<const PolygonOfInterest*>(pOoi);
}

inline PolygonOfInterest* PolygonOfInterest::castFromObjectOfInterest (ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otPgoi)
		throwImpactException("Tried to cast wrong subclass of Ooi to PolygonOfInterest");

	return static_cast<PolygonOfInterest*>(pOoi);
}

inline ObjectOfInterest* PolygonOfInterest::makeShallowCopy() const
{
	return new PolygonOfInterest(*this);
}


inline bool PolygonOfInterest::getFillVisible() const
{
	return ObjectOfInterest::getFillVisible();
}

inline ObjectOfInterest::FillVisibleProxy PolygonOfInterest::getFillVisible()
{
	return ObjectOfInterest::FillVisibleProxy(*this);
}

inline PolygonOfInterest& PolygonOfInterest::setFillVisible(bool bVisible)
{
	ObjectOfInterest::setFillVisible(bVisible);
	return *this;
}


inline bool PolygonOfInterest::getFillTransparent() const
{
	return ObjectOfInterest::getFillTransparent();
}

inline ObjectOfInterest::FillTransparentProxy PolygonOfInterest::getFillTransparent()
{
	return ObjectOfInterest::FillTransparentProxy(*this);
}

inline PolygonOfInterest& PolygonOfInterest::setFillTransparent(bool bTransparent)
{
	ObjectOfInterest::setFillTransparent(bTransparent);
	return *this;
}


inline RGBColor PolygonOfInterest::getFillColor() const
{
	return ObjectOfInterest::getFillColor();
}

inline ObjectOfInterest::FillColorProxy PolygonOfInterest::getFillColor()
{
	return ObjectOfInterest::FillColorProxy(*this);
}

inline PolygonOfInterest& PolygonOfInterest::setFillColor(const RGBColor& color)
{
	ObjectOfInterest::setFillColor(color);
	return *this;
}


inline std::string PolygonOfInterest::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy PolygonOfInterest::getText() 
{
	return ObjectOfInterest::TextProxy(*this);
}

inline PolygonOfInterest& PolygonOfInterest::setText(const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor PolygonOfInterest::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy PolygonOfInterest::getTextColor()
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline PolygonOfInterest& PolygonOfInterest::setTextColor(const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool PolygonOfInterest::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy PolygonOfInterest::getTextVisible()
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline PolygonOfInterest& PolygonOfInterest::setTextVisible(bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline int PolygonOfInterest::getX(int i) const
{
	return ObjectOfInterest::getPos(i*2);
}

inline PolygonOfInterest& PolygonOfInterest::setX(
	int i,
	int x)
{
	ObjectOfInterest::setPos(i*2, x);
	return *this;
}


inline int PolygonOfInterest::getY(int i) const
{
	return ObjectOfInterest::getPos(i*2+1);
}

inline PolygonOfInterest& PolygonOfInterest::setY(
	int i,
	int y)
{
	ObjectOfInterest::setPos(i*2+1, y);
	return *this;
}


inline std::vector<Point> PolygonOfInterest::getPoints() const
{
	return ObjectOfInterest::getPoints();
}

inline PolygonOfInterest& PolygonOfInterest::setPoints(std::vector<Point> const& points)
{
	ObjectOfInterest::setPoints(points);
	return *this;
}

inline LineArrayOfInterest::LineArrayOfInterest()
	: ObjectOfInterest()
{ }

inline LineArrayOfInterest::LineArrayOfInterest(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline LineArrayOfInterest::LineArrayOfInterest(
	const Window & window,
	const std::vector<Point> & points)
	: ObjectOfInterest(window)
{
	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_ALOI_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline LineArrayOfInterest::LineArrayOfInterest(
	const LineArrayOfInterest &s)
	: ObjectOfInterest(s)
{ }

inline LineArrayOfInterest::~LineArrayOfInterest()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const LineArrayOfInterest* LineArrayOfInterest::castFromObjectOfInterest (const ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otAloi)
		throwImpactException("Tried to cast wrong subclass of Ooi to LineArrayOfInterest");

	return static_cast<const LineArrayOfInterest*>(pOoi);
}

inline LineArrayOfInterest* LineArrayOfInterest::castFromObjectOfInterest (ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otAloi)
		throwImpactException("Tried to cast wrong subclass of Ooi to LineArrayOfInterest");

	return static_cast<LineArrayOfInterest*>(pOoi);
}

inline ObjectOfInterest* LineArrayOfInterest::makeShallowCopy() const
{
	return new LineArrayOfInterest(*this);
}

inline int LineArrayOfInterest::getX(int i) const
{
	return ObjectOfInterest::getPos(i*2);
}

inline LineArrayOfInterest& LineArrayOfInterest::setX(
	int i,
	int x)
{
	ObjectOfInterest::setPos(i*2, x);
	return *this;
}


inline int LineArrayOfInterest::getY(int i) const
{
	return ObjectOfInterest::getPos(i*2+1);
}

inline LineArrayOfInterest& LineArrayOfInterest::setY(
	int i,
	int y)
{
	ObjectOfInterest::setPos(i*2+1, y);
	return *this;
}


inline std::vector<Line> LineArrayOfInterest::getLines() const
{
	int n;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_NPOINTS, &n))
		throwImpactError();

	std::vector<Line> lines;

	for (int i=0; i<n; i+=2)
		lines.push_back(Line(getPos(i*2), getPos(i*2+1), getPos(i*2+2), getPos(i*2+3)));

	return lines;
}

inline LineArrayOfInterest& LineArrayOfInterest::setLines(std::vector<Line> const& lines)
{
	int n;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_NPOINTS, &n))
		throwImpactError();

	if (n*2 != (int)lines.size())
		throwImpactException("Parameter mismatch. Point count incorrect.");

	for (int i=0; i<n; i+=2)
	{
		setPos(i*2, lines[i].getP0().getX());
		setPos(i*2+1, lines[i].getP0().getY());
		setPos(i*2+2, lines[i].getP1().getX());
		setPos(i*2+3, lines[i].getP1().getY());
	}

	return *this;
}

inline BarcodeVisualizer::BarcodeVisualizer()
	: ObjectOfInterest()
{ }

inline BarcodeVisualizer::BarcodeVisualizer(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline BarcodeVisualizer::BarcodeVisualizer(
	const Window & window,
	const std::vector<Point> & points)
	: ObjectOfInterest(window)
{
	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_CR1D_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline BarcodeVisualizer::BarcodeVisualizer(
	const BarcodeVisualizer &s)
	: ObjectOfInterest(s)
{ }

inline BarcodeVisualizer::~BarcodeVisualizer()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const BarcodeVisualizer* BarcodeVisualizer::castFromObjectOfInterest (const ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otCr1d)
		throwImpactException("Tried to cast wrong subclass of Ooi to BarcodeVisualizer");

	return static_cast<const BarcodeVisualizer*>(pOoi);
}

inline BarcodeVisualizer* BarcodeVisualizer::castFromObjectOfInterest (ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otCr1d)
		throwImpactException("Tried to cast wrong subclass of Ooi to BarcodeVisualizer");

	return static_cast<BarcodeVisualizer*>(pOoi);
}

inline ObjectOfInterest* BarcodeVisualizer::makeShallowCopy() const
{
	return new BarcodeVisualizer(*this);
}


#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(ShowDetails, bool, BarcodeVisualizer)

#pragma warning(push)


inline bool BarcodeVisualizer::getFillVisible() const
{
	return ObjectOfInterest::getFillVisible();
}

inline ObjectOfInterest::FillVisibleProxy BarcodeVisualizer::getFillVisible()
{
	return ObjectOfInterest::FillVisibleProxy(*this);
}

inline BarcodeVisualizer& BarcodeVisualizer::setFillVisible(bool bVisible)
{
	ObjectOfInterest::setFillVisible(bVisible);
	return *this;
}


inline bool BarcodeVisualizer::getFillTransparent() const
{
	return ObjectOfInterest::getFillTransparent();
}

inline ObjectOfInterest::FillTransparentProxy BarcodeVisualizer::getFillTransparent()
{
	return ObjectOfInterest::FillTransparentProxy(*this);
}

inline BarcodeVisualizer& BarcodeVisualizer::setFillTransparent(bool bTransparent)
{
	ObjectOfInterest::setFillTransparent(bTransparent);
	return *this;
}


inline RGBColor BarcodeVisualizer::getFillColor() const
{
	return ObjectOfInterest::getFillColor();
}

inline ObjectOfInterest::FillColorProxy BarcodeVisualizer::getFillColor()
{
	return ObjectOfInterest::FillColorProxy(*this);
}

inline BarcodeVisualizer& BarcodeVisualizer::setFillColor(const RGBColor& color)
{
	ObjectOfInterest::setFillColor(color);
	return *this;
}


inline std::string BarcodeVisualizer::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy BarcodeVisualizer::getText()
{
	return ObjectOfInterest::TextProxy(*this);
}

inline BarcodeVisualizer& BarcodeVisualizer::setText(const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor BarcodeVisualizer::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy BarcodeVisualizer::getTextColor()
{
	return ObjectOfInterest::TextColorProxy(*this);
}

inline BarcodeVisualizer& BarcodeVisualizer::setTextColor(const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool BarcodeVisualizer::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy BarcodeVisualizer::getTextVisible() 
{
	return ObjectOfInterest::TextVisibleProxy(*this);
}

inline BarcodeVisualizer& BarcodeVisualizer::setTextVisible(bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline bool BarcodeVisualizer::getShowDetails() const
{
	mvIMPACT_C::IPL_BOOL show_details;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_SHOW_DETAILS, &show_details))
		throwImpactError();
	return show_details ? true : false;
}

inline BarcodeVisualizer::ShowDetailsProxy BarcodeVisualizer::getShowDetails()
{
	return BarcodeVisualizer::ShowDetailsProxy(*this);;
}

inline BarcodeVisualizer& BarcodeVisualizer::setShowDetails(bool bShowDetails)
{
	mvIMPACT_C::IPL_BOOL show_details = bShowDetails ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_SHOW_DETAILS, &show_details))
		throwImpactError();
	return *this;
}


inline int BarcodeVisualizer::getX(int i) const
{
	return ObjectOfInterest::getPos(i*2);
}

inline BarcodeVisualizer& BarcodeVisualizer::setX(
	int i,
	int x)
{
	ObjectOfInterest::setPos(i*2, x);
	return *this;
}


inline int BarcodeVisualizer::getY(int i) const
{
	return ObjectOfInterest::getPos(i*2+1);
}

inline BarcodeVisualizer& BarcodeVisualizer::setY(
	int i,
	int y)
{
	ObjectOfInterest::setPos(i*2+1, y);
	return *this;
}


inline std::vector<Point> BarcodeVisualizer::getPoints() const
{
	return ObjectOfInterest::getPoints();
}

inline BarcodeVisualizer& BarcodeVisualizer::setPoints(std::vector<Point> const& points)
{
	ObjectOfInterest::setPoints(points);
	return *this;
}

inline DatamatrixVisualizer::DatamatrixVisualizer()
	: ObjectOfInterest()
{ }

inline DatamatrixVisualizer::DatamatrixVisualizer(
	const Window & window,
	mvIMPACT_C::DISP_OOIHANDLE h)
	: ObjectOfInterest(window, h)
{
	window_->onOoiCreate(this);
}

inline DatamatrixVisualizer::DatamatrixVisualizer(
	const Window & window,
	const std::vector<Point> & points)
	: ObjectOfInterest(window)
{
	if (!mvIMPACT_C::DISP_OoiAlloc(window.getOoiSetHandle(), handle_, DISP_CR2D_DM_TOOL, (mvIMPACT_C::IPL_POINT*)&*points.begin(), (mvIMPACT_C::IPL_ULONG)points.size(), window_->windows_))
		throwImpactError();

	assert(unassigned != getHandle());

	window_->onOoiCreate(this);
}

inline DatamatrixVisualizer::DatamatrixVisualizer(
	const DatamatrixVisualizer &s)
	: ObjectOfInterest(s)
{ }

inline DatamatrixVisualizer::~DatamatrixVisualizer()
{
	// a destructor must never throw
	try
	{
		if (getIsUnique() && window_)
			window_->onOoiDestroy(this);
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline const DatamatrixVisualizer* DatamatrixVisualizer::castFromObjectOfInterest (const ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otCr2dDm)
		throwImpactException("Tried to cast wrong subclass of Ooi to DatamatrixVisualizer");

	return static_cast<const DatamatrixVisualizer*>(pOoi);
}

inline DatamatrixVisualizer* DatamatrixVisualizer::castFromObjectOfInterest (ObjectOfInterest *pOoi)
{
	if (pOoi->getObjectType() != otCr2dDm)
		throwImpactException("Tried to cast wrong subclass of Ooi to DatamatrixVisualizer");

	return static_cast<DatamatrixVisualizer*>(pOoi);
}

inline ObjectOfInterest* DatamatrixVisualizer::makeShallowCopy() const
{
	return new DatamatrixVisualizer(*this);
}


#pragma warning(push)
#pragma warning(disable : 4717)

PROXY_CLASS_SIMPLE_IMPL(ShowDetails, bool, DatamatrixVisualizer)
PROXY_CLASS_SIMPLE_IMPL(Rows, int, DatamatrixVisualizer)
PROXY_CLASS_SIMPLE_IMPL(Columns, int, DatamatrixVisualizer)

#pragma warning(push)


inline bool DatamatrixVisualizer::getFillVisible() const
{
	return ObjectOfInterest::getFillVisible();
}

inline ObjectOfInterest::FillVisibleProxy DatamatrixVisualizer::getFillVisible()
{
	return ObjectOfInterest::getFillVisible();
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setFillVisible(bool bVisible)
{
	ObjectOfInterest::setFillVisible(bVisible);
	return *this;
}


inline bool DatamatrixVisualizer::getFillTransparent() const
{
	return ObjectOfInterest::getFillTransparent();
}

inline ObjectOfInterest::FillTransparentProxy DatamatrixVisualizer::getFillTransparent()
{
	return ObjectOfInterest::getFillTransparent();
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setFillTransparent(bool bTransparent)
{
	ObjectOfInterest::setFillTransparent(bTransparent);
	return *this;
}


inline RGBColor DatamatrixVisualizer::getFillColor() const
{
	return ObjectOfInterest::getFillColor();
}

inline ObjectOfInterest::FillColorProxy DatamatrixVisualizer::getFillColor()
{
	return ObjectOfInterest::getFillColor();
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setFillColor(const RGBColor& color)
{
	ObjectOfInterest::setFillColor(color);
	return *this;
}


inline std::string DatamatrixVisualizer::getText() const
{
	return ObjectOfInterest::getText();
}

inline ObjectOfInterest::TextProxy DatamatrixVisualizer::getText()
{
	return ObjectOfInterest::getText();
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setText(const std::string& text)
{
	ObjectOfInterest::setText(text);
	return *this;
}


inline RGBColor DatamatrixVisualizer::getTextColor() const
{
	return ObjectOfInterest::getTextColor();
}

inline ObjectOfInterest::TextColorProxy DatamatrixVisualizer::getTextColor()
{
	return ObjectOfInterest::getTextColor();
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setTextColor(const RGBColor& color)
{
	ObjectOfInterest::setTextColor(color);
	return *this;
}


inline bool DatamatrixVisualizer::getTextVisible() const
{
	return ObjectOfInterest::getTextVisible();
}

inline ObjectOfInterest::TextVisibleProxy DatamatrixVisualizer::getTextVisible()
{
	return ObjectOfInterest::getTextVisible();
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setTextVisible(bool bVisible)
{
	ObjectOfInterest::setTextVisible(bVisible);
	return *this;
}


inline bool DatamatrixVisualizer::getShowDetails() const
{
	mvIMPACT_C::IPL_BOOL show_details;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_SHOW_DETAILS, &show_details))
		throwImpactError();
	return show_details ? true : false;
}

inline DatamatrixVisualizer::ShowDetailsProxy DatamatrixVisualizer::getShowDetails()
{
	return DatamatrixVisualizer::ShowDetailsProxy(*this);
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setShowDetails(bool bShowDetails)
{
	mvIMPACT_C::IPL_BOOL show_details = bShowDetails ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_OOI_SHOW_DETAILS, &show_details))
		throwImpactError();
	return *this;
}


inline int DatamatrixVisualizer::getRows() const
{
	mvIMPACT_C::IPL_ULONG columns;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_CR2D_DM_ROWS, &columns))
		throwImpactError();
	return columns;
}

inline DatamatrixVisualizer::RowsProxy DatamatrixVisualizer::getRows() 
{
	return DatamatrixVisualizer::RowsProxy(*this);
}


inline DatamatrixVisualizer& DatamatrixVisualizer::setRows(int columns)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_CR2D_DM_ROWS, &columns))
		throwImpactError();
	return *this;
}


inline int DatamatrixVisualizer::getColumns() const
{
	mvIMPACT_C::IPL_ULONG columns;
	if (!mvIMPACT_C::DISP_InquireOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_CR2D_DM_COLUMNS, &columns))
		throwImpactError();
	return columns;
}

inline DatamatrixVisualizer::ColumnsProxy DatamatrixVisualizer::getColumns() 
{
	return DatamatrixVisualizer::ColumnsProxy(*this);
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setColumns(int columns)
{
	if (!mvIMPACT_C::DISP_ControlOoi(getWindow() ? getWindow()->getOoiSetHandle() : -1, getHandle(), mvIMPACT_C::DISP_CR2D_DM_COLUMNS, &columns))
		throwImpactError();
	return *this;
}


inline int DatamatrixVisualizer::getX(int i) const
{
	return ObjectOfInterest::getPos(i*2);
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setX(
	int i,
	int x)
{
	ObjectOfInterest::setPos(i*2, x);
	return *this;
}


inline int DatamatrixVisualizer::getY(int i) const
{
	return ObjectOfInterest::getPos(i*2+1);
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setY(
	int i,
	int y)
{
	ObjectOfInterest::setPos(i*2+1, y);
	return *this;
}


inline std::vector<Point> DatamatrixVisualizer::getPoints() const
{
	return ObjectOfInterest::getPoints();
}

inline DatamatrixVisualizer& DatamatrixVisualizer::setPoints(std::vector<Point> const& points)
{
	ObjectOfInterest::setPoints(points);
	return *this;
}

inline void runMessageLoop(
	const std::string& message)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_KeepRunning(message.c_str()))
		throwImpactError();
}

inline void dispatchMessages()
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_KeepRunning(0))
		throwImpactError();
}


inline ProfileDirection getDefaultProfileDrawingDir()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_PROFILE_DIRECTION, &mode))
		throwImpactError();
	return ProfileDirection(mode);
}





inline void setDefaultProfileDrawingDir(
	ProfileDirection v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_PROFILE_DIRECTION, &mode))
		throwImpactError();
}





inline  HistogramMode getDefaultHistogramDrawingMode()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_HISTOGRAM_MODE, &mode))
		throwImpactError();
	return HistogramMode(mode);
}





inline  void setDefaultHistogramDrawingMode(
	HistogramMode v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_HISTOGRAM_MODE, &mode))
		throwImpactError();
}





inline  ImageMode getDefaultImageDrawingMode()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_IMAGE_MODE, &mode))
		throwImpactError();
	return ImageMode(mode);
}





inline  void setDefaultImageDrawingMode(
	ImageMode v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_IMAGE_MODE, &mode))
		throwImpactError();
}





inline  int getDefaultNumericBase()
//-----------------------------------------------------------------------------
{
	int mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_NUMERIC_BASE, &mode))
		throwImpactError();
	return mode;
}





inline  void setDefaultNumericBase(
	int v)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_NUMERIC_BASE, &v))
		throwImpactError();
}





inline  Palette getDefaultColorPalette()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE v;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_COLOR_LUT, &v))
		throwImpactError();
	return Palette::paletteFactory(v);
}





inline  void setDefaultColorPalette(
	const Palette& v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE vh = v.getHandle();
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_COLOR_LUT, &vh))
		throwImpactError();
}





inline  int getDefaultTextSpacing()
//-----------------------------------------------------------------------------
{
	int mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_TEXT_SPACING, &mode))
		throwImpactError();
	return mode;
}





inline void setDefaultTextSpacing(
//-----------------------------------------------------------------------------
	int v)
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_TEXT_SPACING, &v))
		throwImpactError();
}





inline  int getDefaultTextHeight()
//-----------------------------------------------------------------------------
{
	int mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_TEXT_HEIGHT, &mode))
		throwImpactError();
	return mode;
}





inline void setDefaultTextHeight(
	int v)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_TEXT_HEIGHT, &v))
		throwImpactError();
}





inline  int getDefaultTextWeight()
//-----------------------------------------------------------------------------
{
	int mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_TEXT_WEIGHT, &mode))
		throwImpactError();
	return mode;
}





inline void setDefaultTextWeight(
	int v)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_TEXT_WEIGHT, &v))
		throwImpactError();
}





inline  RGBColor getDefaultTextColor()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_COLOR mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_TEXT_COLOR, &mode))
		throwImpactError();
	return mode;
}





inline  void setDefaultTextColor(
	const RGBColor& color)
//-----------------------------------------------------------------------------
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_TEXT_COLOR, &v))
		throwImpactError();
}





inline  RGBColor getDefaultDrawColor()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_COLOR mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_DRAW_COLOR, &mode))
		throwImpactError();
	return mode;
}





inline  void setDefaultDrawColor(
	const RGBColor& color)
//-----------------------------------------------------------------------------
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_DRAW_COLOR, &v))
		throwImpactError();
}





inline  RGBColor getDefaultFillColor()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_COLOR mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_FILL_COLOR, &mode))
		throwImpactError();
	return mode;
}





inline  void setDefaultFillColor(
	const RGBColor& color)
//-----------------------------------------------------------------------------
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_FILL_COLOR, &v))
		throwImpactError();
}





inline  int getDefaultNormalize()
//-----------------------------------------------------------------------------
{
	int mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_NORMALIZE, &mode))
		throwImpactError();
	return mode;
}





inline  void setDefaultNormalize(
	int v)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_NORMALIZE, &v))
		throwImpactError();
}





inline  CopyClipboardMode getDefaultClipboardMode()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_COPY_WINDOW, &mode))
		throwImpactError();
	return CopyClipboardMode(mode);
}





inline  void setDefaultClipboardMode(
	CopyClipboardMode v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_COPY_WINDOW, &mode))
		throwImpactError();
}





inline  double getDefaultZoomX()
//-----------------------------------------------------------------------------
{
	double mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_ZOOM_WINDOW_X, &mode))
		throwImpactError();
	return mode;
}





inline  void setDefaultZoomX(
	double v)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_ZOOM_WINDOW_X, &v))
		throwImpactError();
}





inline  double getDefaultZoomY()
//-----------------------------------------------------------------------------
{
	double mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_ZOOM_WINDOW_Y, &mode))
		throwImpactError();
	return mode;
}





inline  void setDefaultZoomY(
	double v)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_ZOOM_WINDOW_Y, &v))
		throwImpactError();
}





inline  void setDefaultZoom(
	double v)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_ZOOM_WINDOW, &v))
		throwImpactError();
}





inline  ZoomingMode getDefaultZoomMode()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_ZOOM_MODE, &mode))
		throwImpactError();
	return ZoomingMode(mode);
}





inline void setDefaultZoomMode(
	ZoomingMode v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_ZOOM_MODE, &mode))
		throwImpactError();
}





inline  Palette getDefaultMappingPalette()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE v;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_MAPPING_LUT, &v))
		throwImpactError();
	return Palette::paletteFactory(v);
}





//-----------------------------------------------------------------------------
inline void setDefaultMappingPalette(
	const Palette& v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE vh = v.getHandle();
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_MAPPING_LUT, &vh))
		throwImpactError();
}





inline int getDefaultShift()
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_LONG bits;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_SHIFT, &bits))
		throwImpactError();
	return (int)bits;
}





inline void setDefaultShift(int bits)
	//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_SHIFT, &bits))
		throwImpactError();
}





inline  ScrollingMode getDefaultScrollbarMode()
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_SCROLLBAR_MODE, &mode))
		throwImpactError();
	return ScrollingMode(mode);
}





inline  void setDefaultScrollbarMode(
	ScrollingMode v)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_SCROLLBAR_MODE, &mode))
		throwImpactError();
}





inline RGBColor getDefaultBackgroundColor ()
//-----------------------------------------------------------------------------
{
	RGBColor color;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_BACKGROUND_COLOR, &color))
		throwImpactError();
	return color;
}





inline void setDefaultBackgroundColor (const RGBColor& color)
//-----------------------------------------------------------------------------
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_BACKGROUND_COLOR, &v))
		throwImpactError();
}



inline int getDefaultEmptyWindowWidth ()
{
	int v;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_DEFAULT_EMPTY_WINDOW_WIDTH, &v))
		throwImpactError();
	return v;
}

inline void setDefaultEmptyWindowWidth (int v)
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_DEFAULT_EMPTY_WINDOW_WIDTH, &v))
		throwImpactError();
}

inline int getDefaultEmptyWindowHeight ()
{
	int v;
	if (!mvIMPACT_C::DISP_Inquire(mvIMPACT_C::DISP_DEFAULT_EMPTY_WINDOW_HEIGHT, &v))
		throwImpactError();
	return v;
}

inline void setDefaultEmptyWindowHeight (int v)
{
	if (!mvIMPACT_C::DISP_Control(mvIMPACT_C::DISP_DEFAULT_EMPTY_WINDOW_HEIGHT, &v))
		throwImpactError();
}

inline DeviceContext::DeviceContext()
{
	m_dc = 0;
}

inline DeviceContext::DeviceContext(mvIMPACT_C::IPL_HDC hdc)
{
	m_dc = hdc;
}

inline DeviceContext::DeviceContext(int hdc)
{
	m_dc = reinterpret_cast<mvIMPACT_C::IPL_HDC>(hdc);
}

inline DeviceContext::operator mvIMPACT_C::IPL_HDC() const
{
	return m_dc;
}

inline DeviceContext::operator int() const
{
	return reinterpret_cast<int>(m_dc);
}

inline mvIMPACT_C::IPL_HDC DeviceContext::ConvertToDeviceContextHandle() const
{
	return m_dc;
}

inline int DeviceContext::ConvertToInteger() const
{
	return reinterpret_cast<int>(m_dc);
}

// 64bit support makes sense for 64bit capable compilers only
#ifdef _M_X64
inline long long DeviceContext::ConvertToInteger64() const
{
	return reinterpret_cast<long long>(m_dc);
}
#endif // #ifdef _M_X64

inline void DeviceContext::draw(
	const BufferBase& buffer,
	const Rectangle& source,
	const Rectangle& dest)
{
	mvIMPACT_C::IPL_RECT iplSource = source;
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_Draw(buffer.getHandle(), m_dc, &iplSource, &iplDest))
		throwImpactError();
}

inline void DeviceContext::draw(
	const BufferBase& buffer
	)
{
    if (!mvIMPACT_C::DISP_Draw(buffer.getHandle(), m_dc, 0, 0))
		throwImpactError();
}

inline void DeviceContext::draw(
	const BufferBase& buffer,
	const Rectangle& source)
{
	mvIMPACT_C::IPL_RECT iplSource = source;

	if (!mvIMPACT_C::DISP_Draw(buffer.getHandle(), m_dc, &iplSource, &iplSource))
		throwImpactError();
}


inline void DeviceContext::drawArc
(
	const Rectangle& rect,
	const Point& start,
	const Point& end,
	bool  direction,
	const RGBColor& color
)
{
	mvIMPACT_C::IPL_RECT iplRect = rect;
	mvIMPACT_C::IPL_POINT iplStart = start;
	mvIMPACT_C::IPL_POINT iplEnd = end;
	mvIMPACT_C::IPL_BOOL iplDirection = direction ? IPL_TRUE : IPL_FALSE;

	if (!mvIMPACT_C::DISP_DrawArc(
			m_dc,
			color.asDWord(),
			&iplRect,
			&iplStart,
			&iplEnd,
			iplDirection))
		throwImpactError();
}

inline void DeviceContext::drawEllipse
(
	const Rectangle& rect,
	const RGBColor& color
	)
{
	drawArc(
		rect,
		Point(rect.getLeft(), rect.getTop()),
		Point(rect.getLeft(), rect.getTop()),
		true,
		color);
}


inline void DeviceContext::drawHistogram
(
	const Histogram& histogram
)
{
	if (!mvIMPACT_C::DISP_DrawHistogram(
			histogram.getHandle(),
			m_dc,
			NULL,
			NULL))
		throwImpactError();
}

inline void DeviceContext::drawHistogram
(
	const Histogram& histogram,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawHistogram(
			histogram.getHandle(),
			m_dc,
			NULL,
			&iplDest))
		throwImpactError();
}

inline void DeviceContext::drawHistogram
(
	const Histogram& histogram,
	const Rectangle& source,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplSource = source;
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawHistogram(
			histogram.getHandle(),
			m_dc,
			&iplSource,
			&iplDest))
		throwImpactError();
}

inline void DeviceContext::drawImage
(
	const Image& image
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_DrawImage(
			image.getHandle(),
			m_dc,
			NULL,
			NULL))
		throwImpactError();
}

inline void DeviceContext::drawImage
(
	const Image& image,
	const Rectangle& source,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplSource = source;
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawImage(
			image.getHandle(),
			m_dc,
			&iplSource,
			&iplDest))
		throwImpactError();
}

inline void DeviceContext::drawImage
(
	const Image& image,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawImage(
			image.getHandle(),
			m_dc,
			NULL,
			&iplDest))
		throwImpactError();
}

inline void DeviceContext::drawLine
(
	const Point& start,
	const Point& end,
	const RGBColor& color
)
{
	mvIMPACT_C::IPL_POINT iplStart = start;
	mvIMPACT_C::IPL_POINT iplEnd = end;

	if (!mvIMPACT_C::DISP_DrawLine(
			m_dc,
			color.asDWord(),
			&iplStart,
			&iplEnd))
		throwImpactError();
}


inline void DeviceContext::drawLine
(
	const Line &line,
	const RGBColor& color
)
{
	drawLine(line.getP0(), line.getP1(), color);
}

inline void DeviceContext::drawPalette
(
	const Palette& palette
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_DrawLut(
			palette.getHandle(),
			m_dc,
			NULL,
			NULL))
		throwImpactError();
}

inline void DeviceContext::drawPalette
(
	const Palette& palette,
	const Rectangle& source,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplSource = source;
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawLut(
			palette.getHandle(),
			m_dc,
			&iplSource,
			&iplDest))
		throwImpactError();
}

inline void DeviceContext::drawPalette
(
	const Palette& palette,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawLut(
			palette.getHandle(),
			m_dc,
			NULL,
			&iplDest))
		throwImpactError();
}

inline void DeviceContext::drawPoint
(
	const Point& point,
	const RGBColor& color
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_POINT iplPoint = point;
	if (!mvIMPACT_C::DISP_DrawPoint(
			m_dc,
			color.asDWord(),
			&iplPoint))
		throwImpactError();
}


inline void DeviceContext::drawProfile
(
	const Profile& profile
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::DISP_DrawLut(
			profile.getHandle(),
			m_dc,
			NULL,
			NULL))
		throwImpactError();
}

inline void DeviceContext::drawProfile
(
	const Profile& profile,
	const Rectangle& source,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplSource = source;
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawLut(
			profile.getHandle(),
			m_dc,
			&iplSource,
			&iplDest))
		throwImpactError();
}

inline void DeviceContext::drawProfile
(
	const Profile& profile,
	const Rectangle& dest
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_RECT iplDest = dest;

	if (!mvIMPACT_C::DISP_DrawLut(
			profile.getHandle(),
			m_dc,
			NULL,
			&iplDest))
		throwImpactError();
}


inline Window::Window()
	: counter_(new int(1))
	, handle_(new WindowHandle(0)), must_cleanup_(new bool(false))
	, ooiset_(new mvIMPACT_C::DISP_OOISETHANDLE(-1))
	, buffer_(NULL)
	, windows_(new std::list<Window*>())
	, callbacks_(NULL)
	, oois_(new std::list<ObjectOfInterest*>())
{
	addWindow(this);
	callbacks_ = new callbacks(windows_);
}

inline Window::Window(
	const BufferBase& buffer,
	const WindowHandle* handle,
	bool create,
	bool cleanup)
	: counter_(new int(1))
	, handle_(new WindowHandle(handle ? *handle : (const WindowHandle)0))
	, must_cleanup_(new bool(cleanup))
	, ooiset_(new mvIMPACT_C::DISP_OOISETHANDLE(-1))
	, buffer_(buffer.makeShallowCopy())
	, windows_(new std::list<Window*>())
	, callbacks_(NULL)
	, oois_(new std::list<ObjectOfInterest*>())
{
	addWindow(this);
	callbacks_ = new callbacks(windows_);

	if (create)
	{
		mvIMPACT_C::IPL_HWND createdWindowHandle = (mvIMPACT_C::IPL_HWND)0;

		if (!mvIMPACT_C::DISP_OpenWindow(buffer_->getHandle(), &createdWindowHandle, (mvIMPACT_C::IPL_HWND)getHandle(), windows_))
			throwImpactError();

		*handle_ = createdWindowHandle;
	}
	else
	{
		mvIMPACT_C::IPL_HWND subclassedWindowHandle = *handle_;

		if (!mvIMPACT_C::DISP_OpenWindow(buffer_->getHandle(), &subclassedWindowHandle, 0, windows_))
			throwImpactError();

		*handle_ = subclassedWindowHandle;
	}

	if (!mvIMPACT_C::DISP_OoiSetAlloc(ooiset_))
		throwImpactError();

 	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_OOISET, getHandle(), ooiset_))
 		throwImpactError();

	// set the callbacks:
	setCallbacks();
}

inline Window::Window(
	const Window& r)
	: counter_(r.counter_)
	, handle_(r.handle_), must_cleanup_(r.must_cleanup_)
	, ooiset_(r.ooiset_)
	, buffer_(r.buffer_)
	, windows_(r.windows_)
	, callbacks_(r.callbacks_)
	, oois_(r.oois_)
{
	(*counter_)++;
	addWindow(this);
}

inline Window& Window::operator=(
	const Window& r)
{
	if (this != &r)
	{
		dispose();

		counter_ = r.counter_;
		(*counter_)++;
		handle_ = r.handle_;
		must_cleanup_ = r.must_cleanup_;
		windows_ = r.windows_;
		callbacks_ = r.callbacks_;
		oois_ = r.oois_;

		buffer_ = r.buffer_;
		ooiset_ = r.ooiset_;

		addWindow(this);
	}

	return *this;
}

inline Window::~Window()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline void Window::dispose()
{
	removeWindow(this);
	if (0 == --*counter_)
	{
		if (int(0) != int(getHandle()) && *must_cleanup_)
		{
			if (!mvIMPACT_C::DISP_OoiSetFree(*ooiset_))
				throwImpactError();
			*ooiset_ = -1;

			if (!mvIMPACT_C::DISP_CloseWindow(getHandle()))
				throwImpactError();
			*handle_ = 0;

			delete buffer_;
			buffer_ = 0;
		}

		delete oois_;
		oois_ = 0;

		delete callbacks_;
		callbacks_ = 0;

		delete windows_;
		windows_ = 0;

		delete ooiset_;
		ooiset_ = 0;

		delete handle_;
		handle_ = 0;

		delete counter_;
		counter_ = 0;
	}
}

inline void Window::addWindow(Window* window)
{
	windows_->push_back(window);
}

inline void Window::removeWindow(Window* window)
{
	windows_->remove(window);
}

inline void Window::addOoi(ObjectOfInterest* ooi)
{
	oois_->push_back(ooi);
}

inline void Window::removeOoi(ObjectOfInterest* ooi)
{
	oois_->remove(ooi);
}

inline Window* Window::makeShallowCopy() const
{
	return new Window(*this);
}

inline int Window::getUseCount() const
{
	return *counter_;
}

inline WindowHandle Window::getHandle() const
{
	return *handle_;
}

inline mvIMPACT_C::DISP_OOISETHANDLE Window::getOoiSetHandle() const
{
	return *ooiset_;
}

inline bool Window::getMustCleanup() const
{
	return *must_cleanup_;
}

inline Window& Window::setMustCleanup(bool bClean)
{
	*must_cleanup_ = bClean;
	return *this;
}

inline Window::operator WindowHandle() const
{
	return getHandle();
}

inline void Window::update() const
{
	if (!mvIMPACT_C::DISP_UpdateWindow(getHandle(), mvIMPACT_C::DISP_INVALIDATE))
		throwImpactError();
}

inline void Window::redraw() const
{
	if (!mvIMPACT_C::DISP_UpdateWindow(getHandle(), mvIMPACT_C::DISP_REDRAW))
		throwImpactError();
}

inline void Window::copyToClipboard() const
{
	if (!mvIMPACT_C::DISP_CopyWindow(getHandle()))
		throwImpactError();
}

inline BufferBase& Window::getBuffer() const
{
	return *buffer_;
}

inline Window& Window::setBuffer(
	const BufferBase& buffer)
{
	mvIMPACT_C::IPL_BUFHANDLE bufhandle = buffer.getHandle();
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_BUFHANDLE, getHandle(), &bufhandle))
		throwImpactError();

	(*buffer_) = buffer;

	return *this;
}

inline ProfileDirection Window::getProfileDrawingDir() const
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_PROFILE_DIRECTION, getHandle(), &mode))
		throwImpactError();
	return ProfileDirection(mode);
}

inline Window& Window::setProfileDrawingDir(
	ProfileDirection v)
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_PROFILE_DIRECTION, getHandle(), &mode))
		throwImpactError();

	return *this;
}

inline HistogramMode Window::getHistogramDrawingMode() const
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_HISTOGRAM_MODE, getHandle(), &mode))
		throwImpactError();
	return HistogramMode(mode);
}

inline Window& Window::setHistogramDrawingMode(
	HistogramMode v)
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_HISTOGRAM_MODE, getHandle(), &mode))
		throwImpactError();

	return *this;
}

inline ImageMode Window::getImageDrawingMode() const
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_IMAGE_MODE, getHandle(), &mode))
		throwImpactError();
	return ImageMode(mode);
}

inline Window& Window::setImageDrawingMode(
	ImageMode v)
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_IMAGE_MODE, getHandle(), &mode))
		throwImpactError();

	return *this;
}

inline int Window::getNumericBase() const
{
	int mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_NUMERIC_BASE, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setNumericBase(
	int v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_NUMERIC_BASE, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline Palette Window::getColorPalette() const
{
	mvIMPACT_C::IPL_BUFHANDLE v=-1;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_COLOR_LUT, getHandle(), &v))
		throwImpactError();
	return Palette::paletteFactory(v);
}

inline Window& Window::setColorPalette(
	const Palette& v)
{
	mvIMPACT_C::IPL_BUFHANDLE vh = v.getHandle();
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_COLOR_LUT, getHandle(), &vh))
		throwImpactError();

	return *this;
}

inline int Window::getTextSpacing() const
{
	int mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_TEXT_SPACING, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setTextSpacing(
	int v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_TEXT_SPACING, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline int Window::getTextHeight() const
{
	int mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_TEXT_HEIGHT, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setTextHeight(
	int v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_TEXT_HEIGHT, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline int Window::getTextWeight() const
{
	int mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_TEXT_WEIGHT, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setTextWeight(
	int v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_TEXT_WEIGHT, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline RGBColor Window::getTextColor() const
{
	mvIMPACT_C::IPL_COLOR mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_TEXT_COLOR, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setTextColor(
	const RGBColor color)
{
	mvIMPACT_C::IPL_COLOR iplColor = color;
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_TEXT_COLOR, getHandle(), &iplColor))
		throwImpactError();

	return *this;
}

inline RGBColor Window::getDrawColor() const
{
	mvIMPACT_C::IPL_COLOR mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_DRAW_COLOR, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setDrawColor(
	const RGBColor& color)
{
	mvIMPACT_C::IPL_COLOR iplColor = color;
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_DRAW_COLOR, getHandle(), &iplColor))
		throwImpactError();

	return *this;
}

inline RGBColor Window::getFillColor() const
{
	mvIMPACT_C::IPL_COLOR mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_FILL_COLOR, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setFillColor(
	const RGBColor& color)
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_FILL_COLOR, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline int Window::getNormalize() const
{
	int mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_NORMALIZE, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setNormalize(
	int v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_NORMALIZE, getHandle(), &v))
		throwImpactError();

	return *this;
}

#ifdef PLATFORM_MVBL
inline Window& Window::setDisplayGreenGain( double v )
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_BAYER_DISPLAY_GREEN_GAIN, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline double Window::getDisplayGreenGain() const
{
	double v;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_BAYER_DISPLAY_GREEN_GAIN, getHandle(), &v ))
		throwImpactError();
	return v;
}
#endif	// PLATFORM_MVBL


inline CopyClipboardMode Window::getClipboardMode() const
{
#ifndef linux
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_COPY_WINDOW, getHandle(), &mode))
		throwImpactError();
	return CopyClipboardMode(mode);
#else
	throwImpactException("Not implemented");
	return ccmClient;
#endif
}

inline Window& Window::setClipboardMode(
	CopyClipboardMode v)
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_COPY_WINDOW, getHandle(), &mode))
		throwImpactError();


	return *this;
}

inline double Window::getZoomX() const
{
	double mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_ZOOM_WINDOW_X, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setZoomX(
	double v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_ZOOM_WINDOW_X, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline double Window::getZoomY() const
{
	double mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_ZOOM_WINDOW_Y, getHandle(), &mode))
		throwImpactError();
	return mode;
}

inline Window& Window::setZoomY(
	double v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_ZOOM_WINDOW_Y, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline Window& Window::setZoom(
	double v)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_ZOOM_WINDOW, getHandle(), &v))
		throwImpactError();
	return *this;
}

inline ZoomingMode Window::getZoomMode() const
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_ZOOM_MODE, getHandle(), &mode))
		throwImpactError();
	return ZoomingMode(mode);
}

inline Window& Window::setZoomMode(
	ZoomingMode v)
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_ZOOM_MODE, getHandle(), &mode))
		throwImpactError();

	return *this;
}


inline int Window::getScrollPosX() const
{
	mvIMPACT_C::IPL_LONG pos;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_SCROLLPOS_X, getHandle(), &pos))
		throwImpactError();
	return pos;
}

inline Window& Window::setScrollPosX(
	int v)
{
	mvIMPACT_C::IPL_LONG pos = v;
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_SCROLLPOS_X, getHandle(), &pos))
		throwImpactError();

	return *this;
}

inline int Window::getScrollPosY() const
{
	mvIMPACT_C::IPL_LONG pos;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_SCROLLPOS_Y, getHandle(), &pos))
		throwImpactError();
	return pos;
}

inline Window& Window::setScrollPosY(
	int v)
{
	mvIMPACT_C::IPL_LONG pos = v;
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_SCROLLPOS_Y, getHandle(), &pos))
		throwImpactError();

	return *this;
}

inline Point Window::getScrollPos() const
{
	mvIMPACT_C::IPL_POINT pos;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_SCROLLPOS, getHandle(), &pos))
		throwImpactError();
	return Point(pos.xpos, pos.ypos);
}

inline Window& Window::setScrollPos(
	Point v)
{
	mvIMPACT_C::IPL_POINT pos;
	pos.xpos = v.getX();
	pos.ypos = v.getY();
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_SCROLLPOS, getHandle(), &pos))
		throwImpactError();

	return *this;
}


inline Point Window::getWindowSize() const
{
	mvIMPACT_C::IPL_POINT pos;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_WINDOW_SIZE, getHandle(), &pos))
		throwImpactError();
	return Point(pos.xpos, pos.ypos);
}

inline Window& Window::setWindowSize(
	Point v)
{
	mvIMPACT_C::IPL_POINT pos;
	pos.xpos = v.getX();
	pos.ypos = v.getY();
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_WINDOW_SIZE, getHandle(), &pos))
		throwImpactError();

	return *this;
}


inline Palette Window::getMappingPalette() const
{
	mvIMPACT_C::IPL_BUFHANDLE v=-1;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_MAPPING_LUT, getHandle(), &v))
		throwImpactError();
	return Palette::paletteFactory(v);
}

inline Window& Window::setMappingPalette(
	const Palette& v)
{
	mvIMPACT_C::IPL_BUFHANDLE vh = v.getHandle();
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_MAPPING_LUT, getHandle(), &vh))
		throwImpactError();

	return *this;
}

inline int Window::getShift() const
{
	mvIMPACT_C::IPL_LONG bits;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_SHIFT, getHandle(), &bits))
		throwImpactError();
	return (int)bits;
}

inline Window& Window::setShift(int bits)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_SHIFT, getHandle(), &bits))
		throwImpactError();

	return *this;
}

inline ScrollingMode Window::getScrollbarMode() const
{
	mvIMPACT_C::DISP_MODE mode;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_SCROLLBAR_MODE, getHandle(), &mode))
		throwImpactError();
	return ScrollingMode(mode);
}

inline Window& Window::setScrollbarMode(
	ScrollingMode v)
{
	mvIMPACT_C::DISP_MODE mode(static_cast<mvIMPACT_C::DISP_MODE>(v));
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_SCROLLBAR_MODE, getHandle(), &mode))
		throwImpactError();

	return *this;
}

inline RGBColor Window::getBackgroundColor () const
{
	RGBColor color;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_BACKGROUND_COLOR, getHandle(), &color))
		throwImpactError();
	return color;
}

inline Window& Window::setBackgroundColor (const RGBColor& color)
{
	int v = color.asDWord();
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_BACKGROUND_COLOR, getHandle(), &v))
		throwImpactError();

	return *this;
}

inline std::string Window::getTitle() const
{
	int length = 0;
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_WINDOW_TITLE_TEXT_LENGTH, getHandle(), &length))
		throwImpactError();

	std::vector<char> title(length+1);
	if (!mvIMPACT_C::DISP_InquireWindow(mvIMPACT_C::DISP_WINDOW_TITLE_TEXT, getHandle(), &*title.begin()))
		throwImpactError();

	return std::string(&*title.begin());
}

inline void Window::setTitle(const std::string & title)
{
	if (!mvIMPACT_C::DISP_ControlWindow(mvIMPACT_C::DISP_WINDOW_TITLE_TEXT, getHandle(), title.c_str()))
		throwImpactError();
}

// get/setDefaultEmptyWindowWidth and get/setDefaultEmptyWindowHeight are not wrapped
// since they create an error when applied to windows. They make sense only as default
// settings.

inline void Window::setCallbacks()
{
	callbacks_->setCallbacks();
}

inline Window::callbacks::callbacks(std::list<Window *> * windows)
	: windows_(windows)
{ }

inline void Window::callbacks::setCallback(mvIMPACT_C::DISP_CONTROLTYPE item, void* pCallback)
{
	// Some of the callbacks in the C layer are global by nature, and not associated with
	// a window. Therefore, they need to be set globally as well, so that the C layer
	// can get at the correct pointers in all cases.

	if (!mvIMPACT_C::DISP_Control(item, pCallback))
		throwImpactError();

	for (std::list<Window *>::iterator i=windows_->begin(); i!=windows_->end(); ++i)
	{
		if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
		{
			if (!mvIMPACT_C::DISP_ControlWindow(item, (*i)->getHandle(), pCallback))
				throwImpactError();
		}
	}
}

inline void Window::callbacks::setCallbacks()
{
	setCallback(mvIMPACT_C::DISP_ON_BEFOREDRAW, (void*)DISP_OnBeforeDraw);
	setCallback(mvIMPACT_C::DISP_ON_DRAW, (void*)DISP_OnDraw);
	setCallback(mvIMPACT_C::DISP_ON_MOUSEMOVE, (void*)DISP_OnWindowMouseMove);
	setCallback(mvIMPACT_C::DISP_ON_MOUSEMOVEZOOMED, (void*)DISP_OnMouseMove);
	setCallback(mvIMPACT_C::DISP_ON_BUTTONDOWN, (void*)DISP_OnWindowButtonDown);
	setCallback(mvIMPACT_C::DISP_ON_BUTTONDOWNZOOMED , (void*)DISP_OnButtonDown);
	setCallback(mvIMPACT_C::DISP_ON_BUTTONUP , (void*)DISP_OnWindowButtonUp);
	setCallback(mvIMPACT_C::DISP_ON_BUTTONUPZOOMED , (void*)DISP_OnButtonUp);
	setCallback(mvIMPACT_C::DISP_ON_DBLCLK , (void*)DISP_OnWindowDblClk);
	setCallback(mvIMPACT_C::DISP_ON_DBLCLKZOOMED , (void*)DISP_OnDblClk);
	setCallback(mvIMPACT_C::DISP_ON_KEYDOWN, (void*)DISP_OnKeyDown);
	setCallback(mvIMPACT_C::DISP_ON_SCROLLPOSCHANGED , (void*)DISP_OnScrollPosChange);
	setCallback(mvIMPACT_C::DISP_ON_ZOOMFACTORSCHANGED , (void*)DISP_OnZoomFactorsChange);
	setCallback(mvIMPACT_C::DISP_ON_OOISTARTTRACKING , (void*)DISP_OnOoiStartTracking);
	setCallback(mvIMPACT_C::DISP_ON_OOIENDTRACKING , (void*)DISP_OnOoiEndTracking);
	setCallback(mvIMPACT_C::DISP_ON_OOITRACKTO , (void*)DISP_OnOoiTrackTo);
	setCallback(mvIMPACT_C::DISP_ON_OOITRACKED , (void*)DISP_OnOoiTracked);
	setCallback(mvIMPACT_C::DISP_ON_OOISELECT , (void*)DISP_OnOoiSelect);
}

inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnOoiSelect
(
	mvIMPACT_C::DISP_OOISETHANDLE /*set*/,
	mvIMPACT_C::DISP_OOIHANDLE    hOoi,
	mvIMPACT_C::IPL_BOOL          select,
	mvIMPACT_C::IPL_BOOL*         cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				for (std::list<ObjectOfInterest *>::iterator j=(*i)->oois_->begin(); j!=(*i)->oois_->end(); ++j)
				{
					if (hOoi == (*j)->getHandle())
					{
						*cancel = !(*i)->onOoiSelect(*j, select==IPL_TRUE ? true : false);
						return IPL_TRUE;
					}
				}
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onOoiSelect overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnOoiStartTracking
(
	mvIMPACT_C::DISP_OOISETHANDLE /*set*/,
	mvIMPACT_C::DISP_OOIHANDLE    hOoi,
	mvIMPACT_C::IPL_LONG          handle,
	mvIMPACT_C::IPL_BOOL*         cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				for (std::list<ObjectOfInterest *>::iterator j=(*i)->oois_->begin(); j!=(*i)->oois_->end(); ++j)
				{
					if (hOoi == (*j)->getHandle())
					{
						*cancel = !(*i)->onOoiStartTrack(*j, handle);
						return IPL_TRUE;
					}
				}
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onOoiStartTrack overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnOoiEndTracking
(
	mvIMPACT_C::DISP_OOISETHANDLE /*set*/,
	mvIMPACT_C::DISP_OOIHANDLE    hOoi,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				for (std::list<ObjectOfInterest *>::iterator j=(*i)->oois_->begin(); j!=(*i)->oois_->end(); ++j)
				{
					if (hOoi == (*j)->getHandle())
					{
						(*i)->onOoiStopTrack(*j);
						return IPL_TRUE;
					}
				}
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onOoiStopTrack overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnOoiTrackTo
(
	mvIMPACT_C::DISP_OOISETHANDLE /*set*/,
	mvIMPACT_C::DISP_OOIHANDLE    hOoi,
	mvIMPACT_C::IPL_LONG          handle,
	const mvIMPACT_C::IPL_POINT*  point,
	mvIMPACT_C::IPL_BOOL*         cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				for (std::list<ObjectOfInterest *>::iterator j=(*i)->oois_->begin(); j!=(*i)->oois_->end(); ++j)
				{
					if (hOoi == (*j)->getHandle())
					{
						Point cppPoint = *point;
						*cancel = !(*i)->onOoiTrackTo(*j, handle, &cppPoint);
						return IPL_TRUE;
					}
				}
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onOoiTrackTo overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnOoiTracked
(
	mvIMPACT_C::DISP_OOISETHANDLE /*set*/,
	mvIMPACT_C::DISP_OOIHANDLE    hOoi,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				for (std::list<ObjectOfInterest *>::iterator j=(*i)->oois_->begin(); j!=(*i)->oois_->end(); ++j)
				{
					if (hOoi == (*j)->getHandle())
					{
						(*i)->onOoiTracked(*j);
						return IPL_TRUE;
					}
				}
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onOoiTracked overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnBeforeDraw
(
    mvIMPACT_C::IPL_HWND         /*window*/,
    mvIMPACT_C::IPL_BUFHANDLE    /*buffer*/,
	mvIMPACT_C::IPL_HDC          hdc,
    const mvIMPACT_C::IPL_RECT*  source,
    const mvIMPACT_C::IPL_RECT*  dest,
    mvIMPACT_C::IPL_BOOL*        cancel,
    mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	DeviceContext Dc = hdc;
	Rectangle* pSourceRect    = (Rectangle*)source;
	Rectangle* pDestRect      = (Rectangle*)dest;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onBeforeDraw(&Dc, pSourceRect, pDestRect);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onBeforeDraw overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnDraw
(
	mvIMPACT_C::IPL_HWND         /*window*/,
	mvIMPACT_C::IPL_BUFHANDLE    /*buffer*/,
	mvIMPACT_C::IPL_HDC          hdc,
	const mvIMPACT_C::IPL_RECT * source,
	const mvIMPACT_C::IPL_RECT * dest ,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	DeviceContext Dc = hdc;
	Rectangle* pSourceRect    = (Rectangle*)source;
	Rectangle* pDestRect      = (Rectangle*)dest;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				(*i)->onDraw(&Dc, pSourceRect, pDestRect);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onDraw overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnWindowButtonDown
(
	mvIMPACT_C::IPL_HWND         /*window*/,
	const mvIMPACT_C::IPL_POINT* point,
	mvIMPACT_C::DISP_MOUSEBUTTON button,
	mvIMPACT_C::IPL_BOOL*        cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	Point cppPoint = *point;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseButtonDownWindow((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseButtonDownWindow overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnWindowButtonUp
(
	mvIMPACT_C::IPL_HWND         /*window*/,
	const mvIMPACT_C::IPL_POINT* point,
	mvIMPACT_C::DISP_MOUSEBUTTON button,
	mvIMPACT_C::IPL_BOOL*        cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	Point cppPoint = *point;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseButtonUpWindow((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseButtonUpWindow overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnWindowDblClk
(
	mvIMPACT_C::IPL_HWND         /*window*/,
	const mvIMPACT_C::IPL_POINT* point,
	mvIMPACT_C::DISP_MOUSEBUTTON button,
	mvIMPACT_C::IPL_BOOL*        cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	Point cppPoint = *point;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseDoubleClickWindow((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseDoubleClickWindow overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnWindowMouseMove
(
	mvIMPACT_C::IPL_HWND         /*window*/,
	const mvIMPACT_C::IPL_POINT* point,
	mvIMPACT_C::DISP_MOUSEBUTTON button,
	mvIMPACT_C::IPL_BOOL*        cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	Point cppPoint = *point;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseMoveWindow((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseMoveWindow overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnButtonDown
(
	mvIMPACT_C::IPL_HWND                /*window*/,
	const mvIMPACT_C::IPL_POINT_DOUBLE* point,
	mvIMPACT_C::DISP_MOUSEBUTTON        button,
	mvIMPACT_C::IPL_BOOL*               cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR      userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	PointDouble cppPoint = *point;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseButtonDown((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseButtonDown overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnButtonUp
(
	mvIMPACT_C::IPL_HWND                /*window*/,
	const mvIMPACT_C::IPL_POINT_DOUBLE* point,
	mvIMPACT_C::DISP_MOUSEBUTTON        button,
	mvIMPACT_C::IPL_BOOL*               cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR      userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	PointDouble cppPoint = *point;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseButtonUp((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseButtonUp overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnDblClk
(
	mvIMPACT_C::IPL_HWND                /*window*/,
	const mvIMPACT_C::IPL_POINT_DOUBLE* point,
	mvIMPACT_C::DISP_MOUSEBUTTON        button,
	mvIMPACT_C::IPL_BOOL*               cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR      userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	PointDouble cppPoint = *point;
	try
	{
		*cancel = !(windows->front()->onMouseDoubleClick((MouseButton)button, &cppPoint));
		return IPL_TRUE;
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseDoubleClick((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseDoubleClick overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnMouseMove
(
	mvIMPACT_C::IPL_HWND                /*window*/,
	const mvIMPACT_C::IPL_POINT_DOUBLE* point,
	mvIMPACT_C::DISP_MOUSEBUTTON        button,
	mvIMPACT_C::IPL_BOOL*               cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR      userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	PointDouble cppPoint = *point;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onMouseMove((MouseButton)button, &cppPoint);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onMouseMove overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnKeyDown
(
	mvIMPACT_C::IPL_HWND        /*window*/,
	mvIMPACT_C::IPL_ULONG       character,
	mvIMPACT_C::IPL_BOOL*       cancel,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				*cancel = !(*i)->onKeyDown(character);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onKeyDown overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnScrollPosChange
(
	mvIMPACT_C::IPL_HWND        /*window*/,
	mvIMPACT_C::IPL_ULONG       x,
	mvIMPACT_C::IPL_ULONG       y,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				(*i)->onScrollPosChange(x, y );
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onScrollPosChange overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE Window::callbacks::DISP_OnZoomFactorsChange
(
	mvIMPACT_C::IPL_HWND        /*window*/,
	mvIMPACT_C::IPL_DOUBLE      x,
	mvIMPACT_C::IPL_DOUBLE      y,
	mvIMPACT_C::IPL_CONST_VOID_PTR userPtr
)
{
	std::list<Window *> * windows = (std::list<Window *> *)userPtr;
	try
	{
		for (std::list<Window *>::iterator i=windows->begin(); i!=windows->end(); ++i)
		{
			if (0 != (mvIMPACT_C::IPL_HWND)(*i)->getHandle())
			{
				(*i)->onZoomFactorsChange(x, y);
				return IPL_TRUE;
			}
		}
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace("Caught exception in onZoomFactorsChange overload.");
		return IPL_FALSE;
	}

	return IPL_FALSE;
}


#endif  // !defined(I386_LINUX) && !defined(__ppc_6xx__)

inline bool Tracer::onContourPixel(int /*x*/, int /*y*/)
	{
		return true;
	}


inline mvIMPACT_C::IPL_LONG IPL_C_CALLTYPE Tracer::onEventPixel
	(
		mvIMPACT_C::IPL_LONG  xPos,
		mvIMPACT_C::IPL_LONG  yPos,
		mvIMPACT_C::IPL_VOID_PTR pUser
	)
	{
		Tracer* pTracer = (Tracer*)pUser;

		mvIMPACT_C::IPL_LONG result;

		try {
			if (pTracer->onContourPixel(xPos,yPos)) { result = ATR_CONTINUE; }
			else { result = ATR_STOP; }
		} catch(...) {
			mvIMPACT_C::IPL_Trace(" caught exception in callback");
			result = ATR_STOP; // in case of an error, stop tracing
		}

		return result;
	}


inline ContourList::ContourList()
		: m_pList(NULL), m_bContinue(false)
{
}

inline ContourList::ContourList(const ContourList& c)
		: m_pList(c.m_pList), m_bContinue(c.m_bContinue)
{
}


inline ContourList& ContourList::operator= (const ContourList& c)
{
	m_pList = c.m_pList;
	m_bContinue = c.m_bContinue;
	return *this;
}


inline PointDouble ContourList::getPoint
	(
		int index
	) const
	{
		if (index<0 || index>=(int)m_pList->TotalPoints)
		{
			throwImpactException("Index out of range");
		}

		return m_pList->Points[index];
	}

inline PointDouble ContourList::operator[]
	(
		int index
	) const
	{
		return getPoint(index);
	}

inline ContourList& ContourList::setPoint(int index, PointDouble p)
{
	if(index < 0 || index >= (int)m_pList->TotalPoints)
	{
		throwImpactException("Index out of range");
	}
	m_pList->Points[index] = p;
	return *this;
}


inline int ContourList::getNumberOfPoints () const
	{
		return (int)(m_pList->TotalPoints);
	}

inline int ContourList::getSearchRadius () const
	{
		return (int)(m_pList->SearchRadius);
	}

inline void ContourList::continueTracing
	(
		int fromIndex,
		PointDouble toPoint
	)
	{
		if (fromIndex<0 || fromIndex>=(int)m_pList->TotalPoints)
		{
			throwImpactException("Index out of range");
		}

		m_pList->FromPoint    = fromIndex;
		m_pList->ToPoint.xpos = toPoint.getX();
		m_pList->ToPoint.ypos = toPoint.getY();

		m_bContinue = true;
	}

inline ContourList::ContourList (mvIMPACT_C::ATR_LIST* pList)
	{
        m_pList     = pList;
		m_bContinue = false;
	}

inline bool ContourList::doContinue() const
	{
		return m_bContinue;
	}

inline ContourList::~ContourList()
{
	// actually do nothing here
}

inline void Tracer::onEvent
	(
		TraceStatus /*type*/,
		ContourList& /*contourList*/
	)
	{
	}


inline mvIMPACT_C::IPL_LONG IPL_C_CALLTYPE Tracer::onEventTracer
	(
		mvIMPACT_C::IPL_LONG  status,
		mvIMPACT_C::ATR_LIST* pListPtr,
		mvIMPACT_C::IPL_VOID_PTR pUser
	)
	{
		Tracer *pTracer = (Tracer*)pUser;
		ContourList contourList (pListPtr); // Create an ATR_LIST Wrapper

		mvIMPACT_C::IPL_LONG result;
		try {
			pTracer->onEvent((TraceStatus)status, contourList);
		} catch(...) {
			mvIMPACT_C::IPL_Trace(" caught exception in callback");
			result = ATR_STOP; // in case of an error, stop tracing
		}

		if (contourList.doContinue()) {
			result = ATR_CONTINUE;
		} else {
			result = ATR_STOP;
		}

		return result;
	}


inline TraceResult::TraceResult ()
		:	m_handle(unassigned), m_counter(new int(1)),
			m_numberOfPoints(0), m_Points(NULL),
			m_status(tsOk),  m_PolygonBuffer(unassigned)
{
}


inline TraceStatus TraceResult::getStatus () const
{
	return m_status;
}

inline PointDouble TraceResult::getPoint(int index) const
{
	if (index<0 || index>=getNumberOfPoints())
	{
		throwImpactException("Index out of range");
	}

	return m_Points[index];
}

inline PointDouble TraceResult::operator[](int index) const
{
	return getPoint(index);
}

inline TraceResult& TraceResult::setPoint(int index, PointDouble p)
{
	if (index<0 || index>=getNumberOfPoints())
	{
		throwImpactException("Index out of range");
	}
	m_Points[index] = p;
	return *this;
}

inline int TraceResult::getNumberOfPoints () const
{
	return m_numberOfPoints;
}

inline TraceResult::TraceResult (const TraceResult& m)
{
	copy(m);
}

inline TraceResult& TraceResult::operator= (const TraceResult& rhs)
{
	if (m_counter != rhs.m_counter)
	{
		dispose();
		copy(rhs);
	}
	return *this;
}


 inline TraceResult::TraceResult (mvIMPACT_C::IPL_BUFHANDLE handle)
		:	m_handle(handle), m_counter(new int(1))
{
	if (!mvIMPACT_C::ATR_GetResults(getHandle(), mvIMPACT_C::ATR_STATUS, &m_status))
		throwImpactError();
	if (!mvIMPACT_C::ATR_GetResults(getHandle(), mvIMPACT_C::ATR_NB_POINTS, &m_numberOfPoints))
		throwImpactError();
	if (!mvIMPACT_C::ATR_GetResults(getHandle(), mvIMPACT_C::ATR_POINTS, &m_Points))
		throwImpactError();
	if (!mvIMPACT_C::ATR_GetResults(getHandle(), mvIMPACT_C::ATR_POLYGON_BUFFER, &m_PolygonBuffer))
		throwImpactError();
}

inline int TraceResult::getUseCount() const { return *m_counter; }

inline bool TraceResult::getIsUnique   () const { return 1 == *m_counter; }

inline mvIMPACT_C::IPL_BUFHANDLE TraceResult::getHandle  () const { return m_handle; }

inline mvIMPACT_C::IPL_BUFHANDLE TraceResult::getPolygonBuffer() const
{
	return m_PolygonBuffer;
}

inline void TraceResult::copy(const TraceResult& src)
{
	m_counter = src.m_counter;
	m_handle         = src.m_handle;
	m_status         = src.m_status;
	m_numberOfPoints = src.m_numberOfPoints;
	m_Points        = src.m_Points;
	m_PolygonBuffer = src.m_PolygonBuffer;

	(*m_counter)++;
}

inline void TraceResult::dispose()
{
	if (0 == --*m_counter)
	{
		delete m_counter;
		m_counter = 0;

		if (unassigned != m_handle)
		{
			if (!mvIMPACT_C::ATR_Free(m_handle))
				throwImpactError();
			m_handle = unassigned;
		}
		if(unassigned != m_PolygonBuffer)
		{
			if(!mvIMPACT_C::IPL_BufFree(m_PolygonBuffer))
				throwImpactError();
			m_PolygonBuffer = unassigned;
		}
	}
}

inline TraceResult::~TraceResult ()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
		mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}


inline TraceResult Tracer::trace
	(
		const Image& S,
		const Line& line1,
		int  bandOffset  ,
		int  sliceOffset ,
		int  frameOffset
	)
	{
		return trace(S,line1,line1,bandOffset,sliceOffset,frameOffset);
	}

inline TraceResult Tracer::trace
	(
		const Image& S,
		const Line& line1,
		const Line& line2,
		int  bandOffset  ,
		int  sliceOffset ,
		int  frameOffset
	)
	{
		transferConfiguration();
		setCallbacks();

		mvIMPACT_C::IPL_BUFHANDLE result = -1;

		mvIMPACT_C::IPL_POINT iplL1p0 = line1.getP0();
		mvIMPACT_C::IPL_POINT iplL1p1 = line1.getP1();
		mvIMPACT_C::IPL_POINT iplL2p0 = line2.getP0();
		mvIMPACT_C::IPL_POINT iplL2p1 = line2.getP1();

		if (!mvIMPACT_C::ATR_Trace (
				S.getHandle(),
				bandOffset,
				sliceOffset,
				frameOffset,
				&iplL1p0,
				&iplL1p1,
				&iplL2p0,
				&iplL2p1,
				&result))
			throwImpactError();

		removeCallbacks();

		return TraceResult(result);
	}

inline Tracer::Tracer()
{
	m_maxPoints = 819;
	m_filterSize = tk5x5;
	m_numberOfTestPoints = 10;
	m_startDirection = sdEast;
	m_minThreshold = 0;
	m_maxThreshold = 1;
	m_neighborhoodMode = nm8Connected;
	m_startIndex = -1;
	m_startPolarity = 1;
	m_startThreshold = 0.04;

	m_isEtDoneEnabled = false;
	m_isEtMinimumThresholdEnabled = false;
	m_isEtMaximumThresholdEnabled = false;
	m_isEtPointTwiceEnabled = false;
	m_isEtBorderEnabled = false;
	m_isEtSafeModeEnabled = false;
}

inline Tracer::~Tracer()
{
}

inline void Tracer::configureEvent
	(
		EventType type,
		bool bEnable
	)
	{
		switch (type)
		{
		case etDone:
			m_isEtDoneEnabled = bEnable;
			break;
		case etMinimumThreshold:
			m_isEtMinimumThresholdEnabled = bEnable;
			break;
		case etMaximumThreshold:
			m_isEtMaximumThresholdEnabled = bEnable;
			break;
		case etPointTwice:
			m_isEtPointTwiceEnabled = bEnable;
			break;
		case etBorder:
			m_isEtBorderEnabled = bEnable;
			break;
		case etSafeMode:
			m_isEtSafeModeEnabled = bEnable;
			break;
		default:
			throwImpactException("Unknown event type");
		}
	}

inline void Tracer::transferConfiguration() const
{
	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_MAX_POINTS, (void*)&m_maxPoints))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_FILTER_SIZE, (void*)&m_filterSize))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_TEST_POINTS, (void*)&m_numberOfTestPoints))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_DIR, (void*)&m_startDirection))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_MIN_THRES, (void*)&m_minThreshold))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_MAX_THRES, (void*)&m_maxThreshold))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_LATTICE_MODE, (void*)&m_neighborhoodMode))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_POL_START, (void*)&m_startPolarity))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_INDEX_START, (void*)&m_startIndex))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_Control(mvIMPACT_C::ATR_MIN_GRAD_START, (void*)&m_startThreshold))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_SetEvent (
		mvIMPACT_C::ATR_EVENTFLAGS(etDone),
		m_isEtDoneEnabled ? mvIMPACT_C::ATR_ON : mvIMPACT_C::ATR_OFF))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_SetEvent (
		mvIMPACT_C::ATR_EVENTFLAGS(etMinimumThreshold),
		m_isEtMinimumThresholdEnabled ? mvIMPACT_C::ATR_ON : mvIMPACT_C::ATR_OFF))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_SetEvent (
		mvIMPACT_C::ATR_EVENTFLAGS(etMaximumThreshold),
		m_isEtMaximumThresholdEnabled ? mvIMPACT_C::ATR_ON : mvIMPACT_C::ATR_OFF))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_SetEvent (
		mvIMPACT_C::ATR_EVENTFLAGS(etPointTwice),
		m_isEtPointTwiceEnabled ? mvIMPACT_C::ATR_ON : mvIMPACT_C::ATR_OFF))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_SetEvent (
		mvIMPACT_C::ATR_EVENTFLAGS(etBorder),
		m_isEtBorderEnabled ? mvIMPACT_C::ATR_ON : mvIMPACT_C::ATR_OFF))
	{ throwImpactError(); }

	if (!mvIMPACT_C::ATR_SetEvent (
		mvIMPACT_C::ATR_EVENTFLAGS(etSafeMode),
		m_isEtSafeModeEnabled ? mvIMPACT_C::ATR_ON : mvIMPACT_C::ATR_OFF))
	{ throwImpactError(); }
}


inline Tracer& Tracer::setMaxPoints (int points)
	{
		m_maxPoints = points;
		return *this;
	}

inline int Tracer::getMaxPoints () const
	{
		return m_maxPoints;
	}

inline Tracer& Tracer::setFilterSize (TracingKernel kernel)
	{
		m_filterSize = kernel;
		return *this;
	}

inline TracingKernel Tracer::getFilterSize () const
	{
		return m_filterSize;
	}

inline Tracer& Tracer::setNumberOfTestPoints (int points)
	{
		m_numberOfTestPoints = points;
		return *this;
	}

inline int Tracer::getNumberOfTestPoints () const
	{
		return m_numberOfTestPoints;
	}

inline Tracer& Tracer::setStartDirection (StartDirection dir)
	{
		m_startDirection = dir;
		return *this;
	}

inline StartDirection Tracer::getStartDirection () const
	{
		return m_startDirection;
	}

inline Tracer& Tracer::setMinThreshold (double threshold)
	{
		m_minThreshold = threshold;
		return *this;
	}

inline double Tracer::getMinThreshold () const
	{
		return m_minThreshold;
	}

inline Tracer& Tracer::setMaxThreshold (double threshold)
	{
		m_maxThreshold = threshold;
		return *this;
	}

inline double Tracer::getMaxThreshold () const
	{
		return m_maxThreshold;
	}

inline Tracer& Tracer::setStartPolarity (int polarity)
	{
		m_startPolarity = polarity;
		return *this;
	}

inline int Tracer::getStartPolarity () const
	{
		return m_startPolarity;
	}

inline Tracer& Tracer::setStartIndex (int index)
	{
		m_startIndex = index;
		return *this;
	}

inline int Tracer::getStartIndex () const
	{
		return m_startIndex;
	}

inline Tracer& Tracer::setStartThreshold (double threshold)
	{
		m_startThreshold = threshold;
		return *this;
	}

inline double Tracer::getStartThreshold () const
	{
		return m_startThreshold;
	}

inline Tracer& Tracer::setNeighborhoodMode (NeighborhoodMode mode)
	{
		m_neighborhoodMode = mode;
		return *this;
	}

inline NeighborhoodMode Tracer::getNeighborhoodMode () const
	{
		return m_neighborhoodMode;
	}

inline void Tracer::setCallbacks()
{
	if(!mvIMPACT_C::ATR_SetEventFunction (mvIMPACT_C::ATR_CALLBACK_EVENT, (mvIMPACT_C::IPL_VOID_PTR)onEventTracer, this))
	{ throwImpactError(); }

	if(!mvIMPACT_C::ATR_SetEventFunction (mvIMPACT_C::ATR_CALLBACK_PIXEL,  (mvIMPACT_C::IPL_VOID_PTR)onEventPixel, this))
	{ throwImpactError(); }
}

inline void Tracer::removeCallbacks()
{
	if(!mvIMPACT_C::ATR_SetEventFunction (mvIMPACT_C::ATR_CALLBACK_EVENT, NULL, NULL))
	{ throwImpactError(); }

	if(!mvIMPACT_C::ATR_SetEventFunction (mvIMPACT_C::ATR_CALLBACK_PIXEL, NULL, NULL))
	{ throwImpactError(); }
}


} // namespace mvIMPACT


#endif // #ifndef WRAP_ANY
#endif // DOXYGEN_SHOULD_SKIP_THIS

#endif // MVIMPACT_IMPL_H_
